@@ -1,7 +1,7 @@
 /*
  * fcntl.c - fcntl interface
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: fcntl.c,v 1.24 2008-05-10 13:35:39 shirok Exp $
  */
 
 #define _GNU_SOURCE  /* for Linux, this enables additional features */
@@ -203,11 +201,11 @@ ScmObj Scm_SysFcntl(ScmObj port_or_fd, int op, ScmObj arg)
 
 extern void Scm_Init_fcntlib(ScmModule *mod);
 
-SCM_EXTENSION_ENTRY void Scm_Init_fcntl(void)
+SCM_EXTENSION_ENTRY void Scm_Init_gauche__fcntl(void)
 {
     ScmModule *mod;
 
-    SCM_INIT_EXTENSION(fcntl);
+    SCM_INIT_EXTENSION(gauche__fcntl);
     mod = SCM_FIND_MODULE("gauche.fcntl", SCM_FIND_MODULE_CREATE);
     Scm_InitStaticClass(&Scm_SysFlockClass, "<sys-flock>",
                         mod, flock_slots, 0);
