@@ -11,231 +11,40 @@
 (use file.util)
 (test-module 'file.util)
 
-(test "current-directory" '("/" "/" #t #t)
-      (lambda ()
-        (let* ((cur   (sys-getcwd))
-               (root  (begin (current-directory "/")
-                             (sys-getcwd)))
-               (root* (current-directory))
-               (cur*  (begin (current-directory cur)
-                             (sys-getcwd)))
-               (cur** (current-directory)))
-          (list root root* (string=? cur cur*) (string=? cur* cur**)))))
+;; shorthand of normalizing pathname.  this doesn't do anything on
+;; unix, but on Windows the separator in PATHNAME is replaced.
+;; NB: we remove the drive letter from DOS path to accomodate
+;; different installations.
+(define (n pathname . pathnames)
+  (define (normalize s)
+    (let1 s (sys-normalize-pathname s)
+      (if (#/^[a-zA-Z]:/ s) (string-drop s 2) s)))
+  (if (null? pathnames)
+    (normalize pathname)
+    (map normalize (cons pathname pathnames))))
 
-;; prepare test data set
-(sys-system "rm -rf test.out test2.out")
-(sys-system "mkdir test.out")
-(with-output-to-file "test.out/test1.o" (cut display (make-string 100 #\o)))
-(with-output-to-file "test.out/test2.o" (cut display (make-string 100 #\o)))
-(with-output-to-file "test.out/test3.o" (cut display (make-string 100 #\i)))
-(with-output-to-file "test.out/test4.o" (cut display (make-string 20000 #\i)))
-(with-output-to-file "test.out/test5.o" (cut display (make-string 20000 #\i)))
+;; mingw doesn't have fully compatible permissions as unix.
+;; this procedure compensates it.  
+(define (P perm)
+  (cond-expand
+   [gauche.os.windows (let1 p (ash perm -6)
+                        (logior p (ash p 3) (ash p 6)))]
+   [else perm]))
 
-(cond-expand
- [gauche.sys.symlink
-  (sys-symlink "test1.o" "test.out/test6.o")
-  (sys-symlink "test6.o" "test.out/test7.o")
-  (sys-symlink "test.d" "test.out/test2.d")]
- [else
-  (with-output-to-file "test.out/test6.o" (cut newline))
-  (with-output-to-file "test.out/test7.o" (cut newline))
-  (sys-mkdir "test.out/test2.d" #o777)])
-
-(sys-system "mkdir test.out/test.d")
-(with-output-to-file "test.out/test.d/test10.o"
-  (cut display (make-string 100 #\o)))
-
-(cond-expand
- [gauche.sys.symlink
-  (sys-symlink "../test1.o" "test.out/test.d/test11.o")
-  (test* "file-is-symlink?" #t
-         (file-is-symlink? "test.out/test.d/test11.o"))]
- [else
-  (with-output-to-file "test.out/test.d/test11.o" (cut newline))])
-
-(test* "directory-list"
-       '("." ".." "test.d" "test1.o" "test2.d" "test2.o"
-         "test3.o" "test4.o" "test5.o" "test6.o" "test7.o" )
-       (directory-list "test.out"))
-
-(test* "directory-list :children?"
-       '("test.d" "test1.o" "test2.d" "test2.o"
-         "test3.o" "test4.o" "test5.o" "test6.o" "test7.o" )
-       (directory-list "test.out" :children? #t))
-
-(test* "directory-list :add-path?"
-       '("test.out/." "test.out/.." "test.out/test.d" "test.out/test1.o"
-         "test.out/test2.d" "test.out/test2.o"  "test.out/test3.o"
-         "test.out/test4.o" "test.out/test5.o" "test.out/test6.o"
-         "test.out/test7.o" )
-       (directory-list "test.out/" :add-path? #t))
-
-(test* "directory-list :filter"
-       '("test.out/test1.o"
-         "test.out/test2.o"  "test.out/test3.o" "test.out/test4.o"
-         "test.out/test5.o" "test.out/test6.o" "test.out/test7.o" )
-       (directory-list "test.out" :add-path? #t
-                       :filter (lambda (p) (string-suffix? "o" p))))
-
-(test* "directory-list :filter"
-       '("test.out/test1.o"
-         "test.out/test2.o"  "test.out/test3.o" "test.out/test4.o"
-         "test.out/test5.o" "test.out/test6.o" "test.out/test7.o" )
-       (directory-list "test.out" :add-path? #t :filter-add-path? #t
-                       :filter file-is-regular?))
-
-(test* "directory-list2"
-       '(("." ".." "test.d" "test2.d")
-         ("test1.o" "test2.o" "test3.o" "test4.o"
-          "test5.o" "test6.o" "test7.o" ))
-       (receive x (directory-list2 "test.out") x))
-
-(test* "directory-list2 :add-path"
-       '(("test.out/." "test.out/.." "test.out/test.d" "test.out/test2.d")
-         ("test.out/test1.o" "test.out/test2.o"  "test.out/test3.o"
-          "test.out/test4.o" "test.out/test5.o" "test.out/test6.o"
-          "test.out/test7.o"))
-       (receive x (directory-list2 "test.out" :add-path? #t) x))
-
-(test* "directory-list2 :children"
-       '(("test.out/test.d" "test.out/test2.d")
-         ("test.out/test1.o" "test.out/test2.o"  "test.out/test3.o"
-          "test.out/test4.o" "test.out/test5.o" "test.out/test6.o"
-          "test.out/test7.o"))
-       (receive x (directory-list2 "test.out" :add-path? #t :children? #t) x))
-
-(test* "directory-list2 :filter"
-       '(("test.d" "test2.d")
-         ("test1.o" "test2.o" "test3.o" "test4.o"
-          "test5.o" "test6.o" "test7.o" ))
-       (receive x 
-           (directory-list2 "test.out"
-                            :filter (lambda (p) (string-contains p "test")))
-         x))
-
-(cond-expand
- [gauche.sys.symlink
-  (test* "directory-list2 :follow-link? #f"
-         '(("test.d")
-           ("test1.o" "test2.d" "test2.o" "test3.o" "test4.o"
-            "test5.o" "test6.o" "test7.o" ))
-         (receive x (directory-list2 "test.out" :follow-link? #f :children? #t)
-           x)
-         )]
- [else])
-
-(test* "directory-fold"
-       (cond-expand
-        [gauche.sys.symlink
-         '("test.out/test.d/test10.o" "test.out/test.d/test11.o"
-           "test.out/test1.o"
-           "test.out/test2.d/test10.o" "test.out/test2.d/test11.o"
-           "test.out/test2.o" "test.out/test3.o"
-           "test.out/test6.o" "test.out/test7.o")]
-        [else
-         '("test.out/test.d/test10.o" "test.out/test.d/test11.o"
-           "test.out/test1.o"
-           "test.out/test2.o" "test.out/test3.o"
-           "test.out/test6.o" "test.out/test7.o")])
-       (reverse
-        (directory-fold "test.out"
-                        (lambda (path result)
-                          (if (= (file-size path) 100)
-                              (cons path result)
-                              result))
-                        '()))
-       )
-
-(test* "directory-fold"
-       '("test.out"
-         "test.out/test.d"
-         "test.out/test.d/test10.o" "test.out/test.d/test11.o"
-         "test.out/test1.o"
-         "test.out/test2.d"
-         "test.out/test2.d/test10.o"
-         "test.out/test2.d/test11.o"
-         "test.out/test2.o" "test.out/test3.o"
-         "test.out/test4.o" "test.out/test5.o"
-         "test.out/test6.o" "test.out/test7.o")
-       (reverse
-        (directory-fold "test.out" cons '()
-                        :lister (lambda (path seed)
-                                  (values
-                                   (directory-list path
-                                                   :add-path? #t
-                                                   :children? #t)
-                                   (cons path seed)))))
-       )
-
-(cond-expand
- [gauche.sys.symlink
-  (test* "directory-fold :follow-link? #f"
-         '("test.out/test.d/test10.o" "test.out/test.d/test11.o"
-           "test.out/test1.o"
-           "test.out/test2.o" "test.out/test3.o"
-           "test.out/test6.o" "test.out/test7.o")
-         (reverse 
-          (directory-fold "test.out"
-                          (lambda (path result)
-                            (if (= (file-size path) 100)
-                                (cons path result)
-                                result))
-                          '()
-                          :follow-link? #f)))
-  ;; this tests dangling symlink
-  (sys-symlink "foo" "test.out/test.dangling")
-  (test* "directory-fold :follow-link? #t; dangling link"
-         '("test.out/test.d/test10.o" "test.out/test.d/test11.o"
-           "test.out/test.dangling" "test.out/test1.o"
-           "test.out/test2.d/test10.o" "test.out/test2.d/test11.o"
-           "test.out/test2.o" "test.out/test3.o" "test.out/test4.o"
-           "test.out/test5.o" "test.out/test6.o" "test.out/test7.o")
-         (sort (directory-fold "test.out" cons '())))
-  (sys-unlink "test.out/test.dangling")]
- [else])
-
-(test* "directory-fold :lister"
-       '("test.out/test.d/test10.o" "test.out/test.d/test11.o" "test.out/test1.o")
-       (reverse
-        (directory-fold "test.out" cons '()
-                        :follow-link? #f
-                        :lister (lambda (dir knil)
-                                  (receive (dirs files)
-                                      (directory-list2 dir :add-path? #t :children? #t :follow-link? #f)
-                                    (append dirs
-                                            (filter (cut string-contains <> "test1")
-                                                    files))))))
-       )
+;;=====================================================================
+(test-section "pathname utils")
 
 (test* "build-path" "" (build-path ""))
 (test* "build-path" "." (build-path "."))
-(test* "build-path" "/" (build-path "/"))
-(test* "build-path" "a/b/c" (build-path "a" "b" "c"))
-(test* "build-path" "a/b/c" (build-path "a/" "b/" "c"))
-(test* "build-path" "/a/b/c" (build-path "/" "a/b" "c"))
-(test* "build-path" "./a/b/c" (build-path "." "a/b" "c"))
-(test* "build-path" *test-error* (build-path "." "/a/b"))
-(test* "build-path" "foo" (build-path "" "foo"))
-(test* "build-path" "foo/bar" (build-path "" "foo" "" "bar"))
-(test* "build-path" "foo" (build-path "" "foo" ""))
-
-(test* "resolve-path" "/" (resolve-path "/"))
-(test* "resolve-path" "." (resolve-path "."))
-(test* "resolve-path" "test.out" (resolve-path "test.out"))
-(cond-expand
- [gauche.sys.symlink
-  (test* "resolve-path" "test.out/test1.o"
-         (resolve-path "test.out/test6.o"))
-  (test* "resolve-path" "test.out/test1.o"
-         (resolve-path "test.out/test7.o"))
-  (test* "resolve-path" "test.out/test1.o"
-         (resolve-path "test.out/test2.d/test11.o"))
-  (test* "resolve-path" "test.out/test1.o"
-         (resolve-path "test.out/test2.d/../test.d/test11.o"))
-  (test* "resolve-path" "test.out/test1.o"
-         (resolve-path "test.out/test.d/../test2.d/test11.o"))
-  ]
- [else])
+(test* "build-path" (n "/") (build-path "/"))
+(test* "build-path" (n "a/b/c") (build-path "a" "b" "c"))
+(test* "build-path" (n "a/b/c") (build-path "a/" "b/" "c"))
+(test* "build-path" (n "/a/b/c") (build-path "/" "a/b" "c"))
+(test* "build-path" (n "./a/b/c") (build-path "." "a/b" "c"))
+(test* "build-path" (test-error) (build-path "." "/a/b"))
+(test* "build-path" (n "foo") (build-path "" "foo"))
+(test* "build-path" (n "foo/bar") (build-path "" "foo" "" "bar"))
+(test* "build-path" (n "foo") (build-path "" "foo" ""))
 
 (test* "decompose-path" '("/a/b/c" "d" "e")
        (receive r (decompose-path "/a/b/c/d.e") r))
@@ -290,6 +99,268 @@
 (test* "path-swap-extension" ".a.o" (path-swap-extension ".a.c" "o"))
 
 
+;;=====================================================================
+(test-section "directories")
+
+(test "current-directory" (list (n "/") (n "/") #t #t)
+      (lambda ()
+        (let* ((cur   (sys-getcwd))
+               (root  (begin (current-directory "/")
+                             (n (sys-getcwd))))
+               (root* (n (current-directory)))
+               (cur*  (begin (current-directory cur)
+                             (sys-getcwd)))
+               (cur** (current-directory)))
+          (list root root* (string=? cur cur*) (string=? cur* cur**)))))
+
+;; rm -rf
+(define (cmd-rmrf dir)
+  ;; shorthand of normalizing pathname.  this doesn't do anything on
+  ;; unix, but on Windows the separator in PATHNAME is replaced.
+  (define (n pathname) (sys-normalize-pathname pathname))
+
+  (cond-expand
+   [gauche.os.windows
+    (sys-system #`"rmdir /q /s ,(n dir) > NUL 2>&1")
+    (sys-system #`"del /q ,(n dir) > NUL 2>&1")]
+   [else
+    (sys-system #`"rm -rf ,dir > /dev/null")]))
+
+;; prepare test data set
+(define *test-tree*
+  `(test.out
+    ((test1.o ,(make-string 100 #\o))
+     (test2.o ,(make-string 100 #\o))
+     (test3.o ,(make-string 100 #\i))
+     (test4.o ,(make-string 20000 #\i))
+     (test5.o ,(make-string 20000 #\i))
+     ,@(cond-expand
+        [gauche.sys.symlink
+         '((test6.o :symlink "test1.o")
+           (test7.o :symlink "test6.o")
+           (test2.d :symlink "test.d"))]
+        [else
+         `((test6.o ,(make-string 100 #\o))
+           (test7.o ,(make-string 100 #\o))
+           (test2.d :mode #o777
+		    ((test10.o ,(make-string 100 #\o))
+		     (test11.o ,(make-string 100 #\o))
+                     test12.o)))])
+     (test.d ((test10.o ,(make-string 100 #\o))
+              ,(cond-expand
+                [gauche.sys.symlink
+                 '(test11.o :symlink "../test1.o")]
+                [else
+                 `(test11.o ,(make-string 100 #\o))])
+              test12.o))
+     )))
+
+(cmd-rmrf "test.out")
+
+(test* "create-directory-tree" #t
+       (begin (create-directory-tree "." *test-tree*)
+              (and (file-is-directory? "test.out")
+                   (file-is-regular? "test.out/test1.o")
+                   (file-is-regular? "test.out/test2.o")
+                   (file-is-regular? "test.out/test3.o")
+                   (file-is-regular? "test.out/test4.o")
+                   (file-is-regular? "test.out/test5.o")
+                   (file-is-directory? "test.out/test.d")
+                   (file-is-regular? "test.out/test.d/test10.o")
+                   (file-is-regular? "test.out/test.d/test12.o"))))
+
+(test* "check-directory-tree" #t
+       (check-directory-tree "." *test-tree*))
+           
+(test* "directory-list"
+       '("." ".." "test.d" "test1.o" "test2.d" "test2.o"
+         "test3.o" "test4.o" "test5.o" "test6.o" "test7.o" )
+       (directory-list "test.out"))
+
+(test* "directory-list :children?"
+       '("test.d" "test1.o" "test2.d" "test2.o"
+         "test3.o" "test4.o" "test5.o" "test6.o" "test7.o" )
+       (directory-list "test.out" :children? #t))
+
+(test* "directory-list :add-path?"
+       (n "test.out/." "test.out/.." "test.out/test.d" "test.out/test1.o"
+          "test.out/test2.d" "test.out/test2.o"  "test.out/test3.o"
+          "test.out/test4.o" "test.out/test5.o" "test.out/test6.o"
+          "test.out/test7.o" )
+       (directory-list "test.out/" :add-path? #t))
+
+(test* "directory-list :filter"
+       (n "test.out/test1.o"
+          "test.out/test2.o"  "test.out/test3.o" "test.out/test4.o"
+          "test.out/test5.o" "test.out/test6.o" "test.out/test7.o" )
+       (directory-list "test.out" :add-path? #t
+                       :filter (lambda (p) (string-suffix? "o" p))))
+
+(test* "directory-list :filter"
+       (n "test.out/test1.o"
+          "test.out/test2.o"  "test.out/test3.o" "test.out/test4.o"
+          "test.out/test5.o" "test.out/test6.o" "test.out/test7.o" )
+       (directory-list "test.out" :add-path? #t :filter-add-path? #t
+                       :filter file-is-regular?))
+
+(test* "directory-list2"
+       '(("." ".." "test.d" "test2.d")
+         ("test1.o" "test2.o" "test3.o" "test4.o"
+          "test5.o" "test6.o" "test7.o" ))
+       (receive x (directory-list2 "test.out") x))
+
+(test* "directory-list2 :add-path"
+       `(,(n "test.out/." "test.out/.." "test.out/test.d" "test.out/test2.d")
+         ,(n "test.out/test1.o" "test.out/test2.o"  "test.out/test3.o"
+             "test.out/test4.o" "test.out/test5.o" "test.out/test6.o"
+             "test.out/test7.o"))
+       (receive x (directory-list2 "test.out" :add-path? #t) x))
+
+(test* "directory-list2 :children"
+       `(,(n "test.out/test.d" "test.out/test2.d")
+         ,(n "test.out/test1.o" "test.out/test2.o"  "test.out/test3.o"
+          "test.out/test4.o" "test.out/test5.o" "test.out/test6.o"
+         "test.out/test7.o"))
+       (receive x (directory-list2 "test.out" :add-path? #t :children? #t) x))
+
+(test* "directory-list2 :filter"
+       '(("test.d" "test2.d")
+         ("test1.o" "test2.o" "test3.o" "test4.o"
+          "test5.o" "test6.o" "test7.o" ))
+       (receive x 
+           (directory-list2 "test.out"
+                            :filter (lambda (p) (string-contains p "test")))
+         x))
+
+(cond-expand
+ [gauche.sys.symlink
+  (test* "directory-list2 :follow-link? #f"
+         '(("test.d")
+           ("test1.o" "test2.d" "test2.o" "test3.o" "test4.o"
+            "test5.o" "test6.o" "test7.o" ))
+         (receive x (directory-list2 "test.out" :follow-link? #f :children? #t)
+           x)
+         )]
+ [else])
+
+(test* "directory-fold"
+       (n "test.out/test.d/test10.o" "test.out/test.d/test11.o"
+          "test.out/test1.o"
+          "test.out/test2.d/test10.o" "test.out/test2.d/test11.o"
+          "test.out/test2.o" "test.out/test3.o"
+          "test.out/test6.o" "test.out/test7.o")
+       (reverse
+        (directory-fold "test.out"
+                        (lambda (path result)
+                          (if (= (file-size path) 100)
+                              (cons path result)
+                              result))
+                        '()))
+       )
+
+(test* "directory-fold"
+       (n "test.out"
+          "test.out/test.d"
+          "test.out/test.d/test10.o"
+          "test.out/test.d/test11.o"
+          "test.out/test.d/test12.o"
+          "test.out/test1.o"
+          "test.out/test2.d"
+          "test.out/test2.d/test10.o"
+          "test.out/test2.d/test11.o"
+          "test.out/test2.d/test12.o"
+          "test.out/test2.o" "test.out/test3.o"
+          "test.out/test4.o" "test.out/test5.o"
+          "test.out/test6.o" "test.out/test7.o")
+       (reverse
+        (directory-fold "test.out" cons '()
+                        :lister (lambda (path seed)
+                                  (values
+                                   (directory-list path
+                                                   :add-path? #t
+                                                   :children? #t)
+                                   (cons path seed)))))
+       )
+
+(cond-expand
+ [gauche.sys.symlink
+  (test* "directory-fold :follow-link? #f"
+         (n "test.out/test.d/test10.o"
+            "test.out/test.d/test11.o"
+            "test.out/test1.o"
+            "test.out/test2.o" "test.out/test3.o"
+            "test.out/test6.o" "test.out/test7.o")
+         (reverse 
+          (directory-fold "test.out"
+                          (lambda (path result)
+                            (if (= (file-size path) 100)
+                                (cons path result)
+                                result))
+                          '()
+                          :follow-link? #f)))
+  ;; this tests dangling symlink
+  (sys-symlink "foo" "test.out/test.dangling")
+  (test* "directory-fold :follow-link? #t; dangling link"
+         (n "test.out/test.d/test10.o" "test.out/test.d/test11.o"
+            "test.out/test.d/test12.o"
+            "test.out/test.dangling" "test.out/test1.o"
+            "test.out/test2.d/test10.o" "test.out/test2.d/test11.o"
+            "test.out/test2.d/test12.o"
+            "test.out/test2.o" "test.out/test3.o" "test.out/test4.o"
+            "test.out/test5.o" "test.out/test6.o" "test.out/test7.o")
+         (sort (directory-fold "test.out" cons '())))
+  (sys-unlink "test.out/test.dangling")]
+ [else])
+
+(test* "directory-fold :lister"
+       (cond-expand
+        [gauche.sys.symlink
+         (n "test.out/test.d/test10.o" "test.out/test.d/test11.o"
+            "test.out/test.d/test12.o"
+            "test.out/test1.o")]
+        [else
+         (n "test.out/test.d/test10.o" "test.out/test.d/test11.o"
+            "test.out/test.d/test12.o"
+            "test.out/test2.d/test10.o" "test.out/test2.d/test11.o"
+            "test.out/test2.d/test12.o"
+            "test.out/test1.o")])
+       (reverse
+        (directory-fold "test.out" cons '()
+                        :follow-link? #f
+                        :lister (lambda (dir knil)
+                                  (receive (dirs files)
+                                      (directory-list2 dir :add-path? #t :children? #t :follow-link? #f)
+                                    (append dirs
+                                            (filter (cut string-contains <> "test1")
+                                                    files))))))
+       )
+
+(cmd-rmrf "test.out")
+
+;;=====================================================================
+(test-section "file utils")
+
+(create-directory-tree "." *test-tree*)
+
+;; resolve-path should be here for it uses test.out hierarchy created above.
+'(test* "resolve-path" "/" (resolve-path "/"))
+(test* "resolve-path" "." (resolve-path "."))
+(test* "resolve-path" "test.out" (resolve-path "test.out"))
+(cond-expand
+ [gauche.sys.symlink
+  (test* "resolve-path" "test.out/test1.o"
+         (resolve-path "test.out/test6.o"))
+  (test* "resolve-path" "test.out/test1.o"
+         (resolve-path "test.out/test7.o"))
+  (test* "resolve-path" "test.out/test1.o"
+         (resolve-path "test.out/test2.d/test11.o"))
+  (test* "resolve-path" "test.out/test1.o"
+         (resolve-path "test.out/test2.d/../test.d/test11.o"))
+  (test* "resolve-path" "test.out/test1.o"
+         (resolve-path "test.out/test.d/../test2.d/test11.o"))
+  ]
+ [else])
+
 (test* "file-type" #f
        (file-type "nonexistent/file"))
 (test* "file-type" '(directory directory regular)
@@ -345,7 +416,7 @@
        (number->string (logand #o666 (lognot (sys-umask))) 8)
        (number->string (file-perm "test.out/test.copy") 8))
 
-(test* "copy-file (:if-exists :error)" *test-error*
+(test* "copy-file (:if-exists :error)" (test-error)
        (copy-file "test.out/test5.o" "test.out/test.copy"))
 
 (test* "copy-file (:if-exists #f)" #f
@@ -396,7 +467,7 @@
             (not (file-eq? "test.out/test5.o" "test.out/test.copy"))
             (file-equal? "test.out/test5.o" "test.out/test.copy")))
 
-(test* "copy-file (:if-exists :error, safe)" *test-error*
+(test* "copy-file (:if-exists :error, safe)" (test-error)
        (copy-file "test.out/test5.o" "test.out/test.copy" :safe #t))
 
 (test* "copy-file (:if-exists #f, safe)" #f
@@ -420,13 +491,13 @@
             (file-equal? "test.out/test1.o" "test.out/test.copy")
             (file-equal? "test.out/test5.o" "test.out/test.copy~")))
 
-(test* "copy-file (same file)" *test-error*
+(test* "copy-file (same file)" (test-error)
        (copy-file "test.out/test.copy" "test.out/test.copy"
                   :if-exists :supersede))
 
 (sys-unlink "test.out/test.copy")
 
-(test* "copy-file (keep-mode)" #o642
+(test* "copy-file (keep-mode)" (P #o642)
        (begin (sys-chmod "test.out/test1.o" #o642)
               (copy-file "test.out/test1.o" "test.out/test.copy"
                          :keep-mode #t)
@@ -434,7 +505,7 @@
 
 (sys-unlink "test.out/test.copy")
 
-(test* "copy-file (keep-mode w/safe)" #o624
+(test* "copy-file (keep-mode w/safe)" (P #o624)
        (begin (sys-chmod "test.out/test1.o" #o624)
               (copy-file "test.out/test1.o" "test.out/test.copy"
                          :keep-mode #t)
@@ -448,7 +519,7 @@
             (not (file-exists? "test.out/test.copy"))
             (file-equal? "test.out/test1.o" "test.out/test.move")))
 
-(test* "move-file (:if-exists :error)" *test-error*
+(test* "move-file (:if-exists :error)" (test-error)
        (move-file "test.out/test5.o" "test.out/test.move"))
 
 (test* "move-file (:if-exists :supersede)" #t
@@ -464,13 +535,13 @@
             (file-equal? "test.out/test2.o" "test.out/test.move")
             (file-equal? "test.out/test4.o" "test.out/test.move.orig")))
 
-(test* "move-file (same file)" *test-error*
+(test* "move-file (same file)" (test-error)
        (move-file "test.out/test.move" "test.out/test.move"
                   :if-exists :supersede))
 
 (let ()
   (define (listdir d)
-    (map (cut regexp-replace #/^test2?\.out\// <> "")
+    (map (cut regexp-replace #/^test2?\.out[\/\\]/ <> "")
          (directory-fold d cons '())))
     
   (test* "copy-directory*" (listdir "test.out")
@@ -499,10 +570,10 @@
                                 (sys-readlink "test2.out")))))
         (remove-files "test.out" "test2.out"))
 
-      (tester "copy-file" copy-file *test-error*)
+      (tester "copy-file" copy-file (test-error))
       (tester "copy-file" copy-file #t :follow-link? #f)
       (tester "copy-directory*" copy-directory* #t)
-      (tester "copy-directory*" copy-directory* *test-error* :follow-link? #t)
+      (tester "copy-directory*" copy-directory* (test-error) :follow-link? #t)
       )]
    [else])
   )
@@ -513,7 +584,7 @@
 
 (test* "file->string, if-does-not-exist #f" #f
        (file->string "test.out" :if-does-not-exist #f))
-(test* "file->string, if-does-not-exist :error" *test-error*
+(test* "file->string, if-does-not-exist :error" (test-error)
        (file->string "test.out" :if-does-not-exist :error))
 
 (with-output-to-file "test.out"
@@ -526,4 +597,68 @@
 
 (remove-files "test.out" "test2.out")
 
+;;
+;; with-lock-file
+;;
+
+(define (test-lock-file type)
+  (remove-files "test.out" "test.out.2")
+
+  (test* #`"with-lock-file (,type) just lock&release" '(#t #f)
+         (let1 r (with-lock-file "test.out"
+                                 (^() (file-exists? "test.out"))
+                                 :type type)
+           (list r (file-exists? "test.out"))))
+
+  (remove-files "test.out" "test.out.2")
+
+  (test* #`"with-lock-file (,type) lock giveup" `((,<lock-file-failure> #t) #f)
+         (let1 r
+             (with-lock-file "test.out"
+                             (^()
+                               (let1 r2
+                                   (guard (e (else (class-of e)))
+                                     (with-lock-file "test.out"
+                                                     (^() 'boo!)
+                                                     :type type
+                                                     :abandon-timeout #f
+                                                     :retry-interval 0.1
+                                                     :retry-limit 0.5))
+                                 (list r2 (file-exists? "test.out"))))
+                             :type type)
+           (list r (file-exists? "test.out"))))
+
+  (remove-files "test.out" "test.out.2")
+
+  (test* #`"with-lock-file (,type) stealing" 'got
+         (begin
+           (case type
+             [(file) (touch-file "test.out")]
+             [(directory) (make-directory* "test.out")])
+           (with-lock-file "test.out"
+                           (^() 'got)
+                           :type type
+                           :retry-interval 0.1
+                           :retry-limit 5
+                           :abandon-timeout 1)))
+  (remove-files "test.out" "test.out.2")
+
+  (test* #`"with-lock-file (,type) stealing failure"
+         (test-error <lock-file-failure>)
+         (begin
+           (case type
+             [(file) (make-directory* "test.out")]
+             [(directory) (touch-file "test.out")])
+           (with-lock-file "test.out"
+                           (^() 'got)
+                           :type type
+                           :retry-interval 0.1
+                           :retry-limit 0.5
+                           :abandon-timeout 1)))
+  (remove-files "test.out" "test.out.2")
+  )
+
+(test-lock-file 'file)
+(test-lock-file 'directory)
+
 (test-end)
