@@ -1,7 +1,7 @@
 /*
  * addr.c - socket address
  *
- *  Copyright (c) 2001-2008  Shiro Kawai  <shiro@acm.org>
+ *  Copyright (c) 2001-2011  Shiro Kawai  <shiro@acm.org>
  *
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,12 +29,9 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: addr.c,v 1.34 2008-05-10 13:35:41 shirok Exp $
  */
 
-#include "gauche/net.h"
-#include "gauche/uvector.h"
+#include "gauche-net.h"
 #include <string.h>
 
 static ScmObj key_path = SCM_FALSE;
@@ -44,6 +41,12 @@ static ScmObj key_any = SCM_FALSE;
 static ScmObj key_broadcast = SCM_FALSE;
 static ScmObj key_loopback = SCM_FALSE;
 
+/* NB: built-in socket address structures are allocated as ATOMIC---when
+   you want to extend them, be careful not to introduce sole pointers to
+   allocated objects; GC will collect them prematurely.  The only
+   pointer, the tagged pointer to the class, is protected since they're
+   bound to global variables. */
+
 /*==================================================================
  * Generic Socket Address
  */
@@ -97,8 +100,8 @@ static ScmObj sockaddr_allocate(ScmClass *klass, ScmObj initargs)
 ScmObj Scm_MakeSockAddr(ScmClass *klass, struct sockaddr *saddr, int len)
 {
     ScmSockAddr *addr;
-    addr = SCM_NEW2(ScmSockAddr*,
-                    sizeof(ScmSockAddr) - sizeof(struct sockaddr) + len);
+    addr = SCM_NEW_ATOMIC2(ScmSockAddr*,
+                           sizeof(ScmSockAddr) - sizeof(struct sockaddr) + len);
     if (klass == NULL) {
         switch (saddr->sa_family) {
         case AF_UNIX:
@@ -127,11 +130,6 @@ ScmObj Scm_MakeSockAddr(ScmClass *klass, struct sockaddr *saddr, int len)
  * Unix domain socket
  */
 
-static ScmObj sockaddr_un_allocate(ScmClass *klass, ScmObj initargs);
-
-SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrUnClass, sockaddr_print,
-                         NULL, NULL, sockaddr_un_allocate, Scm_SockAddrCPL);
-
 static ScmObj sockaddr_un_allocate(ScmClass *klass, ScmObj initargs)
 {
     ScmObj path = Scm_GetKeyword(key_path, initargs, SCM_FALSE);
@@ -140,7 +138,7 @@ static ScmObj sockaddr_un_allocate(ScmClass *klass, ScmObj initargs)
     if (!SCM_FALSEP(path) && !SCM_STRINGP(path)) {
         Scm_Error(":path parameter must be a string, but got %S", path);
     }
-    addr = SCM_NEW(ScmSockAddrUn);
+    addr = SCM_NEW_ATOMIC(ScmSockAddrUn);
     SCM_SET_CLASS(addr, SCM_CLASS_SOCKADDR_UN);
     memset(&addr->addr, 0, sizeof(struct sockaddr_un));
 #ifdef HAVE_STRUCT_SOCKADDR_UN_SUN_LEN
@@ -161,15 +159,29 @@ static ScmObj sockaddr_un_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(addr);
 }
 
+static int sockaddr_un_compare(ScmObj x, ScmObj y, int equalp)
+{
+    ScmSockAddrUn *xx = (ScmSockAddrUn*)x;
+    ScmSockAddrUn *yy = (ScmSockAddrUn*)y;
+    if (!equalp) Scm_Error("object %S and %S can't be ordered", x, y);
+    
+    if (xx->addrlen == yy->addrlen
+        && memcmp(xx->addr.sun_path, yy->addr.sun_path,
+                  sizeof(xx->addr.sun_path)) == 0) {
+        return 0;               /* (equal? x y) => #t */
+    } else {
+        return -1;              /* (equal? x y) => #f */
+    }
+}
+
+SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrUnClass, sockaddr_print,
+                         sockaddr_un_compare, NULL,
+                         sockaddr_un_allocate, Scm_SockAddrCPL);
+
 /*==================================================================
  * Inet domain socket
  */
 
-static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs);
-
-SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrInClass, sockaddr_print,
-                         NULL, NULL, sockaddr_in_allocate, Scm_SockAddrCPL);
-
 static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
 {
     ScmObj host = Scm_GetKeyword(key_host, initargs, key_any);
@@ -180,7 +192,7 @@ static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
         Scm_Error(":port parameter must be a small exact integer, but got %S",
                   port);
     }
-    addr = SCM_NEW(ScmSockAddrIn);
+    addr = SCM_NEW_ATOMIC(ScmSockAddrIn);
     SCM_SET_CLASS(addr, SCM_CLASS_SOCKADDR_IN);
     memset(&addr->addr, 0, sizeof(struct sockaddr_in));
 #ifdef HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
@@ -234,17 +246,32 @@ static ScmObj sockaddr_in_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(addr);
 }
 
+static int sockaddr_in_compare(ScmObj x, ScmObj y, int equalp)
+{
+    ScmSockAddrIn *xx = (ScmSockAddrIn*)x;
+    ScmSockAddrIn *yy = (ScmSockAddrIn*)y;
+    if (!equalp) Scm_Error("object %S and %S can't be ordered", x, y);
+
+    if (xx->addrlen == yy->addrlen
+        && xx->addr.sin_family == yy->addr.sin_family
+        && xx->addr.sin_port == yy->addr.sin_port
+        && xx->addr.sin_addr.s_addr == yy->addr.sin_addr.s_addr) {
+        return 0;               /* (equal? x y) => #t */
+    } else {
+        return -1;              /* (equal? x y) => #f */
+    }
+}
+
+SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrInClass, sockaddr_print,
+                         sockaddr_in_compare, NULL,
+                         sockaddr_in_allocate, Scm_SockAddrCPL);
+
 /*==================================================================
  * Inet6 domain socket
  */
 
 #ifdef HAVE_IPV6
 
-static ScmObj sockaddr_in6_allocate(ScmClass *klass, ScmObj initargs);
-
-SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrIn6Class, sockaddr_print,
-                         NULL, NULL, sockaddr_in6_allocate, Scm_SockAddrCPL);
-
 static ScmObj sockaddr_in6_allocate(ScmClass *klass, ScmObj initargs)
 {
     ScmObj host = Scm_GetKeyword(key_host, initargs, key_any);
@@ -255,7 +282,7 @@ static ScmObj sockaddr_in6_allocate(ScmClass *klass, ScmObj initargs)
         Scm_Error(":port parameter must be a small exact integer, but got %S",
                   port);
     }
-    addr = SCM_NEW(ScmSockAddrIn6);
+    addr = SCM_NEW_ATOMIC(ScmSockAddrIn6);
     SCM_SET_CLASS(addr, SCM_CLASS_SOCKADDR_IN6);
     memset(&addr->addr, 0, sizeof(struct sockaddr_in6));
 #ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
@@ -303,6 +330,27 @@ static ScmObj sockaddr_in6_allocate(ScmClass *klass, ScmObj initargs)
     return SCM_OBJ(addr);
 }
 
+static int sockaddr_in6_compare(ScmObj x, ScmObj y, int equalp)
+{
+    ScmSockAddrIn6 *xx = (ScmSockAddrIn6*)x;
+    ScmSockAddrIn6 *yy = (ScmSockAddrIn6*)y;
+    if (!equalp) Scm_Error("object %S and %S can't be ordered", x, y);
+    
+    if (xx->addrlen == yy->addrlen
+        && xx->addr.sin6_family == yy->addr.sin6_family
+        && xx->addr.sin6_port == yy->addr.sin6_port
+        && (memcmp(&xx->addr.sin6_addr, &yy->addr.sin6_addr,
+                   sizeof(xx->addr.sin6_addr)) == 0)) {
+        return 0;               /* (equal? x y) => #t */
+    } else {
+        return -1;              /* (equal? x y) => #f */
+    }
+}
+
+SCM_DEFINE_BUILTIN_CLASS(Scm_SockAddrIn6Class, sockaddr_print,
+                         sockaddr_in6_compare, NULL,
+                         sockaddr_in6_allocate, Scm_SockAddrCPL);
+
 #endif /* HAVE_IPV6 */
 
 /*==================================================================
