@@ -1,7 +1,7 @@
 ;;;
 ;;; net - network interface
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: net.scm,v 1.43 2008-05-10 13:35:41 shirok Exp $
-;;;
 
 #!no-fold-case
 
@@ -46,8 +44,8 @@
           socket-address socket-status socket-input-port socket-output-port
           socket-shutdown socket-close socket-bind socket-connect socket-fd
           socket-listen socket-accept socket-setsockopt socket-getsockopt
-          socket-getsockname socket-getpeername
-          socket-send socket-sendto
+          socket-getsockname socket-getpeername socket-ioctl
+          socket-send socket-sendto socket-sendmsg socket-buildmsg
           socket-recv socket-recv! socket-recvfrom socket-recvfrom!
           <sockaddr> <sockaddr-in> <sockaddr-un> make-sockaddrs
           sockaddr-name sockaddr-family sockaddr-addr sockaddr-port
@@ -64,7 +62,7 @@
   
 (select-module gauche.net)
 
-(dynamic-load "libnet")
+(dynamic-load "gauche--net")
 
 (export-if-defined
  IPPROTO_IP IPPROTO_ICMP IPPROTO_TCP
@@ -83,6 +81,19 @@
  MSG_CTRUNC MSG_DONTROUTE MSG_EOR MSG_OOB MSG_PEEK MSG_TRUNC
  MSG_WAITALL)
 
+;; Netdevice control.  OS specific.
+(export-if-defined
+ SIOCGIFNAME SIOCSIFNAME SIOCGIFINDEX SIOCGIFFLAGS SIOCSIFFLAGS
+ SIOCGIFMETRIC SIOCSIFMETRIC SIOCGIFMTU SIOCSIFMTU
+ SIOCGIFHWADDR SIOCSIFHWADDR SIOCSIFHWBROADCAST
+ SIOCGIFMAP SIOCSIFMAP SIOCADDMULTI SIOCDELMULTI
+ SIOGIFTXQLEN SIOSIFTXQLEN SIOCGIFCONF
+
+ IFF_UP IFF_BROADCAST IFF_DEBUG IFF_LOOPBACK IFF_POINTTOPOINT
+ IFF_RUNNING IFF_NOARP IFF_PROMISC IFF_NOTRAILERS IFF_ALLMULTI
+ IFF_MASTER IFF_SLAVE IFF_MULTICAST IFF_PORTSEL IFF_AUTOMEDIA
+ IFF_DYNAMIC)
+
 ;; if ipv6 is supported, these symbols are defined in the C routine.
 
 (export-if-defined
@@ -95,4 +106,3 @@
  sys-getnameinfo
  NI_NOFQDN NI_NUMERICHOST NI_NAMEREQD NI_NUMERICSERV NI_DGRAM)
 
-(provide "gauche/net")
