@@ -1,7 +1,7 @@
 ;;;
 ;;; netaux.scm - network interface
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: netaux.scm,v 1.14 2008-05-10 13:35:41 shirok Exp $
-;;;
 
 (select-module gauche.net)
 (use srfi-1)
@@ -153,8 +151,30 @@
   (apply make-server-socket-from-addr (car (make-sockaddrs #f port)) args))
 
 (define (make-server-sockets host port . args)
-  (map (cut apply make-server-socket <> args)
-       (make-sockaddrs host port)))
+  (let1 ss (make-sockaddrs host port)
+    ;; Heuristics - if we have both v4 and v6 sockets, we *may* need
+    ;; only v6 sockets if the system defaults to dual-stack socket.
+    ;; Unfortunately the behavior is system dependent.  So we try to
+    ;; open both (first v6, then v4) and if the latter fails to bind
+    ;; we assume v6 socket listens both.
+    (cond-expand
+     [gauche.os.windows
+      ;; mingw doesn't have EADDRINUSE.  it's likely not to have ipv6 either,
+      ;; so we just use the default.  NB: we can't switch here with
+      ;; gauche.sys.ipv6; see the comment on ipv6-capable definition above.
+      (map (cut apply make-server-socket <> args) ss)]
+     [else
+      (let ([v4s (filter (^s (eq? (sockaddr-family s) 'inet)) ss)]
+            [v6s (filter (^s (eq? (sockaddr-family s) 'inet6)) ss)])
+        (if (and (not (null? v4s)) (not (null? v6s)))
+          (fold (^(s ss)
+                  (guard (e [(and (<system-error> e)
+                                  (eqv? (~ e'errno) EADDRINUSE))
+                             ss])         ;ignore s
+                    (cons (apply make-server-socket s args) ss)))
+                (map (cut apply make-server-socket <> args) v6s)
+                v4s)
+          (map (cut apply make-server-socket <> args) ss)))])))
 
 (define (make-sockaddrs host port . maybe-proto)
   (let1 proto (get-optional maybe-proto 'tcp)
@@ -179,9 +199,15 @@
                  (slot-ref hh 'addresses)))
           (list (make <sockaddr-in> :host :any :port port)))))))
 
-(define (call-with-client-socket socket proc)
+(define (call-with-client-socket socket proc
+                                 :key (input-buffering #f) (output-buffering #f))
   (unwind-protect 
-      (proc (socket-input-port socket) (socket-output-port socket))
+      (proc (if input-buffering
+              (socket-input-port socket :buffering input-buffering)
+              (socket-input-port socket))
+            (if output-buffering
+              (socket-output-port socket :buffering output-buffering)
+              (socket-output-port socket)))
     (socket-close socket)))
 
 ;;=================================================================
