@@ -6,6 +6,7 @@
 
 (use gauche.test)
 (use gauche.uvector)
+(use gauche.experimental.app)
 (use srfi-1)
 (use srfi-13)
 (test-start "net")
@@ -18,13 +19,13 @@
 ;;-----------------------------------------------------------------
 (test-section "socket address")
 
-'(test* "sockaddr_un" #t
+(test* "sockaddr_un" #t
         (let ((addr (make <sockaddr-un> :path "/tmp/xxx")))
           (and (eq? (sockaddr-family addr) 'unix)
                (equal? (sockaddr-name addr) "/tmp/xxx")
                #t)))
 
-'(test* "sockaddr_in" #t
+(test* "sockaddr_in" #t
         (let ((addr (make <sockaddr-in> :host "127.0.0.1" :port 80)))
           (and (eq? (sockaddr-family addr) 'inet)
                (equal? (sockaddr-name addr) "127.0.0.1:80")
@@ -62,8 +63,26 @@
               (equal? (sockaddr-name addr) "255.255.255.255:0")
               #t)))
 
+(let ([un1 (make <sockaddr-un> :path "/tmp/a")]
+      [un2 (make <sockaddr-un> :path "/tmp/aa")]
+      [un3 (make <sockaddr-un> :path "/tmp/a")]
+      [in1 (make <sockaddr-in> :host "127.0.0.1" :port 2030)]
+      [in2 (make <sockaddr-in> :host "127.0.0.2" :port 2030)]
+      [in3 (make <sockaddr-in> :host "127.0.0.1" :port 2031)]
+      [in4 (make <sockaddr-in> :host "127.0.0.1" :port 2030)])
+  (let-syntax ([t (syntax-rules ()
+                    [(_ exp a b)
+                     (test* (format "sockaddr equal? ~s" '(equal? a b))
+                            exp (equal? a b))])])
+    (t #f un1 un2)
+    (t #t un1 un3)
+    (t #f in1 in2)
+    (t #f in1 in3)
+    (t #t in1 in4)
+    (t #f un1 in1)))
+
 (cond-expand
- (gauche.net.ipv6
+ [gauche.net.ipv6
   (test* "sockaddr_in6" #t
          (let ((addr (make <sockaddr-in6> :host "2001:200::8002:203:47ff:fea5:3085" :port 23)))
            (and (eq? (sockaddr-family addr) 'inet6)
@@ -75,8 +94,19 @@
          (sockaddr-name (make <sockaddr-in6> :host 1)))
   (test* "sockaddr_in6" "[1:2:3:4:5:6:7:8]:0"
          (sockaddr-name (make <sockaddr-in6> :host '#u8(0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8))))
-  )
- (else #f))
+
+  (let ([4n1 (make <sockaddr-in6> :host 1 :port 80)]
+        [4n2 (make <sockaddr-in6> :host 1 :port 81)]
+        [4n3 (make <sockaddr-in6> :host "1:2::8" :port 80)]
+        [4n4 (make <sockaddr-in6> :host 1 :port 80)])
+    (let-syntax ([t (syntax-rules ()
+                      [(_ exp a b)
+                       (test* (format "sockaddr equal? ~s" '(equal? a b))
+                              exp (equal? a b))])])
+      (t #f 4n1 4n2)
+      (t #f 4n1 4n3)
+      (t #t 4n1 4n4)))]
+ [else])
 
 (let ()
   (define (addr-test desc input exp-val exp-vers)
@@ -102,7 +132,7 @@
   (addr-test "v4-err3" "172.256.4.2" #f #f)
 
   (cond-expand
-   (gauche.net.ipv6
+   [gauche.net.ipv6
     (addr-test "v6-1" "1:2:3:4:fffc:fffd:fffe:ffff"
                #x0001000200030004fffcfffdfffeffff AF_INET6)
     (addr-test "v6-2" "::1"
@@ -125,8 +155,8 @@
                #x000000000000000000000000c0a80102 AF_INET6)
     (addr-test "v6-9" "1::2:0:0:192.168.1.2"
                #x000100000000000200000000c0a80102 AF_INET6)
-    )
-   (else #f))
+    ]
+   [else])
   )
 
 (let ()
@@ -147,7 +177,7 @@
   (addr-test "v4-3" #xfffffc00 AF_INET "255.255.252.0")
 
   (cond-expand
-   (gauche.net.ipv6
+   [gauche.net.ipv6
     (addr-test "v6-1" #x0001000200030004fffcfffdfffeffff AF_INET6
                "1:2:3:4:fffc:fffd:fffe:ffff")
     (addr-test "v6-2" #x00000000000000000000000000000001 AF_INET6
@@ -170,8 +200,8 @@
                "ffe0:0:0:223::4")
     (addr-test "v6-11" #xffe00000000000000001000000000004 AF_INET6
                "ffe0::1:0:0:4")
-    )
-   (else #f))
+    ]
+   [else])
   )
 
 
@@ -183,9 +213,11 @@
          (and host
               (or (equal? (slot-ref host 'name) "localhost")
                   (member "localhost" (slot-ref host 'aliases))
-                  ;; cygwin usually doesn't define "localhost", and
-                  ;; returns hostname.  For now we skip this test.
-                  (string-suffix? "cygwin" (gauche-architecture)))
+                  ;; cygwin and mingw usually doesn't define "localhost", and
+                  ;; returns hostname.
+                  (cond-expand
+                   [(or gauche.os.cygwin gauche.os.windows) #t]
+                   [else #f]))
               (member "127.0.0.1" (slot-ref host 'addresses))
               #t)))
 
@@ -194,9 +226,11 @@
          (and host
               (or (equal? (slot-ref host 'name) "localhost")
                   (member "localhost" (slot-ref host 'aliases))
-                  ;; cygwin usually doesn't define "localhost", and
+                  ;; cygwin and mingw usually doesn't define "localhost", and
                   ;; returns hostname.  For now we skip this test.
-                  (string-suffix? "cygwin" (gauche-architecture)))
+                  (cond-expand
+                   [(or gauche.os.cygwin gauche.os.windows) #t]
+                   [else #f]))
               (member "127.0.0.1" (slot-ref host 'addresses))
               #t)))
 
@@ -254,29 +288,36 @@
 
 (use gauche.process)
 
+;; sockargs is an expression that yields to a list of server sockets
 (define (run-simple-server sockargs)
   (with-output-to-file "testserv.o"
     (lambda ()
       (write '(use gauche.net))
+      (write '(use gauche.selector))
       (write '(use srfi-13))
-      (write '(define (simple-server socket)
+      (write '(define (simple-server sockets)
+                (define (handler fd flag)
+                  (let loop ((clnt (socket-accept
+                                    (find (^s (eq? (socket-fd s) fd)) sockets))))
+                    (let ((in   (socket-input-port clnt))
+                          (out  (socket-output-port clnt)))
+                      (let loop2 ((line (read-line in)))
+                        (cond ((eof-object? line)
+                               (socket-close clnt))
+                              ((string=? line "END")
+                               (socket-close clnt)
+                               (for-each socket-close sockets)
+                               (sys-exit 33))
+                              (else
+                               (display (string-upcase line) out)
+                               (newline out)
+                               (flush out)
+                               (loop2 (read-line in))))))))
                 (newline) (flush) ;; handshake
-                (let loop ((clnt (socket-accept socket)))
-                  (let ((in   (socket-input-port clnt))
-                        (out  (socket-output-port clnt)))
-                    (let loop2 ((line (read-line in)))
-                      (cond ((eof-object? line)
-                             (socket-close clnt)
-                             (loop (socket-accept socket)))
-                            ((string=? line "END")
-                             (socket-close clnt)
-                             (socket-close socket)
-                             (sys-exit 33))
-                            (else
-                             (display (string-upcase line) out)
-                             (newline out)
-                             (flush out)
-                             (loop2 (read-line in)))))))))
+                (let ((sel (make <selector>)))
+                  (dolist (s sockets)
+                    (selector-add! sel (socket-fd s) handler '(r)))
+                  (do () (#f) (selector-select sel)))))
       (write `(define (main args)
                 (simple-server ,sockargs)
                 0)))
@@ -297,48 +338,52 @@
 
 (sys-unlink "sock.o")
 
-(test* "unix server socket" #f
-       (begin
-         (run-simple-server '(make-server-socket 'unix "sock.o"))
-         (let1 stat (sys-stat "sock.o")
-           (not (memq (sys-stat->file-type stat) '(socket fifo))))))
+(cond-expand
+ [gauche.os.windows #f]
+ [else
+  (test* "unix server socket" #f
+	 (begin
+	   (run-simple-server '(list (make-server-socket 'unix "sock.o")))
+	   (let1 stat (sys-stat "sock.o")
+		 (not (memq (sys-stat->file-type stat) '(socket fifo))))))
 
-(test* "unix client socket" '("ABC" "XYZ")
-       (call-with-client-socket (make-client-socket 'unix "sock.o")
-         (lambda (in out)
-           (display "abc\n" out) (flush out)
-           (let ((abc (read-line in)))
-             (display "xyz\n" out) (flush out)
-             (list abc (read-line in))))))
+  (test* "unix client socket" '("ABC" "XYZ")
+         (call-with-client-socket (make-client-socket 'unix "sock.o")
+           (lambda (in out)
+             (display "abc\n" out) (flush out)
+             (let ((abc (read-line in)))
+               (display "xyz\n" out) (flush out)
+               (list abc (read-line in))))))
 
-(test* "unix client socket" #t
-       (call-with-client-socket (make-client-socket 'unix "sock.o")
-         (lambda (in out)
-           (display (make-string *chunk-size* #\a) out)
-           (newline out)
-           (flush out)
-           (string=? (read-line in) (make-string *chunk-size* #\A)))))
+  (test* "unix client socket" #t
+         (call-with-client-socket (make-client-socket 'unix "sock.o")
+           (lambda (in out)
+             (display (make-string *chunk-size* #\a) out)
+             (newline out)
+             (flush out)
+             (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
-(test* "unix client socket" #t
-       (call-with-client-socket (make-client-socket (make <sockaddr-un> :path "sock.o"))
-         (lambda (in out)
-           (display (make-string *chunk-size* #\a) out)
-           (newline out)
-           (flush out)
-           (string=? (read-line in) (make-string *chunk-size* #\A)))))
-
-(test* "unix client socket" 33
-       (call-with-client-socket (make-client-socket 'unix "sock.o")
-         (lambda (in out)
-           (display "END\n" out) (flush out)
-           (receive (pid code) (sys-wait)
-             (sys-wait-exit-status code)))))
+  (test* "unix client socket" #t
+         (call-with-client-socket
+          (make-client-socket (make <sockaddr-un> :path "sock.o"))
+          (lambda (in out)
+            (display (make-string *chunk-size* #\a) out)
+            (newline out)
+            (flush out)
+            (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
+  (test* "unix client socket" 33
+         (call-with-client-socket (make-client-socket 'unix "sock.o")
+           (lambda (in out)
+             (display "END\n" out) (flush out)
+             (receive (pid code) (sys-wait)
+                      (sys-wait-exit-status code)))))
+  ])
 
 (sys-unlink "sock.o")
 
 (test* "inet server socket" #t
-       (run-simple-server `(make-server-socket 'inet ,*inet-port* :reuse-addr? #t)))
+       (run-simple-server `(make-server-sockets #f ,*inet-port* :reuse-addr? #t)))
 
 (test* "inet client socket" '("ABC" "XYZ")
        (call-with-client-socket (make-client-socket 'inet "localhost" *inet-port*)
@@ -348,7 +393,7 @@
              (display "xyz\n" out) (flush out)
              (list abc (read-line in))))))
 
-(test* "inet client socket" #t
+(test* "inet client socket (host,port)" #t
        (call-with-client-socket (make-client-socket "localhost" *inet-port*)
          (lambda (in out)
            (display (make-string *chunk-size* #\a) out)
@@ -356,39 +401,66 @@
            (flush out)
            (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
-;; On IPv6 system, the loopback may have different name than "localhost".
-;; We apply some heuristics here.
-(test* "inet client socket" #t
-       (and-let* ((sock (any (lambda (name)
-                               (guard (e (else #f))
-                                 (make-client-socket
-                                  (make (cond-expand
-                                         (gauche.net.ipv6 <sockaddr-in6>)
-                                         (else <sockaddr-in>))
-                                    :host name :port *inet-port*))
-                                 ))
-                             '("localhost" "ip6-localhost" "ipv6-localhost"
-                               "::1" "127.0.0.1"))))
-         (call-with-client-socket sock
-           (lambda (in out)
-             (display (make-string *chunk-size* #\a) out)
-             (newline out)
-             (flush out)
-             (string=? (read-line in) (make-string *chunk-size* #\A))))))
+(test* "inet client socket (sockaddr)" #t
+       (call-with-client-socket (make-client-socket
+                                 (make <sockaddr-in>
+                                   :host "localhost" :port *inet-port*))
+         (lambda (in out)
+           (display (make-string *chunk-size* #\a) out)
+           (newline out)
+           (flush out)
+           (string=? (read-line in) (make-string *chunk-size* #\A)))))
 
-(test* "inet client socket" 33
+(test* "inet client socket (termination)" 33
        (call-with-client-socket (make-client-socket 'inet "localhost" *inet-port*)
          (lambda (in out)
            (display "END\n" out) (flush out)
            (receive (pid code) (sys-wait)
              (sys-wait-exit-status code)))))
 
-(test* "socket w/ port error handling" *test-error*
+
+(cond-expand
+ [gauche.net.ipv6
+  (test* "inet server socket (ipv6)" #t
+         (run-simple-server `(list (make-server-socket
+                                    (make <sockaddr-in6>
+                                      :host :any :port ,*inet-port*)
+                                    :reuse-addr? #t))))
+
+  ;; On IPv6 system, the loopback may have different name than "localhost".
+  ;; We apply some heuristics here.
+  (define (get-ipv6-sock)
+    (any (lambda (name)
+           (guard (e (else #f))
+             (make-client-socket
+              (make <sockaddr-in6> :host name :port *inet-port*))))
+         '("localhost" "ip6-localhost" "ipv6-localhost" "::1")))
+  
+  (test* "inet client socket (ipv6)" #t
+         (and-let* ((sock (get-ipv6-sock)))
+           (call-with-client-socket sock
+             (lambda (in out)
+               (display (make-string *chunk-size* #\a) out)
+               (newline out)
+               (flush out)
+               (string=? (read-line in) (make-string *chunk-size* #\A))))))
+
+  (test* "inet client socket (ipv6, termination)" 33
+         (and-let* ((sock (get-ipv6-sock)))
+           (call-with-client-socket sock
+             (lambda (in out)
+               (display "END\n" out) (flush out)
+               (receive (pid code) (sys-wait)
+                 (sys-wait-exit-status code))))))
+  ]
+ [else])
+
+(test* "socket w/ port error handling" (test-error)
        (let1 s (make-client-socket 'inet "localhost" *inet-port*)
          (close-socket s)
          (socket-input-port s)))
 
-(test* "socket w/ port error handling" *test-error*
+(test* "socket w/ port error handling" (test-error)
        (let1 s (make-client-socket 'inet "localhost" *inet-port*)
          (close-socket s)
          (socket-output-port s)))
@@ -430,31 +502,73 @@
 (test* "udp client socket" "ABC"
        (let ((sock (make-socket PF_INET SOCK_DGRAM))
              (addr (make <sockaddr-in> :host :loopback :port *inet-port*)))
-         (socket-setsockopt sock SOL_SOCKET SO_REUSEADDR 1)
-         (socket-connect sock addr)
-         (socket-send sock "abc")
+         ;; NB: Cygwin weirdness... Somehow the first udp packet
+         ;; is always dropped on CYGWIN_NT-5.1 1.5.25(0.156/4/2) on XP SP3,
+         ;; so we send it twice.  Curiously such symptom does not appear
+         ;; on Vista.
+         (cond-expand
+          [gauche.os.cygwin
+           (socket-sendto sock "abc" addr)
+           (sys-sleep 1)
+           (socket-sendto sock "abc" addr)]
+          [else
+           (socket-connect sock addr)
+           (socket-send sock "abc")])
          (begin0 (string-incomplete->complete (socket-recv sock 1024))
            (socket-close sock)
            (sys-wait))))
 
-(test* "udp uvector API" '(#t #t)
-       (let ((s-sock (make-socket PF_INET SOCK_DGRAM))
-             (r-sock (make-socket PF_INET SOCK_DGRAM))
-             (s-addr (make <sockaddr-in> :host :loopback :port *inet-port*))
-             (r-addr (make <sockaddr-in> :host :any :port *inet-port*))
-             (from   (make <sockaddr-in>))
-             (data   (make-u8vector 1024 #x3c))
-             (buf    (make-u8vector 1024 #xa5)))
-         (socket-setsockopt r-sock SOL_SOCKET SO_REUSEADDR 1)
-         (socket-bind r-sock r-addr)
-         (socket-sendto s-sock data s-addr)
-         (unwind-protect
-             (receive (size f-addr) (socket-recvfrom! r-sock buf (list from))
-               (list (eq? f-addr from)
-                     (equal? buf data)))
-           (begin
-             (socket-close r-sock)
-             (socket-close s-sock)))
-         ))
+(define (with-sr-udp proc)
+  (let ((s-sock (make-socket PF_INET SOCK_DGRAM))
+        (r-sock (make-socket PF_INET SOCK_DGRAM))
+        (s-addr (make <sockaddr-in> :host :loopback :port *inet-port*))
+        (r-addr (make <sockaddr-in> :host :any :port *inet-port*)))
+    (socket-setsockopt r-sock SOL_SOCKET SO_REUSEADDR 1)
+    (socket-bind r-sock r-addr)
+    (unwind-protect
+        (proc s-sock s-addr r-sock r-addr)
+      (begin (socket-close r-sock)
+             (socket-close s-sock)))))
+
+(with-sr-udp
+ (lambda (s-sock s-addr r-sock r-addr)
+   (let ([from (make <sockaddr-in>)]
+         [data (make-u8vector 1024 #x3c)]
+         [buf  (make-u8vector 1024 #xa5)])
+     (test* "udp uvector API" '(#t #t)
+            (begin
+              (socket-sendto s-sock data s-addr)
+              (receive (size f-addr) (socket-recvfrom! r-sock buf (list from))
+                (list (eq? f-addr from)
+                      (equal? buf data))))))))
+
+(cond-expand
+ ;; NB: as of 0.9, sendmsg fails on cygwin.  We don't have time to track
+ ;; it down yet.  For now, we skip the tests.
+ [(and (not gauche.os.cygwin)
+       (not gauche.os.windows))
+  (with-sr-udp
+   (lambda (s-sock s-addr r-sock r-addr)
+     (let ([from   (make <sockaddr-in>)]
+	   [data   `#(,(make-string 255 #\a) ,(make-u8vector 255 48))]
+	   [sbuf   (make-u8vector 1024)]
+	   [rbuf   (make-u8vector 1024 0)])
+
+       (define (xtest sbuf)
+	 (socket-sendmsg s-sock (socket-buildmsg r-addr data '() 0 sbuf))
+	 (receive (size f-addr) (socket-recvfrom! r-sock rbuf (list from))
+		  (list (eq? f-addr from)
+			(equal? (uvector-alias <u8vector> rbuf 0 size)
+				($ string->u8vector
+				   $ string-concatenate
+				   $ map (lambda (z)
+					   (if (string? z)
+					       z
+					       (u8vector->string z)))
+				   $ vector->list data)))))
+       
+       (test* "udp sendmsg w/sendbuf" '(#t #t) (xtest sbuf))
+       (test* "udp sendmsg w/o sendbuf" '(#t #t) (xtest #f)))))]
+ [else #f])
 
 (test-end)
