@@ -110,10 +110,6 @@
 (test* "length+" '(5 #f)
        (list (length+ '(1 2 3 4 5))
              (length+ (circular-list 1 2 3 4 5))))
-(test* "append" '(1 2 3 4 5) (append '(1 2 3) '() '(4 5)))
-(test* "append" '(1 2 3 . 5) (append '(1 2 3) 5))
-(test* "append!" '(1 2 3 4 5) (append (list 1 2 3) '() '(4 5)))
-(test* "append!" '(1 2 3 . 5) (append (list 1 2 3) 5))
 (test* "concatenate" '(1 2 3 4 5)
        (concatenate '((1 2 3) () (4 5))))
 (test* "concatenate" '(1 2 3 4 . 5)
@@ -122,10 +118,6 @@
        (concatenate (list (list 1 2 3) '() (list 4 5))))
 (test* "concatenate!" '(1 2 3 4 . 5)
        (concatenate (list (list 1 2 3) '() (list 4) 5)))
-(test* "reverse" '(5 4 3 2 1) (reverse '(1 2 3 4 5)))
-(test* "reverse" '() (reverse '()))
-(test* "reverse!" '(5 4 3 2 1) (reverse! (list 1 2 3 4 5)))
-(test* "reverse!" '() (reverse! '()))
 (test* "append-reverse" '(1 2 3 4 5)
        (append-reverse '(3 2 1) '(4 5)))
 (test* "append-reverse" '(1 2 3 4 . 5)
@@ -160,24 +152,6 @@
        (count < '(1 2 4 8) '(2 4 6 8 10 12 14 16)))
 (test* "count" 2
        (count < '(3 1 4 1) (circular-list 1 10)))
-(test* "fold" 55
-       (fold + 0 '(1 2 3 4 5 6 7 8 9 10)))
-(test* "fold" '(e d c b a)
-       (fold cons '() '(a b c d e)))
-(test* "fold" 3
-       (fold (lambda (x c) (if (symbol? x) (+ c 1) c))
-             0
-             '(a 3 b 8 c 9)))
-(test* "fold" '(c 3 b 2 a 1)
-       (fold cons* '() '(a b c) '(1 2 3 4 5)))
-(test* "fold-right" '(1 2 3 4 5)
-       (fold-right cons '() '(1 2 3 4 5)))
-(test* "fold-right" '(2 4 6)
-       (fold-right (lambda (x l) (if (even? x) (cons x l) l))
-                   '()
-                   '(1 2 3 4 5 6 7)))
-(test* "fold-right" '(a 1 b 2 c 3)
-       (fold-right cons* '() '(a b c) '(1 2 3 4 5)))
 (test* "pair-fold" '(5 4 3 2 1)
        (pair-fold (lambda (p t) (set-cdr! p t) p) '()
                   (list 1 2 3 4 5)))
@@ -333,6 +307,8 @@
        (delete-duplicates '(a b c d e) eq?))
 (test* "delete-duplicates" '(a b c d e)
        (delete-duplicates '(a b a c b a d d a e) eq?))
+(test* "delete-duplicates" '(a b c d . e)
+       (delete-duplicates '(a b a c b a d d a . e) eq?))
 (test* "delete-duplicates" '(1.0 2.0 3.0 4.0 5.0)
        (delete-duplicates '(1.0 2.0 1.0 2.0 3.0 3.0 4.0 1.0 5.0) eqv?))
 (test* "delete-duplicates" '("a" "b" "c" "d" "e")
@@ -1070,7 +1046,7 @@
 (define-syntax assert-error
   (syntax-rules ()
     ((_ msg expr)
-     (test* msg *test-error* expr))))
+     (test* msg (test-error) expr))))
 
 (assert-equal "make-vector 0"
               (vector-length (make-vector 5))
