@@ -1,7 +1,7 @@
 /*
  * thread.c - Scheme thread API
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: threads.c,v 1.20 2008-05-10 13:35:45 shirok Exp $
  */
 
 #include <gauche.h>
@@ -83,7 +81,7 @@ ScmObj Scm_MakeThread(ScmProcedure *thunk, ScmObj name)
    With pthread, the real thread is started as "detached" mode; i.e. once
    the thread exits, the resources allocated for the thread by the system
    is collected, including the result of the thread.  During this
-   deconstruction phase, the handler vm_cleanup() runs and saves the
+   deconstruction phase, the handler thread_cleanup() runs and saves the
    thread result to the ScmVM structure.  If nobody cares about the
    result of the thread, ScmVM structure will eventually be GCed.
    This is to prevent exitted thread's system resources from being
@@ -194,7 +192,7 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
     int intr = FALSE, tout = FALSE;
     
     pts = Scm_GetTimeSpec(timeout, &ts);
-    (void)SCM_INTERNAL_MUTEX_LOCK(target->vmlock);
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(target->vmlock);
     while (target->state != SCM_VM_TERMINATED) {
         if (pts) {
             int tr = pthread_cond_timedwait(&(target->cond), &(target->vmlock), pts);
@@ -204,8 +202,11 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
             pthread_cond_wait(&(target->cond), &(target->vmlock));
         }
     }
-    if (!tout) { result = target->result; resultx = target->resultException; }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(target->vmlock);
+    if (!tout) {
+        result = target->result; resultx = target->resultException;
+        target->resultException = SCM_FALSE; /* clear it */
+    }
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
     if (intr) Scm_SigCheck(Scm_VM());
     if (tout) {
         if (SCM_UNBOUNDP(timeoutval)) {
@@ -224,6 +225,121 @@ ScmObj Scm_ThreadJoin(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
 #endif /*!GAUCHE_USE_PTHREADS*/
 }
 
+/* Attempt to stop other thread.   See process_queued_requests() in vm.c
+   for the target VM operation.
+
+   VM Stop protocol:
+
+             target                           inspector
+  ------------------------------------------------------------------------
+                                    * LOCK(target->vmlock)
+                                    * Check if target is already inspected
+                                    * Set target's STOP flag
+                                    * Wait on target->cond
+                                      until target->state becomes STOPPED
+   * If STOP flag is up,
+     set self->state to STOPPED 
+     and signal self->cond
+     then wait on self->cond until 
+     target->state changes from STOPPED
+                                    * Inspect target VM state
+                                    * Set taget->state to RUNNABLE,
+                                      then signal state->cond
+   * Resume execution
+  -------------------------------------------------------------------------
+ */
+
+ScmObj Scm_ThreadStop(ScmVM *target, ScmObj timeout, ScmObj timeoutval)
+{
+#ifdef GAUCHE_USE_PTHREADS
+    struct timespec ts, *pts;
+    ScmVM *vm = Scm_VM();
+    ScmVM *taker = NULL;
+    int timedout;
+    int invalid_state = FALSE;
+    pts = Scm_GetTimeSpec(timeout, &ts);
+
+ retry:
+    timedout = FALSE;
+    pthread_mutex_lock(&target->vmlock);
+    if (target->state != SCM_VM_RUNNABLE
+        && target->state != SCM_VM_STOPPED) {
+        invalid_state = TRUE;
+    } else if (target->inspector != NULL
+        && target->inspector != vm
+        && target->inspector->state != SCM_VM_TERMINATED) {
+        taker = target->inspector;
+    } else {
+        /* NB: if target->inspector == vm, it means we have requested to
+           stop the thread before, and haven't called ThreadCont yet.
+           In that case we don't need to set stopRequest again; either target
+           thread already cleared it and stopped/going to stop, or 
+           the target thread hasn't seen the flag and it's still ON. */
+        if (target->inspector != vm) {
+            target->inspector = vm;
+            target->stopRequest = TRUE;
+            target->attentionRequest = TRUE;
+        }
+        while (target->state != SCM_VM_STOPPED && !timedout) {
+            if (pts) {
+                timedout = pthread_cond_timedwait(&target->cond,
+                                                  &target->vmlock,
+                                                  pts);
+            } else {
+                pthread_cond_wait(&target->cond, &target->vmlock);
+            }
+        }
+    }
+    pthread_mutex_unlock(&target->vmlock);
+
+    if (invalid_state) {
+        Scm_Error("cannot stop a thread %S since it is in neither runnable nor stopped state",
+                  target);
+    }
+    if (taker != NULL) {
+        Scm_Error("target %S is already under inspection by %S", target, taker);
+    }
+
+    if (timedout == EINTR) { Scm_SigCheck(vm); goto retry; }
+    if (timedout == ETIMEDOUT) return timeoutval;
+    return SCM_OBJ(target);
+#else  /*!GAUCHE_USE_PTHREADS*/
+    Scm_Error("not implemented!\n");
+    return SCM_UNDEFINED;
+#endif /*!GAUCHE_USE_PTHREADS*/
+}
+
+
+/* Release inspected thread */
+ScmObj Scm_ThreadCont(ScmVM *target)
+{
+#ifdef GAUCHE_USE_PTHREADS
+    int not_stopped = FALSE;
+    ScmVM *stopped_by_other = NULL;
+
+    pthread_mutex_lock(&target->vmlock);
+    if (target->inspector == NULL) {
+        not_stopped = TRUE;
+    } else if (target->inspector != Scm_VM()
+               && target->inspector->state != SCM_VM_TERMINATED) {
+        stopped_by_other = target->inspector;
+    } else {
+        target->inspector = NULL;
+        target->state = SCM_VM_RUNNABLE;
+        target->stopRequest = FALSE;
+        pthread_cond_broadcast(&target->cond);
+    }
+    pthread_mutex_unlock(&target->vmlock);
+    if (not_stopped) Scm_Error("target %S is not stopped", target);
+    if (stopped_by_other) Scm_Error("target %S is stopped by other thread %S",
+                                    target, stopped_by_other);
+    return SCM_OBJ(target);
+#else  /*!GAUCHE_USE_PTHREADS*/
+    Scm_Error("not implemented!\n");
+    return SCM_UNDEFINED;
+#endif /*!GAUCHE_USE_PTHREADS*/
+}
+
 /* Thread sleep */
 ScmObj Scm_ThreadSleep(ScmObj timeout)
 {
@@ -280,17 +396,10 @@ ScmObj Scm_ThreadTerminate(ScmVM *target)
 /*
  * Initialization.
  */
-extern void Scm_Init_mutex(ScmModule*);
-extern void Scm_Init_thrlib(ScmModule*);
-
-SCM_EXTENSION_ENTRY void Scm_Init_threads(void)
+void Scm_Init_threads(ScmModule *mod)
 {
-    ScmModule *mod = SCM_FIND_MODULE("gauche.threads", SCM_FIND_MODULE_CREATE);
-    SCM_INIT_EXTENSION(threads);
 #ifdef GAUCHE_USE_PTHREADS
     sigfillset(&threadrec.defaultSigmask);
 #endif /*GAUCHE_USE_PTHREADS*/
-    Scm_Init_mutex(mod);
-    Scm_Init_thrlib(mod);
 }
 
