@@ -1,7 +1,7 @@
 /*
  * threads.h - Gauche threads support
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: threads.h,v 1.7 2008-05-10 13:35:45 shirok Exp $
  */
 
 #ifndef GAUCHE_THREADS_H
@@ -48,36 +46,11 @@
 extern ScmObj Scm_MakeThread(ScmProcedure *thunk, ScmObj name);
 extern ScmObj Scm_ThreadStart(ScmVM *vm);
 extern ScmObj Scm_ThreadJoin(ScmVM *vm, ScmObj timeout, ScmObj timeoutval);
+extern ScmObj Scm_ThreadStop(ScmVM *vm, ScmObj timeout, ScmObj timeoutval);
+extern ScmObj Scm_ThreadCont(ScmVM *vm);
 extern ScmObj Scm_ThreadSleep(ScmObj timeout);
 extern ScmObj Scm_ThreadTerminate(ScmVM *vm);
 
-/*---------------------------------------------------------
- * Thread exception classes.
- * NB: The actual structure, ScmThreadException, is defined in
- * src/gauche/exception.h.
- */
-
-SCM_CLASS_DECL(Scm_ThreadExceptionClass);
-#define SCM_CLASS_THREAD_EXCEPTION  (&Scm_ThreadExceptionClass)
-#define SCM_THREAD_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_THREAD_EXCEPTION)
-#define SCM_THREAD_EXCEPTION(obj)   ((ScmThreadException*)(obj))
-
-SCM_CLASS_DECL(Scm_JoinTimeoutExceptionClass);
-#define SCM_CLASS_JOIN_TIMEOUT_EXCEPTION (&Scm_JoinTimeoutExceptionClass)
-#define SCM_JOIN_TIMEOUT_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_JOIN_TIMEOUT_EXCEPTION)
-
-SCM_CLASS_DECL(Scm_AbandonedMutexExceptionClass);
-#define SCM_CLASS_ABANDONED_MUTEX_EXCEPTION (&Scm_AbandonedMutexExceptionClass)
-#define SCM_ABANDONED_MUTEX_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_ABANDONED_MUTEX_EXCEPTION)
-
-SCM_CLASS_DECL(Scm_TerminatedThreadExceptionClass);
-#define SCM_CLASS_TERMINATED_THREAD_EXCEPTION (&Scm_TerminatedThreadExceptionClass)
-#define SCM_TERMINATED_THREAD_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_TERMINATED_THREAD_EXCEPTION)
-
-SCM_CLASS_DECL(Scm_UncaughtExceptionClass);
-#define SCM_CLASS_UNCAUGHT_EXCEPTION (&Scm_UncaughtExceptionClass)
-#define SCM_UNCAUGHT_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_UNCAUGHT_EXCEPTION)
-
 /*---------------------------------------------------------
  * SYNCHRONIZATION DEVICES
  *
