@@ -13,17 +13,14 @@
 ;;;  warranty.  In no circumstances the author(s) shall be liable
 ;;;  for any damages arising out of the use of this software.
 ;;;
-;;; $Id: matrix.scm,v 1.4 2006-04-06 04:09:31 shirok Exp $
-;;;
 
 (select-module gauche.array)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; general array manipulation
 
-(define (array-concatenate a b . opt)
-  (let* ((dimension (get-optional opt 0))
-         (a-start (start-vector-of a))
+(define (array-concatenate a b :optional (dimension 0))
+  (let* ((a-start (start-vector-of a))
          (a-end (end-vector-of a))
          (b-start (start-vector-of b))
          (b-end (end-vector-of b))
@@ -60,52 +57,49 @@
           (make-vector b-rank))
         c))))
 
-(define (array-transpose a . opt)
-  (let-optionals* opt ((dim1 0) (dim2 1))
-    (let* ((sh (copy-object (array-shape a)))
-           (rank (array-rank a))
-           (tmp0 (array-ref sh dim1 0))
-           (tmp1 (array-ref sh dim1 1)))
-      (array-set! sh dim1 0 (array-ref sh dim2 0))
-      (array-set! sh dim1 1 (array-ref sh dim2 1))
-      (array-set! sh dim2 0 tmp0)
-      (array-set! sh dim2 1 tmp1)
-      (let ((res (make-array sh)))
-        (array-for-each-index a
-          (lambda (vec1)
-            (let* ((vec2 (vector-copy vec1))
-                   (tmp (vector-ref vec2 dim1)))
-              (vector-set! vec2 dim1 (vector-ref vec2 dim2))
-              (vector-set! vec2 dim2 tmp)
-              (array-set! res vec2 (array-ref a vec1))))
-          (make-vector rank))
-        res))))
+(define (array-transpose a :optional (dim1 0) (dim2 1))
+  (let* ((sh (copy-object (array-shape a)))
+         (rank (array-rank a))
+         (tmp0 (array-ref sh dim1 0))
+         (tmp1 (array-ref sh dim1 1)))
+    (array-set! sh dim1 0 (array-ref sh dim2 0))
+    (array-set! sh dim1 1 (array-ref sh dim2 1))
+    (array-set! sh dim2 0 tmp0)
+    (array-set! sh dim2 1 tmp1)
+    (let ((res (make-array sh)))
+      (array-for-each-index a
+                            (lambda (vec1)
+                              (let* ((vec2 (vector-copy vec1))
+                                     (tmp (vector-ref vec2 dim1)))
+                                (vector-set! vec2 dim1 (vector-ref vec2 dim2))
+                                (vector-set! vec2 dim2 tmp)
+                                (array-set! res vec2 (array-ref a vec1))))
+                            (make-vector rank))
+      res)))
 
-(define (array-rotate-90 a . opt)
-  (let-optionals* opt ((dim1 0) (dim2 1))
-    (let* ((sh (copy-object (array-shape a)))
-           (rank (array-rank a))
-           (tmp0 (array-ref sh dim1 0))
-           (tmp1 (array-ref sh dim1 1))
-           (off (- (array-ref sh dim1 1) 1)))
-      (array-set! sh dim1 0 (array-ref sh dim2 0))
-      (array-set! sh dim1 1 (array-ref sh dim2 1))
-      (array-set! sh dim2 0 tmp0)
-      (array-set! sh dim2 1 tmp1)
-      (let ((res (make-array sh)))
-        (array-for-each-index a
-          (lambda (vec1)
-            (let* ((vec2 (vector-copy vec1))
-                   (tmp (vector-ref vec2 dim1)))
-              (vector-set! vec2 dim1 (vector-ref vec2 dim2))
-              (vector-set! vec2 dim2 (- off tmp))
-              (array-set! res vec2 (array-ref a vec1))))
-          (make-vector rank))
-        res))))
+(define (array-rotate-90 a :optional (dim1 0) (dim2 1))
+  (let* ((sh (copy-object (array-shape a)))
+         (rank (array-rank a))
+         (tmp0 (array-ref sh dim1 0))
+         (tmp1 (array-ref sh dim1 1))
+         (off (- (array-ref sh dim1 1) 1)))
+    (array-set! sh dim1 0 (array-ref sh dim2 0))
+    (array-set! sh dim1 1 (array-ref sh dim2 1))
+    (array-set! sh dim2 0 tmp0)
+    (array-set! sh dim2 1 tmp1)
+    (let ((res (make-array sh)))
+      (array-for-each-index a
+                            (lambda (vec1)
+                              (let* ((vec2 (vector-copy vec1))
+                                     (tmp (vector-ref vec2 dim1)))
+                                (vector-set! vec2 dim1 (vector-ref vec2 dim2))
+                                (vector-set! vec2 dim2 (- off tmp))
+                                (array-set! res vec2 (array-ref a vec1))))
+                            (make-vector rank))
+      res)))
 
-(define (array-flip! a . opt)
-  (let* ((dimension (get-optional opt 0))
-         (lo (s32vector-ref (start-vector-of a) dimension))
+(define (array-flip! a :optional (dimension 0))
+  (let* ((lo (s32vector-ref (start-vector-of a) dimension))
          (end (end-vector-of a))
          (hi (s32vector-ref end dimension))
          (half (quotient hi 2))
@@ -132,9 +126,8 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; linear algebra
 
-(define (identity-array n . opt)
-  (let ((res (make-array-internal (get-optional opt <array>)
-                                  (shape 0 n 0 n) 0)))
+(define (identity-array n :optional (class <array>))
+  (let ((res (make-array-internal class (shape 0 n 0 n) 0)))
     (do ((i 0 (+ i 1)))
         ((= i n) res)
       (array-set! res i i 1))))
@@ -434,11 +427,14 @@
 ;;
 ;; The API is subject to change.
 
-(define (pretty-print-array a . opt)
-  (let* ((p1 (get-optional opt (current-output-port)))
-         (p (cond ((port? p1) p1)
-                  (p1 (current-output-port))
-                  (else (open-output-string))))
+(define (pretty-print-array a :optional (port (current-output-port))
+                            :key (readable? #t) (fill? #t)
+                            (line-char #f) (left line-char) (right left)
+                            (middle-col #\space) (top line-char)
+                            (bottom top) (center-row #f) (pad-char #\space))
+  (let* ((p (cond [(port? port) port]
+                  [port (current-output-port)]
+                  [else (open-output-string)]))
          (rank (array-rank a))
          (row-dim (- rank 2))
          (col-dim (- rank 1))
@@ -447,102 +443,92 @@
          (inner-shape (take-right shape-list 4))
          (nrows (cadr inner-shape))
          (ncols (cadddr inner-shape)))
-    (let-keywords* (if (pair? opt) (cdr opt) '())
-        ((readable? #t)
-         (fill? #t)
-         (line-char #f)
-         (left line-char)
-         (right left)
-         (middle-col #\space)
-         (top line-char)
-         (bottom top)
-         (center-row #f)
-         (pad-char #\space))
-      (define (replicate x width)
-        (if (char? x)
-          (make-string width x)
-          (let ((len (string-length x)))
-            (if (eq? len 1)
-              (make-string width (string-ref x 0))
-              (receive (quot rem) (quotient&remainder width len)
-                (string-append
-                 (apply string-append (map (lambda (n) x) (iota quot)))
-                 (substring x 0 rem)))))))
-      (define (pad x width)
-        (let* ((res (x->string x))
-               (len (string-length res)))
-          (if (< len width)
-            (string-append (make-string (- width len) pad-char) res )
-            res)))
-      (define (print-width x)
-        (cond ((char? x) 1)
-              ((string? x) (string-length x))
-              (else (string-length (x->string x)))))
-      (let ((fixed-width (+ (if left (print-width left) 0)
-                            (if right (print-width right) 0)
-                            (if middle-col (* (- ncols 1) (print-width middle-col)) 0))))
-        (define (column-widths vec)
-          (let loop1 ((col (caddr inner-shape))
-                      (res '()))
-            (cond
-              ((< col ncols)
-               (vector-set! vec col-dim col)
-               (let loop2 ((row (car inner-shape))
-                           (x 0))
-                 (cond
-                   ((< row nrows)
-                    (vector-set! vec row-dim row)
-                    (loop2 (+ row 1) (max x (print-width (array-ref a vec)))))
-                   (else
-                    (loop1 (+ col 1) (cons x res))))))
-              (else (reverse res)))))
-        (define (print2d vec)
-          (newline p)
-          (let* ((widths (if fill? (column-widths (vector-copy vec)) '()))
-                 (total-width (apply + fixed-width widths)))
-            (when top
-              (display (if fill? (replicate top total-width) top) p)
-              (newline p))
-            (let loop1 ((row (car inner-shape)))
-              (when (< row nrows)
-                (when left (display left p))
+
+    (define (replicate x width)
+      (if (char? x)
+        (make-string width x)
+        (let ((len (string-length x)))
+          (if (eq? len 1)
+            (make-string width (string-ref x 0))
+            (receive (quot rem) (quotient&remainder width len)
+              (string-append
+               (apply string-append (map (lambda (n) x) (iota quot)))
+               (substring x 0 rem)))))))
+    (define (pad x width)
+      (let* ((res (x->string x))
+             (len (string-length res)))
+        (if (< len width)
+          (string-append (make-string (- width len) pad-char) res )
+          res)))
+    (define (print-width x)
+      (cond ((char? x) 1)
+            ((string? x) (string-length x))
+            (else (string-length (x->string x)))))
+    (let ((fixed-width (+ (if left (print-width left) 0)
+                          (if right (print-width right) 0)
+                          (if middle-col (* (- ncols 1) (print-width middle-col)) 0))))
+      (define (column-widths vec)
+        (let loop1 ((col (caddr inner-shape))
+                    (res '()))
+          (cond
+           ((< col ncols)
+            (vector-set! vec col-dim col)
+            (let loop2 ((row (car inner-shape))
+                        (x 0))
+              (cond
+               ((< row nrows)
                 (vector-set! vec row-dim row)
-                (let loop2 ((col (caddr inner-shape))
-                            (w widths))
-                  (when (< col ncols)
-                    (vector-set! vec col-dim col)
-                    (display (if (pair? w)
-                               (pad (array-ref a vec) (car w))
-                               (array-ref a vec))
-                             p)
-                    (when (and middle-col (< (+ col 1) ncols))
-                      (display middle-col p))
-                    (loop2 (+ col 1) (if (pair? w) (cdr w) w))))
-                (when right (display right p))
-                (newline p)
-                (when (and center-row (< (+ row 1) nrows))
-                  (display (if fill? (replicate bottom total-width) bottom))
-                  (newline p))
-                (loop1 (+ row 1))))
-            (when bottom
-              (display (if fill? (replicate bottom total-width) bottom) p)
-              (newline p))))
-        (cond
-          ((< rank 2)
-           (if readable? (write a p) (display a p)))
-          (else
-           (when readable?
-             (format p "#,(~A ~S" (class-name (class-of a)) shape-list))
-           (if (= rank 2)
-             (print2d (make-vector rank))
-             (array-for-each-index-by-dimension
-              a (drop-right (iota rank) 2)
-              print2d
-              (make-vector rank)))
-           (when readable?
-             (format p "  )"))
-           (newline p)))
-        (if (not p1) (get-output-string p))))))
+                (loop2 (+ row 1) (max x (print-width (array-ref a vec)))))
+               (else
+                (loop1 (+ col 1) (cons x res))))))
+           (else (reverse res)))))
+      (define (print2d vec)
+        (newline p)
+        (let* ((widths (if fill? (column-widths (vector-copy vec)) '()))
+               (total-width (apply + fixed-width widths)))
+          (when top
+            (display (if fill? (replicate top total-width) top) p)
+            (newline p))
+          (let loop1 ((row (car inner-shape)))
+            (when (< row nrows)
+              (when left (display left p))
+              (vector-set! vec row-dim row)
+              (let loop2 ((col (caddr inner-shape))
+                          (w widths))
+                (when (< col ncols)
+                  (vector-set! vec col-dim col)
+                  (display (if (pair? w)
+                             (pad (array-ref a vec) (car w))
+                             (array-ref a vec))
+                           p)
+                  (when (and middle-col (< (+ col 1) ncols))
+                    (display middle-col p))
+                  (loop2 (+ col 1) (if (pair? w) (cdr w) w))))
+              (when right (display right p))
+              (newline p)
+              (when (and center-row (< (+ row 1) nrows))
+                (display (if fill? (replicate bottom total-width) bottom))
+                (newline p))
+              (loop1 (+ row 1))))
+          (when bottom
+            (display (if fill? (replicate bottom total-width) bottom) p)
+            (newline p))))
+      (cond
+       ((< rank 2)
+        (if readable? (write a p) (display a p)))
+       (else
+        (when readable?
+          (format p "#,(~A ~S" (class-name (class-of a)) shape-list))
+        (if (= rank 2)
+          (print2d (make-vector rank))
+          (array-for-each-index-by-dimension
+           a (drop-right (iota rank) 2)
+           print2d
+           (make-vector rank)))
+        (when readable?
+          (format p "  )"))
+        (newline p)))
+      (if (not port) (get-output-string p)))))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -596,5 +582,4 @@
           (return <array>) ; already most general array
           (loop next (cdr l)))))))
 
-(provide "gauche/matrix")
 
