@@ -3,6 +3,7 @@
 ;;
 
 (use gauche.test)
+(use gauche.parameter)
 (use srfi-1)
 
 (test-start "uniform vector and array")
@@ -28,6 +29,8 @@
 (test* "#s64()" #f (s64vector? '#(0 1 2 3 4)))
 (test* "#u64()" #t (u64vector? '#u64(0 1 2 3 4)))
 (test* "#u64()" #f (u64vector? '#(0 1 2 3 4)))
+(test* "#f16()" #t (f16vector? '#f16(0.0 1.0 2.0 3.0 4.0)))
+(test* "#f16()" #f (f16vector? '#(0.0 1.0 2.0 3.0 4.0)))
 (test* "#f32()" #t (f32vector? '#f32(0.0 1.0 2.0 3.0 4.0)))
 (test* "#f32()" #f (f32vector? '#(0.0 1.0 2.0 3.0 4.0)))
 (test* "#f64()" #t (f64vector? '#f64(0.0 1.0 2.0 3.0 4.0)))
@@ -60,6 +63,9 @@
 (test* "#u64()" "#u64(0 1 2 3 4)"
        (with-output-to-string
          (lambda () (write (apply u64vector (iota 5))))))
+(test* "#f16()" "#f16()"
+       (with-output-to-string
+         (lambda () (write (f16vector)))))
 (test* "#f32()" "#f32()"
        (with-output-to-string
          (lambda () (write (f32vector)))))
@@ -109,6 +115,10 @@
        (uvrefset-tester make-u64vector u64vector-ref u64vector-set!
                         '(0 1 2 #xffffffffffffffff)
                         '#u64(0 1 2 #xffffffffffffffff)))
+(test* "f16vector-ref|set!" #t
+       (uvrefset-tester make-f16vector f16vector-ref f16vector-set!
+                        '(0.0 -1.0 1.0)
+                        '#f16(0.0 -1.0 1.0)))
 (test* "f32vector-ref|set!" #t
        (uvrefset-tester make-f32vector f32vector-ref f32vector-set!
                         '(0.0 -1.0 1.0)
@@ -222,6 +232,11 @@
                       u64vector->vector vector->u64vector
                       '#u64(0 1 #xffffffffffffffff)
                       '(0 1 #xffffffffffffffff)))
+(test* "f16vector conversion" #t
+       (uvconv-tester f16vector->list list->f16vector
+                      f16vector->vector vector->f16vector
+                      '#f16(0.0 -1.0 1.0)
+                      '(0.0 -1.0 1.0)))
 (test* "f32vector conversion" #t
        (uvconv-tester f32vector->list list->f32vector
                       f32vector->vector vector->f32vector
@@ -233,6 +248,30 @@
                       '#f64(0.0 -1.0 1.0)
                       '(0.0 -1.0 1.0)))
 
+;;-------------------------------------------------------------------
+(test-section "comparison")
+
+(define (uvcomp-tester list-> samples)
+  (dolist (x samples)
+    (dolist (y samples)
+      (test* #`"equal? ,(list-> x) ,(list-> y)"
+             (and (= (length x) (length y))
+                  (every = x y))
+             (equal? (list-> x) (list-> y))))))
+
+(for-each (cut uvcomp-tester <> '(() (0) (1) (0 0) (0 1)))
+          (list list->s8vector list->u8vector
+                list->s16vector list->u16vector
+                list->s32vector list->u32vector
+                list->s64vector list->u64vector))
+
+(for-each (cut uvcomp-tester <> '(() (0) (1) (0 0) (0 1)
+                                  (+inf.0) (-inf.0) (+nan.0)
+                                  (0 +inf.0) (0 -inf.0) (0 +nan.0)))
+          (list list->f16vector
+                list->f32vector
+                list->f64vector))
+
 ;;-------------------------------------------------------------------
 (test-section "copying and filling")
 
@@ -280,6 +319,10 @@
                       u64vector->list list->u64vector
                       '#u64(0 1 #xffffffffffffffff) #x8000000000000000))
 
+(test* "f16vector copy|fill!" #t
+       (uvcopy-tester f16vector-copy f16vector-copy! f16vector-fill!
+                      f16vector->list list->f16vector
+                      '#f16(0 -1.0 1.0) 1.0))
 (test* "f32vector copy|fill!" #t
        (uvcopy-tester f32vector-copy f32vector-copy! f32vector-fill!
                       f32vector->list list->f32vector
@@ -332,9 +375,23 @@
 (uvcopy!-newapi-test "u32vector-copy! newapi" u32vector u32vector-copy!)
 (uvcopy!-newapi-test "s64vector-copy! newapi" s64vector s64vector-copy!)
 (uvcopy!-newapi-test "u64vector-copy! newapi" u64vector u64vector-copy!)
+(uvcopy!-newapi-test "f16vector-copy! newapi" f16vector f16vector-copy!)
 (uvcopy!-newapi-test "f32vector-copy! newapi" f32vector f32vector-copy!)
 (uvcopy!-newapi-test "f64vector-copy! newapi" f64vector f64vector-copy!)
 
+(test* "uvector-copy! (generic)" '#u16(0 0 1 2 65535 0)
+       (rlet1 v (make-u16vector 6 0)
+         (uvector-copy! v 2 '#s16(1 2 -1))))
+(test* "uvector-copy! (generic, start)" '#u16(0 0 1 2 65535 0)
+       (rlet1 v (make-u16vector 6 0)
+         (uvector-copy! v 2 '#s16(0 1 2 -1) 1)))
+(test* "uvector-copy! (generic, start+end)" '#u16(0 0 1 2 65535 0)
+       (rlet1 v (make-u16vector 6 0)
+         (uvector-copy! v 2 '#s16(0 1 2 -1 -2) 1 4)))
+(test* "uvector-copy! (generic, different size)" '#u16(0 0 257 0)
+       (rlet1 v (make-u16vector 4 0)
+         (uvector-copy! v 2 '#u8(1 1))))
+
 ;;-------------------------------------------------------------------
 (test-section "swapping bytes")
 
@@ -413,6 +470,8 @@
        (collection-tester <s64vector> (s64vector 1 2 3 4)))
 (test* "u64vector collection interface" #t
        (collection-tester <u64vector> (u64vector 1 2 3 4)))
+(test* "f16vector collection interface" #t
+       (collection-tester <f16vector> (f16vector 1 2 3 4)))
 (test* "f32vector collection interface" #t
        (collection-tester <f32vector> (f32vector 1 2 3 4)))
 (test* "f64vector collection interface" #t
@@ -638,6 +697,7 @@
          (div (make 0.0 1.0 2.0 3.0) 2.0))
   )
 
+(flonum-arith-test-generate f16)
 (flonum-arith-test-generate f32)
 (flonum-arith-test-generate f64)
 
@@ -836,6 +896,23 @@
 (dotprod-test-generate u64 #u64(1 2 3 4 18446744073709551615)
                        #u64(1 2 3 4 18446744073709551615))
 
+(dotprod-test-generate f16 #f16(0 1 2 3) #f16(4 5 6 7))
+(dotprod-test-generate f16 #f16(0 -1 2 -3) #f16(-4 5 -6 7))
+(dotprod-test-generate f16 #f16(16384 16384 16384 16384 16384)
+                       #f16(16384 16384 16384 16384 16384))
+(dotprod-test-generate f16 #f16(16384 -16384 16384 -16384 16384)
+                       #f16(16384 -16384 16384 -16384 16384))
+(dotprod-test-generate f16 #f16(32767 32767 32767 32767 32767)
+                       #f16(32767 32767 32767 32767 32767))
+(dotprod-test-generate f16 #f16(214748367 214748367 214748367 214748367 214748367)
+                       #f16(214748367 214748367 214748367 214748367 214748367))
+(dotprod-test-generate f16 #f16(9223372036854775807 1 2 3 4)
+                       #f16(9223372036854775807 1 2 3 4))
+(dotprod-test-generate f16 #f16(1 2 3 4 9223372036854775807 1)
+                       #f16(1 2 3 4 9223372036854775807 1))
+(dotprod-test-generate f16 #f16(32767 -32767 32767 -32767 32767)
+                       #f16(32767 -32767 32767 -32767 32767))
+
 (dotprod-test-generate f32 #f32(0 1 2 3) #f32(4 5 6 7))
 (dotprod-test-generate f32 #f32(0 -1 2 -3) #f32(-4 5 -6 7))
 (dotprod-test-generate f32 #f32(16384 16384 16384 16384 16384)
@@ -1175,6 +1252,87 @@
 (clamp-test-generate u64 #u64(127 0 4 200 255)
                      #u64(3 3 3 3 3) #u64(199 199 199 199 199))
 
+;;-------------------------------------------------------------------
+(test-section "block i/o")
+
+(sys-unlink "test.o")
+
+(let* ([data1 '#u8(0 1 2 3 255 254 253 252 4 5 6 7 251 250 249 248)]
+       [data2 '#u8(1 0 3 2 254 255 252 253 5 4 7 6 250 251 248 249)]
+       [data3 '#u8(3 2 1 0 252 253 254 255 7 6 5 4 248 249 250 251)]
+       [data4 '#u8(252 253 254 255 3 2 1 0 248 249 250 251 7 6 5 4)]
+
+       [data5 '#u8(3 0 1 3 3 1 0 3 131 0 1 131 131 1 0 131)]
+       )
+
+  ;; Invaliance:
+  ;; w = (uvector-alias T u8v)  <--> w = read-block! T + write-block u8v
+  (define (test-default-endian data)
+    (lambda (T maker size)
+      (test* (format "native endian ~a" (class-name T))
+             (uvector-alias T data)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o" (cut write-block data <>))
+               (call-with-input-file "test.o" (cut read-block! buf <>))))))
+
+  (define (test-reverse-endian T maker size)
+    (let* ([rev-endian (case (native-endian)
+                         [(big-endian) 'little-endian]
+                         [(little-endian arm-little-endian) 'big-endian])]
+           [rev-vec (cond
+                     [(memq T (list <u8vector> <s8vector>)) data1]
+                     [(memq T (list <u16vector> <s16vector>)) data2]
+                     [(memq T (list <u32vector> <s32vector>)) data3]
+                     [(memq T (list <u64vector> <s64vector>)) data4])])
+      (test* (format "reading ~a ~a" rev-endian (class-name T))
+             (uvector-alias T rev-vec)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o" (cut write-block data1 <>))
+               (call-with-input-file "test.o"
+                 (cut read-block! buf <> 0 -1 rev-endian))))
+      (test* (format "reading ~a ~a, parameter" rev-endian (class-name T))
+             (uvector-alias T rev-vec)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o" (cut write-block data1 <>))
+               (parameterize ((default-endian rev-endian))
+                 (call-with-input-file "test.o" (cut read-block! buf <>)))))
+      (test* (format "writing ~a ~a" rev-endian (class-name T))
+             (uvector-alias T data1)
+             (rlet1 buf (maker size)
+               (call-with-output-file "test.o"
+                 (cut write-block (uvector-alias T rev-vec) <> 0 -1 rev-endian))
+               (call-with-input-file "test.o" (cut read-block! buf <>))))
+      (test* (format "writing ~a ~a, parameter" rev-endian (class-name T))
+             (uvector-alias T data1)
+             (rlet1 buf (maker size)
+               (parameterize ((default-endian rev-endian))
+                 (call-with-output-file "test.o"
+                   (cut write-block (uvector-alias T rev-vec) <>)))
+               (call-with-input-file "test.o" (cut read-block! buf <>))))))
+
+  (define (run-across fn)
+    (let1 s (u8vector-length data1)
+      (fn <u8vector> make-u8vector s)
+      (fn <s8vector> make-s8vector s)
+      (fn <u16vector> make-u16vector (/ s 2))
+      (fn <s16vector> make-s16vector (/ s 2))
+      (fn <u32vector> make-u32vector (/ s 4))
+      (fn <s32vector> make-s32vector (/ s 4))
+      (fn <u64vector> make-u64vector (/ s 8))
+      (fn <s64vector> make-s64vector (/ s 8))
+      ))
+  (define (run-across-f fn)
+    (let1 s (u8vector-length data5)
+      (fn <f16vector> make-f16vector (/ s 2))
+      (fn <f32vector> make-f32vector (/ s 4))
+      (fn <f64vector> make-f64vector (/ s 8))
+      ))
+  
+  (run-across (test-default-endian data1))
+  (run-across-f (test-default-endian data5))
+  (run-across test-reverse-endian)
+  )
+
 ;;-------------------------------------------------------------------
 (test-section "string <-> uvector")
 
@@ -1186,7 +1344,7 @@
        (string->u8vector "@ABCD" 1 4))
 (test* "string->u8vector (start, end)" '#u8(64 65 66 67 68)
        (string->u8vector "@ABCD" 0 5))
-(test* "string->u8vector (OOB)" *test-error*
+(test* "string->u8vector (OOB)" (test-error)
        (string->u8vector "abcde" 2 6))
 
 (test* "string->u8vector!" '#u8(64 65 66 67 68)
@@ -1219,7 +1377,7 @@
        (u8vector->string '#u8(64 65 66 67 68) 2 4))
 (test* "u8vector->string (start, end)" "@ABCD"
        (u8vector->string '#u8(64 65 66 67 68) 0 5))
-(test* "u8vector->string (OOB)" *test-error*
+(test* "u8vector->string (OOB)" (test-error)
        (u8vector->string '#u8(64 65 66 67 68) 0 8))
 
 (test* "string->s8vector" '#s8(64 65 66 67 68)
@@ -1230,7 +1388,7 @@
        (string->s8vector "@ABCD" 1 4))
 (test* "string->s8vector (start, end)" '#s8(64 65 66 67 68)
        (string->s8vector "@ABCD" 0 5))
-(test* "string->s8vector (OOB)" *test-error*
+(test* "string->s8vector (OOB)" (test-error)
        (string->s8vector "abcde" 2 6))
 
 (test* "string->s8vector!" '#s8(64 65 66 67 68)
@@ -1263,7 +1421,7 @@
        (s8vector->string '#s8(64 65 66 67 68) 2 4))
 (test* "s8vector->string (start, end)" "@ABCD"
        (s8vector->string '#s8(64 65 66 67 68) 0 5))
-(test* "s8vector->string (OOB)" *test-error*
+(test* "s8vector->string (OOB)" (test-error)
        (s8vector->string '#s8(64 65 66 67 68) 0 8))
 
 (test* "string->u32vector" '#u32(64 65 66 67 68)
@@ -1274,7 +1432,7 @@
        (string->u32vector "@ABCD" 1 4))
 (test* "string->u32vector (start, end)" '#u32(64 65 66 67 68)
        (string->u32vector "@ABCD" 0 5))
-(test* "string->u32vector (OOB)" *test-error*
+(test* "string->u32vector (OOB)" (test-error)
        (string->u32vector "abcde" 2 6))
 
 (test* "u32vector->string" "@ABCD"
@@ -1285,7 +1443,7 @@
        (u32vector->string '#u32(64 65 66 67 68) 2 4))
 (test* "u32vector->string (start, end)" "@ABCD"
        (u32vector->string '#u32(64 65 66 67 68) 0 5))
-(test* "u32vector->string (OOB)" *test-error*
+(test* "u32vector->string (OOB)" (test-error)
        (u32vector->string '#u32(64 65 66 67 68) 0 8))
 
 (test* "string->s32vector" '#s32(64 65 66 67 68)
@@ -1296,7 +1454,7 @@
        (string->s32vector "@ABCD" 1 4))
 (test* "string->s32vector (start, end)" '#s32(64 65 66 67 68)
        (string->s32vector "@ABCD" 0 5))
-(test* "string->s32vector (OOB)" *test-error*
+(test* "string->s32vector (OOB)" (test-error)
        (string->s32vector "abcde" 2 6))
 
 (test* "s32vector->string" "@ABCD"
@@ -1307,7 +1465,7 @@
        (s32vector->string '#s32(64 65 66 67 68) 2 4))
 (test* "s32vector->string (start, end)" "@ABCD"
        (s32vector->string '#s32(64 65 66 67 68) 0 5))
-(test* "s32vector->string (OOB)" *test-error*
+(test* "s32vector->string (OOB)" (test-error)
        (s32vector->string '#s32(64 65 66 67 68) 0 8))
 
 ;; test for multibyte chars
@@ -1367,21 +1525,21 @@
                       4 8))
 
 ;; test alignment check
-(test* "alias u32 u8 (alignment violation)" *test-error*
+(test* "alias u32 u8 (alignment violation)" (test-error)
        (let* ((src (make-u8vector 9))
               (dst (uvector-alias <u32vector>)))
          dst))
-(test* "alias u32 u8 (alignment violation)" *test-error*
+(test* "alias u32 u8 (alignment violation)" (test-error)
        (let* ((src (make-u8vector 32))
               (dst (uvector-alias <u32vector> 2)))
          dst))
-(test* "alias u32 u8 (alignment violation)" *test-error*
+(test* "alias u32 u8 (alignment violation)" (test-error)
        (let* ((src (make-u8vector 32))
               (dst (uvector-alias <u32vector> 4 5)))
          dst))
 
 ;; test if immutable property propagates
-(test* "immutability violation" *test-error*
+(test* "immutability violation" (test-error)
        (let* ((src '#u8(0 1 2 3))
               (dst (uvector-alias <u8vector> src)))
          (u8vector-set! dst 0 1)))
@@ -1405,9 +1563,9 @@
             (shape 1 2 3 4 5 6 7 8 1 2 3 4 5 6 7 8 1 2 3 4 5 6 7 8)
             #t))
 
-(test* "shape" *test-error* (shape 1))
-(test* "shape" *test-error* (shape 1 2 3))
-(test* "shape" *test-error* (shape 3 1))
+(test* "shape" (test-error) (shape 1))
+(test* "shape" (test-error) (shape 1 2 3))
+(test* "shape" (test-error) (shape 3 1))
 
 (test* "make-array" #t
        (and (make-array (shape))
