@@ -1,7 +1,7 @@
 ;;;
 ;;; dbm - abstract base class for dbm interface
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: dbm.scm,v 1.11 2008-05-10 13:35:48 shirok Exp $
-;;;
 
 (define-module dbm
   (use gauche.collection)
@@ -40,7 +38,8 @@
           dbm-open    dbm-close   dbm-closed? dbm-get
           dbm-put!    dbm-delete! dbm-exists?
           dbm-fold    dbm-for-each  dbm-map
-          dbm-db-exists? dbm-db-remove dbm-db-copy dbm-db-move dbm-db-rename)
+          dbm-db-exists? dbm-db-remove dbm-db-copy dbm-db-move dbm-db-rename
+          dbm-type->class)
   )
 (select-module dbm)
 
@@ -80,27 +79,25 @@
 
 (autoload file.util file-eq? copy-file move-file)
 
-(define (%dbm-copy2 from1 to1 from2 to2 . keys)
-  (let-keywords keys ((if-exists :error))
-    (if (file-eq? from1 from2)
-      (begin ;; dir and pag files are identical
-        (copy-file from1 to1 :safe #t :if-exists if-exists)
-        (sys-link to1 to2))
-      (begin
-        (copy-file from1 to1 :safe #t :if-exists if-exists)
-        (guard (e (else (sys-unlink to1) (sys-unlink to2) (raise e)))
-          (copy-file from2 to2 :safe #t :if-exists if-exists))))))
+(define (%dbm-copy2 from1 to1 from2 to2 :key (if-exists :error))
+  (if (file-eq? from1 from2)
+    (begin ;; dir and pag files are identical
+      (copy-file from1 to1 :safe #t :if-exists if-exists)
+      (sys-link to1 to2))
+    (begin
+      (copy-file from1 to1 :safe #t :if-exists if-exists)
+      (guard (e (else (sys-unlink to1) (sys-unlink to2) (raise e)))
+        (copy-file from2 to2 :safe #t :if-exists if-exists)))))
 
-(define (%dbm-rename2 from1 to1 from2 to2 . keys)
-  (let-keywords keys ((if-exists :error))
-    (if (file-eq? from1 from2)
-      (begin
-        (move-file from1 to1 :if-exists if-exists)
-        (sys-link to1 to2)
-        (sys-unlink from2))
-      (begin
-        (move-file from1 to1 :if-exists if-exists)
-        (move-file from2 to2 :if-exists if-exists)))))
+(define (%dbm-rename2 from1 to1 from2 to2 :key (if-exists :error))
+  (if (file-eq? from1 from2)
+    (begin
+      (move-file from1 to1 :if-exists if-exists)
+      (sys-link to1 to2)
+      (sys-unlink from2))
+    (begin
+      (move-file from1 to1 :if-exists if-exists)
+      (move-file from2 to2 :if-exists if-exists))))
 
 ;;
 ;; DBM-OPEN
@@ -194,6 +191,18 @@
 ;        (lambda () (begin0 buf (set! buf (fetch))))))
 
 ;; Dictionary framework
+(define-method dict-get ((dbm <dbm>) key . maybe-default)
+  (apply dbm-get dbm key maybe-default))
+
+(define-method dict-put! ((dbm <dbm>) key val)
+  (dbm-put! dbm key val))
+
+(define-method dict-exists? ((dbm <dbm>) key)
+  (dbm-exists? dbm key))
+
+(define-method dict-delete! ((dbm <dbm>) key)
+  (dbm-delete! dbm key))
+
 (define-method dict-fold ((dbm <dbm>) proc seed)
   (dbm-fold dbm proc seed))
 
@@ -237,4 +246,14 @@
 
 (define dbm-db-rename dbm-db-move) ; backward compatibility
 
-(provide "dbm")
+;; Try to dynamically load named dbm module and returns the class.
+;; DBMTYPE must be a symbol like 'gdbm'.
+;; Returns #f if it couldn't retrieve the named dbm module.
+
+(define (dbm-type->class dbmtype)
+  (let ([module-name (string->symbol #`"dbm.,dbmtype")]
+        [class-name (string->symbol #`"<,|dbmtype|>")])
+    (and (library-exists? module-name :strict? #t)
+         (guard (e [else #f])
+           (%require (module-name->path module-name))
+           (global-variable-ref (find-module module-name) class-name)))))
