@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.cgen.cise - C in S expression
 ;;;  
-;;;   Copyright (c) 2004-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,24 +30,26 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: cise.scm,v 1.9 2008-05-10 13:35:57 shirok Exp $
-;;;
 
 (define-module gauche.cgen.cise
   (use srfi-1)
+  (use srfi-13)
   (use gauche.sequence)
   (use gauche.parameter)
   (use gauche.cgen.unit)
   (use gauche.cgen.literal)
-  (use gauche.experimental.ref)
+  (use gauche.experimental.app)
+  (use gauche.experimental.lamb)
   (use util.match)
   (use util.list)
-  (export cise-render cise-render-rec
+  (export cise-render cise-render-to-string cise-render-rec
+          cise-translate
           cise-context cise-context-copy cise-register-macro! cise-lookup-macro
           cise-emit-source-line
           define-cise-macro
           define-cise-stmt
           define-cise-expr
+          define-cise-toplevel
           )
   )
 (select-module gauche.cgen.cise)
@@ -59,8 +61,13 @@
 ;; If true, include #line directive in the output.
 (define cise-emit-source-line (make-parameter #t))
 
+;; The global context
+(define-class <cise-context> ()
+  ((macros :init-keyword :macros :init-form (make-hash-table 'eq?))
+   (static-decls  :init-keyword :static-decls  :init-value '())))
+
 ;; Keeps the cise macro bindings.
-(define cise-context (make-parameter (make-hash-table 'eq?)))
+(define cise-context (make-parameter (make <cise-context>)))
 
 ;;=============================================================
 ;; Environment
@@ -69,16 +76,17 @@
 ;; Environment must be treated opaque from outside of CISE module.
 
 (define-class <cise-env> ()
-  ((context :init-keyword :context :init-value 'stmt) ; stmt or expr
-   (decls   :init-keyword :decls   :init-value '())   ; list of extra decls
+  ((context :init-keyword :context) ; toplevel, stmt or expr
+   (decls   :init-keyword :decls)   ; list of extra decls
    ))
 
 (define (make-env context decls)
   (make <cise-env> :context context :decls decls))
-(define (env-ctx env)   [~ env'context])
-(define (env-decls env) [~ env'decls])
+(define (env-ctx env)   (~ env'context))
+(define (env-decls env) (~ env'decls))
 (define (expr-ctx? env) (eq? (env-ctx env) 'expr))
 (define (stmt-ctx? env) (eq? (env-ctx env) 'stmt))
+(define (toplevel-ctx? env) (eq? (env-ctx env) 'toplevel))
 
 (define (null-env)      (make-env 'stmt '()))
 
@@ -89,17 +97,24 @@
 
 (define (ensure-stmt-ctx form env)
   (unless (stmt-ctx? env)
-    (error "cise: statment appears in an expression context:" form)))
+    (if (expr-ctx? env)
+      (error "cise: statment appears in an expression context:" form)
+      (error "cise: statment appears in a toplevel context:" form))))
 
+(define (ensure-toplevel-ctx form env)
+  (unless (toplevel-ctx? env)
+    (error "cise: form can only appear in toplevel:" form)))
+(define (ensure-stmt-or-toplevel-ctx form env)
+  (unless (or (toplevel-ctx? env) (stmt-ctx? env))
+    (error "cise: form can only appear in toplevel or statment context:" form)))
 (define (env-decl-add! env decl)
-  (push! [~ env'decls] decl))
+  (push! (~ env'decls) decl))
 
 (define (wrap-expr form env)
-  (if (stmt-ctx? env) `(,form ";") form))
+  (if (expr-ctx? env) form `(,form ";")))
 
 (define (render-env-decls env)
-  (map (match-lambda
-         ((var type) `(,(cise-render-type type)" ",var";")))
+  (map (^.[(var type) `(,(cise-render-typed-var type var env) ";")])
        (env-decls env)))
 
 ;; Check source-info attribute of the input S-expr, and returns Stree
@@ -111,7 +126,18 @@
       [((? string? file) line)
        `((source-info ,file ,line))]
       [_ '()])))
-   
+
+;;=============================================================
+;; Global decls
+;;
+
+(define (push-static-decl! stree :optional (context (cise-context)))
+  (push! (ref context'static-decls) stree))
+                                
+(define (emit-static-decls port :optional (context (cise-context)))
+  (dolist [stree (reverse (ref context'static-decls))]
+    (render-finalize stree port)))
+
 ;;=============================================================
 ;; Expander
 ;;
@@ -126,25 +152,24 @@
 ;;   EXPANDER takes twi arguments, the form to expand and a
 ;;   opaque cise environmen.
 ;;
-(define (cise-register-macro! name expander . opts)
-  (let-optionals* opts ((context (cise-context)))
-    (hash-table-put! context name expander)))
+(define (cise-register-macro! name expander :optional (context (cise-context)))
+  (hash-table-put! (ref context'macros) name expander))
 
 ;;
 ;; cise-lookup-macro NAME &optional CONTEXT
 ;;
 ;;   Lookup cise macro.
 ;;
-(define (cise-lookup-macro name . opts)
-  (let-optionals* opts ((context (cise-context)))
-    (hash-table-get context name #f)))
+(define (cise-lookup-macro name :optional (context (cise-context)))
+  (hash-table-get (ref context'macros) name #f))
 
 ;;
 ;; copy the current cise context
 ;;
-(define (cise-context-copy . opts)
-  (let-optionals* opts ((context (cise-context)))
-    (hash-table-copy context)))
+(define (cise-context-copy :optional (context (cise-context)))
+  (make <cise-context>
+    :macros (hash-table-copy (ref context'macros))
+    :static-decls  (ref context'static-decls)))
 
 ;;
 ;; define-cise-macro (OP FORM ENV) . BODY
@@ -154,11 +179,14 @@
 (define-syntax define-cise-macro
   (syntax-rules ()
     [(_ (op form env) . body)
-     (cise-register-macro! 'op (lambda (form env) . body))]))
-
+     (cise-register-macro! 'op (lambda (form env) . body))]
+    [(_ op op2)                         ; alias
+     (cise-register-macro! 'op (or (cise-lookup-macro 'op2)
+                                   (error "unknown cise macro:" 'op2)))]))
 ;;
 ;; define-cise-stmt OP [ENV] CLAUSE ... [:where DEFINITION ...]
 ;; define-cise-expr OP [ENV] CLAUSE ... [:where DEFINITION ...]
+;; define-cise-toplevel OP [ENV] CLAUSE ... [:where DEFINITION ...]
 ;;
 (define-syntax define-cise-stmt
   (syntax-rules ()
@@ -173,6 +201,8 @@
     [(_ "clauses" op env (clause ...) (x . y))
      (define-cise-stmt "clauses" op env (clause ... x) y)]
     ;; entry
+    [(_ (op . args) . body)       ; single pattern case
+     (define-cise-stmt "clauses" op env (((_ . args) . body)) ())]
     [(_ op (pat . body) .  clauses) ; (pat . body) rules out a single symbol
      (define-cise-stmt "clauses" op env ((pat . body)) clauses)]
     [(_ op env . clauses)
@@ -193,39 +223,72 @@
     [(_ "clauses" op env (clause ...) (x . y))
      (define-cise-expr "clauses" op env (clause ... x) y)]
     ;; entry
+    [(_ (op . args) . body)       ; single pattern case
+     (define-cise-expr "clauses" op env (((_ . args) . body)) ())]
     [(_ op (pat . body) .  clauses)
      (define-cise-expr "clauses" op env ((pat . body)) clauses)]
     [(_ op env . clauses)
      (define-cise-expr "clauses" op env () clauses)]))
 
+(define-syntax define-cise-toplevel
+  (syntax-rules ()
+    ;; recursion
+    [(_ "clauses" op env clauses (:where defs ...))
+     (define-cise-macro (op form env)
+       defs ...
+       (ensure-toplevel-ctx form env)
+       (match form . clauses))]
+    [(_ "clauses" op env clauses ())
+     (define-cise-toplevel "clauses" op env clauses (:where))]
+    [(_ "clauses" op env (clause ...) (x . y))
+     (define-cise-toplevel "clauses" op env (clause ... x) y)]
+    ;; entry
+    [(_ (op . args) . body)       ; single pattern case
+     (define-cise-toplevel "clauses" op env (((_ . args) . body)) ())]
+    [(_ op (pat . body) .  clauses) ; (pat . body) rules out a single symbol
+     (define-cise-toplevel "clauses" op env ((pat . body)) clauses)]
+    [(_ op env . clauses)
+     (define-cise-toplevel "clauses" op env () clauses)]))
+
 ;;
-;; cise-render cise &optional port as-expr?
+;; cise-render cise &optional port context
+;;
+;; context := 'toplevel | 'stmt | 'expr | #t (expr) | #f (stmt)
 ;;
 ;;   External entry of renderer
 ;;
-(define (cise-render form . opts)
-  (let-optionals* opts ((port (current-output-port))
-                        (expr #f))
-    (define current-file #f)
-    (define current-line 1)
-    (define (render-finish stree)
-      (match stree
-        [('source-info (? string? file) line)
-         (cond ((and (equal? file current-file) (eqv? line current-line)))
-               ((and (equal? file current-file) (eqv? line (+ 1 current-line)))
-                (inc! current-line)
-                (format port "\n"))
-               (else
-                (set! current-file file)
-                (set! current-line line)
-                (format port "\n#line ~a ~s\n" line file)))]
-        [(x . y) (render-finish x) (render-finish y)]
-        [(? (any-pred string? symbol? number?) x) (display x port)]
-        [_ #f]))
-    
-    (let* ((env ((if expr expr-env identity) (null-env)))
-           (stree (render-rec form env)))
-      (render-finish `(,@(render-env-decls env) ,stree)))))
+(define (cise-render form :optional (port (current-output-port)) (ctx 'stmt))
+  (let* ([env (case ctx
+                [(toplevel) (make-env 'toplevel '())]
+                [(stmt #f)  (null-env)]
+                [(expr #t)  (expr-env (null-env))]
+                [else (error "cise-render: invalid context:" ctx)])]
+         [stree (render-rec form env)])
+    (render-finalize `(,@(render-env-decls env) ,stree) port)))
+
+(define (cise-render-to-string form :optional (ctx #f))
+  (call-with-output-string (cut cise-render form <> ctx)))
+
+(define (render-finalize stree port)
+  (define current-file #f)
+  (define current-line 1)
+  (define (rec stree)
+    (match stree
+      [('source-info (? string? file) line)
+       (cond [(and (equal? file current-file) (eqv? line current-line))]
+             [(and (equal? file current-file) (eqv? line (+ 1 current-line)))
+              (inc! current-line)
+              (format port "\n")]
+             [else
+              (set! current-file file)
+              (set! current-line line)
+              (format port "\n#line ~a ~s\n" line file)])]
+      ['|#reset-line| ; reset source info
+       (set! current-file #f) (set! current-line 0)]
+      [(x . y) (rec x) (rec y)]
+      [(? (any-pred string? symbol? number?) x) (display x port)]
+      [_ #f]))
+  (rec stree))
 
 ;;
 ;; cise-render-rec cise stmt/expr env
@@ -242,40 +305,87 @@
                   stmt or expr, but got:" stmt/expr)]))
 
 ;; render-rec :: Cise, Env -> Stree
-;;   Recursively expands 
+;;   Recursively expands Cise and generates Stree
 (define (render-rec form env)
   (match form
-    [((? symbol? key) . args)
-     (cond ((cise-lookup-macro key)
+    [([? symbol? key] . args)
+     (cond [(cise-lookup-macro key)
             => (lambda (expander)
                  `(,@(source-info form env)
-                   ,@(render-rec (expander form env) env))))
-           (else
+                   ,@(render-rec (expander form env) env)))]
+           [(or (type-decl-initial? key)
+                (any type-decl-subsequent? args))
+            (cise-render-typed-var form "" env)]
+           [else
             (let1 eenv (expr-env env)
               (wrap-expr
                `(,@(source-info form env)
                  ,(cise-render-identifier key) "("
                  ,@(intersperse "," (map (cut render-rec <> eenv) args))
                  ")")
-               env))))]
+               env))])]
     [(x . y)     form]   ; already stree
-    [(? symbol?) (wrap-expr (cise-render-identifier form) env)]
-    [(? identifier?) (wrap-expr (cise-render-identifier (unwrap-syntax form))
+    ['|#reset-line| '|#reset-line|] ; special directive to reset line info
+    [[? type-decl-initial?] (wrap-expr (cise-render-typed-var form "" env) env)]
+    [[? symbol?] (wrap-expr (cise-render-identifier form) env)]
+    [[? identifier?] (wrap-expr (cise-render-identifier (unwrap-syntax form))
                                 env)]
-    [(? string?) (wrap-expr (write-to-string form) env)]
-    [(? real?)   (wrap-expr form env)]
+    [[? string?] (wrap-expr (write-to-string form) env)]
+    [[? real?]   (wrap-expr form env)]
     [()          '()]
     [#\'         (wrap-expr "'\\''"  env)]
     [#\\         (wrap-expr "'\\\\'" env)]
     [#\newline   (wrap-expr "'\\n'"  env)]
     [#\return    (wrap-expr "'\\r'"  env)]
     [#\tab       (wrap-expr "'\\t'"  env)]
-    [(? char?)   (wrap-expr `("'" ,(if (char-set-contains? #[[:alnum:]] form)
+    [[? char?]   (wrap-expr `("'" ,(if (char-set-contains? #[[:alnum:]] form)
                                      (string form)
                                      (format "\\x~2'0x" (char->integer form)))
                               "'") env)]
     [_           (error "Invalid CISE form: " form)]))
 
+;;
+;; cise-translate inp outp &key enviroment
+;;
+;;   External interface to translate entire CiSE file into C.
+;;   CiSE expressions are read from INP and the resulting C code
+;;   is written to OUTP.
+;;
+;;   If CISE-TRANSLATE encounters a form (.static-decls),
+;;   it expands the rest of CiSE forms into a temporary string,
+;;   then emits the forward declarations of static functions
+;;   into outp, followed by the accumulated C code.  With this
+;;   you don't need to write forward declarations in CiSE source.
+
+(define (cise-translate inp outp
+                        :key (environment (make-module #f))
+                             (context (cise-context-copy (cise-context))))
+  (define (finish toutp)
+    (unless (eq? outp toutp)
+      (emit-static-decls outp)
+      (display (get-output-string toutp) outp))
+    (newline outp))
+  
+  (eval '(use gauche.cgen.cise) environment)
+  (eval '(use util.match) environment)
+  (parameterize ([cise-context context])
+    (let loop ((toutp outp))
+      (match (read inp)
+        [(? eof-object?) (finish toutp)]
+        [('.raw-c-code . cs)
+         (dolist [c cs] (newline toutp) (display c toutp)) (loop toutp)]
+        [(and ((or 'define-cise-stmt 'define-cise-expr 'define-cise-toplevel)
+               . _)
+              f)
+         (eval f environment)
+         (loop toutp)]
+        [('.static-decls) (loop (open-output-string))]
+        [(and (op . _) f)
+         (if (cise-lookup-macro op)
+           (cise-render f toutp 'toplevel)
+           (eval f environment))
+         (loop toutp)]))))
+
 ;;=============================================================
 ;; Built-in macros
 ;;
@@ -285,37 +395,48 @@
 ;;
 (define-cise-macro (define-cfn form env)
   (define (argchk args)
-    (match args
+    (match (canonicalize-vardecl args)
       [() '()]
       [((var ':: type) . rest) `((,var . ,type) ,@(argchk rest))]
       [(var . rest) `((,var . ScmObj) ,@(argchk rest))]))
-  ;; NB: we need to confine temporary decls within the function body,
-  ;; hence the ugly nested cise-render, since the extra decls handling
-  ;; is done at that level.  Hopefully this is an exception.
+
+  (define (gen-args args env)
+    (let1 eenv (expr-env env)
+      ($ intersperse ","
+         $ map (^.[(var . type) (cise-render-typed-var type var eenv)]) args)))
+
   (define (gen-cfn cls name args rettype body)
     `(,(cise-render-identifier cls) " "
-      ,(cise-render-type rettype)
-      " " ,(cise-render-identifier name) "("
-      ,@(intersperse "," (map (lambda (a)
-                                `(,(cise-render-type (cdr a)) " "
-                                  ,(cise-render-identifier (car a))))
-                              args))
-      ")" "{"
-      ,(call-with-output-string (cut cise-render `(begin ,@body) <>))
-      "}"))
-  ;; NB: this only works at toplevel.  The stmt check doesn't exclude
-  ;; non-toplevel use, and will give an error at C compilation time.
-  ;; Eventually we need to check better one.
-  (ensure-stmt-ctx form env)
+      ,(cise-render-typed-var rettype name env)
+      "(" ,(gen-args args env) ")"
+      "{",(cise-render-to-string `(begin ,@body) 'stmt)"}"))
+  ;; Another ugly hack to allow both :: rettype and ::rettype as
+  ;; return type specification.   Duplication in stub.scm.
+  (define (type-symbol? s)
+    (and (keyword? s) (#/^:[^:]/ (keyword->string s))))
+  (define (type-symbol-type s)
+    (string->symbol (string-drop (keyword->string s) 1)))
+
+  (define (record-static name args ret-type)
+    (push-static-decl!
+     `(,(source-info form env)
+       "static ",ret-type" ",(cise-render-identifier name)
+       "(",(gen-args args env)");")))
+
+  (define (check-static name args ret-type body)
+    (match body
+      [(':static . body) (record-static name args ret-type)
+                         (gen-cfn "static" name args ret-type body)]
+      [_                 (gen-cfn "" name args ret-type body)]))
+  
+  (ensure-toplevel-ctx form env)
   (match form
-    [(_ name (args ...) ':: ret-type ':static . body)
-     (gen-cfn "static" name (argchk args) ret-type body)]
     [(_ name (args ...) ':: ret-type . body)
-     (gen-cfn "" name (argchk args) ret-type body)]
-    [(_ name (args ...) ':static . body)
-     (gen-cfn "static" name (argchk args) 'ScmObj body)]
+     (check-static name (argchk args) ret-type body)]
+    [(_ name (args ...) [? type-symbol? ts] . body)
+     (check-static name (argchk args) (type-symbol-type ts) body)]
     [(_ name (args ...) . body)
-     (gen-cfn "" name (argchk args) 'ScmObj body)]))
+     (check-static name (argchk args) 'ScmObj body)]))
 
 ;;------------------------------------------------------------
 ;; Syntax
@@ -324,10 +445,13 @@
 ;; [cise stmt]  begin STMT ...
 ;;    Grouping.
 (define-cise-macro (begin form env)
-  (ensure-stmt-ctx form env)
-  (match form
-    [(_ . forms)
-     `("{" ,@(map (cut render-rec <> env) forms) "}")]))
+  (cond
+   [(stmt-ctx? env)
+    `("{" ,@(map (cut render-rec <> env) (cdr form)) "}")]
+   [(toplevel-ctx? env)
+    `(,@(map (cut render-rec <> env) (cdr form)))]
+   [else
+    (intersperse "," (map (cut render-rec <> env) (cdr form)))]))
 
 ;; [cise stmt]  let* ((VAR [:: TYPE] [INIT-EXPR]) ...) STMT ...
 ;;    Local variables.   Because of C semantics, we only support
@@ -336,22 +460,24 @@
 (define-cise-macro (let* form env)
   (ensure-stmt-ctx form env)
   (match form
-    [(_ ((var . spec) ...) . body)
-     (let1 eenv (expr-env env)
-       `(begin ,@(map (lambda (var spec)
-                        (receive (type has-init? init)
-                            (match spec
-                              [()         (values 'ScmObj #f #f)]
-                              [(init)     (values 'ScmObj #t init)]
-                              [(':: type) (values type #f #f)]
-                              [(':: type init) (values type #t init)])
-                          `(,(cise-render-type type)" "
-                            ,(cise-render-identifier var)
-                            ,@(cond-list (has-init?
-                                          `("=",(render-rec init eenv))))
-                            ";")))
-                      var spec)
-               ,@(map (cut render-rec <> env) body)))]
+    [(_ vars . body)
+     (match (canonicalize-vardecl vars)
+       [((var . spec) ...)
+        (let1 eenv (expr-env env)
+          `(begin
+             ,@(map (lambda (var spec)
+                      (receive (type has-init? init)
+                          (match spec
+                            [()         (values 'ScmObj #f #f)]
+                            [(init)     (values 'ScmObj #t init)]
+                            [(':: type) (values type #f #f)]
+                            [(':: type init) (values type #t init)])
+                        `(,(cise-render-typed-var type var env)
+                          ,@(cond-list [has-init? `("=",(render-rec init eenv))])
+                          ";")))
+                    var spec)
+             ,@(map (cut render-rec <> env) body)))]
+       [_ (error "invalid variable decls in let* form:" form)])]
     ))
 
 ;; [cise stmt] if TEST-EXPR THEN-STMT [ELSE-STMT]
@@ -362,10 +488,10 @@
     (match form
       [(_ test then)
        `("if (",(render-rec test eenv)")"
-         ,(render-rec then env))]
+         "{",(render-rec then env)"}")]
       [(_ test then else)
        `("if (",(render-rec test eenv)")"
-         ,(render-rec then env)" else " ,(render-rec else env))]
+         "{",(render-rec then env)"} else {" ,(render-rec else env) "}")]
       )))
 
 ;; [cise stmt] when TEST-EXPR STMT ...
@@ -409,7 +535,8 @@
                   `(,@(source-info literals env)
                     ,@(if (eq? literals 'else)
                         '("default: ")
-                        (map (lambda (literal) `("case ",literal" : "))
+                        (map (lambda (literal)
+                               `("case ",(render-rec literal eenv)" : "))
                              literals))
                     ,@(render-rec `(begin ,@clause
                                           ,@(if fallthrough? '() '((break))))
@@ -475,6 +602,14 @@
                          ,@body) env)
          "}")])))
 
+;; [cise stmt] dolist [VAR EXPR] STMT ...
+(define-cise-macro (dolist form env)
+  (ensure-stmt-ctx form env)
+  (let ((eenv (expr-env env)))
+    (match form
+      [(_ (var expr) . body)
+       `(for-each (lambda (,var) ,@body) ,expr)])))
+
 ;; [cise stmt] pair-for-each (lambda (VAR) STMT ...) EXPR
 ;;   Like for-each, but VAR is bound to each 'spine' cell instead of
 ;;   each element of the list.
@@ -489,6 +624,14 @@
          ,(render-rec `(begin ,@body) env)
          )])))
 
+;; [cise stmt] dopairs [VAR EXPR] STMT ...
+(define-cise-macro (dopairs form env)
+  (ensure-stmt-ctx form env)
+  (let ((eenv (expr-env env)))
+    (match form
+      [(_ (var expr) . body)
+       `(pair-for-each (lambda (,var) ,@body) ,expr)])))
+
 ;; [cise stmt] dotimes (VAR EXPR) STMT ...
 ;;   EXPR must yield an integer, N.  Repeat STMT ... by binding VAR from 0
 ;;   to (N-1).
@@ -506,7 +649,8 @@
 (define-cise-macro (return form env)
   (ensure-stmt-ctx form env)
   (match form
-    [(_ expr) `("return (" ,(render-rec expr (expr-env env)) ");")]))
+    [(_ expr) `("return (" ,(render-rec expr (expr-env env)) ");")]
+    [(_)      `("return;")]))
 
 ;; [cise stmt] break
 ;; [cise stmt] continue
@@ -520,12 +664,63 @@
 ;; [cise stmt] label NAME
 ;; [cise stmt] goto NAME
 ;;   Label and goto.
+;;   We always add null statement after the label, so that we can place
+;;   (label NAME) at the end of compound statement.
 (define-cise-stmt label
-  [(_ name) `(,(cise-render-identifier name) " : ")])
+  [(_ name) `(,(cise-render-identifier name) " :; ")])
 
 (define-cise-stmt goto
   [(_ name) `("goto " ,(cise-render-identifier name) ";")])
 
+;;
+;; Preprocessor directives
+;;
+
+;; [cise stmt] .if STRING STMT [STMT]
+;;   c preprocessor directive
+(define-cise-macro (.if form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ condition stmt1)
+     `("#if " ,(x->string condition) "\n" |#reset-line|
+       ,(render-rec stmt1 env) "\n"
+       "#endif /* " ,(x->string condition) " */\n" |#reset-line|)]
+    [(_ condition stmt1 stmt2)
+     `("#if " ,(x->string condition) "\n" |#reset-line|
+       ,(render-rec stmt1 env) "\n"
+       "#else  /* !",(x->string condition) " */\n" |#reset-line|
+       ,(render-rec stmt2 env) "\n"
+       "#endif /* " ,(x->string condition) " */\n" |#reset-line|)]))
+
+(define-cise-macro (.cond form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ (condition . stmts) ...)
+     `("#if 0 /*dummy*/\n"
+       ,@(fold-right (lambda (c ss seed)
+                       `(,(cond [(eq? c 'else) '("#else")]
+                                [else `("#elif " ,(x->string c))])
+                         "\n" |#reset-line|
+                         ,(map (cut render-rec <> env) ss) "\n"
+                         ,@seed))
+                     '("#endif\n" |#reset-line|)
+                     condition stmts))]))
+
+(define-cise-macro (.include form env)
+  (ensure-stmt-or-toplevel-ctx form env)
+  (match form
+    [(_ item ...)
+     (map (lambda (f)
+            `("#include "
+              ,(cond [(string? f) (write-to-string f)]
+                     [(symbol? f) (x->string f)]
+                     [else (error "bad argument to .include:" f)])
+              "\n" |#reset-line|))
+          item)]
+    [(_ . other) (error "malformed .include:" form)]))
+
+(define-cise-macro |#if| .if)           ;backward compat.
+
 ;;------------------------------------------------------------
 ;; Operators
 ;;
@@ -577,6 +772,7 @@
 ;;   shift.
 ;;
 ;; [cise expr] set! LVALUE EXPR LVALUE EXPR ...
+;; [cise expr] =    LVALUE EXPR LVALUE EXPR ...
 ;; [cise expr] +=   LVALUE EXPR
 ;; [cise expr] -=   LVALUE EXPR
 ;; [cise expr] *=   LVALUE EXPR
@@ -642,6 +838,9 @@
 (define-unary pre++  "++")
 (define-unary pre--  "--")
 
+(define-cise-macro inc! pre++)
+(define-cise-macro dec! pre--)
+
 (define-macro (define-post-unary op sop)
   `(define-cise-macro (,op form env)
      (wrap-expr
@@ -708,14 +907,6 @@
           ,(append-map (lambda (ind) `("[",(render-rec ind eenv)"]")) b))])
      env)))
 
-(define-cise-macro (cast form env)
-  (let1 eenv (expr-env env)
-    (wrap-expr
-     (match form
-       [(_ type expr)
-        `("((",(cise-render-type type)")(",(render-rec expr eenv)"))")])
-     env)))
-
 (define-cise-macro (?: form env)
   (let1 eenv (expr-env env)
     (wrap-expr
@@ -737,13 +928,27 @@
                   "=(",(render-rec val eenv)")") ,@r))]
         [_   (error "uneven args for set!:" form)]))))
 
+(define-cise-macro = set!)              ;EXPERIMENTAL
+
+;;------------------------------------------------------------
+;; Type-related expressions
+;;
+
+(define-cise-macro (cast form env)
+  (let1 eenv (expr-env env)
+    (wrap-expr
+     (match form
+       [(_ type expr)
+        `("((",(cise-render-typed-var type "" env)")(",(render-rec expr eenv)"))")])
+     env)))
+
 ;;------------------------------------------------------------
 ;; Convenience expression macros
 ;;
 
-;; Embed raw c code.  NB: I'm not sure yet about the name.  It is
-;; desirable to be consistent with genstub (currently it uses (c <expr>))
-;; I'll think it a bit more.
+;; Embed raw c code.  THIS IS A KLUDGE---SHOULDN'T BE USED.
+;; Allowing raw C code prevents higher-level analysis of cise code.
+;; This should be regarded as a compromise until cise support full C features.
 (define-cise-expr C:
   [(_ stuff) (list (x->string stuff))])
 
@@ -760,7 +965,7 @@
   [(_ a b c)     `(SCM_LIST3 ,a ,b ,c)]
   [(_ a b c d)   `(SCM_LIST4 ,a ,b ,c ,d)]
   [(_ a b c d e) `(SCM_LIST5 ,a ,b ,c ,d ,e)]
-  [(_ x ...)     (fold (lambda (elt r) `(Scm_Cons ,elt ,r)) '() x)])
+  [(_ xs ...)     (fold-right (cut list 'Scm_Cons <> <>) 'SCM_NIL xs)])
 
 (define-cise-expr values
   [(_)           '("Scm_Values(SCM_NIL)")]
@@ -783,13 +988,68 @@
 ;; Other utilities
 ;;
 
-(define (cise-render-type typespec)  ; for the time being
-  (if (list? typespec)
-    (intersperse " " (map x->string typespec))
-    (x->string typespec)))
+;; type-decl-initial? and type-decl-subsequent? are used to determine if
+;; (sym sym2 ...) is a type spec or ordinary expression.  The way
+;; to render it differs depending on whether it is a type spec.
+;; The reason that we have two predicates are that '*' and '&' can
+;; appear in the operator position of a valid expression.
+(define (type-decl-initial? sym)
+  (or (memq sym '(const class enum struct volatile unsigned long
+                  char short int float double .array))
+      (and (symbol? sym)
+           (#/.[*&]$/ (symbol->string sym)))))
+
+(define (type-decl-subsequent? sym)
+  (or (memq sym '(* &))
+      (type-decl-initial? sym)))
+
+      
+(define (cise-render-typed-var typespec var env)
+  (match typespec
+    [('.array spec (dim ...))
+     `(,(cise-render-typed-var spec var env)
+       ,@(map (^.['* "[]"]
+                 [x `("[" ,(render-rec x (expr-env env)) "]")])
+              dim))]
+    [(x ...) `(,(intersperse " " (map x->string x))
+               " " ,(cise-render-identifier var))]
+    [x       `(,(x->string x) " " ,(cise-render-identifier var))]))
 
 (define (cise-render-identifier sym)
   (cgen-safe-name-friendly (x->string sym)))
 
-(provide "gauche/cgen/cise")
+;; Allow var::type as (var :: type)
+;; and (var::type init) as (var :: type init)
+(define (canonicalize-vardecl vardecls)
+  (define (expand-type elt seed)
+    (cond
+     [(symbol? elt)
+      (rxmatch-case (symbol->string elt)
+        [#/^(.+)::$/ (_ v) `(,(string->symbol v) :: ,@seed)]
+        [#/^(.+)::(.+)$/ (_ v t)
+            `(,(string->symbol v) :: ,(string->symbol t) ,@seed)]
+        [else (cons elt seed)])]
+     [(keyword? elt)  ;; The case of (var ::type)
+      (rxmatch-case (keyword->string elt)
+        [#/^:(.+)$/ (_ t) `(:: ,(string->symbol t) ,@seed)]
+        [else (cons elt seed)])]
+     [else (cons elt seed)]))
+
+  (define (err decl) (error "invlaid variable declaration:" decl))
+
+  (define (scan in r)
+    (match in
+      [() (reverse r)]
+      [([? symbol? var] :: type . rest)
+       (scan rest `((,var :: ,type) ,@r))]
+      [([? symbol? var] . rest)
+       (scan rest `((,var :: ScmObj) ,@r))]
+      [(([? symbol? v] [? symbol? t] . args) . rest)
+       (scan rest `(,(expand-type v (expand-type t args)) ,@r))]
+      [(([? symbol? vt] . args) . rest)
+       (scan rest `(,(expand-type vt args) ,@r))]
+      [(xx . rest) (err xx)]))
+
+  (scan (fold-right expand-type '() vardecls) '()))
+
 
