@@ -1,7 +1,7 @@
 ;;;
 ;;; file related utility functions.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: fileutil.scm,v 1.9 2008-05-10 13:35:54 shirok Exp $
-;;;
 
 #!no-fold-case
 
@@ -108,7 +106,7 @@
   (let-keywords opts ((separator #[/])
                       (folder glob-fs-folder))
     (define (rec node matcher seed)
-      (cond [(null? matcher) (proc seed)]
+      (cond [(null? matcher) seed]
             [(eq? (car matcher) '**)
              (rec* node (cdr matcher) seed)]
             [(null? (cdr matcher))
@@ -212,47 +210,68 @@
           (case ch
             [(#\*) (element1* (n) ct)]
             [else  (element1 ch ct)]))
-        `(seq bol ,@(element0 (n) '())))))))
+        `(0 #f bol ,@(element0 (n) '())))))))
 
-(define (make-glob-fs-fold . args)
-  (let-keywords* args ((root-path #f)
-                       (current-path #f))
-    (let ((separ (cond-expand
-                  (gauche.os.windows "\\")
-                  (else "/"))))
-      (define (ensure-dirname s)
-        (and s
-             (or (and-let* ([len (string-length s)]
-                            [ (> len 0) ]
-                            [ (not (eqv? (string-ref s (- len 1))
-                                         (string-ref separ 0))) ])
-                   (string-append s separ))
-                 s)))
-      (define root-path/    (ensure-dirname root-path))
-      (define current-path/ (ensure-dirname current-path))
-      (lambda (proc seed node regexp non-leaf?)
-        (let1 prefix (case node
-                       [(#t) (or root-path/ separ)]
-                       [(#f) (or current-path/ "")]
-                       [else (string-append node separ)])
-          ;; NB: we can't use filter, for it is not built-in.
-          ;; also we can't use build-path, from the same reason.
-          (if (eq? regexp 'dir?)
-            (proc prefix seed)
-            (fold (lambda (child seed)
-                    (or (and-let* ([ (regexp child) ]
-                                   [full (string-append prefix child)]
-                                   [ (or (not non-leaf?)
-                                         (file-is-directory? full)) ])
-                          (proc full seed))
-                        seed))
-                  seed
-                  (sys-readdir (case node
-                                 [(#t) (or root-path/ "/")]
-                                 [(#f) (or current-path/ ".")]
-                                 [else node]))))))
-      )))
+;; if rx is just test perfect match, e.g. #/^string$/, returns
+;; string portion.
+(define (fixed-regexp? rx)
+  (let ((ast (regexp-ast rx)))
+    (and (> (length ast) 4)
+         (eq? (caddr ast) 'bol)
+         (let loop ((cs (cdddr ast)) (r '()))
+           (cond [(null? (cdr cs))
+                  (and (eq? (car cs) 'eol) (list->string (reverse r)))]
+                 [(char? (car cs)) (loop (cdr cs) (cons (car cs) r))]
+                 [else #f])))))
+
+(define (make-glob-fs-fold :key (root-path #f) (current-path #f))
+  (let ((separ (cond-expand
+                (gauche.os.windows "\\")
+                (else "/"))))
+    (define (ensure-dirname s)
+      (and s
+           (or (and-let* ([len (string-length s)]
+                          [ (> len 0) ]
+                          [ (not (eqv? (string-ref s (- len 1))
+                                       (string-ref separ 0))) ])
+                 (string-append s separ))
+               s)))
+    (define root-path/    (ensure-dirname root-path))
+    (define current-path/ (ensure-dirname current-path))
+    (lambda (proc seed node regexp non-leaf?)
+      (let1 prefix (case node
+                     [(#t) (or root-path/ separ)]
+                     [(#f) (or current-path/ "")]
+                     [else (string-append node separ)])
+        ;; NB: we can't use filter, for it is not built-in.
+        ;; also we can't use build-path from the same reason.
+        ;; We treat fixed-regexp specially, since it allows
+        ;; us not to search the directory---sometimes the directory
+        ;; has 'x' permission but not 'r' permission, and it would be
+        ;; unreasonable if we fail to go down the path even if we know
+        ;; the exact name.
+        (cond [(eq? regexp 'dir?) (proc prefix seed)]
+              [(fixed-regexp? regexp)
+               => (^s (let1 full (string-append prefix s)
+                        (if (and (file-exists? full)
+                                 (or (not non-leaf?)
+                                     (file-is-directory? full)))
+                          (proc full seed)
+                          seed)))]
+              [else
+               (fold (lambda (child seed)
+                       (or (and-let* ([ (regexp child) ]
+                                      [full (string-append prefix child)]
+                                      [ (or (not non-leaf?)
+                                            (file-is-directory? full)) ])
+                             (proc full seed))
+                           seed))
+                     seed
+                     (sys-readdir (case node
+                                    [(#t) (or root-path/ "/")]
+                                    [(#f) (or current-path/ ".")]
+                                    [else node])))])))
+    ))
 
 (define glob-fs-folder (make-glob-fs-fold))
 
-(provide "gauche/fileutil")
