@@ -1,7 +1,7 @@
 ;;;
 ;;; interactive/info.scm - online helper
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: info.scm,v 1.11 2008-05-10 13:35:59 shirok Exp $
-;;;
 
 (define-module gauche.interactive.info
   (use srfi-1)
@@ -85,7 +83,8 @@
                             :paths paths
                             :pred (lambda (p)
                                     (or (file-is-readable? p)
-                                        (file-is-readable? #`",|p|.gz"))))
+                                        (file-is-readable? #`",|p|.gz")
+                                        (file-is-readable? #`",|p|.bz2"))))
         (errorf "couldn't find info file ~s in paths: ~s" *info-file* paths))
     ))
 
@@ -101,5 +100,4 @@
     (viewer (ref (info-get-node *info* nodename) 'content)))
   (values))
 
-(provide "gauche/interactive/info")
 
