@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche/listener - listerner utility
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: listener.scm,v 1.11 2008-05-10 13:35:56 shirok Exp $
-;;;
 
 ;; provides functions useful to implement a repl listener
 
@@ -159,9 +157,8 @@
                            (set! (ref self 'rbuf)
                                  (get-remaining-input-string
                                   (current-input-port))))))
-                     (or (listener-show-prompt self) (abort e))
-                     (loop)))))))
-      ))
+                     (and (listener-show-prompt self)
+                          (loop))))))))))
 
   ;; Capture std ports when the handler is created
   (set! (ref self 'original-input-port)  (current-input-port))
@@ -276,4 +273,3 @@
    (else (and (<unhandled-signal-error> e)
               (eqv? (ref e 'signal) SIGPIPE)))))
 
-(provide "gauche/listener")
