@@ -1,7 +1,7 @@
 ;;;
 ;;; logical.scm - logical (bitwise) operations.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: logical.scm,v 1.6 2008-05-10 13:35:56 shirok Exp $
-;;;
 
 (select-module gauche)
 
@@ -65,36 +63,3 @@
                 (ash (logand from mask) start)))
       from))
 
-;;; The following code uses the algorithm from SLIB's logical.scm,
-;;; adapted to Gauche
-
-;;;; "logical.scm", bit access and operations for integers for Scheme
-;;; Copyright (C) 1991, 1993 Aubrey Jaffer.
-;
-;Permission to copy this software, to redistribute it, and to use it
-;for any purpose is granted, subject to the following restrictions and
-;understandings.
-;
-;1.  Any copy made of this software must include this copyright notice
-;in full.
-;
-;2.  I have made no warrantee or representation that the operation of
-;this software will be error-free, and I am under no obligation to
-;provide any services, by way of maintenance, update, or otherwise.
-;
-;3.  In conjunction with products arising from the use of this
-;material, there shall be no use of my name in any advertising,
-;promotional, or sales literature without prior written consent in
-;each case.
-
-(define (integer-length n)
-  (check-arg integer? n)
-  (letrec ((rec
-            (lambda (n)
-              (case n
-                ((0 -1) 0)
-                ((1 -2) 1)
-                ((2 3 -3 -4) 2)
-                ((4 5 6 7 -5 -6 -7 -8) 3)
-                (else (+ 4 (rec (ash n -4))))))))
-    (rec n)))
