@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche.package.fetch - fetch a package
 ;;;  
-;;;   Copyright (c) 2004-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2004-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: fetch.scm,v 1.8 2008-05-10 13:36:04 shirok Exp $
-;;;
 
 ;; *EXPERIMENTAL*
 ;; gauche.package.fetch module is intended to automate fetching package
@@ -55,24 +53,22 @@
 (define *wget-program*     (find-file-in-paths "wget"))
 (define *ncftpget-program* (find-file-in-paths "ncftpget"))
 
-(define (gauche-package-ensure uri . opts)
-  (let-keywords opts ((config '()))
-    (let* ((build-dir (assq-ref config 'build-dir "."))
-           (wget      (assq-ref config 'wget *wget-program*))
-           (ncftpget  (assq-ref config 'ncftpget *ncftpget-program*))
-           (dest      (build-path build-dir (sys-basename uri))))
-      (rxmatch-case uri
-        (#/^https?:/ (#f)
-         (sys-unlink dest)
-         (run #`",wget -P \",build-dir\" \",uri\"")
-         dest)
-        (#/^ftp:/ (#f)
-         (guard (e (else (sys-unlink dest) (raise e)))
-           (run #`",ncftpget -c \",uri\" > \",dest\""))
-         dest)
-        (else
-         (unless (file-is-readable? uri)
-           (error "can't read the package: " uri))
-         uri)))))
+(define (gauche-package-ensure uri :key (config '()))
+  (let* ((build-dir (assq-ref config 'build-dir "."))
+         (wget      (assq-ref config 'wget *wget-program*))
+         (ncftpget  (assq-ref config 'ncftpget *ncftpget-program*))
+         (dest      (build-path build-dir (sys-basename uri))))
+    (rxmatch-case uri
+      (#/^https?:/ (#f)
+                   (sys-unlink dest)
+                   (run #`",wget -P \",build-dir\" \",uri\"")
+                   dest)
+      (#/^ftp:/ (#f)
+                (guard (e (else (sys-unlink dest) (raise e)))
+                  (run #`",ncftpget -c \",uri\" > \",dest\""))
+                dest)
+      (else
+       (unless (file-is-readable? uri)
+         (error "can't read the package: " uri))
+       uri))))
 
-(provide "gauche/package/fetch")
