@@ -1,7 +1,7 @@
 ;;;
 ;;; procedure.scm - auxiliary procedure utilities.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -29,20 +29,18 @@
 ;;;   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-;;;  
-;;;  $Id: procedure.scm,v 1.23 2008-05-10 13:35:56 shirok Exp $
 ;;;
 
 (define-module gauche.procedure
   (use srfi-1)
-  (export compose .$ complement ~$ pa$ map$ for-each$ apply$
+  (export compose .$ complement pa$ map$ for-each$ apply$
           count$ fold$ fold-right$ reduce$ reduce-right$
           filter$ partition$ remove$ find$ find-tail$
           any$ every$ delete$ member$ assoc$
           any-pred every-pred
           arity procedure-arity-includes?
           <arity-at-least> arity-at-least? arity-at-least-value
-          case-lambda disasm
+          case-lambda ~ ref* disasm
           ))
 
 (select-module gauche.procedure)
@@ -151,51 +149,80 @@
   (ref x 'value))
 
 (define (arity proc)
-  (cond
-   ((or (is-a? proc <procedure>) (is-a? proc <method>))
-    (if (ref proc 'optional)
-        (make <arity-at-least> :value (ref proc 'required))
-        (ref proc 'required)))
-   ((is-a? proc <generic>)
-    (map arity (ref proc 'methods)))
-   (else
-    (errorf "cannot get arity of ~s" proc))))
+  (cond [(or (is-a? proc <procedure>) (is-a? proc <method>))
+         (if (ref proc 'optional)
+           (make <arity-at-least> :value (ref proc 'required))
+           (ref proc 'required))]
+        [(is-a? proc <generic>) (map arity (ref proc 'methods))]
+        [else (errorf "cannot get arity of ~s" proc)]))
 
 (define (procedure-arity-includes? proc k)
   (let1 a (arity proc)
     (define (check a)
-      (cond ((integer? a) (= a k))
-            ((arity-at-least? a) (>= k (arity-at-least-value a)))
-            (else (errorf "implementation error in (procedure-arity-includes? ~s ~s)" proc k))))
+      (cond [(integer? a) (= a k)]
+            [(arity-at-least? a) (>= k (arity-at-least-value a))]
+            [else (errorf "implementation error in (procedure-arity-includes? ~s ~s)" proc k)]))
     (if (list? a)
-        (any check a)
-        (check a))))
+      (any check a)
+      (check a))))
 
-;; case-lambda (srfi-16) ---------------------------------------
+;; ~, ref*  ----------------------------------------------------
+;;  (~ a b c d) => (ref (ref (ref a b) c) d)
+;;  TODO: This is here because case-lambda used to be here as well.
+;;  This should really belong to src/objlib.scm.  However, we can't
+;;  do that in 0.9.2, since src/objlib.scm needs to be complied with
+;;  0.9.1.  Move this after 0.9.2 release.
+(define ~
+  (getter-with-setter
+   (case-lambda
+     [(obj selector) (ref obj selector)]
+     [(obj selector . more) (apply ~ (ref obj selector) more)])
+   (case-lambda
+     [(obj selector val) ((setter ref) obj selector val)]
+     [(obj selector selector2 . rest)
+      (apply (setter ~) (ref obj selector) selector2 rest)])))
 
-;; This is a temporary implementation.  There's a plan to replace it
-;; for more efficient dispatching mechanism.  (But I'm not sure when).
+(define ref* ~)                         ;for the backward compatibility
 
-(define-syntax case-lambda
-  (syntax-rules ()
-    ((case-lambda (arg . body) ...)
-     (make-dispatcher (list (lambda arg . body) ...)))
-    ((case-lambda . _)
-     (syntax-error "malformed case-lambda" (case-lambda . _)))))
-
-;; support procedure
-(define (make-dispatcher closures)
-  (lambda args
-    (let ((len (length args)))
-      (cond ((find (lambda (p) (procedure-arity-includes? p len)) closures)
-             => (cut apply <> args))
-            (else
-             (error "wrong number of arguments to case-lambda:" args))))))
-
-;; disassembler.
+;; disassembler ------------------------------------------------
 ;; I'm not sure whether this should be here or not, but fot the time being...
 
 (define (disasm proc)
-  ((with-module gauche.internal vm-dump-code) (closure-code proc)))
-
-(provide "gauche/procedure")
+  (define dump (with-module gauche.internal vm-dump-code))
+  (define (dump-case-lambda min-reqargs proc-array)
+    (print "CASE-LAMBDA")
+    (let1 len (vector-length proc-array)
+      (do ([i 0 (+ i 1)])
+          ((= i len) #t)
+        (when (closure? (vector-ref proc-array i))
+          (if (= i (- len 1))
+            (print ";;; numargs >= " (+ i min-reqargs))
+            (print ";;; numargs = " (+ i min-reqargs)))
+          (dump (closure-code (vector-ref proc-array i)))))))
+    
+  (cond
+   [(closure? proc) (print "CLOSURE " proc) (dump (closure-code proc))]
+   [(is-a? proc <method>)
+    (print "METHOD " proc)
+    (cond [(method-code proc) => dump]
+          [else (print "(defined in C)")])]
+   [(is-a? proc <generic>)
+    (print "GENERIC FUNCTION " proc)
+    (dolist [m (~ proc'methods)]
+      (print ">> method " m)
+    (cond [(method-code m) => dump]
+          [else (print "(defined in C)")]))]
+   [(and (subr? proc)
+         (let1 info (procedure-info proc)
+           ;; NB: Detect case-lamdba.  The format of procedure-info of
+           ;; case-lambda is tentative, and may be changed in future.
+           ;; See the comment of make-case-lambda-dispatcher in intlib.stub.
+           (and (pair? info)
+                (eq? (car info) 'case-lambda-dispatcher)
+                (pair? (cdr info))
+                (integer? (cadr info))
+                (pair? (cddr info))
+                (vector? (caddr info))
+                (apply dump-case-lambda (cdr info)))))]
+   [else (print "Disassemble not applicable for " proc)])
+  (values))
