@@ -1,7 +1,7 @@
 ;;; ftp.scm - FTP library
 ;;;
 ;;;  Copyright (c) 2005 OOHASHI Daichi, All rights reserved.
-;;;  Copyright (c) 2007-2008  Shiro Kawai  <shiro@acm.org>
+;;;  Copyright (c) 2007-2011  Shiro Kawai  <shiro@acm.org>
 ;;;
 ;;;  Redistribution and use in source and binary forms, with or without
 ;;;  modification, are permitted provided that the following conditions
@@ -29,8 +29,7 @@
 ;;;  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;;  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-;;; $Id: ftp.scm,v 1.9 2008-05-10 13:36:08 shirok Exp $
+;;;
 
 ;; RFC  959 FILE TRANSFER PROTOCOL (FTP)
 ;; RFC 1123 Requirements for Internet Hosts -- Application and Support
@@ -44,14 +43,13 @@
   (use util.list)
   (use gauche.net)
   (use gauche.logger)
-  (export <ftp-connection> call-with-ftp-connection
+  (export <ftp-connection> <ftp-error> call-with-ftp-connection
           ftp-chdir ftp-current-directory ftp-get ftp-size
           ftp-help ftp-list ftp-login ftp-ls ftp-mdtm ftp-mtime
           ftp-mkdir ftp-name-list ftp-noop ftp-passive?  ftp-put
           ftp-put-unique ftp-quit ftp-remove ftp-rename ftp-rmdir
           ftp-site ftp-stat ftp-system ftp-transfer-type
           ))
-
 (select-module rfc.ftp)
 
 (define-constant *default-ftp-port* 21)
@@ -74,9 +72,9 @@
   (receive (user host port) (uri-decompose-authority uri)
     (let1 conn (apply ftp-login host
                       (cond-list
-                       (user @ (list :username user))
-                       (port @ (list :port port))
-                       (#t   @ keys)))
+                       [user @ (list :username user)]
+                       [port @ (list :port port)]
+                       [#t   @ keys]))
       (unwind-protect (proc conn)
         (ftp-quit conn)))))
 
@@ -85,44 +83,38 @@
 ;; USER <SP> <username> <CRLF>
 ;; PASS <SP> <password> <CRLF>
 ;; ACCT <SP> <account-information> <CRLF>
-(define (ftp-login host . keys)
-  (let-keywords* keys ((account "")
-		       (passive #f)
-                       (port *default-ftp-port*)
-                       (username *anon-user*)
-		       (password
-                        (if (string=? username *anon-user*) *anon-pass* ""))
-                       (log-drain #f))
-    (define (do-login conn)
-      (let retry ()
-	(if (string-prefix? "120" (get-response conn))
-          (begin (sys-sleep 1) (retry))
-          (let1 r1 (send-command conn "USER" username)
-            (if (not (string-prefix? "3" r1))
-              r1
-              (let1 r2 (send-command conn "PASS" password)
-                (if (not (string-prefix? "3" r2))
-                  r2
-                  (send-command conn "ACCT" account))))))))
-    (let* ((conn (make <ftp-connection>
-                   :passive passive
-                   :socket (make-client-socket 'inet host port)
-                   :log-drain log-drain))
-           (res (do-login conn)))
-      (if (not (string-prefix? "2" res))
-        (ftp-error res)
-        conn))))
-
+(define (ftp-login host :key (account "") (passive #f)
+                   (port *default-ftp-port*) (username *anon-user*)
+                   (password (if (string=? username *anon-user*) *anon-pass* ""))
+                   (log-drain #f))
+  (define (do-login conn)
+    (let retry ()
+      (if (string-prefix? "120" (get-response conn))
+        (begin (sys-sleep 1) (retry))
+        (let1 r1 (send-command conn "USER" username)
+          (if (not (string-prefix? "3" r1))
+            r1
+            (let1 r2 (send-command conn "PASS" password)
+              (if (not (string-prefix? "3" r2))
+                r2
+                (send-command conn "ACCT" account))))))))
+  (let* ((conn (make <ftp-connection>
+                 :passive passive
+                 :socket (make-client-socket 'inet host port)
+                 :log-drain log-drain))
+         (res (do-login conn)))
+    (if (not (string-prefix? "2" res))
+      (ftp-error res)
+      conn)))
 
 ;; rfc 959 6(1) type commands
 
 ;; QUIT <CRLF>
 (define (ftp-quit conn)
   (unwind-protect
-   (simple-command conn "QUIT")
-   (cond ((ref conn 'socket)
-          => (lambda (s) (socket-shutdown s) (set! (ref conn 'socket) #f))))))
-
+      (simple-command conn "QUIT")
+    (if-let1 s (ref conn 'socket)
+      (begin (socket-shutdown s) (set! (ref conn 'socket) #f)))))
 
 ;; *ABOR <CRLF>
 
@@ -137,81 +129,59 @@
     (simple-command conn "CWD" dirname)))
 
 ;; DELE <SP> <pathname> <CRLF>
-(define (ftp-remove conn path)
-  (simple-command conn "DELE" path))
-
+(define (ftp-remove conn path) (simple-command conn "DELE" path))
 
 ;; HELP [<SP> <string>] <CRLF>
-(define (ftp-help conn . opt)
-  (apply simple-command conn "HELP" opt))
-
+(define (ftp-help conn . opt) (apply simple-command conn "HELP" opt))
 
 ;; MKD  <SP> <pathname> <CRLF>
 ;; PWD  <CRLF>
-(define-values (ftp-mkdir ftp-current-directory)
-  (let1 parse-257
-      (lambda (res)
-        (rxmatch-if (#/^257 \"((?:[^\"]|\"\")+)\"/ res) (_ dirname)
-          (regexp-replace-all #/""/ dirname "\"")
-          (ftp-error res)))
-    (values (lambda (conn dirname)
-              (parse-257 (simple-command conn "MKD" dirname)))
-            (lambda (conn)
-              (parse-257 (simple-command conn "PWD"))))))
+(define (ftp-mkdir conn dirname)(parse-257 (simple-command conn "MKD" dirname)))
+(define (ftp-current-directory conn) (parse-257 (simple-command conn "PWD")))
+
+(define (parse-257 res)
+  (rxmatch-if (#/^257 \"((?:[^\"]|\"\")+)\"/ res) (_ dirname)
+    (regexp-replace-all #/""/ dirname "\"")
+    (ftp-error res)))
 
 ;; *MODE <SP> <mode-code> <CRLF>
 ;; *SMNT <SP> <pathname> <CRLF>
 
 
 ;; SITE <SP> <string> <CRLF>
-(define (ftp-site conn arg)
-  (simple-command conn "SITE" arg))
+(define (ftp-site conn arg) (simple-command conn "SITE" arg))
 
 
 ;; *STRU <SP> <structure-code> <CRLF>
 
 
 ;; RMD  <SP> <pathname> <CRLF>
-(define (ftp-rmdir conn dirname)
-  (simple-command conn "RMD" dirname))
+(define (ftp-rmdir conn dirname) (simple-command conn "RMD" dirname))
 
 ;; STAT [<SP> <pathname>] <CRLF>
-(define (ftp-stat conn . opt)
-  (apply simple-command conn "STAT" opt))
-
+(define (ftp-stat conn . opt) (apply simple-command conn "STAT" opt))
 
 ;; SYST <CRLF>
-(define (ftp-system conn)
-  (let1 res (simple-command conn "SYST")
-    (string-drop res 4)))
+(define (ftp-system conn) (string-drop (simple-command conn "SYST") 4))
 
 ;; SIZE
 (define (ftp-size conn path)
-  (let1 res (simple-command conn "SIZE" path)
-    (string->number (string-drop res 4))))
+  (ftp-set-type conn)                   ; SIZE is affected by transfer type
+  (x->integer (string-drop (simple-command conn "SIZE" path) 4)))
 
 ;; MDTM
-(define (ftp-mdtm conn path)
-  (simple-command conn "MDTM" path))
+(define (ftp-mdtm conn path) (simple-command conn "MDTM" path))
 
-(define (ftp-mtime conn path . opt)
+(define (ftp-mtime conn path :optional (local-time? #f))
   (rxmatch-let (#/(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/ (ftp-mdtm conn path))
       (#f year month day hour min sec)
-    (let1 zoffset (if (get-optional opt #f)
-                      (date-zone-offset (current-date))
-                      0)
-      (date->time-utc (make-date 0
-                                 (string->number sec)
-                                 (string->number min)
-                                 (string->number hour)
-                                 (string->number day)
-                                 (string->number month)
-                                 (string->number year)
-                                 zoffset)))))
+    (date->time-utc
+     (make-date 0 (x->integer sec) (x->integer min) (x->integer hour)
+                (x->integer day) (x->integer month) (x->integer year)
+                (if local-time? (date-zone-offset (current-date)) 0)))))
 
 ;; NOOP <CRLF>
-(define (ftp-noop conn)
-  (simple-command conn "NOOP"))
+(define (ftp-noop conn) (simple-command conn "NOOP"))
 
 
 ;;; rfc 959 6(2) type command
@@ -220,15 +190,15 @@
 
 ;; LIST [<SP> <pathname>] <CRLF>
 ;; NLST [<SP> <pathname>] <CRLF>
-(define-values (ftp-list ftp-name-list)
-  (let1 make-lister (lambda (cmd)
-                      (lambda (conn . opt)
-                        (req&recv conn
-                                  (cut apply send-command conn cmd opt)
-                                  (cut port->string-list <>)
-                                  'ascii)))
-    (values (make-lister "LIST") (make-lister "NLST"))))
+(define (make-lister cmd)
+  (lambda (conn . opt)
+    (req&recv conn
+              (cut apply send-command conn cmd opt)
+              (cut port->string-list <>)
+              'ascii)))
 
+(define ftp-list (make-lister "LIST"))
+(define ftp-name-list (make-lister "NLST"))
 (define ftp-ls ftp-name-list)
 
 ;; *REIN <CRLF>
@@ -236,22 +206,18 @@
 ;; *REST <SP> <marker> <CRLF>
 
 ;; RETR <SP> <pathname> <CRLF>
-(define (ftp-get conn path . keys)
-  (let-keywords* keys ((sink (open-output-string))
-		       (flusher get-output-string))
-    (req&recv conn
-              (cut send-command conn "RETR" path)
-              (lambda (in)
-		(copy-port in sink)
-		(flusher sink)))))
+(define (ftp-get conn path :key (sink (open-output-string))
+                                (flusher get-output-string))
+  (req&recv conn
+            (cut send-command conn "RETR" path)
+            (lambda (in) (copy-port in sink) (flusher sink))))
 
 ;; STOR <SP> <pathname> <CRLF>
-(define (ftp-put conn from-file . opts)
-  (let-optionals* opts ((to-file (sys-basename from-file)))
-    (values-ref
-     (call-with-input-file from-file
-       (cute req&send conn (cut send-command conn "STOR" to-file) <>))
-     0)))
+(define (ftp-put conn from-file :optional (to-file (sys-basename from-file)))
+  (values-ref
+   (call-with-input-file from-file
+     (cute req&send conn (cut send-command conn "STOR" to-file) <>))
+   0))
 
 ;; STOU <CRLF>
 (define (ftp-put-unique conn from-file)
@@ -386,15 +352,12 @@
 
 
 ;; TYPE <SP> <type-code> <CRLF>
-(define (ftp-set-type conn . opt)
-  (let1 type (get-optional opt (ftp-transfer-type conn))
-    (simple-command conn
-                    "TYPE"
-                    (case type
-                      ((ascii) "A")
-                      ((binary image) "I")
-                      (else
-                       (error "Invalid transfer type:" type))))))
+(define (ftp-set-type conn :optional (type (ftp-transfer-type conn)))
+  (simple-command conn "TYPE"
+                  (case type
+                    [(ascii) "A"]
+                    [(binary image) "I"]
+                    [else (error "Invalid transfer type:" type)])))
 
 ;; requrest server to send data and receive it
 (define (req&recv conn cmdproc reader . opt)
@@ -422,22 +385,21 @@
       (lambda (get-data-socket)
         (let1 res (cmdproc)
           (rxmatch-case res
-            (#/^1\d\d FILE: (.+)$/ (#f dst-path)
+            [#/^1\d\d FILE: (.+)$/ (#f dst-path)
              ;; RFC 1123 - 4.1.2.9  STOU Command: RFC-959 Section 4.1.3
-             (copy-data get-data-socket) dst-path)
-            (#/^1/ ()
+             (copy-data get-data-socket) dst-path]
+            [#/^1/ ()
              ;; in case if the server doesn't conform RFC1123
-             (copy-data get-data-socket) #f)
-            (else (ftp-error res)))))))
+             (copy-data get-data-socket) #f]
+            [else (ftp-error res)])))))
   (define (retrieve-response res dst-path)
     (rxmatch-case res
-      (#/^2/ () (values res dst-path))
-      ;; vsftpd duplicates the 1XX reply for STOU.
-      (#/^1/ () (retrieve-response (get-response conn) dst-path))
-      (else (ftp-error res))))
+      [#/^2/ () (values res dst-path)]
+      ; vsftpd duplicates the 1XX reply for STOU.
+      [#/^1/ () (retrieve-response (get-response conn) dst-path)]
+      [else (ftp-error res)]))
 
   (ftp-set-type conn)
   (let1 dst-path (send-data)
     (retrieve-response (get-response conn) dst-path)))
 
-(provide "rfc/ftp")
