@@ -1,7 +1,7 @@
 ;;;
 ;;; mime.scm - parsing MIME (rfc2045) message
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: mime.scm,v 1.17 2008-05-10 13:36:08 shirok Exp $
-;;;
 
 ;; RFC2045 Multipurpose Internet Mail Extensions (MIME) Part One:
 ;;            Format of Internet Message Bodies
@@ -56,22 +54,24 @@
   (use util.list)
   (export mime-parse-version
           mime-parse-content-type mime-parse-content-disposition
-          mime-parse-parameter-value
+          mime-parse-parameters mime-compose-parameters
           mime-encode-word mime-encode-text
           mime-decode-word mime-decode-text
           <mime-part>
           mime-parse-message mime-retrieve-body
           mime-body->string mime-body->file
+          mime-make-boundary mime-compose-message mime-compose-message-string
           )
   )
 (select-module rfc.mime)
 
 (autoload rfc.quoted-printable quoted-printable-decode-string
-          quoted-printable-encode-string)
+          quoted-printable-encode quoted-printable-encode-string)
 (autoload rfc.base64 base64-decode-string base64-decode
-          base64-encode-string)
+          base64-encode-string base64-encode)
 (autoload gauche.charconv
           ces-upper-compatible? ces-conversion-supported? ces-convert)
+(autoload srfi-27 random-integer)       ;for MIME boundary generation
 
 ;;===============================================================
 ;; Basic utility
@@ -101,7 +101,7 @@
                       [ (string? subtype) ])
              (list* (string-downcase type)
                     (string-downcase subtype)
-                    (mime-parse-parameter-value input)))))))
+                    (mime-parse-parameters input)))))))
 
 ;; RFC2183 Content-Disposition header field
 ;; returns (<token> (<attribute> . <value>) ...)
@@ -112,14 +112,14 @@
            (and-let* ([token (rfc822-next-token input `(,*ct-token-chars*))]
                       [ (string? token) ])
              (cons (string-downcase token)
-                   (mime-parse-parameter-value input)))))))
+                   (mime-parse-parameters input)))))))
            
 
 ;; parse a parameter-values type header field
 ;;   ;parameter=value;parameter=value
 ;; => ((parameter . value) ...)
 ;; NB: This will support RFC2231, but not yet.
-(define (mime-parse-parameter-value input)
+(define (mime-parse-parameters :optional (input (current-input-port)))
   (let loop ((r '()))
     (cond [(and-let* ([ (eqv? #\; (rfc822-next-token input '())) ]
                       [attr (rfc822-next-token input `(,*ct-token-chars*))]
@@ -134,6 +134,37 @@
            => (lambda (p) (loop (cons p r)))]
           [else (reverse! r)])))
 
+;; Inverse of mime-parse-parameters.
+;; ((parameter . value) ...) => ;parameter=value;parameter=value ...
+;; NB: This will support RFC2231, but not yet.
+(define (mime-compose-parameters pvs
+                                 :optional (port (current-output-port))
+                                 :key (start-column 0))
+  (define (quote-value v)               ; TODO: possibly folding?
+    (if (string-every *ct-token-chars* v)
+      v
+      (string-append "\"" (regexp-replace-all #/[\"\\]/ v "\\\\\\0") "\"")))
+  (define (valid-name p)
+    (rlet1 z (x->string p)
+      (unless (string-every *ct-token-chars* z)
+        (error "invalid parameter value for rfc2822 header:" p))))
+  (define (gen)
+    (fold (lambda (pv column)
+            (match pv
+              [(p . v)
+               (let* ([z #`",(valid-name p)=,(quote-value (x->string v))"]
+                      [len (+ (string-length z) column)])
+                 (cond [(> len 78)
+                        (display ";\r\n ") (display z) (string-length z)]
+                       [else
+                        (display "; ") (display z) (+ len column 2)]))]
+              [_ (error "bad parameter-value entry:" pv)]))
+          start-column pvs))
+  (match port
+    [#f (with-output-to-string gen)]
+    [#t (gen)]
+    [(? port?) (with-output-to-port port gen)]))
+
 ;;===============================================================
 ;; RFC2047 header field encoding
 ;;
@@ -184,97 +215,95 @@
 
 ;; Encode a single word.  This one always encode, even WORD contains
 ;; US-ASCII characters only.
-(define (mime-encode-word word . keys)
-  (let-keywords keys ((charset "utf-8")
-                      (transfer-encoding 'base64))
-    (let ((converted
-           (cond [(ces-upper-compatible? charset (gauche-character-encoding))
-                  word]
-                 [else
-                  (ces-convert word (gauche-character-encoding) charset)]))
-          (enc (%canonical-encoding transfer-encoding)))
-      (format "=?~a?~a?~a?=" charset enc
-              ((if (eq? enc 'B)
-                 base64-encode-string
-                 quoted-printable-encode-string)
-               converted :line-width #f)))))
+(define (mime-encode-word word :key (charset "utf-8") (transfer-encoding 'base64))
+  (let ((converted
+         (cond [(ces-upper-compatible? charset (gauche-character-encoding))
+                word]
+               [else
+                (ces-convert word (gauche-character-encoding) charset)]))
+        (enc (%canonical-encoding transfer-encoding)))
+    (format "=?~a?~a?~a?=" charset enc
+            ((if (eq? enc 'B)
+               base64-encode-string
+               quoted-printable-encode-string)
+             converted :line-width #f))))
 
 ;; Encode entire body if necessary, considering line folding.
 ;; At this moment, this function just treat the body as unstructured
 ;; text.  It's not safe to pass structured text.  In future we may have
 ;; a keyword option to make body parsed as structured text.
-(define (mime-encode-text body . keys)
-  (let-keywords keys ((charset "utf-8")
-                      (transfer-encoding 'base64)
-                      (line-width 76)
-                      (start-column 0)
-                      (force #f))
-    (let ((enc (%canonical-encoding transfer-encoding))
-          (cslen (string-length (x->string charset)))
-          (pass-through? (and (not force)
-                              (ces-upper-compatible? charset 'ascii)
-                              (string-every #[\x00-\x7f] body))))
-      ;; estimates the length of an encoded word.  it is not trivial since
-      ;; we have to ces-convert each segment separately, and there's no
-      ;; general way to estimate the size of ces-converted string.
-      ;; we throw in some heuristics here.
-      (define (estimate-width s i)
-        (let1 na (string-count s #[\x00-\x7f] 0 i)
-          (+ 6 cslen
-             (if (eq? enc 'B)
-               (ceiling (* (+ na (* (- i na) 3)) 4/3))
-               (let1 ng (string-count s #[!-<>-~] 0 i)
-                 (+ (- na ng) (* 3 (+ ng (* (- i na) 3)))))))))
-      (define (encode-word w)
-        (mime-encode-word w :charset charset :transfer-encoding enc))
-      ;; we don't need to pack optimally, so we use some heuristics.
-      (define (encode str width adj)
-        (or (and-let* ([estim (estimate-width str (string-length str))]
-                       [ (< (* adj estim) width) ]
-                       [ew (encode-word str)])
-              (if (<= (string-length ew) width)
-                `(,ew)
-                (encode str width (* adj (/. (string-length ew) width)))))
-            (let loop ((k (min (string-length str) (quotient width 2))))
-              (let1 estim (* adj (estimate-width str k))
-                (if (<= estim width)
-                  (let1 ew (encode-word (string-take str k))
-                    (if (<= (string-length ew) width)
-                      (list* ew "\r\n "
-                             (encode (string-drop str k) (- line-width 1) adj))
-                      (loop (floor->exact
-                             (* k (/ width (string-length ew)))))))
-                  (loop (floor->exact (* k (/ width estim)))))))
-            ))
-      ;; fill pass-through text.  we try our best to look for an appropriate
-      ;; place to insert FWS.  we know STR is all ASCII.
-      (define (fill str width)
-        (if (<= (string-length str) width)
-          `(,str)
-          (or (and-let* ([pos (string-index-right str #\space 0 width)])
-                (list* (string-take str pos) "\r\n "
-                       (fill (string-drop str (+ pos 1)) (- line-width 1))))
-              ;; if we can't find a whitespace, we break in the middle of
-              ;; word for the last resort.
-              (list* (string-take str width) "\r\n "
-                     (fill (string-drop str width) (- line-width 1))))))
-      
-      (cond [(or (not line-width) (zero? line-width))
-             (if pass-through? body (encode-word body))]
-            [(< line-width 30)
-             (errorf "line width (~a) is too short to encode header field body: ~s" line-width body)]
-            [(< (- line-width start-column) 30)
-             ;; just in case if header name is very long.  we insert line break
-             ;; first.
-             (string-concatenate
-              (cons "\r\n " (if pass-through?
-                              (fill body (- line-width 1))
-                              (encode body (- line-width 1) 1.0))))]
-            [else
-             (string-concatenate
-              (if pass-through?
-                (fill body (- line-width start-column))
-                (encode body (- line-width start-column) 1.0)))]))))
+(define (mime-encode-text body :key
+                          (charset "utf-8")
+                          (transfer-encoding 'base64)
+                          (line-width 76)
+                          (start-column 0)
+                          (force #f))
+  (let ((enc (%canonical-encoding transfer-encoding))
+        (cslen (string-length (x->string charset)))
+        (pass-through? (and (not force)
+                            (ces-upper-compatible? charset 'ascii)
+                            (string-every #[\x00-\x7f] body))))
+    ;; estimates the length of an encoded word.  it is not trivial since
+    ;; we have to ces-convert each segment separately, and there's no
+    ;; general way to estimate the size of ces-converted string.
+    ;; we throw in some heuristics here.
+    (define (estimate-width s i)
+      (let1 na (string-count s #[\x00-\x7f] 0 i)
+        (+ 6 cslen
+           (if (eq? enc 'B)
+             (ceiling (* (+ na (* (- i na) 3)) 4/3))
+             (let1 ng (string-count s #[!-<>-~] 0 i)
+               (+ (- na ng) (* 3 (+ ng (* (- i na) 3)))))))))
+    (define (encode-word w)
+      (mime-encode-word w :charset charset :transfer-encoding enc))
+    ;; we don't need to pack optimally, so we use some heuristics.
+    (define (encode str width adj)
+      (or (and-let* ([estim (estimate-width str (string-length str))]
+                     [ (< (* adj estim) width) ]
+                     [ew (encode-word str)])
+            (if (<= (string-length ew) width)
+              `(,ew)
+              (encode str width (* adj (/. (string-length ew) width)))))
+          (let loop ((k (min (string-length str) (quotient width 2))))
+            (let1 estim (* adj (estimate-width str k))
+              (if (<= estim width)
+                (let1 ew (encode-word (string-take str k))
+                  (if (<= (string-length ew) width)
+                    (list* ew "\r\n "
+                           (encode (string-drop str k) (- line-width 1) adj))
+                    (loop (floor->exact
+                           (* k (/ width (string-length ew)))))))
+                (loop (floor->exact (* k (/ width estim)))))))
+          ))
+    ;; fill pass-through text.  we try our best to look for an appropriate
+    ;; place to insert FWS.  we know STR is all ASCII.
+    (define (fill str width)
+      (if (<= (string-length str) width)
+        `(,str)
+        (or (and-let* ([pos (string-index-right str #\space 0 width)])
+              (list* (string-take str pos) "\r\n "
+                     (fill (string-drop str (+ pos 1)) (- line-width 1))))
+            ;; if we can't find a whitespace, we break in the middle of
+            ;; word for the last resort.
+            (list* (string-take str width) "\r\n "
+                   (fill (string-drop str width) (- line-width 1))))))
+    
+    (cond [(or (not line-width) (zero? line-width))
+           (if pass-through? body (encode-word body))]
+          [(< line-width 30)
+           (errorf "line width (~a) is too short to encode header field body: ~s" line-width body)]
+          [(< (- line-width start-column) 30)
+           ;; just in case if header name is very long.  we insert line break
+           ;; first.
+           (string-concatenate
+            (cons "\r\n " (if pass-through?
+                            (fill body (- line-width 1))
+                            (encode body (- line-width 1) 1.0))))]
+          [else
+           (string-concatenate
+            (if pass-through?
+              (fill body (- line-width start-column))
+              (encode body (- line-width start-column) 1.0)))])))
 
 (define (%canonical-encoding transfer-encoding)
   (case transfer-encoding
@@ -406,14 +435,15 @@
 
 ;; message information packet
 (define-class <mime-part> ()
-  ((type     :init-keyword :type)
-   (subtype  :init-keyword :subtype)
-   (parameters :init-keyword :parameters)
-   (transfer-encoding :init-keyword :transfer-encoding)
-   (headers  :init-keyword :headers)
+  ((type     :init-keyword :type :init-value "text")
+   (subtype  :init-keyword :subtype :init-value "plain")
+   (parameters :init-keyword :parameters :init-value '())
+   (transfer-encoding :init-keyword :transfer-encoding :init-value #f)
+   (headers  :init-keyword :headers :init-value '())
    (parent   :init-keyword :parent :init-value #f)
    (index    :init-keyword :index :init-value 0)
-   (content  :init-value #f)
+   (content  :init-keyword :content :init-value #f)
+   (source   :init-keyword :source :init-value #f) ; only used for composing
    ))
 
 (define (mime-parse-message port headers handler)
@@ -421,19 +451,18 @@
                   '("text" "plain" ("charset" . "us-ascii"))))
 
 (define (internal-parse port headers handler parent index default-type)
-  (let* ((ctype (or (mime-parse-content-type
+  (let* ([ctype (or (mime-parse-content-type
                      (rfc822-header-ref headers "content-type"))
-                    default-type))
-         (enc   (rfc822-header-ref headers "content-transfer-encoding" "7bit"))
-         (packet (make <mime-part>
+                    default-type)]
+         [enc   (rfc822-header-ref headers "content-transfer-encoding" "7bit")]
+         [packet (make <mime-part>
                    :type (car ctype)
                    :subtype (cadr ctype)
                    :parameters (cddr ctype)
                    :transfer-encoding enc
                    :parent parent
                    :index index
-                   :headers headers))
-         )
+                   :headers headers)])
     (cond
      [(equal? (car ctype) "multipart")
       (multipart-parse port packet handler)]
@@ -446,32 +475,31 @@
      )))
 
 (define (multipart-parse port packet handler)
-  (let* ((boundary (or (assoc-ref (ref packet 'parameters) "boundary")
-                       (error "No boundary given for multipart message")))
-         (default-type (if (equal? (ref packet 'subtype) "digest")
+  (let* ([boundary (or (assoc-ref (ref packet 'parameters) "boundary")
+                       (error "No boundary given for multipart message"))]
+         [default-type (if (equal? (ref packet 'subtype) "digest")
                          '("message" "rfc822")
-                         '("text" "plain" ("charset" . "us-ascii"))))
-         (mime-port (make-mime-port boundary port))
-         )
+                         '("text" "plain" ("charset" . "us-ascii")))]
+         [mime-port (make-mime-port boundary port)])
     (let loop ((index 0)
                (contents '()))
-      (let* ((headers (rfc822-header->list mime-port))
-             (r (internal-parse mime-port headers handler
+      (let* ([headers (rfc822-header->list mime-port)]
+             [r (internal-parse mime-port headers handler
                                 packet index
-                                default-type)))
+                                default-type)])
         (case (ref mime-port 'state)
-          ((boundary)
+          [(boundary)
            (set! (ref mime-port 'state) 'body)
-           (loop (+ index 1) (cons r contents)))
-          ((eof)
+           (loop (+ index 1) (cons r contents))]
+          [(eof)
            (set! (ref packet 'content) (reverse! (cons r contents)))
-           packet)
-          (else ;; parser returned without readling entire part.
+           packet]
+          [else ;; parser returned without readling entire part.
            ;; discard the rest of the part.
            (let loop ((b (read-byte port)))
              (unless (eof-object? b)
                (loop (read-byte port))))
-           packet))))
+           packet])))
     ))
 
 (define (message-parse port packet handler)
@@ -548,4 +576,123 @@
         (cut mime-retrieve-body packet inp outp))))
   filename)
 
-(provide "rfc/mime")
+;;===============================================================
+;; MIME composer
+;;
+
+;; Each PART must be either an instance of <mime-part>, or a list
+;; of <mime-part-list>, where
+;;   <mime-part-list> : (<content-type> (<header> ...) <body>)
+;;   <content-type>   : (<type> <subtype> <header-param> ...)
+;;   <header-param>   : (<key> . <value>) ...
+;;   <header>         : (<header-name> <encoded-header-value>)
+;;                    | (<header-name (<header-value> <header-param> ...))
+;;   <body>           : string
+;;                    | (file <filename>)
+;;                    | (subparts <part> ...)
+;;
+;; Note: In the first form of <header>, <encoded-header-value> must
+;; already be encoded using RFC2047 or RFC2231 if the original value
+;; contains non-ascii characters.
+;; In the second form, we plan to do RFC2231 encoding on behalf of
+;; the caller; but the current version does not implement it.  The
+;; caller SHOULD NOT pass encoded words in this form, since it may
+;; result double-encoding when we implement the auto encoding feature;
+;; for the time being, the second form restricts ASCII-only values.
+
+;; Emits composed message to PORT, and returns boundary string.
+;; The boundary string is automatically generated unless provided
+;; by the optional argument.
+(define (mime-compose-message parts
+                              :optional (port (current-output-port))
+                              :key (boundary (mime-make-boundary)))
+  (dolist [p parts]
+    (for-each (cut display <> port) `("\r\n--" ,boundary "\r\n"))
+    (mime-generate-one-part (canonical-part p) port))
+  (for-each (cut display <> port) `("\r\n--" ,boundary "--\r\n"))
+  boundary)
+
+;; Returns composed message in string, AND the boundary.
+(define (mime-compose-message-string parts
+                                     :key (boundary (mime-make-boundary)))
+  (values (call-with-output-string
+            (cut mime-compose-message parts <> :boundary boundary))
+          boundary))
+
+(define (mime-make-boundary)
+  (format "boundary-~a" (number->string (* (random-integer (expt 2 64))
+                                           (sys-time) (sys-getpid))
+                                        36)))
+
+;; internal stuff
+(define (canonical-part p)
+  (match p
+    [(? (cut is-a? <> <mime-part>)) p]
+    [((type subtype . params) (headers ...) body)
+     (let1 hs (filter-map canonical-header headers)
+       (apply make <mime-part>
+              :type type :subtype subtype :parameters params :headers hs 
+              :transfer-encoding (rfc822-header-ref hs "content-transfer-encoding")
+              (match body
+                [(? string?) `(:content ,body)]
+                [('file name) `(:source ,name)]
+                [('subparts ps ...) `(:content ,(map canonical-part ps))]
+                [_ (error "Invalid mime part body spec:" body)])))]
+    [_ (error "Invalid mime part spec:" p)]))
+
+(define (canonical-header header)
+  (match header
+    [(name . x) (cons (x->string name) x)]
+    [_ #f]))
+
+(define (mime-generate-one-part part port)
+  (when (list? (ref part'content))
+    (unless (member "boundary" (ref part'parameters))
+      (push! (ref part'parameters) (cons "boundary" (mime-make-boundary)))))
+  (let1 cte (mime-generate-part-header part port)
+    (with-output-to-port port
+      (lambda ()
+        (cond
+         [(ref part'source) =>
+          (cut with-input-from-file <> (cut mime-generate-part-body part cte))]
+         [(list? (ref part'content))
+          (mime-compose-message (ref part'content) port
+                                :boundary (assoc-ref (ref part'parameters)
+                                                     "boundary"))]
+         [(string? (ref part'content))
+          (with-input-from-string (ref part'content)
+            (cut mime-generate-part-body part cte))]
+         [else (error "unsupported MIME part content")])))))
+
+;; returns content-transfer-encoding
+(define (mime-generate-part-header part port)
+  (rlet1 cte (ref part'transfer-encoding)
+    (rfc822-write-headers
+     `(("Content-type"
+        ,(format "~a/~a~a" (ref part'type) (ref part'subtype)
+                 (mime-compose-parameters (ref part'parameters) #f)))
+       ,@(cond-list [cte => (cut list "Content-transfer-encoding" <>)])
+       ,@(filter-map gen-header-1 (ref part'headers)))
+     :output port :check :ignore)))
+
+(define (gen-header-1 h)
+  (match h
+    [("content-transfer-encoding" . _) #f]
+    [("content-type" . _) #f]
+    [(name (value pv ...))
+     (let* ([sval (x->string value)]
+            [spvs (mime-compose-parameters pv #f
+                   :start-column (+ (string-length name) (string-length sval) 2))])
+       `(,name ,(if (null? pv) sval #`",|sval|,|spvs|")))]
+    [(name value) h]))  
+
+;; current-input-port -> current-output-port
+(define (mime-generate-part-body part transfer-enc)
+  (cond
+   [(or (not transfer-enc) (member transfer-enc '("binary" "7bit") string-ci=?))
+    (copy-port (current-input-port) (current-output-port))]
+   [(string-ci=? transfer-enc "base64") (base64-encode)]
+   [(string-ci=? transfer-enc "quoted-printable") (quoted-printable-encode)]
+   [else (error "Unsupported transfer encoding encountered while composing \
+                 mime message: " transfer-enc)]))
+
