@@ -1,7 +1,7 @@
 ;;;
 ;;; srfi-27.scm - Sources of Random Bits
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: srfi-27.scm,v 1.7 2008-05-10 13:35:48 shirok Exp $
-;;;
 
 ;; Implements SRFI-27 interface on top of math.mt-random module.
 
@@ -112,8 +110,8 @@
     (let1 unit (car maybe-unit)
       (unless (< 0 unit 1)
         (error "unit must be between 0.0 and 1.0 (exclusive), but got" unit))
-      (let* ((1/unit (/ unit))
-             (range (inexact->exact (ceiling 1/unit))))
+      (let* ((1/unit (/. unit))
+             (range (ceiling->exact 1/unit)))
         (lambda ()
           (/ (random-integer range) 1/unit))))))
 
@@ -123,4 +121,3 @@
     (values (lambda (n) (mt-random-integer src n))
             (lambda ()  (mt-random-real src)))))
 
-(provide "srfi-27")
