@@ -4,7 +4,6 @@
 ;;  This is a port of Sebastian Egner's reference implementation to Gauche.
 ;;  Ported by Alex Shinn.
 ;;
-;;  $Id: srfi-42.scm,v 1.3 2006-06-05 05:11:24 shirok Exp $
 
 ; <PLAINTEXT>
 ; Eager Comprehensions in [outer..inner|expr]-Convention
@@ -108,26 +107,29 @@
 ;; use the same signature as the hygienic version for compatibility, we
 ;; ignore the middle "collector" arg
 (define-macro (%replace-keywords syn _ . args)
+  (define (rename symbol)               ; bandage for hygiene
+    ((with-module gauche.internal make-identifier)
+     symbol (current-module) '()))
   (define (rewrite x)
     (if (pair? x)
       (if (keyword? (car x))
         (cons
          (case (car x)
-           ((:)            'srfi-42-)
-           ((:list)        'srfi-42-list)
-           ((:string)      'srfi-42-string)
-           ((:vector)      'srfi-42-vector)
-           ((:integers)    'srfi-42-integers)
-           ((:range)       'srfi-42-range)
-           ((:real-range)  'srfi-42-real-range)
-           ((:char-range)  'srfi-42-char-range)
-           ((:port)        'srfi-42-port)
-           ((:dispatched)  'srfi-42-dispatched)
-           ((:do)          'srfi-42-do)
-           ((:let)         'srfi-42-let)
-           ((:parallel)    'srfi-42-parallel)
-           ((:while)       'srfi-42-while)
-           ((:until)       'srfi-42-until)
+           ((:)            (rename 'srfi-42-))
+           ((:list)        (rename 'srfi-42-list))
+           ((:string)      (rename 'srfi-42-string))
+           ((:vector)      (rename 'srfi-42-vector))
+           ((:integers)    (rename 'srfi-42-integers))
+           ((:range)       (rename 'srfi-42-range))
+           ((:real-range)  (rename 'srfi-42-real-range))
+           ((:char-range)  (rename 'srfi-42-char-range))
+           ((:port)        (rename 'srfi-42-port))
+           ((:dispatched)  (rename 'srfi-42-dispatched))
+           ((:do)          (rename 'srfi-42-do))
+           ((:let)         (rename 'srfi-42-let))
+           ((:parallel)    (rename 'srfi-42-parallel))
+           ((:while)       (rename 'srfi-42-while))
+           ((:until)       (rename 'srfi-42-until))
            (else (car x)))
          (rewrite (cdr x)))
         (cons (rewrite (car x)) (rewrite (cdr x))))
@@ -1131,4 +1133,3 @@
     ((every?-ec qualifier expression)
      (first-ec #t qualifier (if (not expression)) #f) )))
 
-(provide "srfi-42")
