@@ -1,7 +1,7 @@
 ;;;
 ;;; collection.scm - collection generics
 ;;;  
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,15 +30,12 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: collection.scm,v 1.9 2008-05-10 13:36:13 shirok Exp $
-;;;
 
 ;; Defines generic operations over collection.   A collection is
 ;; a set of objects, possibly containing infinite objects.
 
 (define-module gauche.collection
   (use srfi-1)
-  (use util.queue)
   (export call-with-iterator with-iterator call-with-iterators
           call-with-builder  with-builder
           fold fold2 fold3 map map-to map-accum for-each
@@ -50,6 +47,15 @@
   )
 (select-module gauche.collection)
 
+;; utility - we can't depend on util.queue, so this is a simple
+;; alternative.
+(define (make-queue)   (let1 anchor (list #f) (cons anchor anchor)))
+(define (enqueue! q x) (set! (cddr q) (list x)) (set! (cdr q) (cddr q)))
+(define (dequeue! q)   (if (null? (cdar q))
+                         (error "queue is empty" q)
+                         (rlet1 v (cadar q) (set! (cdar q) (cddar q)))))
+(define (queue->list q) (cdar q))
+
 ;;-------------------------------------------------
 ;; Call-with-iterator - the fundamental iterator
 ;;
@@ -124,7 +130,7 @@
 
 (define-method call-with-builder ((class <list-meta>) proc . args)
   (let ((q (make-queue)))
-    (proc (cut enqueue! q <>) (cut dequeue-all! q))))
+    (proc (cut enqueue! q <>) (cut queue->list q))))
 
 (define-method call-with-builder ((class <vector-meta>) proc . args)
   (let ((size (get-keyword :size args #f)))
@@ -138,7 +144,7 @@
                 (lambda () v)))
         (let ((q (make-queue)))
           (proc (cut enqueue! q <>)
-                (cut list->vector (dequeue-all! q)))))))
+                (cut list->vector (queue->list q)))))))
 
 (define-method call-with-builder ((class <weak-vector-meta>) proc . args)
   (let ((size (get-keyword :size args #f)))
@@ -179,7 +185,8 @@
 
 (define-method call-with-builder ((class <tree-map-meta>) proc . args)
   (let-keywords* args ((key=? =)
-                       (key<? <))
+                       (key<? <)
+                       (size #f))       ;ignored for now.
     (let1 tree (make-tree-map key=? key<?)
       (proc (lambda (item)
               (unless (pair? item)
@@ -266,7 +273,7 @@
   (if (null? more)
       (with-iterator (coll end? next)
         (do ((q (make-queue)))
-            ((end?) (dequeue-all! q))
+            ((end?) (queue->list q))
           (enqueue! q (proc (next)))))
       (let ((%map (with-module gauche map)))
         (call-with-iterators
@@ -274,7 +281,7 @@
          (lambda (ends? nexts)
            (do ((q (make-queue)))
                ((any (cut <>) ends?)
-                (dequeue-all! q))
+                (queue->list q))
              (enqueue! q (apply proc (%map (cut <>) nexts))))))
         )))
 
@@ -457,7 +464,7 @@
   (let ((q (make-queue)))
     (with-iterator (coll end? next)
       (until (end?) (let ((e (next))) (when (pred e) (enqueue! q e))))
-      (dequeue-all! q))))
+      (queue->list q))))
 
 (define-method filter-to ((class <class>) pred (coll <collection>))
   (with-builder (class add! get)
@@ -480,7 +487,7 @@
   (let ((q (make-queue)))
     (with-iterator (coll end? next)
       (until (end?) (let ((e (next))) (unless (pred e) (enqueue! q e))))
-      (dequeue-all! q))))
+      (queue->list q))))
 
 (define-method remove-to ((class <class>) pred (coll <collection>))
   (with-builder (class add! get)
@@ -567,4 +574,3 @@
                 '()
                 col))))
 
-(provide "gauche/collection")
