@@ -1,7 +1,7 @@
 /*
  * char.c - character and character set operations
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,14 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: char.c,v 1.62 2008-05-10 13:36:17 shirok Exp $
  */
 
 #include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
+#include "gauche/char_attr.h"
+
+#include "char_attr.c"          /* generated tables */
 
 /*=======================================================================
  * Character functions
@@ -192,9 +193,9 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_CharSetClass,
  * Printer
  */
 
-static void charset_print_ch(ScmPort *out, ScmChar ch)
+static void charset_print_ch(ScmPort *out, ScmChar ch, int firstp)
 {
-    if (ch == '[' || ch == ']' || ch == '-') {
+    if (ch == '[' || ch == ']' || ch == '-' || (ch == '^' && firstp)) {
         Scm_Printf(out, "\\%c", ch);
     } else if (ch < 0x20 || ch == 0x7f) {
         Scm_Printf(out, "\\x%02x", ch);
@@ -209,7 +210,7 @@ static void charset_print_ch(ScmPort *out, ScmChar ch)
 
 static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 {
-    int prev, code;
+    int prev, code, first = TRUE;
     ScmCharSet *cs = SCM_CHAR_SET(obj);
     ScmTreeIter iter;
     ScmDictEntry *e;
@@ -217,13 +218,14 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
     Scm_Printf(out, "#[");
     for (prev = -1, code = 0; code < SCM_CHAR_SET_SMALL_CHARS; code++) {
         if (MASK_ISSET(cs, code) && prev < 0) {
-            charset_print_ch(out, code);
+            charset_print_ch(out, code, first);
             prev = code;
+            first = FALSE;
         } 
         else if (!MASK_ISSET(cs, code) && prev >= 0) {
             if (code - prev > 1) {
                 if (code - prev > 2) Scm_Printf(out, "-");
-                charset_print_ch(out, code-1);
+                charset_print_ch(out, code-1, FALSE);
             }
             prev = -1;
         }
@@ -231,15 +233,15 @@ static void charset_print(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
     if (prev >= 0) {
         if (code - prev > 1) {
             if (prev < 0x7e) Scm_Printf(out, "-");
-            charset_print_ch(out, code-1);
+            charset_print_ch(out, code-1, FALSE);
         }
     }
     Scm_TreeIterInit(&iter, &cs->large, NULL);
     while ((e = Scm_TreeIterNext(&iter)) != NULL) {
-        charset_print_ch(out, (int)e->key);
+        charset_print_ch(out, (int)e->key, FALSE);
         if (e->value != e->key) {
             if (e->value - e->key > 2) Scm_Printf(out, "-");
-            charset_print_ch(out, (int)e->value);
+            charset_print_ch(out, (int)e->value, FALSE);
         }
     }
     Scm_Printf(out, "]", obj);
@@ -461,18 +463,18 @@ ScmObj Scm_CharSetComplement(ScmCharSet *cs)
 
     Scm_BitsOperate(cs->small, SCM_BIT_NOT1, cs->small, NULL,
                     0, SCM_CHAR_SET_SMALL_CHARS);
-    last = SCM_CHAR_SET_SMALL_CHARS;
+    last = SCM_CHAR_SET_SMALL_CHARS-1;
     /* we can't use treeiter, since we modify the tree while traversing it. */
     while ((e = Scm_TreeCoreNextEntry(&cs->large, last)) != NULL) {
         Scm_TreeCoreSearch(&cs->large, e->key, SCM_DICT_DELETE);
         if (last < e->key-1) {
-            n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
+            n = Scm_TreeCoreSearch(&cs->large, last+1, SCM_DICT_CREATE);
             n->value = e->key-1;
         }
-        last = (int)e->value+1;
+        last = (int)e->value;
     }
     if (last < SCM_CHAR_MAX) {
-        n = Scm_TreeCoreSearch(&cs->large, last, SCM_DICT_CREATE);
+        n = Scm_TreeCoreSearch(&cs->large, last+1, SCM_DICT_CREATE);
         n->value = SCM_CHAR_MAX;
     }
     return SCM_OBJ(cs);
@@ -799,6 +801,198 @@ ScmObj read_predef_charset(ScmPort *input, ScmObj *chars, int error_p)
     return SCM_FALSE;
 }
 
+/*-----------------------------------------------------------------
+ * Character attributes
+ */
+
+int Scm_CharGeneralCategory(ScmChar ch)
+{
+    return (int)(Scm__LookupCharCategory(ch) & SCM_CHAR_CATEGORY_MASK);
+}
+
+int Scm_CharAlphabeticP(ScmChar ch)
+{
+    return (SCM_CHAR_ALPHA_MASK & Scm__LookupCharCategory(ch)) != 0;
+}
+
+int Scm_CharUppercaseP(ScmChar ch)
+{
+    return ((SCM_CHAR_ALPHA_MASK & Scm__LookupCharCategory(ch))
+            == SCM_CHAR_UPPERCASE_BITS);
+}
+
+int Scm_CharLowercaseP(ScmChar ch)
+{
+    return ((SCM_CHAR_ALPHA_MASK & Scm__LookupCharCategory(ch))
+            == SCM_CHAR_LOWERCASE_BITS);
+}
+
+int Scm_CharTitlecaseP(ScmChar ch)
+{
+    return (Scm_CharGeneralCategory(ch) == SCM_CHAR_CATEGORY_Lt);
+}
+
+int Scm_CharNumericP(ScmChar ch)
+{
+    return (Scm_CharGeneralCategory(ch) == SCM_CHAR_CATEGORY_Nd);
+}
+
+/* An internal entry to extract case mapping info.
+ * Internal table is compressed, so the caller must provide
+ * the buffer for ScmCharCaseMap.
+ * The function returns either the pointer to the given buffer
+ * with information filled, or a pointer to a static read-only
+ * data structure in the internal table.
+ */
+static const ScmCharCaseMap casemap_identity = {
+    0, 0, 0, {-1}, {-1}, {-1}
+};
+
+const ScmCharCaseMap *Scm__CharCaseMap(ScmChar ch,
+                                       ScmCharCaseMap *buf,
+                                       int full)
+{
+    if (ch < 0x10000) {
+        int subtable = casemap_000[(ch >> 8) & 0xff];
+        unsigned short cmap;
+        
+        if (subtable == 255) return &casemap_identity;
+
+        cmap = casemap_subtable[subtable][(unsigned char)(ch & 0xff)];
+        if (cmap == SCM_CHAR_NO_CASE_MAPPING) return &casemap_identity;
+        if (cmap & 0x8000) {
+            /* mapping is extended. */
+            return &(extended_casemaps[cmap & 0x7fff]);
+        } else {
+            /* mapping is simple */
+            int off = (cmap & 0x2000)? (signed int)(cmap|~0x1fff) : cmap&0x1fff;
+            if (cmap & 0x4000) {
+                buf->to_upper_simple = off;
+                buf->to_lower_simple = 0;
+                buf->to_title_simple = off;
+            } else {
+                buf->to_upper_simple = 0;
+                buf->to_lower_simple = off;
+                buf->to_title_simple = 0;
+            }
+            if (full) {
+                /* indicate no special mappings */
+                buf->to_upper_full[0] = -1;
+                buf->to_lower_full[0] = -1;
+                buf->to_title_full[0] = -1;
+            }
+            return buf;
+        }
+    } else {
+        /* TODO: 104xx*/
+        return &casemap_identity;
+    }
+}
+
+/*
+ * Case conversion API.  For the time being, CharCaseMap works on Unicode
+ * codepoints, so we have to convert from/to ScmChar if the internal encoding
+ * is either EUC-JP or SJIS.
+ */
+#define SIMPLE_CASE(code, buf, field) \
+    (ScmChar)((code) + Scm__CharCaseMap((code), (buf), FALSE)->SCM_CPP_CAT3(to_, field, _simple))
+
+#define SIMPLE_CASE_CV(code, buf, field)    \
+    ((code) = (ScmChar)Scm_CharToUcs((int)(code)), \
+     (code) = SIMPLE_CASE(code, buf, field),       \
+     Scm_UcsToChar((int)(code)))
+
+ScmChar Scm_CharUpcase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (ch < 0x80) return SIMPLE_CASE(ch, &cm, upper);
+    else if (Scm__CharInUnicodeP(ch)) return SIMPLE_CASE_CV(ch, &cm, upper);
+    else           return ch;
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    return SIMPLE_CASE(ch, &cm, upper);
+#else
+    /* Latin-1 mapping and Unicode mapping differ in U+00B5 (MICRO SIGN)
+       and U+00FF (LATIN SMALL LETTER Y WITH DIAERESIS).  In Unicode
+       they map to U+039C and U+0178, respectively.  In Latin-1 we don't
+       have those characters, so we leave them alone. */
+    if (ch == 0xb5 || ch == 0xff) return ch;
+    else return SIMPLE_CASE(ch, &cm, upper);
+#endif
+}
+
+ScmChar Scm_CharDowncase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (ch < 0x80) return SIMPLE_CASE(ch, &cm, lower);
+    else if (Scm__CharInUnicodeP(ch)) return SIMPLE_CASE_CV(ch, &cm, lower);
+    else           return ch;
+#else
+    return SIMPLE_CASE(ch, &cm, lower);
+#endif
+}
+
+ScmChar Scm_CharTitlecase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (ch < 0x80) return SIMPLE_CASE(ch, &cm, title);
+    else if (Scm__CharInUnicodeP(ch)) return SIMPLE_CASE_CV(ch, &cm, title);
+    else           return ch;
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    return SIMPLE_CASE(ch, &cm, title);
+#else
+    /* In Latin-1, titlecase is the same as upcase. */
+    return Scm_CharUpcase(ch);
+#endif
+}
+
+ScmChar Scm_CharFoldcase(ScmChar ch)
+{
+    ScmCharCaseMap cm;
+    const ScmCharCaseMap *pcm;
+#if defined(GAUCHE_CHAR_ENCODING_EUC_JP) || defined(GAUCHE_CHAR_ENCODING_SJIS)
+    if (Scm__CharInUnicodeP(ch)) {
+        ScmChar ucs = (ScmChar)Scm_CharToUcs(ch);
+        pcm = Scm__CharCaseMap(ucs, &cm, FALSE);
+        if (pcm->to_lower_simple == 0 && pcm->to_upper_simple == 0) {
+            /* we don't have case folding */
+            return ch;
+        }
+        /* Otherwise, we do (char-downcase (char-upcase ch)) */
+        if (pcm->to_upper_simple != 0) {
+            ucs += pcm->to_upper_simple;
+            pcm = Scm__CharCaseMap(ucs, &cm, FALSE);
+        }
+        return Scm_UcsToChar((int)(ucs + pcm->to_lower_simple));
+    } else {
+        return ch;
+    }
+#elif defined(GAUCHE_CHAR_ENCODING_UTF_8)
+    if (ch == 0x130 || ch == 0x131) {
+        /* char-foldcase is identity for
+           U+0130 Turkish I (LATIN CAPITAL LETTER I WITH DOT ABOVE) and
+           U+0131 Turkish i (LATIN SMALL LETTER DOTLESS I) */
+        return ch;
+    }
+    pcm = Scm__CharCaseMap(ch, &cm, FALSE);
+    if (pcm->to_lower_simple == 0 && pcm->to_upper_simple == 0) {
+        /* we don't have case folding */
+        return ch;
+    }
+    /* Otherwise, we do (char-downcase (char-upcase ch)) */
+    if (pcm->to_upper_simple != 0) {
+        ch += pcm->to_upper_simple;
+        pcm = Scm__CharCaseMap(ch, &cm, FALSE);
+    }
+    return ch + pcm->to_lower_simple;
+#else
+    /* In Latin-1 range, foldcase is the same as donwcase. */
+    return SIMPLE_CASE(ch, &cm, lower);
+#endif
+}
+
 /*-----------------------------------------------------------------
  * Pre-defined charset
  */
