@@ -1,7 +1,7 @@
 /*
  * compaux.c - C API bridge for the compiler
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: compaux.c,v 1.16 2008-05-10 13:36:17 shirok Exp $
  */
 
 /* This file serves as a bridge to the compiler, which is implemented
@@ -63,13 +61,12 @@ static ScmInternalMutex compile_finish_mutex;
 
 ScmObj Scm_Compile(ScmObj program, ScmObj env)
 {
-    return Scm_ApplyRec(SCM_GLOC_GET(compile_gloc), SCM_LIST2(program, env));
+    return Scm_ApplyRec2(SCM_GLOC_GET(compile_gloc), program, env);
 }
 
 ScmObj Scm_CompilePartial(ScmObj program, ScmObj env)
 {
-    return Scm_ApplyRec(SCM_GLOC_GET(compile_partial_gloc),
-                        SCM_LIST2(program, env));
+    return Scm_ApplyRec2(SCM_GLOC_GET(compile_partial_gloc), program, env);
 }
 
 void Scm_CompileFinish(ScmCompiledCode *cc)
@@ -78,8 +75,7 @@ void Scm_CompileFinish(ScmCompiledCode *cc)
         SCM_INTERNAL_MUTEX_LOCK(compile_finish_mutex);
         SCM_UNWIND_PROTECT {
             if (cc->code == NULL) {
-                Scm_ApplyRec(SCM_GLOC_GET(compile_finish_gloc),
-                             SCM_LIST1(SCM_OBJ(cc)));
+                Scm_ApplyRec1(SCM_GLOC_GET(compile_finish_gloc), SCM_OBJ(cc));
             }
         }
         SCM_WHEN_ERROR {
@@ -328,6 +324,6 @@ void Scm__InitCompaux(void)
     INIT_GLOC(compile_partial_gloc, "compile-partial", gi);
     INIT_GLOC(compile_finish_gloc,  "compile-finish",  gi);
 
-    Scm_ApplyRec(SCM_GLOC_GET(init_compiler_gloc), SCM_NIL);
+    Scm_ApplyRec0(SCM_GLOC_GET(init_compiler_gloc));
 }
 
