@@ -1,7 +1,7 @@
 ;;;
 ;;; gauche-init.scm - initialize standard environment
 ;;;
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: gauche-init.scm,v 1.125 2008-05-10 13:36:19 shirok Exp $
-;;;
 
 (select-module gauche)
 
@@ -53,11 +51,11 @@
 ;; Preferred way
 ;;  (use x.y.z) ==> (require "x/y/z") (import x.y.z)
 
-(define-macro (use module)
+(define-macro (use module . options)
   `(begin
      (with-module gauche
        (require ,(module-name->path module)))
-     (import ,module)))
+     (import (,module ,@options))))
 
 ;; create built-in modules, so that (use srfi-6) won't complain, for example.
 (define-module srfi-2 )
