@@ -1,7 +1,7 @@
 /*
  * gauche.h - Gauche scheme system header
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: gauche.h,v 1.508 2008-06-02 01:13:13 shirok Exp $
  */
 
 #ifndef GAUCHE_H
@@ -55,6 +53,7 @@
 #include <errno.h>
 #include <gauche/int64.h>
 #include <gauche/float.h>
+#include <gauche/arch.h>
 
 #if defined(LIBGAUCHE_BODY)
 #if !defined(GC_DLL)
@@ -115,8 +114,13 @@ SCM_DECL_BEGIN
 #define SCM_DEBUG_HELPER      TRUE
 #endif
 
-#define SCM_INLINE_MALLOC_PRIMITIVES 0
+/* Define this to 0 to turn off fast flonum extension.  See the comment in
+   gauche/number.h for the details. */
+#define GAUCHE_FFX 1
 
+/* Include appropriate threading interface.  Threading primitives are
+   abstracted with SCM_INTERNAL_* macros and ScmInternal* typedefs.
+   See gauche/uthread.h for the semantics of these primitives. */
 #ifdef GAUCHE_USE_PTHREADS
 # include <gauche/pthread.h>
 #else  /* !GAUCHE_USE_PTHREADS */
@@ -125,6 +129,22 @@ SCM_DECL_BEGIN
 
 #define SCM_WORD_BITS   (SIZEOF_LONG*8)
 
+/* Newer gcc/glibc adds lots of __attribute__((warn_unused_result)) that
+   causes excessive warnings for the code that intentionally ignores the
+   return value.  Casting the result to void won't silence it.
+   Hence this macro. */
+#define SCM_IGNORE_RESULT(expr)  do { if(expr) {} } while(0)
+
+/* ScmFlonum and ScmClass must always be aligned in 8-byte boundaries.
+   (All other Scheme objects can be in 4-byte boundary.)
+   Some platform doesn't align static double in 8-byte boundaries, so
+   we try this as well.  */
+#ifdef __GNUC__
+#define SCM_ALIGN8  __attribute__ ((aligned (8)))
+#else  /* !__GNUC__ */
+#define SCM_ALIGN8  /*empty*/
+#endif /* !__GNUC__ */
+
 /*-------------------------------------------------------------
  * BASIC TYPES
  */
@@ -160,28 +180,38 @@ typedef struct ScmClassRec ScmClass;
  * [Pointer]
  *      -------- -------- -------- ------00
  *      Points to a pair or other heap-allocated objects.
+ *      If the lower 3 bits of the pointed word are '111',
+ *      it's a heap object (see below).  Otherwise, it's
+ *      a pair.
  *
  * [Fixnum]
  *      -------- -------- -------- ------01
- *      30-bit signed integer
+ *      30 or 62-bit signed integer
+ *
+ * [Flonum]
+ *      -------- -------- -------- -----M10
+ *      Points to C double.  M=0 if the double is in the VM
+ *      register, M=1 if it is on the heap.  See the comment in
+ *      gauche/number.h for the details.
  *
  * [Character]
- *      -------- -------- -------- -----010
- *      29-bit
+ *      -------- -------- -------- 00000011
+ *      24-bit.  20bits are enough to cover all UCS, but we
+ *      reserve a few extra bits for possible future extension.
  *
  * [Miscellaneous]
- *      -------- -------- -------- ----0110
+ *      -------- -------- -------- 00001011
  *      #f, #t, '(), eof-object, undefined
  *
  * [Pattern variable]
- *      -------- -------- -------- ----1110
+ *      -------- -------- -------- 00010011
  *      Used in macro expander.
  *
  * [Heap object]
- *      -------- -------- -------- ------11
+ *      -------- -------- -------- -----111
  *      Only appears at the first word of heap-allocated
- *      objects except pairs.   Masking lower 2bits gives
- *      a pointer to ScmClass.  
+ *      objects except pairs and flonums.   Masking lower
+ *      3bits gives a pointer to ScmClass.  
  */
 
 /* Type coercer */
@@ -193,17 +223,31 @@ typedef struct ScmClassRec ScmClass;
  * PRIMARY TAG IDENTIFICATION
  */
 
-#define	SCM_TAG(obj)     (SCM_WORD(obj) & 0x03)
-#define SCM_PTRP(obj)    (SCM_TAG(obj) == 0)
+#define SCM_TAG1(obj)    (SCM_WORD(obj) & 0x01)
+#define SCM_TAG2(obj)    (SCM_WORD(obj) & 0x03)
+#define SCM_TAG3(obj)    (SCM_WORD(obj) & 0x07)
+#define SCM_TAG8(obj)    (SCM_WORD(obj) & 0xff)
+
+/* Check if the ScmObj is a 'pointer'---either to a pair,
+   a heap object, or a ScmFlonum. */
+#define SCM_PTRP(obj)    (SCM_TAG1(obj) == 0)
+
+/* Check if the ScmObj is a pointer to either a pair or a heap
+   (That is, we can safely take SCM_OBJ(obj)->tag) */
+#define SCM_HPTRP(obj)   (SCM_TAG2(obj) == 0)
+
+/* This macro further takes the lower three bits of the word pointed
+   by OBJ, to distinguish whether it's a pair or a heap object. */
+#define SCM_HTAG(obj)    (SCM_WORD(SCM_OBJ(obj)->tag)&7)
 
 /*
  * IMMEDIATE OBJECTS
  */
 
-#define SCM_IMMEDIATEP(obj) ((SCM_WORD(obj)&0x0f) == 6)
-#define SCM_ITAG(obj)       (SCM_WORD(obj)>>4)
+#define SCM_IMMEDIATEP(obj) (SCM_TAG8(obj) == 0x0b)
+#define SCM_ITAG(obj)       (SCM_WORD(obj)>>8)
 
-#define SCM__MAKE_ITAG(num)  (((num)<<4) + 6)
+#define SCM__MAKE_ITAG(num)  (((num)<<8) + 0x0b)
 #define SCM_FALSE           SCM_OBJ(SCM__MAKE_ITAG(0)) /* #f */
 #define SCM_TRUE            SCM_OBJ(SCM__MAKE_ITAG(1)) /* #t  */
 #define SCM_NIL             SCM_OBJ(SCM__MAKE_ITAG(2)) /* '() */
@@ -244,12 +288,24 @@ SCM_EXTERN int Scm_EqualM(ScmObj x, ScmObj y, int mode);
  * FIXNUM
  */
 
-#define SCM_INTP(obj)        (SCM_TAG(obj) == 1)
+#define SCM_INTP(obj)        (SCM_TAG2(obj) == 1)
 #define SCM_INT_VALUE(obj)   (((signed long int)SCM_WORD(obj)) >> 2)
 #define SCM_MAKE_INT(obj)    SCM_OBJ(((intptr_t)(obj) << 2) + 1)
 
 #define SCM_UINTP(obj)       (SCM_INTP(obj)&&((signed long int)SCM_WORD(obj)>=0))
 
+/*
+ * FLONUM
+ */
+
+typedef struct ScmFlonumRec {
+    double val;
+} ScmFlonum SCM_ALIGN8;
+
+#define SCM_FLONUM(obj)            ((ScmFlonum*)(SCM_WORD(obj)&~0x07))
+#define SCM_FLONUMP(obj)           (SCM_TAG2(obj) == 2)
+#define SCM_FLONUM_VALUE(obj)      (SCM_FLONUM(obj)->val)
+
 /*
  * CHARACTERS
  *
@@ -260,18 +316,20 @@ SCM_EXTERN int Scm_EqualM(ScmObj x, ScmObj y, int mode);
  */
 
 #define	SCM_CHAR(obj)           ((ScmChar)(obj))
-#define	SCM_CHARP(obj)          ((SCM_WORD(obj)&0x07L) == 2)
-#define	SCM_CHAR_VALUE(obj)     SCM_CHAR(SCM_WORD(obj) >> 3)
-#define	SCM_MAKE_CHAR(ch)       SCM_OBJ(SCM_WORD((ch) << 3) + 2)
+#define	SCM_CHARP(obj)          ((SCM_WORD(obj)&0xff) == 3)
+#define	SCM_CHAR_VALUE(obj)     SCM_CHAR(((unsigned long)SCM_WORD(obj)) >> 8)
+#define	SCM_MAKE_CHAR(ch)       SCM_OBJ((long)((ch) << 8) + 3)
 
 #define SCM_CHAR_INVALID        ((ScmChar)(-1)) /* indicate invalid char */
-#define SCM_CHAR_MAX            (0x1fffffff)
+#define SCM_CHAR_MAX            (0xffffff)
 
 #define SCM_CHAR_ASCII_P(ch)    ((ch) < 0x80)
-#define SCM_CHAR_UPPER_P(ch)    (('A' <= (ch)) && ((ch) <= 'Z'))
-#define SCM_CHAR_LOWER_P(ch)    (('a' <= (ch)) && ((ch) <= 'z'))
-#define SCM_CHAR_UPCASE(ch)     (SCM_CHAR_LOWER_P(ch)?((ch)-('a'-'A')):(ch))
-#define SCM_CHAR_DOWNCASE(ch)   (SCM_CHAR_UPPER_P(ch)?((ch)+('a'-'A')):(ch))
+
+/* The following four macros are obsoleted; use API version instead.*/
+#define SCM_CHAR_UPPER_P(ch)    Scm_CharUppercaseP(ch)
+#define SCM_CHAR_LOWER_P(ch)    Scm_CharLowercaseP(ch)
+#define SCM_CHAR_UPCASE(ch)     Scm_CharUpcase(ch)
+#define SCM_CHAR_DOWNCASE(ch)   Scm_CharDowncase(ch)
 
 SCM_EXTERN int Scm_DigitToInt(ScmChar ch, int radix);
 SCM_EXTERN ScmChar Scm_IntToDigit(int n, int radix);
@@ -281,6 +339,18 @@ SCM_EXTERN ScmObj Scm_CharEncodingName(void);
 SCM_EXTERN const char **Scm_SupportedCharacterEncodings(void);
 SCM_EXTERN int Scm_SupportedCharacterEncodingP(const char *encoding);
 
+SCM_EXTERN int Scm_CharGeneralCategory(ScmChar ch);
+SCM_EXTERN int Scm_CharAlphabeticP(ScmChar ch);
+SCM_EXTERN int Scm_CharUppercaseP(ScmChar ch);
+SCM_EXTERN int Scm_CharLowercaseP(ScmChar ch);
+SCM_EXTERN int Scm_CharTitlecaseP(ScmChar ch);
+SCM_EXTERN int Scm_CharNumericP(ScmChar ch);
+
+SCM_EXTERN ScmChar Scm_CharUpcase(ScmChar ch);
+SCM_EXTERN ScmChar Scm_CharDowncase(ScmChar ch);
+SCM_EXTERN ScmChar Scm_CharTitlecase(ScmChar ch);
+SCM_EXTERN ScmChar Scm_CharFoldcase(ScmChar ch);
+
 SCM_EXTERN void Scm__InstallCharconvHooks(ScmChar (*u2c)(int),
                                           int (*c2u)(ScmChar));
 
@@ -298,18 +368,21 @@ SCM_EXTERN void Scm__InstallCharconvHooks(ScmChar (*u2c)(int),
  * HEAP ALLOCATED OBJECTS
  *
  *  A heap allocated object has its class tag in the first word
- *  (except pairs).  Masking the lower two bits of class tag
+ *  (except pairs).  Masking the lower three bits of class tag
  *  gives a pointer to the class object.
  */
 
-#define SCM_HOBJP(obj)  (SCM_PTRP(obj)&&SCM_TAG(SCM_OBJ(obj)->tag)==3)
+#define SCM_HOBJP(obj)  (SCM_HPTRP(obj)&&(SCM_HTAG(obj)==7))
 
 #define SCM_CPP_CAT(a, b)   a##b
 #define SCM_CPP_CAT3(a, b, c)  a ## b ## c
 
 /* We use a pointer to the class structure (with low-bit tag) as
-   the generic type tag. */
-#define SCM_CLASS2TAG(klass)  ((ScmByte*)(klass) + 3)
+   the generic type tag.   NB: The ScmClass structure is always
+   aligned on 8-byte boundary, so +7 makes the tag's lower
+   3 bits '111'.  Such pattern never appears in tagged pointer,
+   so we can distinguish heap allocated objects from ScmPair.  */
+#define SCM_CLASS2TAG(klass)  ((ScmByte*)(klass) + 7)
 
 /* A common header for heap-allocated objects */
 typedef struct ScmHeaderRec {
@@ -321,8 +394,9 @@ typedef struct ScmHeaderRec {
 #define SCM_HEADER       ScmHeader hdr /* for declaration */
 
 /* Here comes the ugly part.  To understand the general idea, just ignore
-   GAUCHE_BROKEN_LINKER_WORKAROUND part; it's pretty simple.  Every heap
-   allocated object contains (pointer to its class + 3) in its tag field.  */
+   GAUCHE_BROKEN_LINKER_WORKAROUND part; except that, it's pretty simple.
+   Every heap allocated object contains (pointer to its class + 7) in its
+   tag field.  */
 #if !defined(GAUCHE_BROKEN_LINKER_WORKAROUND)
 
 # define SCM_CLASS_DECL(klass) extern ScmClass klass
@@ -331,26 +405,26 @@ typedef struct ScmHeaderRec {
 
 /* Extract the class pointer from the tag.
    You can use these only if SCM_HOBJP(obj) != FALSE */
-# define SCM_CLASS_OF(obj)      SCM_CLASS((SCM_OBJ(obj)->tag - 3))
-# define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)(k) + 3)
+# define SCM_CLASS_OF(obj)      SCM_CLASS((SCM_OBJ(obj)->tag - 7))
+# define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)(k) + 7)
 
 /* Check if classof(OBJ) equals to an extended class KLASS.
-   We can check SCM_PTRP instead of SCM_HOBJP here, since a pair never
+   We can check SCM_HPTRP instead of SCM_HOBJP here, since a pair never
    satisfies the second test. */
 # define SCM_XTYPEP(obj, klass) \
-    (SCM_PTRP(obj)&&(SCM_OBJ(obj)->tag == SCM_CLASS2TAG(klass)))
+    (SCM_HPTRP(obj)&&(SCM_OBJ(obj)->tag == SCM_CLASS2TAG(klass)))
 
 #else  /*GAUCHE_BROKEN_LINKER_WORKAROUND*/
 
 /* You don't want to understand these. */
 # define SCM_CLASS_DECL(klass) \
-    SCM_EXTERN ScmClass klass;                  \
+    SCM_EXTERN ScmClass klass; \
     extern ScmClass *SCM_CPP_CAT(_imp__, klass) 
 # define SCM_CLASS_STATIC_PTR(klass) ((ScmClass*)(&SCM_CPP_CAT(_imp__,klass)))
 # define SCM_CLASS_STATIC_TAG(klass) SCM_CLASS2TAG(SCM_CLASS_STATIC_PTR(klass))
 
-# define SCM_CLASS_OF(obj)      (*(ScmClass**)((SCM_OBJ(obj)->tag - 3)))
-# define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)((k)->classPtr) + 3)
+# define SCM_CLASS_OF(obj)      (*(ScmClass**)((SCM_OBJ(obj)->tag - 7)))
+# define SCM_SET_CLASS(obj, k)  (SCM_OBJ(obj)->tag = (ScmByte*)((k)->classPtr) + 7)
 
 # define SCM_XTYPEP(obj, klass) \
     (SCM_HOBJP(obj)&&(SCM_CLASS_OF(obj) == klass))
@@ -400,7 +474,6 @@ typedef struct ScmDStringRec   ScmDString;
 typedef struct ScmVectorRec    ScmVector;
 typedef struct ScmBignumRec    ScmBignum;
 typedef struct ScmRatnumRec    ScmRatnum;
-typedef struct ScmFlonumRec    ScmFlonum;
 typedef struct ScmCompnumRec   ScmCompnum;
 typedef struct ScmPortRec      ScmPort;
 typedef struct ScmHashTableRec ScmHashTable;
@@ -457,17 +530,17 @@ typedef struct ScmEvalPacketRec {
     ScmModule *module;          /* 'Current module' after evaluation */
 } ScmEvalPacket;
 
-#if defined(GAUCHE_API_0_8_8) || defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
+#if !defined(GAUCHE_API_PRE_0_9)
 SCM_EXTERN int Scm_Eval(ScmObj form, ScmObj env, ScmEvalPacket *packet);
 SCM_EXTERN int Scm_EvalCString(const char *form, ScmObj env,
                                ScmEvalPacket *packet);
 SCM_EXTERN int Scm_Apply(ScmObj proc, ScmObj args,
                          ScmEvalPacket *packet);
-#else  /*!GAUCHE_API_0_8_8*/
+#else  /*GAUCHE_API_PRE_0_9*/
 #define Scm_Eval(f, e)        Scm_EvalRec(f, e)
-#define Scm_EvalCString(f, e) Scm_EvalCString(f, e)
+#define Scm_EvalCString(f, e) Scm_EvalCStringRec(f, e)
 #define Scm_Apply(p, a)       Scm_ApplyRec(p, a)
-#endif /*!GAUCHE_API_0_8_8*/
+#endif /*GAUCHE_API_PRE_0_9*/
 
 /* Calls VM recursively to evaluate the Scheme code.  These
    ones does not capture exceptions. */
@@ -509,6 +582,7 @@ SCM_EXTERN ScmObj Scm_VMEval(ScmObj expr, ScmObj env);
 SCM_EXTERN ScmObj Scm_VMCall(ScmObj *args, int argcnt, void *data);
 
 SCM_EXTERN ScmObj Scm_VMCallCC(ScmObj proc);
+SCM_EXTERN ScmObj Scm_VMCallPC(ScmObj proc);
 SCM_EXTERN ScmObj Scm_VMDynamicWind(ScmObj pre, ScmObj body, ScmObj post);
 SCM_EXTERN ScmObj Scm_VMDynamicWindC(ScmSubrProc *before,
                                      ScmSubrProc *body,
@@ -532,6 +606,22 @@ SCM_EXTERN ScmObj Scm_VMGetResult(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStackLite(ScmVM *vm);
 SCM_EXTERN ScmObj Scm_VMGetStack(ScmVM *vm);
 
+/* A box is to keep a reference.  It is mainly used for mutable local variables.
+ */
+
+typedef struct ScmBoxRec {
+    SCM_HEADER;
+    ScmObj value;
+} ScmBox;
+
+SCM_CLASS_DECL(Scm_BoxClass);
+#define SCM_CLASS_BOX            (&Scm_BoxClass)
+#define SCM_BOX(obj)             ((ScmBox*)(obj))
+#define SCM_BOXP(obj)            (SCM_XTYPEP(obj, SCM_CLASS_BOX))
+#define SCM_BOX_VALUE(obj)       (SCM_BOX(obj)->value)
+#define SCM_BOX_SET(obj, val)    (SCM_BOX(obj)->value = (val))
+
+SCM_EXTERN ScmBox *Scm_MakeBox(ScmObj value);
 
 /*---------------------------------------------------------
  * CLASS
@@ -549,9 +639,15 @@ typedef ScmObj (*ScmClassAllocateProc)(ScmClass *klass, ScmObj initargs);
 /* See class.c for the description of function pointer members.
    There's a lot of voodoo magic in class structure, so don't touch
    those fields casually.  Also, the order of these fields must be
-   reflected to the class definition macros below */
+   reflected to the class definition macros below. */
 struct ScmClassRec {
-    SCM_INSTANCE_HEADER;
+    /* A trick to align statically allocated class structure on 8-byte
+       boundary.  This doesn't guarantee, though, so we use __alignment__
+       attribute as well, whenever possible (see SCM_ALIGN8 macro). */
+    union {
+        SCM_INSTANCE_HEADER;
+        double align_dummy;
+    } classHdr;
 #if defined(GAUCHE_BROKEN_LINKER_WORKAROUND)
     ScmClass **classPtr;
 #endif
@@ -583,14 +679,17 @@ struct ScmClassRec {
                                    otherwise #f */
     ScmInternalMutex mutex;     /* to protect from MT hazard */
     ScmInternalCond cv;         /* wait on this while a class being updated */
-    void   *data;               /* extra data to do nasty trick */
-};
+    void   *data;               /* extra data to do nasty trick.  See the note
+                                   in class.c */
+} SCM_ALIGN8;
 
 typedef struct ScmClassStaticSlotSpecRec ScmClassStaticSlotSpec;
 
 #define SCM_CLASS(obj)        ((ScmClass*)(obj))
 #define SCM_CLASSP(obj)       SCM_ISA(obj, SCM_CLASS_CLASS)
 
+#define SCM_CLASS_NUM_INSTANCE_SLOTS(obj)  SCM_CLASS(obj)->numInstanceSlots
+
 /* Class categories
 
    In C level, there are four categories of classes.  The category of
@@ -598,11 +697,10 @@ typedef struct ScmClassStaticSlotSpecRec ScmClassStaticSlotSpec;
 
    SCM_CLASS_BUILTIN
        An instance of this class doesn't have "slots" member (thus
-       cannot be casted to ScmInstance).   From Scheme level, this
-       class cannot be inherited, nor redefined.  In C you can create
-       subclasses, by making sure the subclass' instance structure
-       to include this class's instance structure.  Such "hard-wired"
-       inheritance only forms a tree, i.e. no multiple inheritance.
+       cannot be cast to ScmInstance*).   From Scheme level, this
+       class cannot be redefined.   It cannot be inherited in Scheme
+       code with the standard inheritance mechanism; though it can have
+       subclasses, provided a special allocator and initializer.
 
    SCM_CLASS_ABSTRACT 
        This class is defined in C, but doesn't allowed to create an
@@ -613,15 +711,37 @@ typedef struct ScmClassStaticSlotSpecRec ScmClassStaticSlotSpec;
    SCM_CLASS_BASE
        This class is defined in C, and can be subclassed in Scheme.
        An instance of this class must have "slots" member and be
-       able to be casted to ScmInstance.  The instance may have other
+       able to be cast to ScmInstance.  The instance may have other
        C members.  This class cannot be redefined.
 
    SCM_CLASS_SCHEME
-       A Scheme-defined class.  This class should have at most one
-       SCM_CLASS_BASE class in its CPL, except the <object> class,
-       which is always in the CPL of Scheme-defined class.  All other
-       classes in CPL must be either SCM_CLASS_ABSTRACT or
-       SCM_CLASS_SCHEME.  This class can be redefined.
+       A Scheme-defined class.  This class will have one or more
+       SCM_CLASS_BASE classes in its CPL.  Specifically, <object>
+       class is always included in its CPL.  This class can be
+       redefined.
+
+   This classification and its rules are to integrate C structures
+   and Scheme classes.   C structure level inheritance has to be
+   single-inheritance, with the subclass structure including its
+   parent structure.  Scheme level inheritance is more flexible,
+   but for that flexibility it has to have "slots" member in its
+   instance (i.e. it has to be castable to ScmInstance*).
+
+   Here's the basic inheritance rules:
+                   
+   - First, ABSTRACT class can be inserted at any place in the
+     inheritance chain.  It doesn't affect C-level operation.  It is
+     only to add the type information in Scheme-level.
+     In the following rules we ignore ABSTRACT classes.
+
+   - BASE class can be inherited from BASE classes, and its
+     inheritance chain must form a single inheritance.
+
+   - BUILTIN class can be inherited from BUILTIN classes, and
+     its inheritance chain must form a single inheritance
+     
+   - SCHEME class can be inherited from SCHEME or BASE classes.
+     It can inherite from multiple SCHEME and/or BASE classes.
 */
 
 enum {
@@ -632,13 +752,23 @@ enum {
 
     /* A special flag that only be used for "natively applicable"
        objects, which basically inherits ScmProcedure. */
-    SCM_CLASS_APPLICABLE = 0x04
+    SCM_CLASS_APPLICABLE = 0x04,
+
+    /* If this flag is set, important slots such as class-precedence-list
+       or class-slots becomes settable.
+       We reset this flag at the end of class initialization, so that
+       we can avoid the behavior of a class from being accidentally
+       chnaged.  The flag may be set during updating a class metaobject
+       triggered by metaclass change (see lib/gauche/redefutil.scm).
+     */
+    SCM_CLASS_MALLEABLE = 0x08
 };
 
-#define SCM_CLASS_FLAGS(obj)     (SCM_CLASS(obj)->flags)
+#define SCM_CLASS_FLAGS(obj)        (SCM_CLASS(obj)->flags)
 #define SCM_CLASS_APPLICABLE_P(obj) (SCM_CLASS_FLAGS(obj)&SCM_CLASS_APPLICABLE)
 
-#define SCM_CLASS_CATEGORY(obj)  (SCM_CLASS_FLAGS(obj)&3)
+#define SCM_CLASS_CATEGORY(obj)     (SCM_CLASS_FLAGS(obj)&3)
+#define SCM_CLASS_MALLEABLE_P(obj)  (SCM_CLASS_FLAGS(obj)&SCM_CLASS_MALLEABLE)
 
 SCM_EXTERN void Scm_InitStaticClass(ScmClass *klass, const char *name,
                                     ScmModule *mod,
@@ -669,10 +799,13 @@ SCM_EXTERN int Scm_SubtypeP(ScmClass *sub, ScmClass *type);
 SCM_EXTERN int Scm_TypeP(ScmObj obj, ScmClass *type);
 SCM_EXTERN ScmClass *Scm_BaseClassOf(ScmClass *klass);
 
+SCM_EXTERN void   Scm_ClassMalleableSet(ScmClass *klass, int flag);
+
 SCM_EXTERN ScmObj Scm_VMSlotRef(ScmObj obj, ScmObj slot, int boundp);
 SCM_EXTERN ScmObj Scm_VMSlotSet(ScmObj obj, ScmObj slot, ScmObj value);
 SCM_EXTERN ScmObj Scm_VMSlotBoundP(ScmObj obj, ScmObj slot);
 
+
 /* built-in classes */
 SCM_CLASS_DECL(Scm_TopClass);
 SCM_CLASS_DECL(Scm_BoolClass);
@@ -722,7 +855,7 @@ extern ScmClass *Scm_ObjectCPL[];
 
 #define SCM__DEFINE_CLASS_COMMON(cname, coreSize, flag, printer, compare, serialize, allocate, cpa) \
     ScmClass cname = {                           \
-        { SCM_CLASS_STATIC_TAG(Scm_ClassClass), NULL },\
+        {{ SCM_CLASS_STATIC_TAG(Scm_ClassClass), NULL }},       \
         SCM__CLASS_PTR_SLOT(cname)               \
         printer,                                 \
         compare,                                 \
@@ -856,7 +989,7 @@ struct ScmExtendedPairRec {
     ScmObj attributes;          /* should be accessed via API func. */
 };
 
-#define SCM_PAIRP(obj)  (SCM_PTRP(obj)&&SCM_TAG(SCM_OBJ(obj)->tag)!=0x03)
+#define SCM_PAIRP(obj)          (SCM_HPTRP(obj)&&SCM_HTAG(obj)!=7)
 
 #define SCM_PAIR(obj)           ((ScmPair*)(obj))
 #define SCM_CAR(obj)            (SCM_PAIR(obj)->car)
@@ -935,6 +1068,7 @@ SCM_EXTERN ScmObj Scm_Conses(ScmObj elt, ...);
 SCM_EXTERN ScmObj Scm_VaList(va_list elts);
 SCM_EXTERN ScmObj Scm_VaCons(va_list elts);
 SCM_EXTERN ScmObj Scm_ArrayToList(ScmObj *elts, int nelts);
+SCM_EXTERN ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail);
 SCM_EXTERN ScmObj *Scm_ListToArray(ScmObj list, int *nelts, ScmObj *store,
 				   int alloc);
 
@@ -953,6 +1087,8 @@ SCM_EXTERN ScmObj Scm_Append2(ScmObj list, ScmObj obj);
 SCM_EXTERN ScmObj Scm_Append(ScmObj args);
 SCM_EXTERN ScmObj Scm_ReverseX(ScmObj list);
 SCM_EXTERN ScmObj Scm_Reverse(ScmObj list);
+SCM_EXTERN ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail);
+SCM_EXTERN ScmObj Scm_Reverse2(ScmObj list, ScmObj tail);
 SCM_EXTERN ScmObj Scm_ListTail(ScmObj list, int i, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_ListRef(ScmObj list, int i, ScmObj fallback);
 SCM_EXTERN ScmObj Scm_LastPair(ScmObj list);
@@ -973,6 +1109,7 @@ SCM_EXTERN ScmObj Scm_DeleteDuplicates(ScmObj list, int cmpmode);
 SCM_EXTERN ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode);
 
 SCM_EXTERN ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences);
+SCM_EXTERN ScmObj Scm_MonotonicMerge1(ScmObj sequences);
 SCM_EXTERN ScmObj Scm_Union(ScmObj list1, ScmObj list2);
 SCM_EXTERN ScmObj Scm_Intersection(ScmObj list1, ScmObj list2);
 
@@ -1099,88 +1236,7 @@ SCM_EXTERN ScmObj Scm_Vsprintf(const char *fmt, va_list args, int sharedp);
  * MODULE
  */
 
-struct ScmModuleRec {
-    SCM_HEADER;
-    ScmSymbol *name;
-    ScmObj imported;            /* list of imported modules */
-    ScmObj exported;            /* list of exported symbols */
-    int    exportAll;           /* TRUE if (export-all) */
-    ScmObj parents;             /* direct parent modules */
-    ScmObj mpl;                 /* module precedence list */
-    ScmObj depended;            /* list of modules that are depended by this
-                                   module for compilation */
-    ScmHashTable *table;        /* binding table */
-};
-
-#define SCM_MODULE(obj)       ((ScmModule*)(obj))
-#define SCM_MODULEP(obj)      SCM_XTYPEP(obj, SCM_CLASS_MODULE)
-
-SCM_CLASS_DECL(Scm_ModuleClass);
-#define SCM_CLASS_MODULE     (&Scm_ModuleClass)
-
-SCM_EXTERN ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists);
-
-/* Flags for Scm_FindBinding and Scm_GlobalVariableRef */
-enum {
-    /* do not search parent/imported */
-    SCM_BINDING_STAY_IN_MODULE = (1L<<0)
-};
-
-SCM_EXTERN ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
-				    int flags);
-SCM_EXTERN ScmObj Scm_GlobalVariableRef(ScmModule *module,
-                                        ScmSymbol *symbol,
-                                        int flags);
-
-SCM_EXTERN ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol,
-			     ScmObj value);
-SCM_EXTERN ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol,
-                                  ScmObj value);
-
-SCM_EXTERN ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers);
-SCM_EXTERN ScmObj Scm_ImportModules(ScmModule *module, ScmObj list);
-SCM_EXTERN ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list);
-SCM_EXTERN ScmObj Scm_ExportAll(ScmModule *module);
-SCM_EXTERN ScmModule *Scm_FindModule(ScmSymbol *name, int flags);
-SCM_EXTERN ScmObj Scm_AllModules(void);
-SCM_EXTERN void   Scm_SelectModule(ScmModule *mod);
-
-/* Flags for Scm_FindModule
-   NB: Scm_FindModule's second arg has been changed since 0.8.6;
-   before, it was just a boolean value to indicate whether a new
-   module should be created (TRUE) or not (FALSE).  We added a
-   new flag value to make Scm_FindModule raises an error if the named
-   module doesn't exist.  This change should be transparent as far
-   as the caller's using Gauche's definition of TRUE. */
-enum {
-    SCM_FIND_MODULE_CREATE = 1, /* Create if there's no named module */
-    SCM_FIND_MODULE_QUIET  = 2  /* Do not signal an error if there's no
-                                   named module, but return NULL instead. */
-};
-
-#define SCM_FIND_MODULE(name, flags) \
-    Scm_FindModule(SCM_SYMBOL(SCM_INTERN(name)), flags)
-
-SCM_EXTERN ScmObj Scm_ModuleNameToPath(ScmSymbol *name);
-SCM_EXTERN ScmObj Scm_PathToModuleName(ScmString *path);
-
-SCM_EXTERN ScmModule *Scm_NullModule(void);
-SCM_EXTERN ScmModule *Scm_SchemeModule(void);
-SCM_EXTERN ScmModule *Scm_GaucheModule(void);
-SCM_EXTERN ScmModule *Scm_UserModule(void);
-SCM_EXTERN ScmModule *Scm_CurrentModule(void);
-
-#define SCM_DEFINE(module, cstr, val)           \
-    Scm_Define(SCM_MODULE(module),              \
-               SCM_SYMBOL(SCM_INTERN(cstr)),    \
-               SCM_OBJ(val))
-
-/* OBSOLETED */
-#define Scm_SymbolValue(m, s) Scm_GlobalVariableRef(m, s, FALSE)
-/* OBSOLETED */
-#define SCM_SYMBOL_VALUE(module_name, symbol_name)                      \
-    Scm_SymbolValue(SCM_FIND_MODULE(module_name, 0),                    \
-                    SCM_SYMBOL(SCM_INTERN(symbol_name)))
+#include <gauche/module.h>
 
 /*--------------------------------------------------------
  * SYMBOL
@@ -1217,17 +1273,66 @@ typedef ScmObj (*ScmTransformerProc)(ScmObj self, ScmObj form, ScmObj env,
 /* Base structure */
 struct ScmProcedureRec {
     SCM_INSTANCE_HEADER;
-    unsigned int required : 10;    /* # of required args */
-    unsigned int optional : 1;     /* 1 if it takes rest args */
+    unsigned int required : 16;    /* # of required args */
+    unsigned int optional : 8;     /* >=1 if it takes opt args. see below.*/
     unsigned int type     : 3;     /* ScmProcedureType */
     unsigned int locked   : 1;     /* setter locked? */
     unsigned int currying : 1;     /* autocurrying */
+    unsigned int constant : 1;     /* constant procedure. see below. */
+    unsigned int reserved : 2;     /* unused yet. */
     ScmObj info;                   /* source code info */
     ScmObj setter;                 /* setter, if exists. */
-    ScmObj inliner;                /* inliner.  NB: for backward compatibility,
-                                      this may be initialized by NULL. */
+    ScmObj inliner;                /* inliner information.  see below. */
 };
 
+/* About optional slot:
+   If this slot is non-zero, the procedure takes optional arguments.
+   For Standard Scheme procedures with 'rest' arguments, this slot is 1
+   and all excessive arguments are 'folded' in a list.
+
+   This slot may have a value more than 1.  If it is N (>1), then up to N-1
+   optional arguments are passed wihtout being folded (that is, passed
+   'on the stack'.  Only when the given argument is more than or equal to
+   N + reqargs, the excessive arguments are folded and passed in a list.
+   Thus, such procedure may get between reqargs values and N+reqargs values
+   after folding (NB: Fixed argument procedure always get regargs values,
+   and standard Scheme variable argument procedure always get reqargs+1 values
+   after argument folding).
+
+   This special treatment is to avoid unnecessary consing of argumets;
+   if we know the callee immeidately unfolds the rest argument, it's no 
+   use to fold excessive arguments anyway.
+ */
+
+/* About 'constant' flag:
+   This flag being TRUE means this procedure returns the same constant
+   value if given same constant arguments, and it does not have any other
+   external effects.   The compiler may use this info to replace a call
+   of this proc with the resulting value, if all the arguments are known
+   at compile-time.  The resulting value must be serializable to the
+   precompiled file.  The result shouldn't be affected
+   by the timing of the compile, architecture on which the compiler runs,
+   or the compiler configuration (e.g. internal encoding).
+ */
+
+/* About procedure inliner:
+   This slot holds information to inline procedures.  The value of this slot
+   can be one of the following kinds:
+   
+   #f: No inliner associated to this procedure.  (For historical
+      reasons, the code that access to this slot expects this slot can be
+      NULL and treats it as SCM_FALSE)
+
+   <integer>: Only appears in some built-in procedures, and specifies
+      the VM instruction number.  This should be considered as a special
+      hack.   The set of procedures that can have this type of inliner
+      is tied to the VM definition.
+
+   <vector>: Procedures defined with define-inline have this.  The vector
+      encodes intermediate form (IForm) of the procedure code, which will be
+      expanded into the caller.
+ */
+
 /* procedure type */
 enum ScmProcedureType {
     SCM_PROC_SUBR,
@@ -1264,12 +1369,15 @@ SCM_CLASS_DECL(Scm_ProcedureClass);
     SCM_PROCEDURE(obj)->type = typ,                     \
     SCM_PROCEDURE(obj)->locked = FALSE,                 \
     SCM_PROCEDURE(obj)->currying = FALSE,               \
+    SCM_PROCEDURE(obj)->constant = FALSE,               \
+    SCM_PROCEDURE(obj)->reserved = 0,                   \
     SCM_PROCEDURE(obj)->info = inf,                     \
     SCM_PROCEDURE(obj)->setter = SCM_FALSE,             \
     SCM_PROCEDURE(obj)->inliner = SCM_FALSE
 
-#define SCM__PROCEDURE_INITIALIZER(klass, req, opt, typ, inf, inl)  \
-    { { klass }, (req), (opt), (typ), FALSE, FALSE, (inf), SCM_FALSE, (inl) }
+#define SCM__PROCEDURE_INITIALIZER(klass, req, opt, typ, cst, inf, inl)  \
+    { { klass }, (req), (opt), (typ), FALSE, FALSE, cst, 0,              \
+      (inf), SCM_FALSE, (inl) }
 
 SCM_EXTERN ScmObj Scm_CurryProcedure(ScmObj proc, ScmObj *given,
                                      int ngiven, int foldlen);
@@ -1290,6 +1398,7 @@ SCM_EXTERN ScmObj Scm_MakeClosure(ScmObj code, ScmEnvFrame *env);
 /* Subr - C defined procedure */
 struct ScmSubrRec {
     ScmProcedure common;
+    int flags;
     ScmSubrProc *func;
     void *data;
 };
@@ -1297,16 +1406,35 @@ struct ScmSubrRec {
 #define SCM_SUBRP(obj) \
     (SCM_PROCEDUREP(obj)&&(SCM_PROCEDURE_TYPE(obj)==SCM_PROC_SUBR))
 #define SCM_SUBR(obj)              ((ScmSubr*)(obj))
+#define SCM_SUBR_FLAGS(obj)        SCM_SUBR(obj)->flags
 #define SCM_SUBR_FUNC(obj)         SCM_SUBR(obj)->func
 #define SCM_SUBR_DATA(obj)         SCM_SUBR(obj)->data
 
-#define SCM_DEFINE_SUBR(cvar, req, opt, inf, func, inliner, data)           \
+/* flags */
+#define SCM_SUBR_IMMEDIATE_ARG  (1L<<0) /* This subr will not retain a reference
+                                           to the flonums given to args.  VM
+                                           can safely pass the register flonums
+                                           to the subr.  This is added when
+                                           the :fast-flonum flag is given to
+                                           define-cproc. */
+
+#define SCM__DEFINE_SUBR_INT(cvar, req, opt, cst, inf, flags, func, inliner, data) \
     ScmSubr cvar = {                                                        \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_ProcedureClass),\
-                                   req, opt, SCM_PROC_SUBR, inf, inliner),  \
-        (func), (data)                                                      \
+            req, opt, SCM_PROC_SUBR, cst, inf, inliner),                    \
+        flags, (func), (data)                                               \
     }
 
+#define SCM_DEFINE_SUBR(cvar, req, opt, inf, func, inliner, data) \
+    SCM__DEFINE_SUBR_INT(cvar, req, opt, 0, inf, 0, func, inliner, data)
+#define SCM_DEFINE_SUBRX(cvar, req, opt, cst, inf, flags, func, inliner, data) \
+    SCM__DEFINE_SUBR_INT(cvar, req, opt, cst, inf, flags, func, inliner, data)
+
+/* old interface.  will be gone. */
+#define SCM_DEFINE_SUBRI(cvar, req, opt, inf, func, inliner, data) \
+    SCM__DEFINE_SUBR_INT(cvar, req, opt, inf, SCM_SUBR_IMMEDIATE_ARG, \
+                         func, inliner, data)
+
 SCM_EXTERN ScmObj Scm_MakeSubr(ScmSubrProc *func,
 			       void *data,
 			       int required, int optional,
@@ -1338,8 +1466,8 @@ SCM_CLASS_DECL(Scm_GenericClass);
 #define SCM_DEFINE_GENERIC(cvar, cfunc, data)                           \
     ScmGeneric cvar = {                                                 \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_GenericClass),\
-                                   0, 0, SCM_PROC_GENERIC, SCM_FALSE,   \
-                                   NULL),                               \
+                                   0, 0, SCM_PROC_GENERIC, 0,           \
+                                   SCM_FALSE, NULL),                    \
         SCM_NIL, 0, cfunc, data                                         \
     }
 
@@ -1373,7 +1501,7 @@ SCM_CLASS_DECL(Scm_MethodClass);
 #define SCM_DEFINE_METHOD(cvar, gf, req, opt, specs, func, data)        \
     ScmMethod cvar = {                                                  \
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_MethodClass),\
-                                   req, opt, SCM_PROC_METHOD,           \
+                                   req, opt, SCM_PROC_METHOD, 0,        \
                                    SCM_FALSE, NULL),                    \
         gf, specs, func, data, NULL                                     \
     }
@@ -1624,6 +1752,9 @@ SCM_EXTERN void Scm_Exit(int code);
 SCM_EXTERN void Scm_Abort(const char *msg);
 SCM_EXTERN void Scm_Panic(const char *msg, ...);
 
+SCM_EXTERN void Scm_SimpleMain(int argc, const char *argv[],
+                               const char *script, u_long flags);
+
 SCM_EXTERN void Scm_GC(void);
 SCM_EXTERN void Scm_PrintStaticRoots(void);
 SCM_EXTERN void Scm_RegisterDL(void *data_start, void *data_end,
@@ -1647,6 +1778,7 @@ SCM_EXTERN ScmObj Scm_LibraryDirectory(void);
 SCM_EXTERN ScmObj Scm_ArchitectureDirectory(void);
 SCM_EXTERN ScmObj Scm_SiteLibraryDirectory(void);
 SCM_EXTERN ScmObj Scm_SiteArchitectureDirectory(void);
+SCM_EXTERN ScmObj Scm__RuntimeDirectory(void); /* private */
 
 /* Compare and Sort */
 SCM_EXTERN int Scm_Compare(ScmObj x, ScmObj y);
@@ -1682,6 +1814,7 @@ SCM_EXTERN ScmObj Scm_SortListX(ScmObj objs, ScmObj fn);
 
 #endif /* !GAUCHE_RECKLESS */
 
+#include <gauche/scmconst.h>
 
 SCM_DECL_END
 
