@@ -1,7 +1,7 @@
 /*
  * arith.h - macros for arithmetic
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: arith.h,v 1.9 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_ARITH_H
@@ -66,6 +64,8 @@
 /* Include processor-specific macros */
 #if defined(SCM_TARGET_I386)
 #include "arith_i386.h"
+#elif defined(SCM_TARGET_X86_64)
+#include "arith_x86_64.h"
 #endif
 
 /*-----------------------------------------------------------------
