@@ -1,7 +1,7 @@
 /*
  * char_utf8.h - UTF8 encoding interface
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: char_utf_8.h,v 1.14 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef SCM_CHAR_ENCODING_BODY
@@ -128,6 +126,16 @@ SCM_EXTERN void Scm_CharUtf8Putc(unsigned char *, ScmChar);
         }                                                       \
     } while (0)
 
+/* C is an ScmChar > 0x80.  Returns true if C is a whitespace character. */
+#define SCM_CHAR_EXTRA_WHITESPACE(c) \
+    ((c) <= 0x3000 && Scm__CharIsExtraWhiteSpace(c, FALSE))
+/* Like SCM_CHAR_EXTRA_WHITESPACE, but excludes Zl and Zp.
+   See R6RS on the intraline whitespaces. */
+#define SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(c) \
+    ((c) <= 0x3000 && Scm__CharIsExtraWhiteSpace(c, TRUE))
+
+extern int Scm__CharIsExtraWhiteSpace(ScmChar c, int intraline);
+
 #else  /* !SCM_CHAR_ENCODING_BODY */
 /*==================================================================
  * This part is included in char.c
@@ -260,4 +268,38 @@ void Scm_CharUtf8Putc(unsigned char *cp, ScmChar ch)
     }
 }
 
+int Scm__CharIsExtraWhiteSpace(ScmChar c, int intraline)
+{
+    if (c < 0x2000) {
+        return (c == 0x00a0         /* Zs NO-BREAK SPACE */
+                || c == 0x1680      /* Zs OGHAM SPACE MARK */
+                || c == 0x180e);    /* Zs MONGOLIAN VOWEL SEPARATOR */
+    } else if (c <= 0x200a) {
+        /* 0x2000 - 0x200a are all Zs's */
+        return TRUE;
+    } else {
+        return (c == 0x3000         /* Zs IDEOGRAPHIC SPACE */
+                || c == 0x202f      /* Zs NARROW NO-BREAK SPACE */
+                || c == 0x205f      /* Zs MEDIUM MATHEMATICAL SPACE */
+                || (!intraline
+                    && (c == 0x2029        /* Zp PARAGRAPH SEPARATOR */
+                        || c == 0x2028))); /* Zl LINE SEPARATOR */
+    }
+}
+
+/*
+ * Lookup character category.  The tables are in char_attr.c, automatically
+ * generated by gen-unicode.scm.
+ */
+static inline unsigned char Scm__LookupCharCategory(ScmChar ch)
+{
+    if (ch == SCM_CHAR_INVALID || ch >= 0x10ffff) {
+        return SCM_CHAR_CATEGORY_Cn;
+    } else if (ch <= 0x20000) {
+        return ucs_general_category_00000[ch];
+    } else {
+        return ucs_general_category_20000(ch);
+    }
+}
+
 #endif /* !SCM_CHAR_ENCODING_BODY */
