@@ -1,7 +1,7 @@
 /*
  * code.h - Virtual machine code
  *
- *   Copyright (c) 2005-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2005-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,13 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: code.h,v 1.11 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_CODE_H
 #define GAUCHE_CODE_H
 
+SCM_DECL_BEGIN
+
 #include <gauche/vm.h>
 
 /*
@@ -146,7 +146,7 @@ SCM_EXTERN void   Scm_CompiledCodeEmit(ScmCompiledCode *cc,
 /* Macros for transition to the packed code vector of NVM.
    In the packed code vector, VM insns are stored untagged.
    It eliminates the shift in the dispatcher. */
-#define SCM_VM_INSN_CODE(obj)       ((u_int)(SCM_WORD(obj)&0x0ff))
+#define SCM_VM_INSN_CODE(obj)       ((u_int)(SCM_WORD(obj)&0x0fff))
 #define SCM_VM_INSN_ARG(obj)        ((signed long)SCM_WORD(obj) >> 12)
 #define SCM_VM_INSN_ARG0(obj)       ((int)((SCM_WORD(obj) >> 12) & 0x03ff))
 #define SCM_VM_INSN_ARG1(obj)       ((int)((SCM_WORD(obj) >> 22) & 0x03ff))
@@ -172,6 +172,6 @@ SCM_EXTERN int Scm_VMInsnOperandType(u_int code);
 SCM_EXTERN int Scm_VMInsnNameToCode(ScmObj name);
 SCM_EXTERN ScmWord Scm_VMInsnBuild(ScmObj insn);
 
-
+SCM_DECL_END
 
 #endif /* GAUCHE_CODE_H */
