@@ -1,7 +1,7 @@
 /*
  * exception.h - more exception classes
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: exception.h,v 1.17 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_EXCEPTION_H
@@ -230,14 +228,28 @@ typedef struct ScmThreadExceptionRec {
                                 */
 } ScmThreadException;
 
-/* NB: Actual exception classes are defined in ext/threads.
-   Scm_ThreadExceptionClass
-   Scm_JoinTimeoutExceptionClass
-   Scm_AbandonedMutexExceptionClass
-   Scm_TerminatedThreadExceptionClass
-   Scm_UncaughtExceptionClass
-*/
-
 SCM_EXTERN ScmObj Scm_MakeThreadException(ScmClass*, ScmVM*);
 
+SCM_CLASS_DECL(Scm_ThreadExceptionClass);
+#define SCM_CLASS_THREAD_EXCEPTION  (&Scm_ThreadExceptionClass)
+#define SCM_THREAD_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_THREAD_EXCEPTION)
+#define SCM_THREAD_EXCEPTION(obj)   ((ScmThreadException*)(obj))
+
+SCM_CLASS_DECL(Scm_JoinTimeoutExceptionClass);
+#define SCM_CLASS_JOIN_TIMEOUT_EXCEPTION (&Scm_JoinTimeoutExceptionClass)
+#define SCM_JOIN_TIMEOUT_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_JOIN_TIMEOUT_EXCEPTION)
+
+SCM_CLASS_DECL(Scm_AbandonedMutexExceptionClass);
+#define SCM_CLASS_ABANDONED_MUTEX_EXCEPTION (&Scm_AbandonedMutexExceptionClass)
+#define SCM_ABANDONED_MUTEX_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_ABANDONED_MUTEX_EXCEPTION)
+
+SCM_CLASS_DECL(Scm_TerminatedThreadExceptionClass);
+#define SCM_CLASS_TERMINATED_THREAD_EXCEPTION (&Scm_TerminatedThreadExceptionClass)
+#define SCM_TERMINATED_THREAD_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_TERMINATED_THREAD_EXCEPTION)
+
+SCM_CLASS_DECL(Scm_UncaughtExceptionClass);
+#define SCM_CLASS_UNCAUGHT_EXCEPTION (&Scm_UncaughtExceptionClass)
+#define SCM_UNCAUGHT_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_UNCAUGHT_EXCEPTION)
+
+
 #endif /*GAUCHE_EXCEPTION_H*/
