@@ -1,7 +1,7 @@
 /*
  * hash.h - Public API for hashtables
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: hash.h,v 1.8 2008-05-10 13:36:25 shirok Exp $
  */
 
 /* This file is included from gauche.h */
@@ -107,7 +105,7 @@ struct ScmHashIterRec {
     void *next;
 };
 
-#if defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
+#if !defined(GAUCHE_API_PRE_0_9)
 SCM_EXTERN void Scm_HashIterInit(ScmHashIter *iter, ScmHashCore *core);
 SCM_EXTERN ScmDictEntry *Scm_HashIterNext(ScmHashIter *iter);
 #endif /* See the compatibility section below for old APIs */
@@ -178,14 +176,14 @@ SCM_EXTERN ScmObj Scm_MakeHashTable(ScmHashProc *hashfn,
 				    ScmHashCompareProc *cmpfn,
 				    unsigned int initSize);
 
-#if !defined(GAUCHE_API_0_9) && !defined(LIBGAUCHE_BODY)
+#if defined(GAUCHE_API_PRE_0_9)
 #define Scm_HashIterInit(table, iter) Scm__HashIterInitCompat(table, iter)
 #define Scm_HashIterNext(iter)        Scm__HashIterNextCompat(iter)
 
 SCM_EXTERN void          Scm__HashIterInitCompat(ScmHashTable *table,
                                                  ScmHashIter *iter);
 SCM_EXTERN ScmHashEntry *Scm__HashIterNextCompat(ScmHashIter *iter);
-#endif
+#endif /*GAUCHE_API_PRE_0_9*/
 
 #endif /* GAUCHE_HASH_H */
 
