@@ -1,7 +1,7 @@
 /*
  * load.h - Public API for loading files
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: load.h,v 1.5 2008-05-16 18:58:16 shirok Exp $
  */
 
 /* This file is included from gauche.h */
@@ -42,22 +40,26 @@
  * Loading Scheme files
  */
 
-/* Flags for Scm_VMLoad, Scm_Load, amd Scm_Require. (not for Scm_VMLoadPort) */
+/* Flags for Scm_VMLoad, Scm_Load, Scm_Require, and Scm_VMLoadPort.
+   NB: Scm_VMLoadPort ignores all flags except SCM_LOAD_MAIN_SCRIPT.
+ */
 typedef enum {
     SCM_LOAD_QUIET_NOFILE = (1L<<0),  /* do not signal an error if the file
                                          does not exist; just return #f. */
     SCM_LOAD_IGNORE_CODING = (1L<<1), /* do not use coding-aware port to honor
                                          'coding' magic comment */
-    SCM_LOAD_PROPAGATE_ERROR = (1L<<2) /* do not capture an error; let the
+    SCM_LOAD_PROPAGATE_ERROR = (1L<<2),/* do not capture an error; let the
                                          caller handle it.  Not effective
                                          for Scm_VMLoad. */
+    SCM_LOAD_MAIN_SCRIPT = (1L<<3)    /* indicates we're loading the file
+                                         as a "main script"---a script file
+                                         given to gosh to load. */
 } ScmLoadFlags;
 
 /* A structure to obtain a detailed result of loading. */
 typedef struct ScmLoadPacketRec {
     ScmObj exception; /* OUT: exception object in case of LOAD_EVAL_ERROR */
     int    loaded;    /* OUT: TRUE iff file is successfully loaded.  */
-    ScmObj paths;     /* reserved */
 } ScmLoadPacket;
 
 SCM_EXTERN ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
@@ -65,15 +67,24 @@ SCM_EXTERN ScmObj Scm_VMLoadFromPort(ScmPort *port, ScmObj next_paths,
 SCM_EXTERN ScmObj Scm_VMLoad(ScmString *file, ScmObj paths, ScmObj env,
 			     int flags);
 
-#if defined(GAUCHE_API_0_8_10) || defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
-SCM_EXTERN int Scm_LoadFromPort(ScmPort *port, int flags, ScmLoadPacket *p);
-SCM_EXTERN int Scm_Load(const char *file, int flags, ScmLoadPacket *p);
-#else  /*!GAUCHE_API_0_9*/
+#if !defined(GAUCHE_API_PRE_0_9)
+SCM_EXTERN void Scm_LoadPacketInit(ScmLoadPacket *p);
+SCM_EXTERN int Scm_LoadFromPort(ScmPort *port, u_long flags, ScmLoadPacket *p);
+SCM_EXTERN int Scm_Load(const char *file, u_long flags, ScmLoadPacket *p);
+#else  /*!GAUCHE_API_PRE_0_9*/
 #define Scm_LoadFromPort(port, flags)  Scm__LoadFromPortCompat(port, flags) 
 #define Scm_Load(port, flags)          Scm__LoadCompat(port, flags) 
 SCM_EXTERN void Scm__LoadFromPortCompat(ScmPort *port, int flags);
 SCM_EXTERN int  Scm__LoadCompat(const char *file, int flags);
-#endif /*!GAUCHE_API_0_9*/
+#endif /*!GAUCHE_API_PRE_0_9*/
+
+/*=================================================================
+ * Dynamic state access
+ */
+SCM_EXTERN ScmObj Scm_CurrentLoadHistory(void);
+SCM_EXTERN ScmObj Scm_CurrentLoadNext(void);
+SCM_EXTERN ScmObj Scm_CurrentLoadPort(void);
+SCM_EXTERN ScmObj Scm_LoadMainScript(void);
 
 /*=================================================================
  * Load path management
@@ -86,18 +97,18 @@ SCM_EXTERN ScmObj Scm_AddLoadPath(const char *cpath, int afterp);
  * Dynamic Loading
  */
 
-SCM_EXTERN ScmObj Scm_DynLoad(ScmString *path, ScmObj initfn, int export_);
+SCM_EXTERN ScmObj Scm_DynLoad(ScmString *path, ScmObj initfn, u_long flags);
 
 /*=================================================================
  * Require & Provide
  */
 
-#if defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
+#if !defined(GAUCHE_API_PRE_0_9)
 SCM_EXTERN int Scm_Require(ScmObj feature, int flags, ScmLoadPacket *p);
-#else  /*!GAUCHE_API_0_9*/
+#else  /*GAUCHE_API_PRE_0_9*/
 #define Scm_Require(feature) Scm__RequireCompat(feature)
 SCM_EXTERN ScmObj Scm__RequireCompat(ScmObj feature);
-#endif /*!GAUCHE_API_0_9*/
+#endif /*GAUCHE_API_PRE_0_9*/
 SCM_EXTERN ScmObj Scm_Provide(ScmObj feature);
 SCM_EXTERN int    Scm_ProvidedP(ScmObj feature);
 
