@@ -1,7 +1,7 @@
 /*
  * macro.h - structures used internally in macro expander
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,13 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: macro.h,v 1.12 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_MACRO_H
 #define GAUCHE_MACRO_H
 
+SCM_DECL_BEGIN
+
 /*
  * SyntaxPattern object is an internal object used to expand r5rs macro.
  */
@@ -85,8 +85,8 @@ SCM_EXTERN ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj lietrals,
 /*
  * Pattern variable reference object 
  */
-#define SCM_PVREF_TAG              0x0e
-#define SCM_PVREF_P(obj)           ((SCM_WORD(obj)&0x0f) == SCM_PVREF_TAG)
+#define SCM_PVREF_TAG              0x13
+#define SCM_PVREF_P(obj)           ((SCM_WORD(obj)&0xff) == SCM_PVREF_TAG)
 #define SCM_PVREF_LEVEL(obj)       ((SCM_WORD(obj)>>24) & 0xff)
 #define SCM_PVREF_COUNT(obj)       ((SCM_WORD(obj)>>16) & 0xff)
 
@@ -97,4 +97,6 @@ SCM_EXTERN ScmObj Scm_CompileSyntaxRules(ScmObj name, ScmObj lietrals,
 SCM_EXTERN ScmObj Scm_MakeMacroTransformerOld(ScmSymbol *name,
                                               ScmProcedure *proc);
 
+SCM_DECL_END
+
 #endif /* GAUCHE_MACRO_H */
