@@ -1,7 +1,7 @@
 /*
  * gauche/port.h - Port API
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: port.h,v 1.28 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_PORT_H
@@ -349,6 +347,10 @@ SCM_EXTERN ScmObj Scm_Stdin(void);
 SCM_EXTERN ScmObj Scm_Stdout(void);
 SCM_EXTERN ScmObj Scm_Stderr(void);
 
+SCM_EXTERN ScmObj Scm_SetStdin(ScmPort *port);
+SCM_EXTERN ScmObj Scm_SetStdout(ScmPort *port);
+SCM_EXTERN ScmObj Scm_SetStderr(ScmPort *port);
+
 #define SCM_CURIN    SCM_VM_CURRENT_INPUT_PORT(Scm_VM())
 #define SCM_CUROUT   SCM_VM_CURRENT_OUTPUT_PORT(Scm_VM())
 #define SCM_CURERR   SCM_VM_CURRENT_ERROR_PORT(Scm_VM())
@@ -364,12 +366,12 @@ SCM_EXTERN ScmObj Scm_SetCurrentErrorPort(ScmPort *port);
 SCM_EXTERN ScmObj Scm_MakeInputStringPort(ScmString *str, int privatep);
 SCM_EXTERN ScmObj Scm_MakeOutputStringPort(int privatep);
 
-#if defined(GAUCHE_API_0_8_10) || defined(GAUCHE_API_0_9) || defined(LIBGAUCHE_BODY)
+#if !defined(GAUCHE_API_PRE_0_9)
 SCM_EXTERN ScmObj Scm_GetOutputString(ScmPort *port, int flags);
 SCM_EXTERN ScmObj Scm_GetOutputStringUnsafe(ScmPort *port, int flags);
 SCM_EXTERN ScmObj Scm_GetRemainingInputString(ScmPort *port, int flags);
 
-#else  /* !defined(GAUCHE_API_0_8_10) && !defined(GAUCHE_API_0_9) && !defined(LIBGAUCHE_BODY) */
+#else  /* GAUCHE_API_PRE_0_9 */
 #define Scm_GetOutputString(p) Scm__GetOutputStringCompat(p)
 #define Scm_GetOutputStringUnsafe(p) Scm__GetOutputStringUnsafeCompat(p)
 #define Scm_GetRemainingInputString(p) Scm__GetRemainingInputStringCompat(p)
@@ -378,7 +380,7 @@ SCM_EXTERN ScmObj Scm_GetRemainingInputString(ScmPort *port, int flags);
 SCM_EXTERN ScmObj Scm__GetOutputStringCompat(ScmPort *port);
 SCM_EXTERN ScmObj Scm__GetOutputStringUnsafeCompat(ScmPort *port);
 SCM_EXTERN ScmObj Scm__GetRemainingInputStringCompat(ScmPort *port);
-#endif /* !defined(GAUCHE_API_0_8_10) && !defined(GAUCHE_API_0_9) && !defined(LIBGAUCHE_BODY) */
+#endif /* GAUCHE_API_PRE_0_9 */
 
 /*================================================================
  * Other type of ports
@@ -401,6 +403,11 @@ SCM_EXTERN ScmObj Scm_MakeCodingAwarePort(ScmPort *iport);
 
 SCM_EXTERN void Scm__InstallCodingAwarePortHook(ScmPort *(*)(ScmPort*, const char*));
 
+/* Windows-specific initialization */
+#if defined(GAUCHE_WINDOWS)
+void Scm__SetupPortsForWindows(int has_console);
+#endif /*defined(GAUCHE_WINDOWS)*/
+
 /*================================================================
  * Locking the ports
  *
