@@ -1,7 +1,7 @@
 /*
  * regexp.h - Regexp module internal definitions
  *
- *   Copyright (c) 2006-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2006-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: regexp.h,v 1.4 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_REGEXP_H
@@ -38,14 +36,18 @@
 
 struct ScmRegexpRec {
     SCM_HEADER;
-    ScmString *pattern;
-    const unsigned char *code;
-    int numGroups;
-    int numCodes;
-    ScmCharSet **sets;
-    ScmObj grpNames;
-    int numSets;
-    int flags;
+    ScmObj pattern;      /* Source string.  For debugging/introspection.
+                            Can be #f if this regexp is created from AST.
+                            regexp->string will construct the string
+                            representation and fill this slot in such case. */
+    ScmObj ast;          /* Parsed AST. */
+    const u_char *code;  /* byte code vector */
+    int numGroups;       /* # of captured groups */
+    int numCodes;        /* size of byte code vector */
+    ScmCharSet **sets;   /* array of charset literals referred from code */
+    ScmObj grpNames;     /* list of names for named groups. */
+    int numSets;         /* # of charsets in sets */
+    int flags;           /* internal; CASE_FOLD, BOL_ANCHORED etc. */
     ScmString *mustMatch;
 };
 
@@ -68,4 +70,7 @@ struct ScmRegMatchRec {
 #define SCM_REG_MATCH_SINGLE_BYTE_P(rm) \
     ((rm)->inputSize == (rm)->inputLen)
 
+/* Note: The structure of ScmRegexp is changed on 0.9.1.  Shuold be safe,
+   for it should never be statically allocated. */
+
 #endif /* GAUCHE_REGEXP_H */
