@@ -1,7 +1,7 @@
 /*
  * system.h - Gauche system interface
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: system.h,v 1.8 2008-05-10 13:36:25 shirok Exp $
  */
 
 #ifndef GAUCHE_SYSTEM_H
@@ -84,6 +82,7 @@ SCM_EXTERN ScmObj Scm_ReadDirectory(ScmString *pathname);
 #define SCM_PATH_CANONICALIZE   (1L<<2)
 #define SCM_PATH_FOLLOWLINK     (1L<<3) /* not supported yet */
 SCM_EXTERN ScmObj Scm_NormalizePathname(ScmString *pathname, int flags);
+SCM_EXTERN ScmObj Scm_TmpDir(void);
 SCM_EXTERN ScmObj Scm_DirName(ScmString *filename);
 SCM_EXTERN ScmObj Scm_BaseName(ScmString *filename);
 
@@ -97,6 +96,7 @@ SCM_CLASS_DECL(Scm_SysStatClass);
 #define SCM_CLASS_SYS_STAT    (&Scm_SysStatClass)
 #define SCM_SYS_STAT(obj)     ((ScmSysStat*)(obj))
 #define SCM_SYS_STAT_P(obj)   (SCM_XTYPEP(obj, SCM_CLASS_SYS_STAT))
+#define SCM_SYS_STAT_STAT(obj) (&SCM_SYS_STAT(obj)->statrec)
 
 SCM_EXTERN ScmObj Scm_MakeSysStat(void); /* returns empty SysStat */
 
@@ -209,11 +209,14 @@ SCM_EXTERN int    Scm_IsSugid(void);
 
 /* flags for Scm_SysExec */
 enum {
-    SCM_EXEC_WITH_FORK = (1L<<0)
+    SCM_EXEC_WITH_FORK = (1L<<0), /* fork() before exec(), i.e. spawn(). */
+    SCM_EXEC_DETACHED = (1L<<1)   /* try to detach from process group.
+                                     good for daemoninzing. */
 };
 
 SCM_EXTERN ScmObj Scm_SysExec(ScmString *file, ScmObj args,
-                              ScmObj iomap, ScmSysSigset *mask, int flags);
+                              ScmObj iomap, ScmSysSigset *mask,
+                              ScmString *dir, int flags);
 SCM_EXTERN int   *Scm_SysPrepareFdMap(ScmObj iomap);
 SCM_EXTERN void   Scm_SysSwapFds(int *fds);
 
@@ -260,11 +263,15 @@ SCM_EXTERN ScmObj Scm_SysMkstemp(ScmString *tmpl);
  */
 
 #if defined(GAUCHE_WINDOWS)
-/* special object to wrap windows process handle */
+/* special object to wrap windows handle */
+SCM_EXTERN ScmObj Scm_MakeWinHandle(HANDLE h, ScmObj type);
+SCM_EXTERN int    Scm_WinHandleP(ScmObj obj, ScmObj type);
+SCM_EXTERN HANDLE Scm_WinHandle(ScmObj h, ScmObj type);
+
 SCM_EXTERN ScmObj Scm_MakeWinProcess(HANDLE h);
 SCM_EXTERN int    Scm_WinProcessP(ScmObj p);
 SCM_EXTERN pid_t  Scm_WinProcessPID(ScmObj p);
-SCM_EXTERN HANDLE Scm_WinProcessHandle(ScmObj p);
+SCM_EXTERN HANDLE Scm_WinProcess(ScmObj p);
 #endif /* GAUCHE_WINDOWS */
 
 #endif /* GAUCHE_SYSTEM_H */
