@@ -1,7 +1,7 @@
 /*
  * vm.h - Virtual machine
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: vm.h,v 1.116 2008-06-02 01:13:14 shirok Exp $
  */
 
 #ifndef GAUCHE_VM_H
@@ -45,10 +43,6 @@
 /* Finalizer queue size */
 #define SCM_VM_FINQ_SIZE       32
 
-/* QueueNotEmpty flag */
-#define SCM_VM_SIGQ_MASK       1
-#define SCM_VM_FINQ_MASK       2
-
 #define SCM_PCTYPE ScmWord*
 
 #if defined(ITIMER_PROF) && defined(SIGPROF)
@@ -56,6 +50,26 @@
 #define GAUCHE_PROFILE
 #endif /* defined(ITIMER_PROF) && defined(SIGPROF) */
 
+
+#ifdef __GNUC__
+/* in GNUC, we inline Scm_VMReturnFlonum using statement expressions. */
+# if GAUCHE_FFX
+SCM_EXTERN void Scm_VMFlushFPStack(ScmVM *vm);
+#   define Scm_VMReturnFlonum(_val)                                     \
+        ({ ScmVM *vm__ = Scm_VM(); ScmFlonum *fp;                       \
+           if (vm__->fpsp == vm__->fpstackEnd) Scm_VMFlushFPStack(vm__);\
+           fp = vm__->fpsp++;                                           \
+           SCM_FLONUM_VALUE(fp) = (_val);                               \
+           SCM_MAKE_FLONUM_REG(fp);                                     \
+        })
+# else  /* !GAUCHE_FFX */
+#   define Scm_VMReturnFlonum(val)  Scm_MakeFlonum(val)
+# endif /* !GAUCHE_FFX */
+#else  /* !__GNUC__ */
+#   define Scm_VMReturnFlonum(val)  Scm_MakeFlonum(val)
+#endif /* !__GNUC__ */
+
+
 /* Actual structure is defined in code.h */
 typedef struct ScmCompiledCodeRec ScmCompiledCode;
 
@@ -222,7 +236,11 @@ typedef struct ScmEscapePointRec {
     ScmObj ehandler;            /* handler closure */
     ScmContFrame *cont;         /* saved continuation */
     ScmObj handlers;            /* saved dynamic handler chain */
-    ScmCStack *cstack;          /* C stack */
+    ScmCStack *cstack;          /* vm->cstack when escape point is created.
+                                   this will be used to rewind cstack.
+                                   this is NULL for partial continuations,
+                                   for they can be executed on anywhere
+                                   w.r.t. cstack. */
     ScmObj xhandler;            /* saved exception handler */
     int errorReporting;         /* state of SCM_VM_ERROR_REPORTING flag
                                    when this ep is captured.  The flag status
@@ -268,7 +286,7 @@ typedef struct ScmEscapePointRec {
  *  is invoked only once.
  *
  *  The C-level signal handler does count each signal until it is cleared
- *  by VM loop.  If one kind of signal arraives more than a certain limit
+ *  by VM loop.  If one kind of signal arrives more than a certain limit
  *  (set/get by Scm_{Set|Get}SignalPendingLimit), Gauche thinks something
  *  went wrong and bail out.   It is useful, for example, to interrupt
  *  unresponsive program from the terminal.
@@ -289,7 +307,7 @@ SCM_EXTERN void Scm_SetSignalPendingLimit(int val);
     ((vm)->queueNotEmpty&SCM_VM_SIGQ_MASK)
 
 #define SCM_SIGCHECK(vm) \
-    do { if (vm->queueNotEmpty&SCM_VM_SIGQ_MASK) Scm_SigCheck(vm); } while (0)
+    do { if (vm->signalPending) Scm_SigCheck(vm); } while (0)
 
 SCM_EXTERN void   Scm_SigCheck(ScmVM *vm);
 
@@ -336,27 +354,28 @@ typedef struct ScmVMProfilerRec ScmVMProfiler;
  *  Most fields of VM are private to the thread that owns the VM.
  *  Only the fields marked as "PUBLIC" should be modified by other
  *  thread, and only with obtaining the lock by VMLOCK mutex.
- *
- *  Note that some fields like "name" and "specific" are not marked
+ *  (Note that some fields like "name" and "specific" are not marked
  *  as PUBLIC although they can be referenced or modified by other
  *  thread (via Scheme call thread-specific-set! etc.)   It is the
- *  user program's responsibility to use a mutex.
- *  When you should introspect other thread (like stack trace), make
- *  sure you stopped that thread, or you may get inconsistent result.
+ *  user program's responsibility to use a mutex.)
+ *
+ *  When you need inspect other thread's private data (like stack
+ *  trace), make the thread is either stopped or terminated, or
+ *  you may get inconsistent result.
  */
 
 struct ScmVMRec {
     SCM_HEADER;
-#ifdef GAUCHE_USE_PTHREADS
-    pthread_t thread;           /* the thread executing this VM. */
-#endif /*!GAUCHE_USE_PTHREADS*/
+    ScmInternalThread thread;   /* the system thread executing this VM. */
     int state;                  /* thread state. PUBLIC. */
     ScmInternalMutex  vmlock;   /* mutex to be used to lock this VM
                                    structure.  PUBLIC. */
     ScmInternalCond cond;       /* the condition variable to wait for state
                                    change of this VM.  PUBLIC. */
-    ScmVM *canceller;           /* the thread that called thread-terminate!
+    ScmVM *canceller;           /* the thread which called thread-terminate!
                                    on this thread.  PUBLIC. */
+    ScmVM *inspector;           /* the thread which requested to stop this
+                                   thread.  PUBLIC. */
     ScmObj name;                /* Scheme thread name. */
     ScmObj specific;            /* Scheme thread specific data. */
     ScmProcedure *thunk;        /* Entry point of this VM. */
@@ -368,7 +387,21 @@ struct ScmVMRec {
                                    "C stack rewinding" below. */
     unsigned int runtimeFlags;  /* Runtime flags */
     unsigned int compilerFlags; /* Compiler flags */
-    unsigned int queueNotEmpty; /* Bitmask if sigq or finq is not empty */
+    intptr_t attentionRequest;  /* Flag if VM needs to check signal queue,
+                                   finalizer queue, or stop request.
+                                   This flag can be turned on asynchronously.
+                                   Only this VM can turn off this flag. */
+    intptr_t signalPending;     /* Flag if there are pending signals.
+                                   Turned on by sig_handle(), turned off
+                                   by Scm_SigCheck(), both in signal.c. */
+    intptr_t finalizerPending;  /* Flag if there are pending finalizers.
+                                   Turned on by finalizable() callback,
+                                   and turned off by Scm_VMFinalizerRun(),
+                                   both in core.c */
+    intptr_t stopRequest;       /* Flag if there is a pending stop request.
+                                   Turned on by Scm_ThreadStop() in
+                                   ext/threads/threads.c, and turned off by
+                                   process_queued_requests() in vm.c */
 
     ScmPort *curin;             /* current input port */
     ScmPort *curout;            /* current output port */
@@ -396,6 +429,14 @@ struct ScmVMRec {
     ScmObj *stackBase;          /* base of current stack area  */
     ScmObj *stackEnd;           /* end of current stack area */
 
+#if GAUCHE_FFX
+    ScmFlonum *fpsp;            /* flonum stack pointer.  we call it 'stack'
+                                   for historical reasons, but it's more like
+                                   a nursery. */
+    ScmFlonum *fpstack;         /* flonum stack */
+    ScmFlonum *fpstackEnd;      /* flonum stack limit */
+#endif
+
     /* Escape handling */
     ScmObj exceptionHandler;    /* the current exception handler installed by
                                    with-exception-handler. */
@@ -415,9 +456,9 @@ struct ScmVMRec {
     ScmObj defaultEscapeHandler;
 
     /* Program information */
-    ScmObj load_next;           /* list of the directories to be searched */
-    ScmObj load_history;        /* history of the nested load */
-    ScmObj load_port;           /* current port from which we are loading */
+    ScmObj dummy0;              /* for the ABI compatibility.  remove on 1.0. */
+    ScmObj dummy1;              /* ditto */
+    ScmObj dummy2;              /* ditto */
     int    evalSituation;       /* eval situation (related to eval-when) */
 
     /* Signal information */
@@ -437,6 +478,7 @@ SCM_EXTERN void   Scm_VMDefaultExceptionHandler(ScmObj exc);
 SCM_EXTERN ScmObj Scm_VMThrowException(ScmVM *vm, ScmObj exc);
 SCM_EXTERN ScmObj Scm_VMGetSourceInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
 SCM_EXTERN ScmObj Scm_VMGetBindInfo(ScmCompiledCode *code, SCM_PCTYPE pc);
+SCM_EXTERN void   Scm_VMSetResult(ScmObj obj);
 
 SCM_CLASS_DECL(Scm_VMClass);
 #define SCM_CLASS_VM              (&Scm_VMClass)
@@ -452,10 +494,8 @@ enum {
                                    initialized. */
     SCM_VM_RUNNABLE,            /* This VM is attached to a thread which is
                                    runnable or blocked. */
-    SCM_VM_BLOCKED,             /* The thread attached to this VM is stopped
-                                   because of thread-yield! or thread-sleep!.
-                                   Note that if the thread is blocked by
-                                   system call, VM's state is still RUNNABLE.*/
+    SCM_VM_STOPPED,             /* The thread attached to this VM is stopped
+                                   by the inspector thread for debugging. */
     SCM_VM_TERMINATED           /* The thread attached to this VM is
                                    terminated. */
 };
@@ -589,7 +629,11 @@ enum {
     SCM_COMPILE_NOSOURCE = (1L<<3),        /* Do not insert source info */
     SCM_COMPILE_SHOWRESULT = (1L<<4),      /* Display each result of
                                               compilation */
-    SCM_COMPILE_NOCOMBINE = (1L<<5)        /* Do not combine instructions */
+    SCM_COMPILE_NOCOMBINE = (1L<<5),       /* Do not combine instructions */
+    SCM_COMPILE_NO_POST_INLINE_OPT = (1L<<6), /* Do not run post-inline
+                                                 optimization (pass3). */
+    SCM_COMPILE_NO_LIFTING = (1L<<7)       /* Do not run lambda lifting pass
+                                              (pass4). */
 };
 
 #define SCM_VM_COMPILER_FLAG_IS_SET(vm, flag) ((vm)->compilerFlags & (flag))
