@@ -1,7 +1,7 @@
 /*
  * list.c - List related functions
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,10 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: list.c,v 1.48 2008-05-10 13:36:19 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/memory.h"
 
 /*
  * Classes
@@ -59,8 +56,13 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_NullClass, NULL, NULL, NULL, NULL, list_cpl);
 
 ScmObj Scm_Cons(ScmObj car, ScmObj cdr)
 {
-    ScmPair *z;
-    SCM_MALLOC_WORDS(z, sizeof(ScmPair)/sizeof(GC_word), ScmPair*);
+    ScmPair *z = SCM_NEW(ScmPair);
+    /* NB: these ENSURE_MEMs are moved here from vm loop to reduce
+       the register pressure there.  In most cases these increases
+       just a couple of mask-and-test instructions on the data on
+       the register. */
+    SCM_FLONUM_ENSURE_MEM(car);
+    SCM_FLONUM_ENSURE_MEM(cdr);
     SCM_SET_CAR(z, car);
     SCM_SET_CDR(z, cdr);
     return SCM_OBJ(z);
@@ -68,9 +70,8 @@ ScmObj Scm_Cons(ScmObj car, ScmObj cdr)
 
 ScmObj Scm_Acons(ScmObj caar, ScmObj cdar, ScmObj cdr)
 {
-    ScmPair *y, *z;
-    SCM_MALLOC_WORDS(y, sizeof(ScmPair)/sizeof(GC_word), ScmPair*);
-    SCM_MALLOC_WORDS(z, sizeof(ScmPair)/sizeof(GC_word), ScmPair*);
+    ScmPair *y = SCM_NEW(ScmPair);
+    ScmPair *z = SCM_NEW(ScmPair);
     SCM_SET_CAR(y, caar);
     SCM_SET_CDR(y, cdar);
     SCM_SET_CAR(z, SCM_OBJ(y));
@@ -117,14 +118,12 @@ ScmObj Scm_VaList(va_list pvar)
     {
 	if (SCM_NULLP(start)) {
             start = SCM_OBJ(SCM_NEW(ScmPair));
-            /*SCM_SET_CLASS(start, SCM_CLASS_PAIR);*/
             SCM_SET_CAR(start, obj);
             SCM_SET_CDR(start, SCM_NIL);
             cp = start;
         } else {
             ScmObj item;
             item = SCM_OBJ(SCM_NEW(ScmPair));
-            /*SCM_SET_CLASS(item, SCM_CLASS_PAIR);*/
             SCM_SET_CDR(cp, item);
             SCM_SET_CAR(item, obj);
             SCM_SET_CDR(item, SCM_NIL);
@@ -142,14 +141,18 @@ ScmObj Scm_VaCons(va_list pvar)
 }
 
 ScmObj Scm_ArrayToList(ScmObj *elts, int nelts)
+{
+    return Scm_ArrayToListWithTail(elts, nelts, SCM_NIL);
+}
+
+ScmObj Scm_ArrayToListWithTail(ScmObj *elts, int nelts, ScmObj tail)
 {
     ScmObj h = SCM_NIL, t = SCM_NIL;
     if (elts) {
         int i;
-        for (i=0; i<nelts; i++) {
-            SCM_APPEND1(h, t, *elts++);
-        }
+        for (i=0; i<nelts; i++) SCM_APPEND1(h, t, *elts++);
     }
+    if (!SCM_NULLP(tail)) SCM_APPEND(h, t, tail);
     return h;
 }
 
@@ -323,39 +326,52 @@ ScmObj Scm_Append(ScmObj args)
     return start;
 }
 
-/* Scm_Reverse(list)
- *    Reverse LIST.  If LIST is not a pair, return LIST itself.
- *    If LIST is improper list, cdr of the last pair is ignored.
+/* Scm_Reverse2(list, tail)
+ *    Reverse LIST, and append TAIL to the result.
+ *    If LIST is an improper list, cdr of the last pair is ignored.
+ *    If LIST is not a pair, TAIL is returned.
+ * Scm_Reverse(list)
+ *    Scm_Reverse2(list, SCM_NIL).  Just for the backward compatibility.
  */
 
-ScmObj Scm_Reverse(ScmObj list)
+ScmObj Scm_Reverse2(ScmObj list, ScmObj tail)
 {
     ScmObj cp, result;
     ScmPair *p;
 
-    if (!SCM_PAIRP(list)) return list;
+    if (!SCM_PAIRP(list)) return tail;
 
-    SCM_NEW_PAIR(p, SCM_NIL, SCM_NIL);
+    p = SCM_NEW(ScmPair);
+    SCM_SET_CAR(p, SCM_NIL);
+    SCM_SET_CDR(p, tail);
     result = SCM_OBJ(p);
     SCM_FOR_EACH(cp, list) {
 	SCM_SET_CAR(result, SCM_CAR(cp));
-        SCM_NEW_PAIR(p, SCM_NIL, result);
+        p = SCM_NEW(ScmPair);
+        SCM_SET_CAR(p, SCM_NIL);
+        SCM_SET_CDR(p, result);
         result = SCM_OBJ(p);
     }
     return SCM_CDR(result);
 }
 
+ScmObj Scm_Reverse(ScmObj list)
+{
+    return Scm_Reverse2(list, SCM_NIL);
+}
+
     
-/* Scm_ReverseX(list)
+/* Scm_Reverse2X(list, tail)
  *   Return reversed list of LIST.  Pairs in previous LIST is used to
- *   create new list.  If LIST is not a pair, return LIST itself.
+ *   create new list.  TAIL is appended to the result.
+ *   If LIST is not a pair, returns TAIL.
  *   If LIST is an improper list, cdr of the last cell is ignored.
  */
 
-ScmObj Scm_ReverseX(ScmObj list)
+ScmObj Scm_Reverse2X(ScmObj list, ScmObj tail)
 {
-    ScmObj first, next, result = SCM_NIL;
-    if (!SCM_PAIRP(list)) return list;
+    ScmObj first, next, result = tail;
+    if (!SCM_PAIRP(list)) return result;
     for (first = list; SCM_PAIRP(first); first = next) {
         next = SCM_CDR(first);
         SCM_SET_CDR(first, result);
@@ -364,6 +380,11 @@ ScmObj Scm_ReverseX(ScmObj list)
     return result;
 }
 
+ScmObj Scm_ReverseX(ScmObj list)
+{
+    return Scm_Reverse2X(list, SCM_NIL);
+}
+
 /* Scm_ListTail(list, i, fallback)
  * Scm_ListRef(list, i, fallback)
  *    Note that i is C-INTEGER.  If i is out of bound, signal error.
@@ -583,7 +604,7 @@ ScmObj Scm_DeleteDuplicates(ScmObj list, int cmpmode)
             SCM_APPEND1(result, tail, SCM_CAR(lp));
         }
     }
-    if (!SCM_NULLP(lp)) SCM_SET_CDR(lp, tail);
+    if (!SCM_NULLP(lp) && !SCM_NULLP(tail)) SCM_SET_CDR(tail, lp);
     return result;
 }
 
@@ -616,11 +637,26 @@ ScmObj Scm_DeleteDuplicatesX(ScmObj list, int cmpmode)
  *    http://www.webcom.com/~haahr/dylan/linearization-oopsla96.html.
  *  Since the algorithm is generally useful, I implement the core routine
  *  of the algorithm here.
+ *
+ *  TODO at 1.0: I noticed START argument acutally isn't used in the
+ *  algorithm at all.  We can drop it and the caller can just say
+ *  Scm_Cons(start, Scm_MonotonicMerge(sequences)).   We can't change it
+ *  now because of ABI compatibility, but it will be nice to do so when
+ *  releasing 1.0.
  */
 
+/* DEPRECATED 1.0 */
 ScmObj Scm_MonotonicMerge(ScmObj start, ScmObj sequences)
 {
-    ScmObj result = Scm_Cons(start, SCM_NIL), next, h;
+    ScmObj r = Scm_MonotonicMerge1(sequences);
+    if (!SCM_FALSEP(r)) r = Scm_Cons(start, r);
+    return r;
+}
+
+/* WILL RENAME IN 1.0 */
+ScmObj Scm_MonotonicMerge1(ScmObj sequences)
+{
+    ScmObj result = SCM_NIL, next, h;
     ScmObj *seqv, *sp, *tp;
     int nseqs = Scm_Length(sequences);
 
