@@ -1,7 +1,7 @@
 /*
  * main.c - interpreter main program
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,15 +29,13 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: main.c,v 1.104 2008-05-10 13:36:19 shirok Exp $
  */
 
-#define GAUCHE_API_0_9        /* temporary compatibility stuff */
 #include "gauche.h"
 
 #include <signal.h>
 #include <ctype.h>
+#include <fcntl.h>		/* for _O_BINMODE on windows. */
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -88,11 +86,12 @@ void usage(void)
             "      load-verbose    report while loading files\n"
             "      no-inline       don't inline procedures & constants (combined\n"
             "                      no-inline-globals, no-inline-locals, and\n"
-            "                      no-inline-constants.\n"
+            "                      no-inline-constants.)\n"
             "      no-inline-globals don't inline global procedures.\n"
             "      no-inline-locals  don't inline local procedures.\n"
             "      no-inline-constants don't inline constants.\n"
-            "      no-source-info  don't preserve source information for debug\n"
+            "      no-post-inline-pass  don't run post-inline optimization pass.\n"
+            "      no-source-info  don't preserve source information for debugging\n"
             "      test            test mode, to run gosh inside the build tree\n"
             );
     exit(1);
@@ -106,8 +105,8 @@ void usage(void)
 
 void version(void)
 {
-    printf("Gauche scheme interpreter, version %s [%s%s]\n",
-           GAUCHE_VERSION, SCM_CHAR_ENCODING_NAME, PTHREAD_OPT);
+    printf("Gauche scheme shell, version %s [%s%s], %s\n",
+           GAUCHE_VERSION, SCM_CHAR_ENCODING_NAME, PTHREAD_OPT, GAUCHE_ARCH);
     exit(0);
 }
 
@@ -128,6 +127,12 @@ void further_options(const char *optarg)
         SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NOINLINE_LOCALS);
         SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NOINLINE_CONSTS);
     }
+    else if (strcmp(optarg, "no-post-inline-pass") == 0) {
+        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NO_POST_INLINE_OPT);
+    }
+    else if (strcmp(optarg, "no-lambda-lifting-pass") == 0) {
+        SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NO_LIFTING);
+    }
     else if (strcmp(optarg, "no-source-info") == 0) {
         SCM_VM_COMPILER_FLAG_SET(vm, SCM_COMPILE_NOSOURCE);
     }
@@ -159,7 +164,7 @@ void further_options(const char *optarg)
     }
     else {
         fprintf(stderr, "unknown -f option: %s\n", optarg);
-        fprintf(stderr, "supported options are: -fcase-fold or -fload-verbose, -fno-inline, -fno-inline-globals, -fno-inline-locals, -fno-inline-constants, -fno-source-info, -ftest\n");
+        fprintf(stderr, "supported options are: -fcase-fold or -fload-verbose, -fno-inline, -fno-inline-globals, -fno-inline-locals, -fno-inline-constants, -fno-source-info, -fno-post-inline-pass, -fno-lambda-lifting-pass, -ftest\n");
         exit(1);
     }
 }
@@ -248,6 +253,30 @@ static void sig_setup(void)
     Scm_SetMasterSigmask(&set);
 }
 
+/* Path setup.  When we're ran with -ftest flag, add paths to refer
+   to the files in the source tree.  This is called after option processing
+   and before loading init file. */
+void test_paths_setup(void)
+{
+    /* The order of directories is important.  'lib' should
+       be searched first (hence it should come latter), since some
+       extension modules are built from the file in src then linked
+       from lib, and we want to test the one in lib. */
+    if (access("../src/stdlib.stub", R_OK) == 0
+        && access("../libsrc/srfi-1.scm", R_OK) == 0
+        && access("../lib/srfi-0.scm", R_OK) == 0) {
+        Scm_AddLoadPath("../src", FALSE);
+        Scm_AddLoadPath("../libsrc", FALSE);
+        Scm_AddLoadPath("../lib", FALSE);
+    } else if (access("../../src/stdlib.stub", R_OK) == 0
+               && access("../../libsrc/srfi-1.scm", R_OK) == 0
+               && access("../../lib/srfi-0.scm", R_OK) == 0) {
+        Scm_AddLoadPath("../../src", FALSE);
+        Scm_AddLoadPath("../../libsrc", FALSE);
+        Scm_AddLoadPath("../../lib", FALSE);
+    }
+}
+
 /* Load gauche-init.scm */
 void load_gauche_init(void)
 {
@@ -267,7 +296,7 @@ void cleanup_main(void *data)
     if (profiling_mode) {
         Scm_ProfilerStop();
         Scm_EvalCString("(profiler-show)",
-                        SCM_OBJ(SCM_FIND_MODULE("gauche.vm.profiler", 0)),
+                        SCM_OBJ(Scm_GaucheModule()),
                         NULL); /* ignore errors */
     }
     
@@ -275,7 +304,7 @@ void cleanup_main(void *data)
     if (stats_mode) {
         fprintf(stderr, "\n;; Statistics (*: main thread only):\n");
         fprintf(stderr,
-                ";;  GC: %dbytes heap, %dbytes allocated\n",
+                ";;  GC: %zubytes heap, %zubytes allocated\n",
                 GC_get_heap_size(), GC_get_total_bytes());
         fprintf(stderr,
                 ";;  stack overflow*: %ldtimes, %.2fms total/%.2fms avg\n",
@@ -291,7 +320,7 @@ void cleanup_main(void *data)
         Scm_Eval(SCM_LIST2(SCM_INTERN("profiler-show-load-stats"),
                            SCM_LIST2(SCM_INTERN("quote"),
                                      vm->stat.loadStat)),
-                 SCM_OBJ(SCM_FIND_MODULE("gauche.vm.profiler", 0)),
+                 SCM_OBJ(Scm_GaucheModule()),
                  NULL);    /* ignore errors */
     }
 }
@@ -308,6 +337,30 @@ void error_exit(ScmObj c)
     Scm_Exit(1);
 }
 
+/* Returns FALSE if the process doesn't have a console. */
+static int init_console(void)
+{
+#if defined(GAUCHE_WINDOWS)
+#  if defined(GAUCHE_WINDOWS_NOCONSOLE)
+    close(0);               /* just in case */
+    close(1);               /* ditto */
+    close(2);               /* ditto */
+    open("NUL", O_RDONLY);
+    open("NUL", O_WRONLY);
+    open("NUL", O_WRONLY);
+    return FALSE;
+#  else /*!defined(GAUCHE_WINDOWS_NOCONSOLE)*/
+    /* This saves so much trouble */
+    _setmode(_fileno(stdin),  _O_BINARY);
+    _setmode(_fileno(stdout), _O_BINARY);
+    _setmode(_fileno(stderr), _O_BINARY);
+    return TRUE;
+#  endif /*!defined(GAUCHE_WINDOS_NOCONSOLE)*/
+#else  /*!defined(GAUCHE_WINDOWS)*/
+    return TRUE;
+#endif /*!defined(GAUCHE_WINDOWS)*/
+}
+    
 /*-----------------------------------------------------------------
  * MAIN
  */
@@ -320,36 +373,26 @@ int main(int argc, char **argv)
     int exit_code = 0;
     ScmEvalPacket epak;
     ScmLoadPacket lpak;
+    int has_console;
 
+    /* Initial setup. */
+    has_console = init_console();
     GC_INIT();
     Scm_Init(GAUCHE_SIGNATURE);
     sig_setup();
+#if defined(GAUCHE_WINDOWS)
+    /* Ugly... need some abstraction. */
+    Scm__SetupPortsForWindows(has_console);
+#endif /*defined(GAUCHE_WINDOWS)*/
 
+    /* Check command-line options */
     argind = parse_options(argc, argv);
 
     /* If -ftest option is given and we seem to be in the source
        tree, adds build directories to the library path _before_
        loading init file.   This is to help development of Gauche
        itself; normal user should never need this. */
-    if (test_mode) {
-        /* The order of directories is important.  'lib' should
-           be searched first (hence it should come latter), since some
-           extension modules are built from the file in src then linked
-           from lib, and we want to test the one in lib. */
-        if (access("../src/stdlib.stub", R_OK) == 0
-            && access("../libsrc/srfi-1.scm", R_OK) == 0
-            && access("../lib/srfi-0.scm", R_OK) == 0) {
-            Scm_AddLoadPath("../src", FALSE);
-            Scm_AddLoadPath("../libsrc", FALSE);
-            Scm_AddLoadPath("../lib", FALSE);
-        } else if (access("../../src/stdlib.stub", R_OK) == 0
-                   && access("../../libsrc/srfi-1.scm", R_OK) == 0
-                   && access("../../lib/srfi-0.scm", R_OK) == 0) {
-            Scm_AddLoadPath("../../src", FALSE);
-            Scm_AddLoadPath("../../libsrc", FALSE);
-            Scm_AddLoadPath("../../lib", FALSE);
-        }
-    }
+    if (test_mode) test_paths_setup();
 
     /* load init file */
     if (load_initfile) load_gauche_init();
@@ -422,8 +465,8 @@ int main(int argc, char **argv)
                             0, &lpak) < 0) {
                 error_exit(lpak.exception);
             }
-            Scm_ImportModules(SCM_CURRENT_MODULE(),
-                              SCM_LIST1(Scm_Intern(SCM_STRING(v))));
+            Scm_ImportModule(SCM_CURRENT_MODULE(), Scm_Intern(SCM_STRING(v)),
+                             SCM_FALSE, 0);
             break;
         case 'e':
             if (Scm_EvalCString(Scm_GetStringConst(SCM_STRING(v)),
@@ -496,8 +539,9 @@ int main(int argc, char **argv)
             if (Scm_Require(SCM_MAKE_STR("gauche/interactive"), 0, &lpak) < 0) {
                 Scm_Warn("couldn't load gauche.interactive\n");
             } else {
-                Scm_ImportModules(SCM_CURRENT_MODULE(),
-                                  SCM_LIST1(SCM_INTERN("gauche.interactive")));
+                Scm_ImportModule(SCM_CURRENT_MODULE(),
+                                 SCM_INTERN("gauche.interactive"),
+                                 SCM_FALSE, 0);
             }
         }
 
@@ -509,9 +553,9 @@ int main(int argc, char **argv)
         }
     }
 
-    /* All is done. */
+    /* All is done.  */
     Scm_Exit(exit_code);
-    return 0;
+    return 0;                   /*NOTREACHED*/
 }
 
 #if defined(MSVC)
