@@ -1,7 +1,7 @@
 /*
  * module.c - module implementation
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,12 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: module.c,v 1.71 2008-05-10 13:36:19 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/builtin-syms.h"
+#include "gauche/class.h"
 
 /*
  * Modules
@@ -47,7 +46,7 @@
  *
  *  Modules are registered to global hash table using their names
  *  as keys, so that the module is retrieved by its name.  The exception
- *  is "anonymous modules", which have '#' as the name field
+ *  is "anonymous modules", which have #f as the name field
  *  and not registered in the global table.   Anonymous modules are especially
  *  useful for certain applications that need temporary, segregated
  *  namespace---for example, a 'sandbox' environment to evaluate an
@@ -74,7 +73,14 @@
 
 static void module_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    Scm_Printf(port, "#<module %A>", SCM_MODULE(obj)->name);
+    if (SCM_MODULEP(SCM_MODULE(obj)->origin)) {
+        Scm_Printf(port, "#<module %A$%A @%p>",
+                   SCM_MODULE(obj)->name,
+                   SCM_MODULE(SCM_MODULE(obj)->origin)->name,
+                   obj);
+    } else {
+        Scm_Printf(port, "#<module %A>", SCM_MODULE(obj)->name);
+    }
 }
 
 SCM_DEFINE_BUILTIN_CLASS(Scm_ModuleClass,
@@ -83,12 +89,10 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_ModuleClass,
 
 /* Global module table */
 static struct {
-    ScmObj anon_name;       /* Name used for anonymous modules.
-                               Symbol '#', set by init */
     ScmHashTable *table;    /* Maps name -> module. */
     ScmInternalMutex mutex; /* Lock for table.  Only register_module and
                                lookup_module may hold the lock. */
-} modules = { SCM_UNBOUND, NULL };
+} modules;
 
 /* Predefined modules - slots will be initialized by Scm__InitModule */
 #define DEFINE_STATIC_MODULE(cname) \
@@ -108,7 +112,7 @@ static ScmObj defaultMpl =     SCM_NIL; /* will be initialized */
  * Constructor
  */
 
-static void init_module(ScmModule *m, ScmSymbol *name)
+static void init_module(ScmModule *m, ScmObj name)
 {
     m->name = name;
     m->imported = m->exported = m->depended = SCM_NIL;
@@ -116,10 +120,11 @@ static void init_module(ScmModule *m, ScmSymbol *name)
     m->parents = defaultParents;
     m->mpl = Scm_Cons(SCM_OBJ(m), defaultMpl);
     m->table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
+    m->origin = m->prefix = SCM_FALSE;
 }
 
 /* Internal */
-static ScmObj make_module(ScmSymbol *name)
+static ScmObj make_module(ScmObj name)
 {
     ScmModule *m;
     m = SCM_NEW(ScmModule);
@@ -148,7 +153,7 @@ static ScmModule *lookup_module_create(ScmSymbol *name, int *created)
                            (intptr_t)name,
                            SCM_DICT_CREATE);
     if (e->value == 0) {
-        (void)SCM_DICT_SET_VALUE(e, make_module(name));
+        (void)SCM_DICT_SET_VALUE(e, make_module(SCM_OBJ(name)));
         *created = TRUE;
     } else {
         *created = FALSE;
@@ -160,9 +165,8 @@ static ScmModule *lookup_module_create(ScmSymbol *name, int *created)
 ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists)
 {
     ScmObj r;
-    if (name == NULL) name = SCM_SYMBOL(modules.anon_name);
-    if (SCM_EQ(SCM_OBJ(name), modules.anon_name)) {
-        r = make_module(name);
+    if (name == NULL) {
+        r = make_module(SCM_FALSE);
     } else {
         int created;
         r = SCM_OBJ(lookup_module_create(name, &created));
@@ -178,27 +182,71 @@ ScmObj Scm_MakeModule(ScmSymbol *name, int error_if_exists)
     return r;
 }
 
+/* internal API to create an anonymous wrapper module */
+ScmObj Scm__MakeWrapperModule(ScmModule *origin, ScmObj prefix)
+{
+    ScmModule *m = SCM_MODULE(make_module(SCM_FALSE));
+    m->parents = SCM_LIST1(SCM_OBJ(origin));
+    m->mpl = Scm_Cons(SCM_OBJ(m), origin->mpl);
+    m->prefix = prefix;
+    while (SCM_MODULEP(origin->origin)) {
+        origin = SCM_MODULE(origin->origin);
+    }
+    m->origin = SCM_OBJ(origin);
+    return SCM_OBJ(m);
+}
+
 /*----------------------------------------------------------------------
  * Finding and modifying bindings
  */
 
 #define SEARCHED_ARRAY_SIZE  64
 
-ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
-                         int flags)
+/* Keep record of searched modules.  we use stack array for small # of
+   modules, in order to avoid consing for typical cases. */
+typedef struct {
+    int num_searched;
+    ScmObj searched[SEARCHED_ARRAY_SIZE];
+    ScmObj more_searched;
+} module_cache;
+
+static inline void init_module_cache(module_cache *c)
+{
+    c->num_searched = 0;
+    c->more_searched = SCM_NIL;
+}
+
+static inline int module_visited_p(module_cache *c, ScmObj m)
+{
+    int i;
+    for (i=0; i<c->num_searched; i++) {
+        if (SCM_EQ(m, c->searched[i])) return TRUE;
+    }
+    if (!SCM_NULLP(c->more_searched)) {
+        if (!SCM_FALSEP(Scm_Memq(m, c->more_searched))) return TRUE;
+    }
+    return FALSE;
+}
+
+static inline void module_add_visited(module_cache *c, ScmObj m)
+{
+    if (c->num_searched < SEARCHED_ARRAY_SIZE) {
+        c->searched[c->num_searched++] = m;
+    } else {
+        c->more_searched = Scm_Cons(m, c->more_searched);
+    }
+}
+
+ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol, int flags)
 {
     ScmModule *m = module;
     ScmObj v, p, mp;
     ScmGloc *gloc = NULL;
     int stay_in_module = flags&SCM_BINDING_STAY_IN_MODULE;
+    module_cache searched;
 
-    /* keep record of searched modules.  we use stack array for small # of
-       modules, in order to avoid consing for typical cases. */
-    ScmObj searched[SEARCHED_ARRAY_SIZE];
-    int num_searched = 0, i;
-    ScmObj more_searched = SCM_NIL;
-
-    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+    init_module_cache(&searched);
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
 
     /* first, search from the specified module.
        NB: we directly check gloc->value instead of calling
@@ -207,55 +255,62 @@ ScmGloc *Scm_FindBinding(ScmModule *module, ScmSymbol *symbol,
     v = Scm_HashTableRef(m->table, SCM_OBJ(symbol), SCM_FALSE);
     if (SCM_GLOCP(v)) {
         gloc = SCM_GLOC(v);
-        if (!SCM_UNBOUNDP(gloc->value)) goto found;
+        if (!SCM_GLOC_PHANTOM_BINDING_P(gloc)) goto out;
     }
+    if (stay_in_module) goto out;
     
-    if (!stay_in_module) {
-        /* Next, search from imported modules */
-        SCM_FOR_EACH(p, module->imported) {
-            SCM_ASSERT(SCM_MODULEP(SCM_CAR(p)));
-            SCM_FOR_EACH(mp, SCM_MODULE(SCM_CAR(p))->mpl) {
-                ScmGloc *g;
-                
-                SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
-                
-                for (i=0; i<num_searched; i++) {
-                    if (SCM_EQ(SCM_CAR(mp), searched[i])) goto skip;
-                }
-                if (!SCM_NULLP(more_searched)) {
-                    if (!SCM_FALSEP(Scm_Memq(SCM_CAR(mp), more_searched))) {
-                        goto skip;
-                    }
-                }
-                
-                m = SCM_MODULE(SCM_CAR(mp));
-                v = Scm_HashTableRef(m->table, SCM_OBJ(symbol), SCM_FALSE);
-                /* see above comment about the check of gloc->value */
-                if (SCM_GLOCP(v) && (g = SCM_GLOC(v))->exported
-                    && !SCM_UNBOUNDP(g->value)) {
-                    gloc = g;
-                    goto found;
-                }
+    /* Next, search from imported modules */
+    SCM_FOR_EACH(p, module->imported) {
+        ScmObj elt = SCM_CAR(p);
+        ScmModule *mod = NULL;
+        ScmObj sym = SCM_OBJ(symbol);
 
-                if (num_searched < SEARCHED_ARRAY_SIZE) {
-                    searched[num_searched++] = SCM_OBJ(m);
-                } else {
-                    more_searched = Scm_Cons(SCM_OBJ(m), more_searched);
+        SCM_ASSERT(SCM_MODULEP(elt));
+        mod = SCM_MODULE(elt);
+        
+        SCM_FOR_EACH(mp, mod->mpl) {
+            ScmGloc *g;
+                
+            SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
+
+            if (module_visited_p(&searched, SCM_CAR(mp))) continue;
+            m = SCM_MODULE(SCM_CAR(mp));
+            if (SCM_SYMBOLP(m->prefix)) {
+                sym = Scm_SymbolSansPrefix(SCM_SYMBOL(sym),
+                                           SCM_SYMBOL(m->prefix));
+                if (!SCM_SYMBOLP(sym)) goto skip;
+            }
+            
+            v = Scm_HashTableRef(m->table, SCM_OBJ(sym), SCM_FALSE);
+            /* see above comment about the check of gloc->value */
+            if (SCM_GLOCP(v)) {
+                g = SCM_GLOC(v);
+                if (g->hidden) break;
+                if (g->exported && !SCM_UNBOUNDP(g->value)) {
+                    gloc = g;
+                    goto out;
                 }
             }
-          skip:;
-        }
-        /* Then, search from parent modules */
-        SCM_ASSERT(SCM_PAIRP(module->mpl));
-        SCM_FOR_EACH(mp, SCM_CDR(module->mpl)) {
-            SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
-            m = SCM_MODULE(SCM_CAR(mp));
-            v = Scm_HashTableRef(m->table, SCM_OBJ(symbol), SCM_FALSE);
-            if (SCM_GLOCP(v)) { gloc = SCM_GLOC(v); goto found; }
+            module_add_visited(&searched, SCM_OBJ(m));
         }
+    skip:;
     }
-  found:
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+
+    /* Then, search from parent modules */
+    SCM_ASSERT(SCM_PAIRP(module->mpl));
+    SCM_FOR_EACH(mp, SCM_CDR(module->mpl)) {
+        SCM_ASSERT(SCM_MODULEP(SCM_CAR(mp)));
+        m = SCM_MODULE(SCM_CAR(mp));
+        if (SCM_SYMBOLP(m->prefix)) {
+            ScmObj sym = Scm_SymbolSansPrefix(symbol, SCM_SYMBOL(m->prefix));
+            if (!SCM_SYMBOLP(sym)) goto out;
+            symbol = SCM_SYMBOL(sym);
+        }
+        v = Scm_HashTableRef(m->table, SCM_OBJ(symbol), SCM_FALSE);
+        if (SCM_GLOCP(v)) { gloc = SCM_GLOC(v); goto out; }
+    }
+ out:
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
     return gloc;
 }
 
@@ -277,62 +332,30 @@ ScmObj Scm_GlobalVariableRef(ScmModule *module,
 
 /*
  * Definition.
- *  TODO: consolidate the common code between Scm_Define and Scm_DefineConst.
  */
-ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol, ScmObj value)
-{
-    ScmGloc *g;
-    ScmObj v;
-    int redefining = FALSE;
-    
-    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-    v = Scm_HashTableRef(module->table, SCM_OBJ(symbol), SCM_FALSE);
-    if (SCM_GLOCP(v)) {
-        g = SCM_GLOC(v);
-        if (SCM_GLOC_CONST_P(g)) {
-            redefining = TRUE;
-            g->setter = NULL;
-        }
-        SCM_GLOC_SET(g, value);
-    } else {
-        g = SCM_GLOC(Scm_MakeGloc(symbol, module));
-        SCM_GLOC_SET(g, value);
-        Scm_HashTableSet(module->table, SCM_OBJ(symbol), SCM_OBJ(g), 0);
-        /* If module is marked 'export-all', export this binding by default */
-        if (module->exportAll) {
-            g->exported = TRUE;
-            module->exported = Scm_Cons(SCM_OBJ(g->name), module->exported);
-        }
-    }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
-    
-    if (redefining) {
-        Scm_Warn("redefining constant %S::%S", g->module, g->name);
-    }
-    return SCM_OBJ(g);
-}
-
-ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
+ScmGloc *Scm_MakeBinding(ScmModule *module, ScmSymbol *symbol,
+                         ScmObj value, int flags)
 {
     ScmGloc *g;
     ScmObj v;
     ScmObj oldval = SCM_UNDEFINED;
-    int redefining = FALSE;
+    int prev_kind = 0;
+    int kind = ((flags&SCM_BINDING_CONST)
+                ? SCM_BINDING_CONST
+                : ((flags&SCM_BINDING_INLINABLE)
+                   ? SCM_BINDING_INLINABLE
+                   : 0));
 
-    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
     v = Scm_HashTableRef(module->table, SCM_OBJ(symbol), SCM_FALSE);
     /* NB: this function bypasses check of gloc setter */
     if (SCM_GLOCP(v)) {
         g = SCM_GLOC(v);
-        if (SCM_GLOC_CONST_P(g)) {
-            redefining = TRUE;
-            oldval = g->value;
-        }
-        g->setter = Scm_GlocConstSetter;
-        g->value  = value;
+        if (Scm_GlocConstP(g))          prev_kind = SCM_BINDING_CONST;
+        else if (Scm_GlocInlinableP(g)) prev_kind = SCM_BINDING_INLINABLE;
+        oldval = g->value;
     } else {
-        g = SCM_GLOC(Scm_MakeConstGloc(symbol, module));
-        g->value = value;
+        g = SCM_GLOC(Scm_MakeGloc(symbol, module));
         Scm_HashTableSet(module->table, SCM_OBJ(symbol), SCM_OBJ(g), 0);
         /* If module is marked 'export-all', export this binding by default */
         if (module->exportAll) {
@@ -340,37 +363,168 @@ ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
             module->exported = Scm_Cons(SCM_OBJ(g->name), module->exported);
         }
     }
-    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
 
-    if (redefining && !Scm_EqualP(value, oldval)) {
-        Scm_Warn("redefining constant %S::%S", g->module->name, g->name);
+    g->value = value;
+    Scm_GlocMark(g, kind);
+
+    if (prev_kind != 0) {
+        /* TODO: value and oldval may have circular structure, and we should
+           avoid diverging. */
+        if (prev_kind != kind || !Scm_EqualP(value, oldval)) {
+            Scm_Warn("redefining %s %S::%S",
+                     (prev_kind == SCM_BINDING_CONST)? "constant" : "inlinable",
+                     g->module->name, g->name);
+        }
     }
-    return SCM_OBJ(g);
+    return g;
 }
 
+/* Convenience wrapper (return value is ScmObj for the backward compatibility)*/
+ScmObj Scm_Define(ScmModule *module, ScmSymbol *symbol, ScmObj value)
+{
+    return SCM_OBJ(Scm_MakeBinding(module, symbol, value, 0));
+}
+
+ScmObj Scm_DefineConst(ScmModule *module, ScmSymbol *symbol, ScmObj value)
+{
+    return SCM_OBJ(Scm_MakeBinding(module, symbol, value, SCM_BINDING_CONST));
+}
+
+/*
+ * Injecting hidden binding
+ *   This inserts a dummy binding with hidden==true so that 
+ *   the module effectively removes the binding of the given symbol
+ *   inherited from parent.
+ *   This is not for genreral use.  It is intended to be used for
+ *   intermediate anonymous modules, created by import handling
+ *   routine to implement :except and :rename qualifiers.
+ */
+void Scm_HideBinding(ScmModule *module, ScmSymbol *symbol)
+{
+    ScmGloc *g;
+    ScmObj v;
+    int err_exists = FALSE;
+    
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+    v = Scm_HashTableRef(module->table, SCM_OBJ(symbol), SCM_FALSE);
+    if (!SCM_FALSEP(v)) {
+        err_exists = TRUE;
+    } else {
+        g = SCM_GLOC(Scm_MakeGloc(symbol, module));
+        g->hidden = TRUE;
+        Scm_HashTableSet(module->table, SCM_OBJ(symbol), SCM_OBJ(g), 0);
+    }
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+
+    if (err_exists) {
+        Scm_Error("hide-binding: binding already exists: %S", SCM_OBJ(symbol));
+    }
+}
+
+/*
+ * Binding aliasing
+ *   This is a special operation to realize :only and :rename import option.
+ *   The name ORIGINNAME is looked up in the module ORIGIN to get a gloc.
+ *   Then the gloc is directly inserted into the module TARGET under the name
+ *   TARGETNAME.
+ *   Since gloc is shared, subsequent changes in the binding are also shared.
+ *
+ *   If the original binding doesn't exist, or isn't exported, noop and
+ *   FALSE is returned.  Otherwise TRUE is returned.
+ *
+ *   CAVEATS:
+ *
+ *   - gloc's module remains the same.
+ *   - autoload won't resolved.
+ *   - TARGETNAME shouldn't be bound in TARGET beforehand.  We don't check
+ *     it and just insert the gloc.  If there is an existing binding,
+ *     it would become orphaned, possibly causing problems.
+ *
+ *   NB: This is the only operation that causes a gloc to be shared between
+ *   more than one modules.  I'm not yet clear on the implication of such
+ *   sharing in general, so this should be used with care.  At least it
+ *   won't cause much trouble if the target module is an implicit anonymous
+ *   module created by :only and :rename import options.
+ */
+int Scm_AliasBinding(ScmModule *target, ScmSymbol *targetName,
+                     ScmModule *origin, ScmSymbol *originName)
+{
+    ScmObj v;
+    ScmGloc *g = Scm_FindBinding(origin, originName, 0);
+
+    if (g == NULL || !(g->exported)) return FALSE;
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(modules.mutex);
+    Scm_HashTableSet(target->table, SCM_OBJ(targetName), SCM_OBJ(g), 0);
+    target->exported = Scm_Cons(SCM_OBJ(targetName), target->exported);
+    SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
+    return TRUE;
+}
+
+/*
+ * Import
+ */
+ScmObj Scm_ImportModule(ScmModule *module,
+                        ScmObj imported,
+                        ScmObj prefix,
+                        u_long flags) /* reserved for future use */
+{
+    ScmModule *imp = NULL;
+    ScmObj p;
+    if (SCM_MODULEP(imported)) {
+        imp = SCM_MODULE(imported);
+    } else if (SCM_SYMBOLP(imported)) {
+        imp = Scm_FindModule(SCM_SYMBOL(imported), 0);
+    } else if (SCM_IDENTIFIERP(imported)) {
+        imp = Scm_FindModule(SCM_IDENTIFIER(imported)->name, 0);
+    } else {
+        Scm_Error("module name or module required, but got %S", imported);
+    }
+
+    if (SCM_SYMBOLP(prefix)) {
+        imp = SCM_MODULE(Scm__MakeWrapperModule(imp, prefix));
+    }
+
+    /* Preallocate a pair, so that we won't call malloc during locking */
+    p = Scm_Cons(SCM_OBJ(imp), SCM_NIL);
+
+    /* Prepend imported module to module->imported list. */
+    (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
+    {
+        ScmObj ms, prev = p;
+        SCM_SET_CDR(p, module->imported);
+        /* Remove duplicate module, if any. */
+        SCM_FOR_EACH(ms, SCM_CDR(p)) {
+            ScmModule *m = SCM_MODULE(SCM_CAR(ms));
+            ScmObj b0 = SCM_MODULEP(m->origin)? m->origin : SCM_OBJ(m);
+            ScmObj b1 = SCM_MODULEP(imp->origin)? imp->origin : SCM_OBJ(imp);
+            if (!SCM_EQ(b0, b1)) {
+                prev = ms;
+                continue;
+            }
+            SCM_SET_CDR(prev, SCM_CDR(ms));
+            break;
+        }
+        module->imported = p;
+    }
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+
+    return module->imported;
+}
+
+/* Deprecated */
 ScmObj Scm_ImportModules(ScmModule *module, ScmObj list)
 {
     ScmObj lp;
-    ScmModule *mod;
-    ScmSymbol *name = NULL;
     SCM_FOR_EACH(lp, list) {
-        if (SCM_SYMBOLP(SCM_CAR(lp))) {
-            name = SCM_SYMBOL(SCM_CAR(lp));
-        } else if (SCM_IDENTIFIERP(SCM_CAR(lp))) {
-            name = SCM_IDENTIFIER(SCM_CAR(lp))->name;
-        } else {
-            Scm_Error("module name required, but got %S", SCM_CAR(lp));
-        }
-        mod = Scm_FindModule(name, 0);
-        (void)SCM_INTERNAL_MUTEX_LOCK(modules.mutex);
-        module->imported =
-            Scm_Cons(SCM_OBJ(mod),
-                     Scm_DeleteX(SCM_OBJ(mod), module->imported, SCM_CMP_EQ));
-        (void)SCM_INTERNAL_MUTEX_UNLOCK(modules.mutex);
+        Scm_ImportModule(module, SCM_CAR(lp), SCM_FALSE, 0);
     }
     return module->imported;
 }
 
+/*
+ * Export
+ */
 ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
 {
     ScmObj lp, syms, badsym = SCM_FALSE;
@@ -382,7 +536,7 @@ ScmObj Scm_ExportSymbols(ScmModule *module, ScmObj list)
     /* We used to do something like
      *  (set! (module-exports module)
      *        (delete-duplicates (union (module-exports module) list)))
-     * This is slow when we export lots of symbols.  As of 0.8.6,
+     * This was slow when we exported lots of symbols.  As of 0.8.6,
      * each GLOC has exported flag, so we can check whether a binding
      * is exported or not in O(1).   Module-exports list is kept
      * for backward compatibility.
@@ -465,12 +619,12 @@ ScmObj Scm_ExtendModule(ScmModule *module, ScmObj supers)
     }
     SCM_APPEND1(seqh, seqt, supers);
     module->parents = supers;
-    mpl = Scm_MonotonicMerge(SCM_OBJ(module), seqh);
+    mpl = Scm_MonotonicMerge1(seqh);
     if (SCM_FALSEP(mpl)) {
         Scm_Error("can't extend those modules simultaneously because of inconsistent precedence lists: %S", supers);
     }
-    module->mpl = mpl;
-    return mpl;
+    module->mpl = Scm_Cons(SCM_OBJ(module), mpl);
+    return module->mpl;
 }
 
 /*----------------------------------------------------------------------
@@ -563,6 +717,73 @@ ScmObj Scm_PathToModuleName(ScmString *path)
     return SCM_INTERN(buf);
 }
 
+/*----------------------------------------------------------------------
+ * Module introspection
+ */
+
+static ScmObj module_name(ScmObj m)
+{
+    return SCM_MODULE(m)->name;
+}
+
+static ScmObj module_imported(ScmObj m)
+{
+    return SCM_MODULE(m)->imported;
+}
+
+static ScmObj module_exported(ScmObj m)
+{
+    return SCM_MODULE(m)->exported;
+}
+
+static ScmObj module_exportAll(ScmObj m)
+{
+    return SCM_MAKE_BOOL(SCM_MODULE(m)->exportAll);
+}
+
+static ScmObj module_parents(ScmObj m)
+{
+    return SCM_MODULE(m)->parents;
+}
+
+static ScmObj module_mpl(ScmObj m)
+{
+    return SCM_MODULE(m)->mpl;
+}
+
+static ScmObj module_depended(ScmObj m)
+{
+    return SCM_MODULE(m)->depended;
+}
+
+static ScmObj module_table(ScmObj m)
+{
+    return SCM_OBJ(SCM_MODULE(m)->table);
+}
+
+static ScmObj module_origin(ScmObj m)
+{
+    return SCM_MODULE(m)->origin;
+}
+
+static ScmObj module_prefix(ScmObj m)
+{
+    return SCM_MODULE(m)->prefix;
+}
+
+static ScmClassStaticSlotSpec module_slots[] = {
+    SCM_CLASS_SLOT_SPEC("name", module_name, NULL),
+    SCM_CLASS_SLOT_SPEC("mpl", module_mpl, NULL),
+    SCM_CLASS_SLOT_SPEC("parents", module_parents, NULL),
+    SCM_CLASS_SLOT_SPEC("imports", module_imported, NULL),
+    SCM_CLASS_SLOT_SPEC("exports", module_exported, NULL),
+    SCM_CLASS_SLOT_SPEC("export-all", module_exportAll, NULL),
+    SCM_CLASS_SLOT_SPEC("table", module_table, NULL),
+    SCM_CLASS_SLOT_SPEC("depends", module_depended, NULL),
+    SCM_CLASS_SLOT_SPEC("origin", module_origin, NULL),
+    SCM_CLASS_SLOT_SPEC("prefix", module_prefix, NULL),
+    SCM_CLASS_SLOT_SPEC_END()
+};
 
 /*----------------------------------------------------------------------
  * Predefined modules and initialization
@@ -602,8 +823,8 @@ ScmModule *Scm_CurrentModule(void)
 #define INIT_MOD(mod, mname, mpl)                                           \
     do {                                                                    \
       SCM_SET_CLASS(&mod, SCM_CLASS_MODULE);                                \
-      init_module(&mod, SCM_SYMBOL(mname));                                 \
-      Scm_HashTableSet(modules.table, SCM_OBJ((mod).name), SCM_OBJ(&mod), 0);\
+      init_module(&mod, mname);                                             \
+      Scm_HashTableSet(modules.table, (mod).name, SCM_OBJ(&mod), 0);        \
       mod.parents = (SCM_NULLP(mpl)? SCM_NIL : SCM_LIST1(SCM_CAR(mpl)));    \
       mpl = mod.mpl = Scm_Cons(SCM_OBJ(&mod), mpl);                         \
     } while (0)
@@ -625,9 +846,17 @@ void Scm__InitModule(void)
     mpl = SCM_CDR(mpl);  /* default mpl doesn't include user module */
     defaultParents = SCM_LIST1(SCM_CAR(mpl));
     defaultMpl = mpl;
-    modules.anon_name = SCM_SYM_SHARP;
 
     /* other modules */
     mpl = defaultMpl;
     INIT_MOD(internalModule, SCM_SYM_GAUCHE_INTERNAL, mpl);
 }
+
+void Scm__InitModulePost(void)
+{
+    Scm_InitStaticClassWithMeta(&Scm_ModuleClass, "<module>", &gaucheModule,
+                                NULL, /* auto-generate meta */
+                                SCM_FALSE, /* calculate supers from cpl */
+                                module_slots,
+                                0);
+}
