@@ -1,7 +1,7 @@
 /*
  * portapi.c - port common API
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: portapi.c,v 1.33 2008-05-10 13:36:20 shirok Exp $
  */
 
 /* This file is included _twice_ by port.c to define safe- and unsafe-
@@ -940,11 +938,16 @@ ScmObj Scm_PortSeekUnsafe(ScmPort *p, ScmObj off, int whence)
             /* NB: possible optimization: the specified position is within
                the current buffer, we can avoid calling seeker. */
             if (SCM_PORT_DIR(p)&SCM_PORT_INPUT) {
+                char *c = p->src.buf.current; /* save current ptr */
                 if (whence == SEEK_CUR) {
-                    o -= (off_t)(p->src.buf.end - p->src.buf.current);
+                    o -= (off_t)(p->src.buf.end - c);
                 }
                 p->src.buf.current = p->src.buf.end; /* invalidate buffer */
                 SAFE_CALL(p, r = p->src.buf.seeker(p, o, whence));
+                if (r == (off_t)-1) {
+                    /* This may happend if seeker somehow gave up */
+                    p->src.buf.current = c;
+                }
             } else {
                 SAFE_CALL(p, bufport_flush(p, 0, TRUE));
                 SAFE_CALL(p, r = p->src.buf.seeker(p, o, whence));
