@@ -1,7 +1,7 @@
 /*
  * proc.c - Procedures
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: proc.c,v 1.47 2008-05-10 13:36:20 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -482,7 +480,12 @@ static ScmObj proc_required(ScmProcedure *p)
 
 static ScmObj proc_optional(ScmProcedure *p)
 {
-    return SCM_MAKE_BOOL(p->optional);
+    return SCM_MAKE_BOOL(p->optional); /* for backward compatibility */
+}
+
+static ScmObj proc_optcount(ScmProcedure *p)
+{
+    return SCM_MAKE_INT(p->optional);
 }
 
 static ScmObj proc_locked(ScmProcedure *p)
@@ -495,6 +498,11 @@ static ScmObj proc_currying(ScmProcedure *p)
     return SCM_MAKE_BOOL(p->currying);
 }
 
+static ScmObj proc_constant(ScmProcedure *p)
+{
+    return SCM_MAKE_BOOL(p->constant);
+}
+
 static ScmObj proc_info(ScmProcedure *p)
 {
     return p->info;
@@ -508,8 +516,10 @@ static ScmObj proc_setter(ScmProcedure *p)
 static ScmClassStaticSlotSpec proc_slots[] = {
     SCM_CLASS_SLOT_SPEC("required", proc_required, NULL),
     SCM_CLASS_SLOT_SPEC("optional", proc_optional, NULL),
+    SCM_CLASS_SLOT_SPEC("optcount", proc_optcount, NULL),
     SCM_CLASS_SLOT_SPEC("locked", proc_locked, NULL),
     SCM_CLASS_SLOT_SPEC("currying", proc_currying, NULL),
+    SCM_CLASS_SLOT_SPEC("constant", proc_constant, NULL),
     SCM_CLASS_SLOT_SPEC("info", proc_info, NULL),
     SCM_CLASS_SLOT_SPEC("setter", proc_setter, NULL),
     SCM_CLASS_SLOT_SPEC_END()
