@@ -1,7 +1,7 @@
 /*
  * read.c - reader
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: read.c,v 1.99 2008-05-10 13:36:20 shirok Exp $
  */
 
 #include <stdio.h>
@@ -60,7 +58,7 @@ static ScmObj read_word(ScmPort *port, ScmChar initial, ScmReadContext *ctx,
 static ScmObj read_symbol(ScmPort *port, ScmChar initial, ScmReadContext *ctx);
 static ScmObj read_number(ScmPort *port, ScmChar initial, ScmReadContext *ctx);
 static ScmObj read_symbol_or_number(ScmPort *port, ScmChar initial, ScmReadContext *ctx);
-static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim);
+static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim, int interned);
 static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx);
 static ScmObj read_regexp(ScmPort *port);
 static ScmObj read_charset(ScmPort *port);
@@ -179,7 +177,6 @@ ScmObj Scm_ReadList(ScmObj port, ScmChar closer)
 
 ScmReadContext *Scm_MakeReadContext(ScmReadContext *proto)
 {
-    ScmVM *vm = Scm_VM();
     ScmReadContext *ctx = SCM_NEW(ScmReadContext);
     SCM_SET_CLASS(ctx, SCM_CLASS_READ_CONTEXT);
     if (proto == NULL) {
@@ -395,27 +392,37 @@ static void read_nested_comment(ScmPort *port, ScmReadContext *ctx)
     }
 }
 
+static void read_comment(ScmPort *port) /* leading semicolon is already read */
+{
+    for (;;) {
+        /* NB: comment may contain unexpected character code.
+           for the safety, we read bytes here. */
+        int c = Scm_GetbUnsafe(port);
+        if (c == '\n') {
+            /* oops.  ugly. */
+            port->line++;
+            break;
+        }
+        if (c == EOF) break;
+    }
+}
+
 static int skipws(ScmPort *port, ScmReadContext *ctx)
 {
+    ScmChar c0;
+    
     for (;;) {
         int c = Scm_GetcUnsafe(port);
         if (c == EOF) return c;
-        if (c <= 256 && isspace(c)) continue;
-        if (c == ';') {
-            for (;;) {
-                /* NB: comment may contain unexpected character code.
-                   for the safety, we read bytes here. */
-                c = Scm_GetbUnsafe(port);
-                if (c == '\n') {
-                    /* oops.  ugly. */
-                    port->line++;
-                    break;
-                }
-                if (c == EOF) return EOF;
+        if (c <= 127) {
+            if (isspace(c)) continue;
+            if (c == ';') {
+                read_comment(port);
+                continue;
             }
-            continue;
+            return c;
         }
-        return c;
+        else if (!SCM_CHAR_EXTRA_WHITESPACE(c)) return c;
     }
 }
 
@@ -502,6 +509,17 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
                     if (c2 == '"') return read_string(port, TRUE, ctx);
                     Scm_ReadError(port, "unsupported #*-syntax: #*%C", c2);
                 }
+            case ':':
+                /* #:name - uninterned symbol */
+                {
+                    int c2 = Scm_GetcUnsafe(port);
+                    if (c2 == '|') {
+                        return read_escaped_symbol(port, c2, FALSE);
+                    } else {
+                        ScmObj name = read_word(port, c2, ctx, FALSE);
+                        return Scm_MakeSymbol(SCM_STRING(name), FALSE);
+                    }
+                }
             case ';':
                 /* #;expr - comment out sexpr */
                 {
@@ -532,7 +550,7 @@ static ScmObj read_internal(ScmPort *port, ScmReadContext *ctx)
             }
         }
     case '|':
-        return read_escaped_symbol(port, '|');
+        return read_escaped_symbol(port, '|', TRUE);
     case '[':
         /* TODO: make it customizable */
         return read_list(port, ']', ctx);
@@ -601,9 +619,10 @@ static ScmObj read_list_int(ScmPort *port, ScmChar closer,
                 goto baddot;
             } else if (c2 == EOF) {
                 goto eoferr;
-            } else if (isspace(c2)) {
-                /* dot pair at the end */
+            } else if (!char_word_constituent(c2)) {
+                /* can be a dot pair at the end */
                 if (start == SCM_NIL) goto baddot;
+                Scm_UngetcUnsafe(c2, port);
                 item = read_item(port, ctx);
                 if (SCM_READ_REFERENCE_P(item)) ref_seen = TRUE;
                 SCM_SET_CDR(last, item);
@@ -737,6 +756,8 @@ static ScmObj read_string(ScmPort *port, int incompletep,
 #define ACCUMULATE(var)                                 \
     if (incompletep) { SCM_DSTRING_PUTB(&ds, var); }    \
     else             { SCM_DSTRING_PUTC(&ds, var); }
+#define INTRALINE_WS(var)                               \
+    ((var)==' ' || (var)=='\t' || SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(var))
 
     for (;;) {
         FETCH(c);
@@ -774,6 +795,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                 break;
             }
                 /* R6RS-style line continuation handling*/
+            line_continuation:
             case ' ':
             case '\t': {
                 for (;;) {
@@ -781,7 +803,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                     if (c == EOF) goto eof_exit;
                     if (c == '\r') goto cont_cr;
                     if (c == '\n') goto cont_lf;
-                    if (c != ' ' && c != '\t') goto cont_err;
+                    if (!INTRALINE_WS(c)) goto cont_err;
                 }
             }
                 /*FALLTHROUGH*/
@@ -790,7 +812,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                 FETCH(c);
                 if (c == EOF)  goto eof_exit;
                 if (c == '\\') goto backslash;
-                if (c != '\n' && c != ' ' && c != '\t') {
+                if (c != '\n' && !INTRALINE_WS(c)) {
                     ACCUMULATE(c);
                     break;
                 }
@@ -802,7 +824,7 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                     FETCH(c);
                     if (c == EOF) goto eof_exit;
                     if (c == '\\') goto backslash;
-                    if (c != ' ' && c != '\t') {
+                    if (!INTRALINE_WS(c)) {
                         ACCUMULATE(c);
                         break;
                     }
@@ -810,10 +832,11 @@ static ScmObj read_string(ScmPort *port, int incompletep,
                 break;
             }
             default:
-                ACCUMULATE(c); break;
+                if (SCM_CHAR_EXTRA_WHITESPACE_INTRALINE(c)) goto line_continuation;
+                else ACCUMULATE(c);
             }
             break;
-        }       
+        } 
         default: ACCUMULATE(c); break;
         }
     }
@@ -833,23 +856,26 @@ static ScmObj read_string(ScmPort *port, int incompletep,
 
 static struct char_name {
     const char *name;
+    int size;
     ScmObj ch;
 } char_names[] = {
-    { "space",        SCM_MAKE_CHAR(' ')  },
-    { "newline",      SCM_MAKE_CHAR('\n') },
-    { "nl",           SCM_MAKE_CHAR('\n') },
-    { "lf",           SCM_MAKE_CHAR('\n') },
-    { "return",       SCM_MAKE_CHAR('\r') },
-    { "cr",           SCM_MAKE_CHAR('\r') },
-    { "tab",          SCM_MAKE_CHAR('\t') },
-    { "ht",           SCM_MAKE_CHAR('\t') },
-    { "page",         SCM_MAKE_CHAR('\f') },
-    { "escape",       SCM_MAKE_CHAR(0x1b) },
-    { "esc",          SCM_MAKE_CHAR(0x1b) },
-    { "delete",       SCM_MAKE_CHAR(0x7f) },
-    { "del",          SCM_MAKE_CHAR(0x7f) },
-    { "null",         SCM_MAKE_CHAR(0)    },
-    { NULL, 0 }
+#define DEFCHAR(name, char) \
+    { #name, sizeof(#name)-1, SCM_MAKE_CHAR(char) }
+    DEFCHAR(space,   ' '),
+    DEFCHAR(newline, '\n'),
+    DEFCHAR(nl,      '\n'),
+    DEFCHAR(lf,      '\n'),
+    DEFCHAR(return,  '\r'),
+    DEFCHAR(cr,      '\r'),
+    DEFCHAR(tab,     '\t'),
+    DEFCHAR(ht,      '\t'),
+    DEFCHAR(page,    '\f'),
+    DEFCHAR(escape,  0x1b),
+    DEFCHAR(esc,     0x1b),
+    DEFCHAR(delete,  0x7f),
+    DEFCHAR(del,     0x7f),
+    DEFCHAR(null,    0),
+    { NULL, 0, 0 }
 };
 
 static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
@@ -897,7 +923,10 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
         }
 
         while (cntab->name) {
-            if (strncmp(cntab->name, cname, namesize) == 0) return cntab->ch;
+            if (cntab->size == namesize
+                && strncmp(cntab->name, cname, namesize) == 0) {
+                return cntab->ch;
+            }
             cntab++;
         }
       unknown:
@@ -965,11 +994,20 @@ static ScmObj read_symbol_or_number(ScmPort *port, ScmChar initial, ScmReadConte
 
 static ScmObj read_keyword(ScmPort *port, ScmReadContext *ctx)
 {
-    ScmString *s = SCM_STRING(read_word(port, SCM_CHAR_INVALID, ctx, FALSE));
-    return Scm_MakeKeyword(s);
+    int c2 = Scm_GetcUnsafe(port);
+    ScmObj name;
+    
+    if (c2 == '|') {
+        name = read_escaped_symbol(port, c2, FALSE); /* read as uninterned */
+        return Scm_MakeKeyword(SCM_SYMBOL_NAME(name));
+    } else {
+        Scm_UngetcUnsafe(c2, port);
+        name = read_word(port, SCM_CHAR_INVALID, ctx, FALSE);
+        return Scm_MakeKeyword(SCM_STRING(name));
+    }
 }
 
-static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim)
+static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim, int interned)
 {
     int c = 0;
     ScmDString ds;
@@ -981,7 +1019,7 @@ static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim)
             goto err;
         } else if (c == delim) {
             ScmString *s = SCM_STRING(Scm_DStringGet(&ds, 0));
-            return Scm_Intern(s);
+            return Scm_MakeSymbol(s, interned);
         } else if (c == '\\') {
             /* CL-style single escape */
             c = Scm_GetcUnsafe(port);
@@ -1105,7 +1143,12 @@ static ScmObj read_reference(ScmPort *port, ScmChar ch, ScmReadContext *ctx)
  * SRFI-10 support
  */
 
-ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher)
+/* NB: The 'module' argument of DefineReaderCtor and GetReaderCtor may
+   be used in future to make reader-ctor binding associated with modules.
+   For now, it is not used and the caller should pass SCM_FALSE. */
+
+ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher,
+                            ScmObj module /*reserved*/)
 {
     ScmObj pair;
     if (!SCM_PROCEDUREP(proc)) {
@@ -1118,6 +1161,15 @@ ScmObj Scm_DefineReaderCtor(ScmObj symbol, ScmObj proc, ScmObj finisher)
     return SCM_UNDEFINED;
 }
 
+ScmObj Scm_GetReaderCtor(ScmObj symbol, ScmObj module /*reserved*/)
+{
+    ScmObj r;
+    (void)SCM_INTERNAL_MUTEX_LOCK(readCtorData.mutex);
+    r = Scm_HashTableRef(readCtorData.table, symbol, SCM_FALSE);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(readCtorData.mutex);
+    return r;
+}
+
 static ScmObj read_sharp_comma(ScmPort *port, ScmReadContext *ctx)
 {
     int len, has_ref, line=-1;
@@ -1161,7 +1213,7 @@ static ScmObj process_sharp_comma(ScmPort *port, ScmObj key, ScmObj args,
 static ScmObj reader_ctor(ScmObj *args, int nargs, void *data)
 {
     ScmObj optarg = (nargs > 2? args[2] : SCM_FALSE);
-    return Scm_DefineReaderCtor(args[0], args[1], optarg);
+    return Scm_DefineReaderCtor(args[0], args[1], optarg, SCM_FALSE);
 }
 
 /*----------------------------------------------------------------
@@ -1201,8 +1253,7 @@ static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
         /*NOTREACHED*/
     } else {
         ScmObj id = read_symbol(port, c2, ctx), e, r;
-        if (SCM_EOFP(id)) return SCM_EOF; /* we may want warn? */
-        
+        SCM_ASSERT(SCM_SYMBOLP(id));
         (void)SCM_INTERNAL_MUTEX_LOCK(hashBangData.mutex);
         e = Scm_HashTableRef(hashBangData.table, id, SCM_FALSE);
         (void)SCM_INTERNAL_MUTEX_UNLOCK(hashBangData.mutex);
@@ -1210,16 +1261,15 @@ static ScmObj read_shebang(ScmPort *port, ScmReadContext *ctx)
             Scm_Warn("Ignoring unrecognized hash-bang directive: #!%S", id);
             return SCM_UNDEFINED;
         }
-        /* Reader directive may return zero or one value. */
+        /* Reader directive may return zero or one value.  When it returns
+           no values, we call Scm_VMSetResult to adjust the number of values.
+         */
         r = Scm_ApplyRec3(e, id, SCM_OBJ(port), SCM_OBJ(ctx));
-        if (Scm_VMGetNumResults(Scm_VM()) == 0) return SCM_UNDEFINED;
-        else                                    return r;
+        if (Scm_VMGetNumResults(Scm_VM()) == 1) return r;
+        else { Scm_VMSetResult(SCM_UNDEFINED); return SCM_UNDEFINED; }
     }
 }
 
-
-
-
 /*----------------------------------------------------------------
  * Uvector
  */
@@ -1299,7 +1349,7 @@ void Scm__InitRead(void)
     Scm_DefineReaderCtor(SCM_SYM_DEFINE_READER_CTOR,
                          Scm_MakeSubr(reader_ctor, NULL, 2, 1,
                                       SCM_SYM_DEFINE_READER_CTOR),
-                         SCM_FALSE);
+                         SCM_FALSE, SCM_FALSE);
 
     hashBangData.table =
         SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 0));
