@@ -1,7 +1,7 @@
 ;;;
 ;;; scmlib.scm - more Scheme libraries
 ;;;
-;;;   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+;;;   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,8 +30,6 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: scmlib.scm,v 1.20 2008-05-13 05:44:15 shirok Exp $
-;;;
 
 ;; This file contains builtin library functions that are easier to be
 ;; written in Scheme instead of as stubs.
@@ -40,7 +38,7 @@
 (select-module gauche)
 
 ;;;=======================================================
-;;; Optional and keyword arguments
+;;; Extending procedures
 ;;;
 
 ;; Extended lambda formals (:optional, :key, :rest etc) are
@@ -53,21 +51,28 @@
 
 ;; KLUDGE: We need these to include the compiled macro transformer
 ;; in the binary.  Will be gone in future.
-(export let-keywords let-keywords* let-optionals*)
+(export let-keywords let-keywords* let-optionals* define-compiler-macro)
 
 (define-macro (let-optionals* arg specs . body)
   (define (rec arg vars&inits rest)
-    (if (null? vars&inits)
-      (if (null? rest) body `((let ((,rest ,arg)) ,@body)))
-      (let ((g (gensym))
-            (v (caar vars&inits))
-            (i (cdar vars&inits)))
+    (cond
+     [(null? (cdr vars&inits))
+      `((let ((,(caar vars&inits)
+               (if (null? ,arg) ,(cdar vars&inits) (car ,arg)))
+              ,@(if (null? rest)
+                  '()
+                  `((,rest (if (null? ,arg) '() (cdr ,arg))))))
+          ,@body))]
+     [else
+      (let ([g (gensym)]
+            [v (caar vars&inits)]
+            [i (cdar vars&inits)])
         ;; NB: if the compiler were more clever, we could use receive
         ;; or apply to make (null? ,arg) test once.  For now, testing it
         ;; twice is faster.
         `((let ((,v (if (null? ,arg) ,i (car ,arg)))
                 (,g (if (null? ,arg) '() (cdr ,arg))))
-            ,@(rec g (cdr vars&inits) rest))))))
+            ,@(rec g (cdr vars&inits) rest))))]))
   (let1 g (gensym)
     `(let ((,g ,arg))
        ,@(rec g (map (lambda (s)
@@ -85,7 +90,7 @@
   (%let-keywords-rec arg specs body 'let 'errorf))
 
 (define-macro (let-keywords* arg specs . body)
-  (%let-keywords-rec arg specs body 'let* 'warn))
+  (%let-keywords-rec arg specs body 'let* 'errorf))
 
 (define (%let-keywords-rec arg specs body %let %error/warn)
   (define (triplet var&default)
@@ -100,6 +105,9 @@
                          (unwrap-syntax (cadr var&default))
                          (caddr var&default))]
             [else #f]))
+        (and-let* ([var (unwrap-syntax var&default)]
+                   [ (symbol? var) ])
+          (values var (make-keyword var) (undefined)))
         (error "bad binding form in let-keywords" var&default)))
   (define (process-specs specs)
     (let loop ((specs specs)
@@ -162,6 +170,50 @@
               ]))))
   )
 
+(define (delete-keywords ks kvlist)
+  (define (rec kvs)
+    (cond [(null? kvs) '()]
+          [(null? (cdr kvs)) (error "incomplete key list" kvlist)]
+          [(memv (car kvs) ks) (rec (cddr kvs))]
+          [else (list* (car kvs) (cadr kvs) (rec (cddr kvs)))]))
+  (rec kvlist))
+
+(define (delete-keywords! ks kvlist)
+  (define (head kvs)
+    (cond [(null? kvs) '()]
+          [(null? (cdr kvs)) (error "incomplete key list" kvlist)]
+          [(memv (car kvs) ks) (head (cddr kvs))]
+          [else (cut-tail! (cddr kvs) kvs) kvs]))
+  (define (cut-tail! kvs prev)
+    (cond [(null? kvs)]
+          [(null? (cdr kvs)) (error "incomplete key list" kvlist)]
+          [(memv (car kvs) ks)
+           (set-cdr! (cdr prev) (cddr kvs))
+           (cut-tail! (cddr kvs) prev)]
+          [else (cut-tail! (cddr kvs) kvs)]))
+  (head kvlist))
+
+;; Tentative compiler macro.
+;;
+;;  (define-compiler-macro <name>
+;;    (er-transformer
+;;     (lambda (form rename compare) ...)))
+;;
+;; Er-transformer is a wrapper to indicate the transformer is
+;; explicit-renaming.  It leaves room to support other type of macro
+;; transformers in future.
+;; The transformer itself must return <FORM> itself if it aborts
+;; expansion.
+
+(define-macro (define-compiler-macro name xformer-spec)
+  ;; TODO: Rewrite this after we get builtin patter matching.
+  (unless (and (= (length xformer-spec) 2)
+               (eq? (unwrap-syntax (car xformer-spec)) 'er-transformer))
+    (error "malformed define-compiler-macro: "
+           `(define-compiler-macro ,name ,xformer-spec)))
+  `((with-module gauche.internal %bind-inline-er-transformer)
+    (current-module) ',name ,(cadr xformer-spec)))
+
 ;;;=======================================================
 ;;; Inline stub and declarations
 ;;;
@@ -171,13 +223,14 @@
 ;; pre-compiled into C.  Technically we can kick C compiler at
 ;; runtime, but it'll need some more work, so we signal an error
 ;; when inline-stub form is evaluated at runtime.  The precompiler
-;; (gencomp) handles this form specially.
+;; (precomp) handles this form specially.
 
 (define-macro (inline-stub . _)
   (warn "The inline-stub form can only be used for Scheme source \
          to be pre-compiled.  Since you're loading the file without \
          pre-compilation, the definitions and expressions in the \
-         inline-stub form are ignored."))
+         inline-stub form are ignored.  (Current module=~s)"
+        (current-module)))
 
 ;; The form (declare ...) may be used in wider purpose.  For the time
 ;; being we use it in limited purposes for compilers.  In interpreter
@@ -231,39 +284,53 @@
 ;; (hence we need to define here)
 
 (define-inline (null-list? l)
-  (cond ((null? l))
-        ((pair? l) #f)
-        (else (error "argument must be a list, but got:" l))))
+  (cond [(null? l)]
+        [(pair? l) #f]
+        [else (error "argument must be a list, but got:" l)]))
 
 (with-module gauche.internal
   (define (%zip-nary-args arglists . seed)
-    (let loop ((as arglists)
-               (cars '())
-               (cdrs '()))
-      (cond ((null? as)
+    (let loop ([as arglists]
+               [cars '()]
+               [cdrs '()])
+      (cond [(null? as)
              (values (reverse! (if (null? seed) cars (cons (car seed) cars)))
-                     (reverse! cdrs)))
-            ((null? (car as)) (values #f #f)) ;;exhausted
-            ((pair? (car as))
-             (loop (cdr as) (cons (caar as) cars) (cons (cdar as) cdrs)))
-            (else
+                     (reverse! cdrs))]
+            [(null? (car as)) (values #f #f)] ;;exhausted
+            [(pair? (car as))
+             (loop (cdr as) (cons (caar as) cars) (cons (cdar as) cdrs))]
+            [else
              (error "argument lists contained an improper list ending with:"
-                    (car as))))))
+                    (car as))])))
   )
 
 (define (any pred lis . more)
   (if (null? more)
     (and (not (null-list? lis))
          (let loop ((head (car lis)) (tail (cdr lis)))
-           (cond ((null-list? tail) (pred head)) ; tail call
-                 ((pred head))
-                 (else (loop (car tail) (cdr tail))))))
+           (cond [(null-list? tail) (pred head)] ; tail call
+                 [(pred head)]
+                 [else (loop (car tail) (cdr tail))])))
     (let loop ((liss (cons lis more)))
       (receive (cars cdrs)
           ((with-module gauche.internal %zip-nary-args) liss)
-        (cond ((not cars) #f)
-              ((apply pred cars))
-              (else (loop cdrs)))))))
+        (cond [(not cars) #f]
+              [(apply pred cars)]
+              [else (loop cdrs)])))))
+
+(define (every pred lis . more)
+  (if (null? more)
+    (or (null-list? lis)
+        (let loop ((head (car lis)) (tail (cdr lis)))
+          (cond [(null-list? tail) (pred head)] ; tail call
+                [(not (pred head)) #f]
+                [else (loop (car tail) (cdr tail))])))
+    (let loop ((liss (cons lis more)))
+      (receive (cars cdrs)
+          ((with-module gauche.internal %zip-nary-args) liss)
+        (cond [(not cars)]
+              [(not (apply pred cars)) #f]
+              [else (loop cdrs)])))))
 
 (define (fold kons knil lis . more)
   (if (null? more)
@@ -289,17 +356,51 @@
           (apply kons (append! cars (list (rec cdrs))))
           knil)))))
 
+(with-module gauche.internal
+  (define (%map*-1 fn tail-fn lis)
+    (if (pair? lis)
+      (cons (fn (car lis)) (%map*-1 fn tail-fn (cdr lis)))
+      (tail-fn lis)))
+  (define (%map*-n fn tail-fn liss)
+    (if (every pair? liss)
+      (receive (cars cdrs) (%zip-nary-args liss)
+        (cons (apply fn cars) (%map*-n fn tail-fn cdrs)))
+      (apply tail-fn liss))))
+
+(define (map* fn tail-fn lis . more)
+  (if (null? more)
+    ((with-module gauche.internal %map*-1) fn tail-fn lis)
+    ((with-module gauche.internal %map*-n) fn tail-fn (cons lis more))))
+
+;; TODO: The definition of map* can be as simple as follows if we use
+;; 2-argument reverse to be in 0.9.2.  Since scmlib.scm needs to be compiled
+;; with 0.9.1 compiler which inlines reverse, we need to postpone rewriting
+;; after 0.9.2 release.
+;;
+;; (define (map* fn tail-fn lis . more)
+;;   (if (null? more)
+;;     (let rec ((xs lis) (rs '()))
+;;       (if (pair? xs)
+;;         (rec (cdr xs) (cons (fn (car xs)) rs))
+;;         (reverse rs (tail-fn xs))))
+;;     (let rec ((xss (cons lis more)) (rs '()))
+;;       (if (every pair? xss)
+;;         (receive (cars cdrs) ((with-module gauche.internal %zip-nary-args xss))
+;;           (rec cdrs (cons (apply fn cars) rs)))
+;;         (reverse rs (apply tail-fn xss))))))
+
+
 (define (find pred lis)
   (let loop ((lis lis))
-    (cond ((not (pair? lis)) #f)
-          ((pred (car lis)) (car lis))
-          (else (loop (cdr lis))))))
+    (cond [(not (pair? lis)) #f]
+          [(pred (car lis)) (car lis)]
+          [else (loop (cdr lis))])))
 
 (define (split-at lis i)
   (let loop ((i i) (rest lis) (r '()))
-    (cond ((= i 0) (values (reverse! r) rest))
-          ((null? rest) (error "given list is too short:" lis))
-          (else (loop (- i 1) (cdr rest) (cons (car rest) r))))))
+    (cond [(= i 0) (values (reverse! r) rest)]
+          [(null? rest) (error "given list is too short:" lis)]
+          [else (loop (- i 1) (cdr rest) (cons (car rest) r))])))
 
 ;;;=======================================================
 ;;; string stuff
@@ -314,7 +415,7 @@
   (check-arg integer? k)
   (check-arg exact? k)
   (check-arg char? ch)
-  (let ((len (string-length str)))
+  (let1 len (string-length str)
     (when (or (< k 0) (<= len k))
       (error "string index out of range:" k))
     (%string-replace-body! str
@@ -329,8 +430,8 @@
   (check-arg integer? k)
   (check-arg exact? k)
   (check-arg integer? b)
-  (let ((siz (string-size str))
-        (out (open-output-string :private? #t)))
+  (let ([siz (string-size str)]
+        [out (open-output-string :private? #t)])
     (when (or (< k 0) (<= siz k))
       (error "string index out of range:" k))
     (display (byte-substring str 0 k) out)
@@ -340,24 +441,20 @@
 
 (set! (setter string-byte-ref) string-byte-set!)
 
-(define (string-fill! str c . opts)
-  (check-arg string? str)
-  (check-arg char? c)
+(define (string-fill! str c :optional (start 0) (end (string-length str)))
   (let1 len (string-length str)
-    (let-optionals* opts ((start 0)
-                          (end   len))
-      (when (or (< start 0) (< len start))
-        (error "start index out of range:" start))
-      (when (or (< end 0) (< len end))
-        (error "end index out of range:" end))
-      (when (< end start)
-        (errorf "end index ~s is smaller than start index ~s" end start))
-      (if (and (= start 0) (= end len))
-        (%string-replace-body! str (make-string len c))
-        (%string-replace-body! str
-                               (string-append (substring str 0 start)
-                                              (make-string (- end start) c)
-                                              (substring str end len)))))))
+    (when (or (< start 0) (< len start))
+      (error "start index out of range:" start))
+    (when (or (< end 0) (< len end))
+      (error "end index out of range:" end))
+    (when (< end start)
+      (errorf "end index ~s is smaller than start index ~s" end start))
+    (if (and (= start 0) (= end len))
+      (%string-replace-body! str (make-string len c))
+      (%string-replace-body! str
+                             (string-append (substring str 0 start)
+                                            (make-string (- end start) c)
+                                            (substring str end len))))))
 
 (define-reader-ctor 'string-interpolate
   (lambda (s) (string-interpolate s))) ;;lambda is required to delay loading
@@ -372,15 +469,11 @@
 ;;;
 (define-values (error errorf)
   (let ()
-    (define (compose-error-message msg args) ;; srfi-23 style message
+    (define (mkmsg msg args) ;; srfi-23 style message
       (let1 p (open-output-string)
         (display msg p)
-        (let loop ((args args))
-          (if (null? args)
-            (get-output-string p)
-            (begin (display " " p)
-                   (write/ss (car args) p)
-                   (loop (cdr args)))))))
+        (dolist [obj args] (display " " p) (write/ss obj p))
+        (get-output-string p)))
     (define (scan-keys args)
       (let loop ((args args)
                  (keys '()))
@@ -393,26 +486,22 @@
     (define (error msg . args)
       (raise
        (cond
-        ((is-a? msg <condition-meta>)
+        [(is-a? msg <condition-meta>)
          (receive (keys msgs) (scan-keys args)
            (if (null? msgs)
              (apply make msg keys)
-             (apply make msg
-                    :message (compose-error-message (car msgs) (cdr msgs))
-                    keys))))
-        (else (make <error> :message (compose-error-message msg args))))))
+             (apply make msg :message (mkmsg (car msgs) (cdr msgs)) keys)))]
+        [else (make <error> :message (mkmsg msg args))])))
 
     (define (errorf fmt . args)
       (raise
        (cond
-        ((is-a? fmt <condition-meta>)
+        [(is-a? fmt <condition-meta>)
          (receive (keys msgs) (scan-keys args)
            (if (null? msgs)
              (apply make fmt keys)
-             (apply make fmt
-                    :message (apply format/ss #f msgs)
-                    keys))))
-        (else (make <error> :message (apply format/ss #f fmt args))))))
+             (apply make fmt :message (apply format/ss #f msgs) keys)))]
+        [else (make <error> :message (apply format/ss #f fmt args))])))
 
     (values error errorf)))
 
@@ -427,18 +516,17 @@
     (%vm-parameter-set! index id
                         (lambda (code fmt args)
                           (when fmt
-                            (apply format (standard-error-port) fmt args))))
-    (lambda maybe-arg                   ;todo: replace with :optional syntax
-      (let1 old (%vm-parameter-ref index id) ;todo: replace with rlet1
+                            (apply format (standard-error-port) fmt args)
+                            (newline (standard-error-port)))))
+    (lambda maybe-arg
+      (rlet1 old (%vm-parameter-ref index id)
         (when (pair? maybe-arg)
-          (%vm-parameter-set! index id (car maybe-arg)))
-        old))))
+          (%vm-parameter-set! index id (car maybe-arg)))))))
 
 (define (exit . args)
   (let-optionals* args ([code 0] [fmt #f] . args)
     (cond [(exit-handler)
-           => (lambda (h)
-                (guard (e [(<error> e) #f]) (h code fmt args)))])
+           => (lambda (h) (guard (e [(<error> e) #f]) (h code fmt args)))])
     (%exit code)))
 
 ;;;=======================================================
@@ -454,13 +542,34 @@
   (receive vals (producer) (apply consumer vals)))
 
 ;;;=======================================================
-;;; signal utility
+;;; other system utilities
 ;;;
 (define (sys-sigset . signals)
   (if (null? signals)
     (make <sys-sigset>)
     (apply sys-sigset-add! (make <sys-sigset>) signals)))
 
+;; This is originally a part of shell-escape-string in gauche.process,
+;; but the lower level function Scm_Exec() requires this to build
+;; windows command line string from given argument list.   It would be
+;; clumsy to implement this in C, so we provide this here to be shared
+;; by Scm_Exec() and shell-escape-string.
+;; NB:  There seems no reliable way to escape command line arguments on
+;; windows, since the parsing is up to every application.  However,
+;; the standard C runtime seems to obey that (a) whitespaces can be
+;; embedded if the argument is surrounded by double quotes, and (b)
+;; within double-quotes, consecutive two double-quotes are replaced
+;; for one double-quote.
+;; NB: The second condition would be clearer if we use string-index, but
+;; it is in srfi-13.  string-split, otoh, is built-in.  The overhead of
+;; using string-split here would be negligible.
+(define (%sys-escape-windows-command-line s)
+  (cond [(not (string? s))
+         (%sys-escape-windows-command-line (write-to-string s))]
+        [(equal? s "") "\"\""]
+        [(null? (cdr (string-split s #[\s\"]))) s]
+        [else (string-append "\"" (regexp-replace-all #/\"/ s "\"\"") "\"")]))
+
 ;;;=======================================================
 ;;; standard hash-bang tokens
 ;;;
@@ -486,9 +595,8 @@
 ;;; srfi-17
 ;;;
 (define (getter-with-setter get set)
-  (let ((proc (lambda x (apply get x))))
-    (set! (setter proc) set)
-    proc))
+  (rlet1 proc (lambda x (apply get x))
+    (set! (setter proc) set)))
 
 ;;;=======================================================
 ;;; srfi-38
@@ -497,8 +605,8 @@
 (define read-with-shared-structure read)
 (define read/ss read)
 
-(define (write-with-shared-structure obj . args)
-  (write* obj (if (pair? args) (car args) (current-output-port))))
+(define (write-with-shared-structure obj :optional (port (current-output-port)))
+  (write* obj port))
 (define write/ss write-with-shared-structure)
 
 ;;;=======================================================
@@ -510,13 +618,13 @@
 (define-values (format format/ss)
   (letrec ((format-int
             (lambda (port fmt args shared?)
-              (cond ((eqv? port #f)
+              (cond [(eqv? port #f)
                      (let ((out (open-output-string :private? #t)))
                        (%format out fmt args shared?)
-                       (get-output-string out)))
-                    ((eqv? port #t)
-                     (%format (current-output-port) fmt args shared?))
-                    (else (%format port fmt args shared?)))))
+                       (get-output-string out))]
+                    [(eqv? port #t)
+                     (%format (current-output-port) fmt args shared?)]
+                    [else (%format port fmt args shared?)])))
            (format
             (lambda (fmt . args)
               (if (string? fmt)
@@ -540,63 +648,57 @@
     ;; "abc\\1de\\3" => '("abc" 1 "de" 3)
     (define (regexp-parse-subpattern sub)
       (cond
-       ((string? sub)
+       [(string? sub)
         (let loop ((sub sub) (r '()))
-          (cond ((rxmatch #/\\(?:(\d+)|k<([^>]+)>|(.))/ sub)
+          (cond [(rxmatch #/\\(?:(\d+)|k<([^>]+)>|(.))/ sub)
                  => (lambda (m)
                       (define (loop2 elem)
                         (loop (rxmatch-after m)
                               (list* elem (rxmatch-before m) r)))
-                      (cond ((rxmatch-substring m 1)
-                             => (lambda (d) (loop2 (string->number d))))
-                            ((rxmatch-substring m 2)
-                             => (lambda (s) (loop2 (string->symbol s))))
-                            (else
-                             (loop2 (rxmatch-substring m 3))))))
-                (else (reverse (cons sub r))))))
-       ((procedure? sub) sub)
-       (else (error "string or procedure required, but got" sub))))
+                      (cond [(rxmatch-substring m 1)
+                             => (lambda (d) (loop2 (string->number d)))]
+                            [(rxmatch-substring m 2)
+                             => (lambda (s) (loop2 (string->symbol s)))]
+                            [else (loop2 (rxmatch-substring m 3))]))]
+                [else (reverse (cons sub r))]))]
+       [(procedure? sub) sub]
+       [else (error "string or procedure required, but got" sub)]))
 
     ;; internal loop
-    (define (regexp-replace-rec match subpat out rec)
-      (display (rxmatch-before match) out)
+    (define (regexp-replace-rec match subpat rec)
+      (display (rxmatch-before match))
       (if (procedure? subpat)
-        (display (subpat match) out)
-        (for-each (lambda (pat)
-                    (display (if (or (number? pat) (symbol? pat))
-                               (rxmatch-substring match pat)
-                               pat)
-                             out))
-                  subpat))
+        (display (subpat match))
+        (dolist [pat subpat]
+          (display (if (or (number? pat) (symbol? pat))
+                     (rxmatch-substring match pat)
+                     pat))))
       (rec (rxmatch-after match)))
 
     (define (regexp-replace rx string sub)
-      (let ((subpat (regexp-parse-subpattern sub))
-            (match  (rxmatch rx string)))
+      (let ([subpat (regexp-parse-subpattern sub)]
+            [match  (rxmatch rx string)])
         (if match
-          (call-with-output-string
-            (lambda (out)
-              (regexp-replace-rec match subpat out
-                                  (lambda (str) (display str out)))))
+          (with-output-to-string (cut regexp-replace-rec match subpat display))
           string)))
 
     ;; The inner call is awkward to avoid creation of output string
     ;; when no match at all.
     (define (regexp-replace-all rx string sub)
-      (let ((subpat (regexp-parse-subpattern sub))
-            (match  (rxmatch rx string)))
+      (let ([subpat (regexp-parse-subpattern sub)]
+            [match  (rxmatch rx string)])
         (if match
-          (call-with-output-string
-            (lambda (out)
+          (with-output-to-string
+            (lambda ()
               (define (loop str)
                 (unless (equal? str "")
-                  (cond ((rxmatch rx str)
+                  (cond [(rxmatch rx str)
                          => (lambda (match)
                               (when (= (rxmatch-start match) (rxmatch-end match))
                                 (error "regexp-replace-all: matching zero-length string causes infinite loop:" rx))
-                              (regexp-replace-rec match subpat out loop)))
-                        (else (display str out)))))
-              (regexp-replace-rec match subpat out loop)))
+                              (regexp-replace-rec match subpat loop))]
+                        [else (display str)])))
+              (regexp-replace-rec match subpat loop)))
           string)))
     (values regexp-replace regexp-replace-all)))
 
@@ -605,17 +707,16 @@
   (let ()
     (define (regexp-replace-driver name func-1)
       (lambda (string rx sub . more)
-        (cond ((null? more)
-               (func-1 rx string sub))
-              (else
+        (cond [(null? more) (func-1 rx string sub)]
+              [else
                (unless (zero? (modulo (length more) 2))
                  (errorf "~a: regexp and subsitution don't pair up" name))
-               (let loop ((s (func-1 rx string sub))
-                          (args more))
+               (let loop ([s (func-1 rx string sub)]
+                          [args more])
                  (if (null? args)
                    s
                    (loop (func-1 (car args) s (cadr args))
-                         (cddr args))))))))
+                         (cddr args))))])))
     (values
      (regexp-replace-driver 'regexp-replace* regexp-replace)
      (regexp-replace-driver 'regexp-replace-all* regexp-replace-all))))
@@ -630,6 +731,11 @@
           (write-char c)
           (loop (read-char)))))))
 
+(define (regexp->string rx)
+  (or (%regexp-pattern rx)
+      (rlet1 s (regexp-unparse (regexp-ast rx))
+        (set! (%regexp-pattern rx) s))))
+
 (define (rxmatch->string rx str . sel)
   (cond [(null? sel) (rxmatch-substring (rxmatch rx str))]
         [(eq? (car sel) 'after)
@@ -658,52 +764,31 @@
 ;; File ports.
 
 (define-in-module scheme (call-with-input-file filename proc . flags)
-  (let ((port (apply open-input-file filename flags)))
-    (with-error-handler
-     (lambda (e)
-       (when port (close-input-port port))
-       (raise e))
-     (lambda ()
-       (receive r (proc port)
-         (when port (close-input-port port))
-         (apply values r))))))
+  (let1 port (apply open-input-file filename flags)
+    (unwind-protect (proc port)
+      (when port (close-input-port port)))))
 
 (define-in-module scheme (call-with-output-file filename proc . flags)
-  (let ((port (apply open-output-file filename flags)))
-    (with-error-handler
-     (lambda (e)
-       (when port (close-output-port port))
-       (raise e))
-     (lambda ()
-       (receive r (proc port)
-         (when port (close-output-port port))
-         (apply values r))))))
+  (let1 port (apply open-output-file filename flags)
+    (unwind-protect (proc port)
+      (when port (close-output-port port)))))
 
 (define-in-module scheme (with-input-from-file filename thunk . flags)
-  (let ((port (apply open-input-file filename flags)))
+  (let1 port (apply open-input-file filename flags)
     (and port
-         (with-error-handler
-          (lambda (e) (close-input-port port) (raise e))
-          (lambda ()
-            (receive r (with-input-from-port port thunk)
-              (close-input-port port)
-              (apply values r)))))))
-                  
+         (unwind-protect (with-input-from-port port thunk)
+           (close-input-port port)))))
 
 (define-in-module scheme (with-output-to-file filename thunk . flags)
-  (let ((port (apply open-output-file filename flags)))
+  (let1 port (apply open-output-file filename flags)
     (and port
-         (with-error-handler
-          (lambda (e) (close-output-port port) (raise e))
-          (lambda ()
-            (receive r (with-output-to-port port thunk)
-              (close-output-port port)
-              (apply values r)))))))
+         (unwind-protect (with-output-to-port port thunk)
+           (close-output-port port)))))
 
 ;; String ports
 
 (define (with-output-to-string thunk)
-  (let ((out (open-output-string)))
+  (let1 out (open-output-string)
     (with-output-to-port out thunk)
     (get-output-string out)))
 
@@ -711,13 +796,12 @@
   (with-input-from-port (open-input-string str) thunk))
 
 (define (call-with-output-string proc)
-  (let ((out (open-output-string)))
+  (let1 out (open-output-string)
     (proc out)
     (get-output-string out)))
 
 (define (call-with-input-string str proc)
-  (let ((in (open-input-string str)))
-    (proc in)))
+  (proc (open-input-string str)))
 
 (define (call-with-string-io str proc)
   (let ((out (open-output-string))
@@ -726,14 +810,10 @@
     (get-output-string out)))
 
 (define (with-string-io str thunk)
-  (with-output-to-string
-    (lambda ()
-      (with-input-from-string str
-        thunk))))
+  (with-output-to-string (cut with-input-from-string str thunk)))
 
-(define (write-to-string obj . args)
-  (with-output-to-string
-    (lambda () ((if (pair? args) (car args) write) obj))))
+(define (write-to-string obj :optional (writer write))
+  (with-output-to-string (cut writer obj)))
 
 (define (read-from-string string . args)
   (with-input-from-string
@@ -744,17 +824,16 @@
 
 (define-syntax %with-ports
   (syntax-rules ()
-    ((_ "tmp" (tmp ...) () (port ...) (param ...) thunk)
+    [(_ "tmp" (tmp ...) () (port ...) (param ...) thunk)
      (let ((tmp #f) ...)
        (dynamic-wind
            (lambda () (when port (set! tmp (param port))) ...)
            thunk
-           (lambda () (when tmp (param tmp)) ...))))
-    ((_ "tmp" tmps (port . more) ports params thunk)
-     (%with-ports "tmp" (tmp . tmps) more ports params thunk))
-    ((_ ((param port) ...) thunk)
-     (%with-ports "tmp" () (port ...) (port ...) (param ...) thunk))
-    ))
+           (lambda () (when tmp (param tmp)) ...)))]
+    [(_ "tmp" tmps (port . more) ports params thunk)
+     (%with-ports "tmp" (tmp . tmps) more ports params thunk)]
+    [(_ ((param port) ...) thunk)
+     (%with-ports "tmp" () (port ...) (port ...) (param ...) thunk)]))
 
 (define (with-input-from-port port thunk)
   (%with-ports ((current-input-port port)) thunk))
@@ -770,3 +849,61 @@
                 (current-output-port oport)
                 (current-error-port eport))
                thunk))
+
+;; Get the file path currently loading from.
+;; We may swap the implementation with more reliable way in future.
+(define (current-load-path)
+  (and-let* ([p (current-load-port)]
+             [info (port-name p)]
+             [ (string? info) ]
+             [ (not (#/^\(.*\)$/ info)) ])
+    info))
+
+;; Auxiliary procedures for case-lambda.
+(define-in-module gauche.internal (make-case-lambda minarg maxarg
+                                                    formals closures)
+
+  (define (fill-dispatch-vector! v formals closure)
+    (define (%set n)
+      (let1 i (- n minarg)
+        (unless (vector-ref v i) (vector-set! v i closure))))
+    (let loop ([formals formals] [n 0])
+      (cond [(> n (+ maxarg 1))]
+            [(null? formals) (%set n)]
+            [(pair? formals) (loop (cdr formals) (+ n 1))]
+            [else (%set n) (loop formals (+ n 1))])))
+  
+  (let1 v (make-vector (+ (- maxarg minarg) 2) #f)
+    (for-each (cut fill-dispatch-vector! v <> <>) formals closures)
+    ((with-module gauche.internal make-case-lambda-dispatcher) v minarg)))
+
+;;; TEMPORARY for 0.9.x series
+;;; Remove this after 1.0 release!!!
+;;;
+;;; Add 0.9 directories, which doesn't follow the new directory structure,
+;;; to support the extension modules that are installed with 0.9.  
+;;; A few extension packages installs files into the "sys" directory hierarchy
+;;; instead of "site" one.  It is banned after 0.9.1, but we need to add 0.9's
+;;; sys directories for the backward compatibility.
+;;; NB: we set! to *load-path* etc here, which is an emergency workaround.
+;;; Ordinary programs should never modify *load-path*/*dynamic-load-path*
+;;; directly.
+(let* ([archdir (gauche-architecture-directory)]
+       [m (rxmatch #/gauche-0\.9[\/\\]0\.9[^\/\\]*[\/\\]/ archdir)]
+       [oldsitedir (string-append (rxmatch-before m)
+                                  "gauche/site/0.9/"
+                                  (rxmatch-after m))]
+       [oldarchdir (string-append (rxmatch-before m)
+                                  "gauche/0.9/"
+                                  (rxmatch-after m))])
+  (set! *dynamic-load-path*
+        (append *dynamic-load-path* (list oldsitedir oldarchdir))))
+(let* ([libdir (gauche-library-directory)]
+       [m (rxmatch #/gauche-0\.9[\/\\]0\.9[^\/\\]*[\/\\]/ libdir)]
+       [oldsitedir (string-append (rxmatch-before m)
+                                  "gauche/site/"
+                                  (rxmatch-after m))]
+       [oldlibdir  (string-append (rxmatch-before m)
+                                  "gauche/0.9/"
+                                  (rxmatch-after m))])
+  (set! *load-path* (append *load-path* (list oldsitedir oldlibdir))))
