@@ -1,7 +1,7 @@
 /*
  * string.c - string implementation
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: string.c,v 1.90 2008-05-10 13:36:21 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -408,8 +406,15 @@ int Scm_StringCmp(ScmString *x, ScmString *y)
     sizy = SCM_STRING_BODY_SIZE(yb);
     siz = (sizx < sizy)? sizx : sizy;
     r = memcmp(SCM_STRING_BODY_START(xb), SCM_STRING_BODY_START(yb), siz);
-    if (r == 0) return (sizx - sizy);
-    else return r;
+    if (r == 0) {
+        if (sizx == sizy) return 0;
+        if (sizx < sizy)  return -1;
+        else              return 1;
+    } else if (r < 0) {
+        return -1;
+    } else {
+        return 1;
+    }
 }
 
 /* single-byte case insensitive comparison */
@@ -418,8 +423,8 @@ static int sb_strcasecmp(const char *px, int sizx,
 {
     char cx, cy;
     for (; sizx > 0 && sizy > 0; sizx--, sizy--, px++, py++) {
-        cx = tolower(*px);
-        cy = tolower(*py);
+        cx = tolower((u_char)*px);
+        cy = tolower((u_char)*py);
         if (cx == cy) continue;
         return (cx - cy);
     }
@@ -458,8 +463,8 @@ int Scm_StringCiCmp(ScmString *x, ScmString *y)
         Scm_Error("cannot compare incomplete strings in case-insensitive way: %S, %S",
                   SCM_OBJ(x), SCM_OBJ(y));
     }
-    sizx = SCM_STRING_BODY_SIZE(xb); lenx = SCM_STRING_BODY_SIZE(xb);
-    sizy = SCM_STRING_BODY_SIZE(yb); leny = SCM_STRING_BODY_SIZE(yb);
+    sizx = SCM_STRING_BODY_SIZE(xb); lenx = SCM_STRING_BODY_LENGTH(xb);
+    sizy = SCM_STRING_BODY_SIZE(yb); leny = SCM_STRING_BODY_LENGTH(yb);
     px = SCM_STRING_BODY_START(xb);
     py = SCM_STRING_BODY_START(yb);
     
@@ -765,7 +770,7 @@ ScmObj Scm_StringReplaceBody(ScmString *str, const ScmStringBody *newbody)
 static ScmObj substring(const ScmStringBody *xb, int start, int end,
                         int byterange)
 {
-    int len = SCM_STRING_BODY_LENGTH(xb);
+    int len = byterange? SCM_STRING_BODY_SIZE(xb) : SCM_STRING_BODY_LENGTH(xb);
     int flags = SCM_STRING_BODY_FLAGS(xb) & ~SCM_STRING_IMMUTABLE;
     SCM_CHECK_START_END(start, end, len);
 
@@ -1543,18 +1548,18 @@ void Scm_DStringDump(FILE *out, ScmDString *dstr)
     if (dstr->anchor) {
         ScmDStringChain *chain; int i;
         fprintf(out, "  chunk0[%3d] = \"", dstr->init.bytes);
-        fwrite(dstr->init.data, 1, dstr->init.bytes, out);
+        SCM_IGNORE_RESULT(fwrite(dstr->init.data, 1, dstr->init.bytes, out));
         fprintf(out, "\"\n");
         for (i=1, chain = dstr->anchor; chain; chain = chain->next, i++) {
             int size = (chain->next? chain->chunk->bytes : (int)(dstr->current - dstr->tail->chunk->data));
             fprintf(out, "  chunk%d[%3d] = \"", i, size);
-            fwrite(chain->chunk->data, 1, size, out);
+            SCM_IGNORE_RESULT(fwrite(chain->chunk->data, 1, size, out));
             fprintf(out, "\"\n");
         }
     } else {
         int size = (int)(dstr->current - dstr->init.data);
         fprintf(out, "  chunk0[%3d] = \"", size);
-        fwrite(dstr->init.data, 1, size, out);
+        SCM_IGNORE_RESULT(fwrite(dstr->init.data, 1, size, out));
         fprintf(out, "\"\n");
     }
 }
