@@ -1,7 +1,7 @@
 /*
  * system.c - system interface
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: system.c,v 1.107 2008-05-10 13:36:22 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -261,12 +259,15 @@ const char *Scm_PathDelimiter(void)
 #endif /* GAUCHE_WINDOWS */
 }
 
+/* On Windows, '/' is *allowed* to be an alternative separator. */
 #if defined(GAUCHE_WINDOWS)
 #define SEPARATOR '\\'
 #define ROOTDIR   "\\"
+#define SEPARATOR_P(c)  ((c) == SEPARATOR || (c) == '/')
 #else
 #define SEPARATOR '/'
 #define ROOTDIR   "/"
+#define SEPARATOR_P(c)  ((c) == SEPARATOR)
 #endif
 
 /* Returns the pointer to the first path separator character,
@@ -275,7 +276,7 @@ static const char *get_first_separator(const char *path, const char *end)
 {
     const char *p = path;
     while (p < end) {
-        if (*p == '/' || *p == '\\') return p;
+        if (SEPARATOR_P(*p)) return p;
         p += SCM_CHAR_NFOLLOWS(*p)+1;
     }
     return NULL;
@@ -287,7 +288,7 @@ static const char *get_last_separator(const char *path, const char *end)
 {
     const char *p = path, *last = NULL;
     while (p < end) {
-        if (*p == '/' || *p == '\\') last = p;
+        if (SEPARATOR_P(*p)) last = p;
         p += SCM_CHAR_NFOLLOWS(*p)+1;
     }
     return last;
@@ -296,7 +297,7 @@ static const char *get_last_separator(const char *path, const char *end)
 static const char *skip_separators(const char *p, const char *end)
 {
     while (p < end) {
-        if (*p != '/' && *p != '\\') break;
+        if (!SEPARATOR_P(*p)) break;
         p += SCM_CHAR_NFOLLOWS(*p)+1;
     }
     return p;
@@ -378,7 +379,7 @@ static void put_current_dir(ScmDString *dst)
     }
     dirlen = (int)strlen(p);
     Scm_DStringPutz(dst, p, dirlen);
-    if (p[dirlen-1] != '/' && p[dirlen-1] != '\\') {
+    if (!SEPARATOR_P(p[dirlen-1])) {
         Scm_DStringPutc(dst, SEPARATOR);
     }
 #undef GETCWD_PATH_MAX
@@ -392,7 +393,7 @@ static void copy_win32_path(ScmDString *dst,
 {
     while (srcp < end) {
         ScmChar ch;
-        if (*srcp == '/' || *srcp == '\\') {
+        if (SEPARATOR_P(*srcp)) {
             ch = SEPARATOR;
         } else {
             SCM_CHAR_GET(srcp, ch);
@@ -437,12 +438,29 @@ ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
 	return Scm_DStringGet(&buf, 0);
     }
 #else /* GAUCHE_WINDOWS */
-    if (endp > srcp+1 && isalpha(*srcp) && *(srcp+1) == ':') {
+    if ((flags & SCM_PATH_EXPAND) && size >= 1 && *str == '~') {
+        const char *home;
+        if (size >= 2 && strchr("/\\", str[1]) == NULL) {
+            Scm_Error("On windows native platforms, getting other user's home "
+                      "directory is not supported (yet): %S",
+                      SCM_OBJ(pathname));
+        }
+        srcp++;
+        if ((home = getenv("HOME")) != NULL) { /* MSYS */
+            Scm_DStringPutz(&buf, home, -1);
+        } else if ((home = getenv("HOMEDRIVE")) != NULL) { /* cmd.exe */
+            Scm_DStringPutz(&buf, home, -1);
+            if ((home = getenv("HOMEPATH")) != NULL) {
+                Scm_DStringPutz(&buf, home, -1);
+            }
+        }
+        /*FALLTHROUGH - if no env is set we use root dir. */
+    } else if (endp > srcp+1 && isalpha(*srcp) && *(srcp+1) == ':') {
         /* We first process the Evil Drive Letter */
         Scm_DStringPutc(&buf, *srcp++);
         Scm_DStringPutc(&buf, *srcp++);
     }
-    if (endp > srcp && (*srcp == '/' || *srcp == '\\')) {
+    if (endp > srcp && (SEPARATOR_P(*srcp))) {
         if (flags & SCM_PATH_CANONICALIZE) {
             Scm_DStringPutc(&buf, SEPARATOR);
             srcp = skip_separators(srcp, endp);
@@ -517,6 +535,32 @@ ScmObj Scm_NormalizePathname(ScmString *pathname, int flags)
     return Scm_DStringGet(&buf, 0);        
 }
 
+/* Returns system's temporary directory. */
+ScmObj Scm_TmpDir(void)
+{
+#if defined(GAUCHE_WINDOWS)
+# define TMP_PATH_MAX 1024
+    TCHAR buf[TMP_PATH_MAX+1], *tbuf = buf;
+    DWORD r, r2;
+    /* According to the windows document, this API checks environment
+       variables TMP, TEMP, and USERPROFILE.  Fallback is the Windows
+       directory. */
+    r = GetTempPath(TMP_PATH_MAX, buf);
+    if (r == 0) Scm_SysError("GetTempPath failed");
+    if (r > TMP_PATH_MAX) {
+        tbuf = SCM_NEW_ATOMIC_ARRAY(TCHAR, r+1);
+        r2 = GetTempPath(r, tbuf);
+        if (r2 != r) Scm_SysError("GetTempPath failed");
+    }
+    return SCM_MAKE_STR_COPYING(SCM_WCS2MBS(tbuf));
+#else  /*!GAUCHE_WINDOWS*/
+    const char *s;
+    if ((s = getenv("TMPDIR")) != NULL) return SCM_MAKE_STR_COPYING(s);
+    if ((s = getenv("TMP")) != NULL) return SCM_MAKE_STR_COPYING(s);
+    else return SCM_MAKE_STR("/tmp"); /* fallback */
+#endif /*!GAUCHE_WINDOWS*/
+}
+
 /* Basename and dirname.
    On Win32, we need to treat drive names specially, e.g.:
    (sys-dirname "C:/a") == (sys-dirname "C:/") == (sys-dirname "C:") == "C:\\"
@@ -620,7 +664,7 @@ int Scm_Mkstemp(char *templat)
         flags = O_CREAT|O_EXCL|O_WRONLY;
 #endif /* !GAUCHE_WINDOWS */
         for (numtry=0; numtry<MKSTEMP_MAX_TRIALS; numtry++) {
-            snprintf(suffix, 7, "%06lx", seed&0xffffff);
+            snprintf(suffix, 7, "%06lx", (seed>>8)&0xffffff);
             memcpy(templat+siz-6, suffix, 7);
             SCM_SYSCALL(fd, open(templat, flags, 0600));
             if (fd >= 0) break;
@@ -679,37 +723,37 @@ ScmObj Scm_MakeSysStat(void)
 
 static ScmObj stat_type_get(ScmSysStat *stat)
 {
-  if (S_ISDIR(stat->statrec.st_mode)) return (SCM_SYM_DIRECTORY);
-  if (S_ISREG(stat->statrec.st_mode)) return (SCM_SYM_REGULAR);
-  if (S_ISCHR(stat->statrec.st_mode)) return (SCM_SYM_CHARACTER);
-  if (S_ISBLK(stat->statrec.st_mode)) return (SCM_SYM_BLOCK);
-  if (S_ISFIFO(stat->statrec.st_mode)) return (SCM_SYM_FIFO);
+    if (S_ISDIR(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_DIRECTORY);
+    if (S_ISREG(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_REGULAR);
+    if (S_ISCHR(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_CHARACTER);
+    if (S_ISBLK(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_BLOCK);
+    if (S_ISFIFO(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_FIFO);
 #ifdef S_ISLNK
-  if (S_ISLNK(stat->statrec.st_mode)) return (SCM_SYM_SYMLINK);
+    if (S_ISLNK(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_SYMLINK);
 #endif
 #ifdef S_ISSOCK
-  if (S_ISSOCK(stat->statrec.st_mode)) return (SCM_SYM_SOCKET);
+    if (S_ISSOCK(SCM_SYS_STAT_STAT(stat)->st_mode)) return (SCM_SYM_SOCKET);
 #endif
-  return (SCM_FALSE);
+    return (SCM_FALSE);
 }
 
 static ScmObj stat_perm_get(ScmSysStat *stat)
 {
-    return Scm_MakeIntegerFromUI(stat->statrec.st_mode & 0777);
+    return Scm_MakeIntegerFromUI(SCM_SYS_STAT_STAT(stat)->st_mode & 0777);
 }
 
 static ScmObj stat_size_get(ScmSysStat *stat)
 {
-    return Scm_OffsetToInteger(stat->statrec.st_size);
+    return Scm_OffsetToInteger(SCM_SYS_STAT_STAT(stat)->st_size);
 }
 
 
 #define STAT_GETTER_UI(name) \
   static ScmObj SCM_CPP_CAT3(stat_, name, _get)(ScmSysStat *s) \
-  { return Scm_MakeIntegerFromUI((u_long)s->statrec.SCM_CPP_CAT(st_, name)); }
+  { return Scm_MakeIntegerFromUI((u_long)(SCM_SYS_STAT_STAT(s)->SCM_CPP_CAT(st_, name))); }
 #define STAT_GETTER_TIME(name) \
   static ScmObj SCM_CPP_CAT3(stat_, name, _get)(ScmSysStat *s) \
-  { return Scm_MakeSysTime(s->statrec.SCM_CPP_CAT(st_, name)); }
+  { return Scm_MakeSysTime(SCM_SYS_STAT_STAT(s)->SCM_CPP_CAT(st_, name)); }
 
 STAT_GETTER_UI(mode)
 STAT_GETTER_UI(ino)
@@ -763,7 +807,16 @@ static ScmObj time_allocate(ScmClass *klass, ScmObj initargs)
 static void time_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmTime *t = SCM_TIME(obj);
-    Scm_Printf(port, "#<%S %S.%09lu>", t->type, Scm_MakeInteger64(t->sec), t->nsec);
+    ScmObj sec = Scm_MakeInteger64(t->sec);
+    long nsec = t->nsec;
+    /* t->sec can be negative for time-difference. */
+    if (Scm_Sign(sec) < 0 && t->nsec > 0) {
+        sec = Scm_Abs(Scm_Add(sec, SCM_MAKE_INT(1)));
+        nsec = 1000000000L - nsec;
+        Scm_Printf(port, "#<%S -%S.%09lu>", t->type, sec, nsec);
+    } else {
+        Scm_Printf(port, "#<%S %S.%09lu>", t->type, sec, nsec);
+    }
 }
 
 static int time_compare(ScmObj x, ScmObj y, int equalp)
@@ -1315,8 +1368,6 @@ int Scm_IsSugid(void)
  *   unclaimed.
  *   One issue is that we cannot wait() for child processes that
  *   are created by Gauche extension code and not using Scm_SysExec API.
- *   I don't think it's a big problem, for such extensions must take
- 
  */
 #if defined(GAUCHE_WINDOWS)
 static struct process_mgr_rec {
@@ -1368,7 +1419,7 @@ void win_process_cleanup(void *data)
     ScmObj cp;
     SCM_INTERNAL_MUTEX_LOCK(process_mgr.mutex);
     SCM_FOR_EACH(cp, process_mgr.children) {
-        CloseHandle(Scm_WinProcessHandle(SCM_CAR(cp)));
+        CloseHandle(Scm_WinHandle(SCM_CAR(cp), SCM_FALSE));
     }
     process_mgr.children = SCM_NIL;
     SCM_INTERNAL_MUTEX_UNLOCK(process_mgr.mutex);
@@ -1390,13 +1441,18 @@ char *win_create_command_line(ScmObj args)
 {
     ScmObj ap, out;
     ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
-    SCM_FOR_EACH(ap, args) Scm_Printf(SCM_PORT(ostr), "%S ", SCM_CAR(ap));
+    ScmObj proc = Scm_GlobalVariableRef(Scm_GaucheModule(),
+                                        SCM_SYMBOL(SCM_INTERN("%sys-escape-windows-command-line")),
+                                        0);
+    SCM_FOR_EACH(ap, args) {
+      ScmObj escaped = Scm_ApplyRec1(proc, SCM_CAR(ap));
+      Scm_Printf(SCM_PORT(ostr), "%A ", escaped);
+    }
     out = Scm_GetOutputStringUnsafe(SCM_PORT(ostr), 0);
     return Scm_GetString(SCM_STRING(out));
 }
 #endif /*GAUCHE_WINDOWS*/
 
-
 /* Scm_SysExec
  *   execvp(), with optionally setting stdios correctly.
  *
@@ -1417,19 +1473,16 @@ char *win_create_command_line(ScmObj args)
  *   On Windows port, this returns a process handle obejct instead of   
  *   pid of the child process in fork mode.  We need to keep handle, or
  *   the process exit status will be lost when the child process terminates.
- *
- *   On Windows/MinGW port, I'm not sure we can do I/O swapping in
- *   reasonable way.  For now, iomap is ignored.
  */
-
 ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
-                   ScmSysSigset *mask, int flags)
+                   ScmSysSigset *mask, ScmString *dir, int flags)
 {
     int argc = Scm_Length(args);
     char **argv;
-    const char *program;
+    const char *program, *cdir = NULL;
     pid_t pid = 0;
     int forkp = flags & SCM_EXEC_WITH_FORK;
+    int detachp = flags & SCM_EXEC_DETACHED;
     int *fds;
 
     if (argc < 1) {
@@ -1443,20 +1496,46 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
     /* setting up iomap table */
     fds = Scm_SysPrepareFdMap(iomap);
     
+    /*
+     * From now on, we have totally different code for Unix and Windows.
+     */
 #if !defined(GAUCHE_WINDOWS)
+    /*
+     * Unix path
+     */
+    if (dir != NULL) cdir = Scm_GetStringConst(dir);
+
     /* When requested, call fork() here. */
     if (forkp) {
         SCM_SYSCALL(pid, fork());
         if (pid < 0) Scm_SysError("fork failed");
     }
 
-    /* Now we swap file descriptors and exec(). */
-    if (!forkp || pid == 0) {
+    if (!forkp || pid == 0) {   /* possibly the child process */
+
+        /* If we're running the daemon, we fork again to detach the parent,
+           and also reset the session id. */
+        if (detachp) {
+            SCM_SYSCALL(pid, fork());
+            if (pid < 0) Scm_SysError("fork failed");
+            if (pid > 0) exit(0);   /* not Scm_Exit(), for we don't want to
+                                       run the cleanup stuff. */
+            setsid();
+        }
+
+        if (cdir != NULL) {
+            if (chdir(cdir) < 0) {
+                Scm_Panic("chdir to %s failed before executing %s: %s",
+                          cdir, program, strerror(errno));
+            }
+        }
+
         Scm_SysSwapFds(fds);
         if (mask) {
             Scm_ResetSignalHandlers(&mask->set);
             Scm_SysSigmask(SIG_SETMASK, mask);
         }
+
         execvp(program, (char *const*)argv);
         /* here, we failed */
         Scm_Panic("exec failed: %s: %s", program, strerror(errno));
@@ -1465,12 +1544,36 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
     /* We come here only when fork is requested. */
     return Scm_MakeInteger(pid);
 #else  /* GAUCHE_WINDOWS */
+    /*
+     * Windows path
+     */
+    if (dir != NULL) {
+        /* we need full path for CreateProcess. */
+        dir = SCM_STRING(Scm_NormalizePathname(dir, SCM_PATH_ABSOLUTE|SCM_PATH_CANONICALIZE));
+        cdir = Scm_GetStringConst(dir);
+        
+        /* If the program is given in relative pathname,
+           it must be adjusted relative to the specified directory. */
+        if (program[0] != '/' && program[0] != '\\'
+            && !(program[0] && program[1] == ':')) {
+            ScmDString ds;
+            int c = cdir[strlen(cdir)-1];
+            Scm_DStringInit(&ds);
+            Scm_DStringPutz(&ds, cdir, -1);
+            if (c != '/' && c != '\\') Scm_DStringPutc(&ds, SCM_CHAR('/'));
+            Scm_DStringPutz(&ds, program, -1);
+            program = Scm_DStringGetz(&ds);
+        }
+    }
+    
     if (forkp) {
         TCHAR  program_path[MAX_PATH+1], *filepart;
         HANDLE *hs = win_prepare_handles(fds);
         BOOL r, pathlen;
         STARTUPINFO si;
         PROCESS_INFORMATION pi;
+        LPCTSTR curdir = NULL;
+        DWORD creation_flags = 0;
 
         pathlen = SearchPath(NULL, SCM_MBS2WCS(program),
                              _T(".exe"), MAX_PATH, program_path,
@@ -1486,14 +1589,20 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
             si.hStdError  = hs[2];
         }
 
+        if (cdir != NULL) curdir = SCM_MBS2WCS(cdir);
+
+        if (detachp) {
+            creation_flags |= CREATE_NEW_PROCESS_GROUP;
+        }
+
         r = CreateProcess(program_path,
                           SCM_MBS2WCS(win_create_command_line(args)),
                           NULL, /* process attr */
                           NULL, /* thread addr */
                           TRUE, /* inherit handles */
-                          0,    /* creation flags */
+                          creation_flags, /* creation flags */
                           NULL, /* nenvironment */
-                          NULL, /* current dir */
+                          curdir, /* current dir */
                           &si,  /* startup info */
                           &pi); /* process info */
         if (r == 0) Scm_SysError("spawning %s failed", program);
@@ -1501,6 +1610,13 @@ ScmObj Scm_SysExec(ScmString *file, ScmObj args, ScmObj iomap,
         return win_process_register(Scm_MakeWinProcess(pi.hProcess));
     } else {
         Scm_SysSwapFds(fds);
+        if (cdir != NULL) {
+            if (_chdir(cdir) < 0) {
+                Scm_SysError("Couldn't chdir to %s", cdir);
+            }
+        }
+        /* TODO: We should probably use Windows API to handle various
+           options consistently with fork-and-exec case above. */
 	execvp(program, (const char *const*)argv);
 	Scm_Panic("exec failed: %s: %s", program, strerror(errno));	
     }
@@ -1633,7 +1749,22 @@ static HANDLE *win_prepare_handles(int *fds)
     for (i=0; i<count; i++) {
         int to = fds[i+1], from = fds[i+1+count];
         if (to >= 0 && to < 3) {
-            hs[to] = (HANDLE)_get_osfhandle(from);
+            if (from >= 3) {
+                /* from_fd may be a pipe. */
+                HANDLE zh;
+                if (!DuplicateHandle(GetCurrentProcess(),
+                                     (HANDLE)_get_osfhandle(from),
+                                     GetCurrentProcess(),
+                                     &zh,
+                                     0, TRUE, 
+                                     DUPLICATE_CLOSE_SOURCE
+                                     |DUPLICATE_SAME_ACCESS)) {
+                    Scm_SysError("DuplicateHandle failed");
+                }
+                hs[to] = zh;
+            } else {
+                hs[to] = (HANDLE)_get_osfhandle(from);
+            }
         }
     }
     for (i=0; i<3; i++) {
@@ -1683,9 +1814,11 @@ void Scm_SysKill(ScmObj process, int signal)
             p = OpenProcess(PROCESS_TERMINATE, FALSE, pid);
             if (p == NULL) Scm_SysError("OpenProcess failed for pid %d", pid);
         } else {
-            p = Scm_WinProcessHandle(process);
+            p = Scm_WinProcess(process);
         }
-        r = TerminateProcess(p, SIGKILL);
+        /* We send 0xff00 + KILL, so that the receiving process (if it is
+           Gauche) can yield an exit status that indicates it is kill. */
+        r = TerminateProcess(p, SIGKILL+0xff00);
         errcode = GetLastError();
         if (pid_given) CloseHandle(p);
         SetLastError(errcode);
@@ -1786,7 +1919,7 @@ ScmObj Scm_SysWait(ScmObj process, int options)
             }
             handles = SCM_NEW_ATOMIC_ARRAY(HANDLE, num_children);
             for (i=0; i<num_children; i++) {
-                handles[i] = Scm_WinProcessHandle(children[i]);
+                handles[i] = Scm_WinProcess(children[i]);
             }
             r = win_wait_for_handles(handles, num_children, options, &status);
             if (r == -2) goto timeout;
@@ -1801,7 +1934,7 @@ ScmObj Scm_SysWait(ScmObj process, int options)
             SetLastError(ERROR_WAIT_NO_CHILDREN);
             Scm_SysError("waitpid failed");
         }
-        handle = Scm_WinProcessHandle(process);
+        handle = Scm_WinProcess(process);
         r = win_wait_for_handles(&handle, 1, options, &status);
         if (r == -2) goto timeout;
         if (r == -1) goto error;
@@ -2039,38 +2172,7 @@ static ScmObj get_relative_processes(int childrenp)
     }
 }
 
-/* Windows "HANDLE" wrapper.  */
-
-void handle_cleanup(ScmObj handle)
-{
-    CloseHandle(SCM_FOREIGN_POINTER_REF(HANDLE, handle));
-}
-
-void handle_print(ScmObj handle, ScmPort *sink, ScmWriteContext *mode)
-{
-    Scm_Printf(sink, "#<win:process-handle %d @%p>",
-               Scm_WinProcessPID(handle), handle);
-}
-
-static ScmClass *WinProcessHandleClass = NULL;
-
-ScmObj Scm_MakeWinProcess(HANDLE h)
-{
-    return Scm_MakeForeignPointer(WinProcessHandleClass, (void*)h);
-}
-
-int Scm_WinProcessP(ScmObj p)
-{
-    return SCM_XTYPEP(p, WinProcessHandleClass);
-}
-
-HANDLE Scm_WinProcessHandle(ScmObj handle)
-{
-    if (!SCM_XTYPEP(handle, WinProcessHandleClass)) {
-        SCM_TYPE_ERROR(handle, "<win:process-handle>");
-    }
-    return SCM_FOREIGN_POINTER_REF(HANDLE, handle);
-}
+/* Retrieve PID from windows process handle wrapper.  */
 
 pid_t Scm_WinProcessPID(ScmObj handle)
 {
@@ -2083,6 +2185,10 @@ pid_t Scm_WinProcessPID(ScmObj handle)
     static DWORD (WINAPI *pGetProcessId)(HANDLE) = NULL;
     static int queried = FALSE;
 
+    if (!Scm_WinProcessP(handle)) {
+        SCM_TYPE_ERROR(handle, "<win:handle process>");
+    }
+
     if (pGetProcessId == NULL) {
         if (queried) return (pid_t)-1;
         pGetProcessId = get_api_entry(_T("kernel32.dll"), "GetProcessId",
@@ -2092,7 +2198,7 @@ pid_t Scm_WinProcessPID(ScmObj handle)
             return (pid_t)-1;
         }
     }
-    return pGetProcessId(Scm_WinProcessHandle(handle));
+    return pGetProcessId(Scm_WinProcess(handle));
 }
 
 /*
@@ -2105,13 +2211,13 @@ pid_t Scm_WinProcessPID(ScmObj handle)
 
 static void convert_user(const USER_INFO_2 *wuser, struct passwd *res)
 {
-    res->pw_name    = SCM_WCS2MBS(wuser->usri2_name);
+    res->pw_name    = (const char*)SCM_WCS2MBS(wuser->usri2_name);
     res->pw_passwd  = "*";
     res->pw_uid     = 0;
     res->pw_gid     = 0;
-    res->pw_comment = SCM_WCS2MBS(wuser->usri2_comment);
-    res->pw_gecos   = SCM_WCS2MBS(wuser->usri2_full_name);
-    res->pw_dir     = SCM_WCS2MBS(wuser->usri2_home_dir);
+    res->pw_comment = (const char*)SCM_WCS2MBS(wuser->usri2_comment);
+    res->pw_gecos   = (const char*)SCM_WCS2MBS(wuser->usri2_full_name);
+    res->pw_dir     = (const char*)SCM_WCS2MBS(wuser->usri2_home_dir);
     res->pw_shell   = "";
 }
 
@@ -2121,7 +2227,8 @@ static struct passwd pwbuf = { "dummy" };
 struct passwd *getpwnam(const char *name)
 {
     USER_INFO_2 *res;
-    if (NetUserGetInfo(NULL, SCM_MBS2WCS(name), 2, (LPBYTE*)&res) != NERR_Success) {
+    if (NetUserGetInfo(NULL, (LPCWSTR)SCM_MBS2WCS(name), 2, (LPBYTE*)&res)
+        != NERR_Success) {
 	return NULL;
     }
     convert_user(res, &pwbuf);
@@ -2199,7 +2306,10 @@ int fork(void)
 int pipe(int fd[])
 {
 #define PIPE_BUFFER_SIZE 512
-    int r = _pipe(fd, PIPE_BUFFER_SIZE, O_BINARY);
+    /* NB: We create pipe with NOINHERIT to avoid complication when spawning
+       child process.  Sys_Exec will dups the handle with inheritable flag
+       for the children.  */
+    int r = _pipe(fd, PIPE_BUFFER_SIZE, O_BINARY|O_NOINHERIT);
     return r;
 }
 
@@ -2297,10 +2407,39 @@ int link(const char *existing, const char *newpath)
                                                            CREATEHARDLINK,
                                                            TRUE);
     }
-    r = pCreateHardLink(SCM_MBS2WCS(newpath), SCM_MBS2WCS(existing), NULL);
+    r = pCreateHardLink((LPTSTR)SCM_MBS2WCS(newpath),
+                        (LPTSTR)SCM_MBS2WCS(existing), NULL);
     return r? 0 : -1;
 }
 
+/* nanosleep.  We emulate it with Sleep(). */
+int nanosleep(const struct timespec *req, struct timespec *rem)
+{
+    DWORD msecs = 0;
+    time_t sec;
+    u_long overflow = 0, c;
+    const DWORD MSEC_OVERFLOW = 4294967; /* 4294967*1000 = 0xfffffed8 */
+
+    /* It's very unlikely that we overflow msecs, but just in case... */
+    if (req->tv_sec > 0 || (req->tv_sec == 0 && req->tv_nsec > 0)) {
+        if (req->tv_sec >= MSEC_OVERFLOW) {
+            overflow = req->tv_sec / MSEC_OVERFLOW;
+            sec = req->tv_sec % MSEC_OVERFLOW;
+        } else {
+            sec = req->tv_sec;
+        }
+        msecs = (sec * 1000 + (req->tv_nsec + 999999)/1000000);
+    }
+    Sleep (msecs);
+    for (c = 0; c < overflow; c++) {
+        Sleep(MSEC_OVERFLOW * 1000);
+    }
+    if (rem) {
+        rem->tv_sec = rem->tv_nsec = 0;
+    }
+    return 0;
+}
+
 /* Winsock requires some obscure initialization.
    We perform initialization here, since winsock module is used
    in both gauche.net and gauche.auxsys. */
@@ -2349,10 +2488,6 @@ void Scm__InitSystem(void)
 #ifdef GAUCHE_WINDOWS
     init_winsock();
     Scm_AddCleanupHandler(fini_winsock, NULL);
-    WinProcessHandleClass =
-        Scm_MakeForeignPointerClass(mod, "<win:process-handle>",
-                                    handle_print, handle_cleanup,
-                                    SCM_FOREIGN_POINTER_KEEP_IDENTITY);
     Scm_AddCleanupHandler(win_process_cleanup, NULL);
 #endif
 }
