@@ -1,7 +1,7 @@
 /*
  * vm.c - virtual machine
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,13 +29,10 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: vm.c,v 1.280 2008-06-02 01:13:13 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
 #include "gauche.h"
-#include "gauche/memory.h"
 #include "gauche/class.h"
 #include "gauche/exception.h"
 #include "gauche/builtin-syms.h"
@@ -82,8 +79,6 @@ static ScmCompiledCode internal_apply_compiled_code =
                                         SCM_NIL, SCM_FALSE,
                                         SCM_FALSE, SCM_FALSE);
 
-
-
 /*
  * The VM. 
  *
@@ -111,6 +106,7 @@ static ScmSubr default_exception_handler_rec;
 static ScmObj throw_cont_calculate_handlers(ScmEscapePoint *, ScmVM *);
 static ScmObj throw_cont_body(ScmObj, ScmEscapePoint*, ScmObj);
 static void   process_queued_requests(ScmVM *vm);
+static void   vm_finalize(ScmObj vm, void *data);
 
 static ScmEnvFrame *get_env(ScmVM *vm);
 
@@ -154,13 +150,14 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     (void)SCM_INTERNAL_MUTEX_INIT(v->vmlock);
     (void)SCM_INTERNAL_COND_INIT(v->cond);
     v->canceller = NULL;
+    v->inspector = NULL;
     v->name = name;
     v->specific = SCM_FALSE;
     v->thunk = NULL;
     v->result = SCM_UNDEFINED;
     v->resultException = SCM_UNDEFINED;
     v->module = proto ? proto->module : Scm_SchemeModule();
-    v->cstack = proto ? proto->cstack : NULL;
+    v->cstack = NULL;
     
     v->curin  = proto? proto->curin  : SCM_PORT(Scm_Stdin());
     v->curout = proto? proto->curout : SCM_PORT(Scm_Stdout());
@@ -170,7 +167,10 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
 
     v->compilerFlags = proto? proto->compilerFlags : 0;
     v->runtimeFlags = proto? proto->runtimeFlags : 0;
-    v->queueNotEmpty = 0;
+    v->attentionRequest = 0;
+    v->signalPending = 0;
+    v->finalizerPending = 0;
+    v->stopRequest = 0;
 
 #ifdef USE_CUSTOM_STACK_MARKER
     v->stack = (ScmObj*)GC_generic_malloc((SCM_VM_STACK_SIZE+1)*sizeof(ScmObj),
@@ -182,6 +182,11 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     v->sp = v->stack;
     v->stackBase = v->stack;
     v->stackEnd = v->stack + SCM_VM_STACK_SIZE;
+#if GAUCHE_FFX
+    v->fpstack = SCM_NEW_ATOMIC_ARRAY(ScmFlonum, SCM_VM_STACK_SIZE);
+    v->fpstackEnd = v->fpstack + SCM_VM_STACK_SIZE;
+    v->fpsp = v->fpstack;
+#endif /* GAUCHE_FFX */
 
     v->env = NULL;
     v->argp = v->stack;
@@ -201,9 +206,6 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     v->escapeData[1] = NULL;
     v->defaultEscapeHandler = SCM_FALSE;
 
-    v->load_history = SCM_NIL;
-    v->load_next = SCM_NIL;
-    v->load_port = SCM_FALSE;
     v->evalSituation = SCM_VM_EXECUTING;
 
     sigemptyset(&v->sigMask);
@@ -216,10 +218,9 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
     v->profilerRunning = FALSE;
     v->prof = NULL;
 
-#ifdef GAUCHE_USE_PTHREADS
-    v->thread = (pthread_t)NULL;
-#endif /*GAUCHE_USE_PTHREADS*/
+    (void)SCM_INTERNAL_THREAD_INIT(v->thread);
 
+    Scm_RegisterFinalizer(SCM_OBJ(v), vm_finalize, NULL);
     return v;
 }
 
@@ -229,7 +230,7 @@ ScmVM *Scm_NewVM(ScmVM *proto, ScmObj name)
 int Scm_AttachVM(ScmVM *vm)
 {
 #ifdef GAUCHE_USE_PTHREADS
-    if (vm->thread != (pthread_t)NULL) return FALSE;
+    if (SCM_INTERNAL_THREAD_INITIALIZED_P(vm->thread)) return FALSE;
     if (theVM != NULL) return FALSE;
 
     if (pthread_setspecific(Scm_VMKey(), vm) != 0) return FALSE;
@@ -274,6 +275,10 @@ ScmVM *Scm_VM(void)
     return theVM;
 }
 
+/* Some macros inserts Scm_VM() in its output.  If such macros are expanded
+   below, we can safely replace Scm_VM() to theVM. */
+#define Scm_VM() theVM
+
 /*
  * Get VM key
  */
@@ -284,6 +289,19 @@ pthread_key_t Scm_VMKey(void)
 }
 #endif /*GAUCHE_USE_PTHREADS*/
 
+/* Warn if VM is terminated by uncaught exception, and GC-ed without
+   joining.  It is cleary an unexpected case and worth reporting. */
+static void vm_finalize(ScmObj obj, void *data)
+{
+    ScmVM *vm = SCM_VM(obj);
+    ScmObj re = vm->resultException;
+
+    if (SCM_UNCAUGHT_EXCEPTION_P(re)) {
+        Scm_Warn("A thread %S died a lonely death with uncaught exception %S.",
+                 vm->name, SCM_THREAD_EXCEPTION(re)->data);
+    }
+}
+
 /*====================================================================
  * VM interpreter
  *
@@ -338,6 +356,13 @@ pthread_key_t Scm_VMKey(void)
 /* pop the top object of the stack and store it to VAR */
 #define POP_ARG(var)       ((var) = *--SP)
 
+#define SHIFT_FRAME(from, to, size)                     \
+    do {                                                \
+        ScmObj *f = (from), *t = (to);                  \
+        int c;                                          \
+        for (c=0; c<(size); c++, f++, t++) *t = *f;     \
+    } while (0)
+
 /* VM registers.  We've benchmarked if keeping some of those registers
    local variables makes VM loop run faster; however, it turned out
    that more local variables tended to make them spill from machine
@@ -386,6 +411,7 @@ pthread_key_t Scm_VMKey(void)
     do {                                                                \
         if (CONT->argp == NULL) {                                       \
             void *data__[SCM_CCONT_DATA_SIZE];                          \
+            ScmObj v__ = VAL0;                                          \
             ScmCContinuationProc *after__;                              \
             void **d__ = data__;                                        \
             void **s__ = (void**)((ScmObj*)CONT + CONT_FRAME_SIZE);     \
@@ -398,9 +424,10 @@ pthread_key_t Scm_VMKey(void)
             ENV = CONT->env;                                            \
             ARGP = SP;                                                  \
             PC = PC_TO_RETURN;                                          \
-            CONT = CONT->prev;                                          \
             BASE = CONT->base;                                          \
-            VAL0 = CALL_CCONT(after__, VAL0, data__);                   \
+            CONT = CONT->prev;                                          \
+            SCM_FLONUM_ENSURE_MEM(v__);                                 \
+            VAL0 = CALL_CCONT(after__, v__, data__);                    \
         } else if (IN_STACK_P((ScmObj*)CONT)) {                         \
             SP   = CONT->argp + CONT->size;                             \
             ENV  = CONT->env;                                           \
@@ -599,7 +626,7 @@ pthread_key_t Scm_VMKey(void)
 #define DISPATCH    /*empty*/
 #define NEXT                                            \
     do {                                                \
-        if (vm->queueNotEmpty) goto process_queue;      \
+        if (vm->attentionRequest) goto process_queue;   \
         FETCH_INSN(code);                               \
         goto *dispatch_table[SCM_VM_INSN_CODE(code)];   \
     } while (0)
@@ -690,7 +717,7 @@ static void run_loop()
     for (;;) {
         DISPATCH;
         /*VM_DUMP("");*/
-        if (vm->queueNotEmpty) goto process_queue;
+        if (vm->attentionRequest) goto process_queue;
         FETCH_INSN(code);
         SWITCH(SCM_VM_INSN_CODE(code)) {
 #define VMLOOP
@@ -759,20 +786,22 @@ static inline ScmEnvFrame *save_env(ScmVM *vm, ScmEnvFrame *env_begin)
     if (!IN_STACK_P((ScmObj*)e)) return e;
 
     do {
-        int esize = (int)e->size, i;
+        long esize = (long)e->size, i;
         ScmObj *d, *s;
 
-        if (e->size < 0) {
+        if (esize < 0) {
             /* forwaded frame */
             if (prev) prev->up = FORWARDED_ENV(e);
             return head;
         }
 
         d = SCM_NEW2(ScmObj*, ENV_SIZE(esize) * sizeof(ScmObj));
-        for (i=ENV_SIZE(esize), s = (ScmObj*)e - esize; i>0; i--) {
+        for (i=esize, s = (ScmObj*)e - esize; i>0; i--) {
+            SCM_FLONUM_ENSURE_MEM(*s);
             *d++ = *s++;
         }
-        saved = (ScmEnvFrame*)(d - ENV_HDR_SIZE);
+        *(ScmEnvFrame*)d = *e; /* copy env header */
+        saved = (ScmEnvFrame*)d;
         if (prev) prev->up = saved;
         if (head == NULL) head = saved;
         next = e->up;
@@ -823,6 +852,7 @@ static void save_cont(ScmVM *vm)
                 s = c->argp;
                 d = (ScmObj*)csave + CONT_FRAME_SIZE;
                 for (i=c->size; i>0; i--) {
+                    SCM_FLONUM_ENSURE_MEM(*s);
                     *d++ = *s++;
                 }
             }
@@ -832,6 +862,8 @@ static void save_cont(ScmVM *vm)
             s = (ScmObj*)c;
             d = (ScmObj*)csave;
             for (i=CONT_FRAME_SIZE + c->size; i>0; i--) {
+                /* NB: C continuation frame contains opaque pointer,
+                   so we shouldn't ENSURE_MEM. */
                 *d++ = *s++;
             }
         }
@@ -914,6 +946,105 @@ static ScmEnvFrame *get_env(ScmVM *vm)
     return e;
 }
 
+#if GAUCHE_FFX
+/* Move all the FLONUM_REGs to heap and clear the fpstack.
+   We cache small number of visited env frames to avoid duplicate scanning
+   (if there are more env frames, linear search in the cache gets even
+   more costly than duplicate scanning).
+ */
+
+#define ENV_CACHE_SIZE 32
+
+#undef COUNT_FLUSH_FPSTACK
+
+#ifdef COUNT_FLUSH_FPSTACK
+static int flush_fpstack_count = 0;
+static u_long flush_fpstack_time = 0;
+static void print_flush_fpstack_count(void*z)
+{
+    fprintf(stderr, "fpstack count = %d  time = %ldus (avg %fus)\n",
+            flush_fpstack_count, flush_fpstack_time,
+            flush_fpstack_time/(double)flush_fpstack_count);
+}
+#endif
+
+void Scm_VMFlushFPStack(ScmVM *vm)
+{
+    ScmEnvFrame *visited[ENV_CACHE_SIZE], *e;
+    ScmContFrame *c;
+    int visited_index = 0, i;
+    ScmObj *p;
+#ifdef COUNT_FLUSH_FPSTACK
+    struct timeval t0, t1;
+    gettimeofday(&t0, NULL);
+#endif
+
+    /* first, scan value registers and incomplete frames */
+    SCM_FLONUM_ENSURE_MEM(VAL0);
+    for (i=0; i<SCM_VM_MAX_VALUES; i++) {
+        SCM_FLONUM_ENSURE_MEM(vm->vals[i]);
+    }
+    if (IN_STACK_P(ARGP)) {
+        for (p = ARGP; p < SP; p++) SCM_FLONUM_ENSURE_MEM(*p);
+    }
+
+    /* scan the main environment chain */
+    e = ENV;
+    while (IN_STACK_P((ScmObj*)e)) {
+        for (i = 0; i < visited_index; i++) {
+            if (visited[i] == e) goto next;
+        }
+        if (visited_index < ENV_CACHE_SIZE) {
+            visited[visited_index++] = e;
+        }
+        
+        for (i = 0; i < e->size; i++) {
+            p = &ENV_DATA(e, i);
+            SCM_FLONUM_ENSURE_MEM(*p);
+        }
+      next:
+        e = e->up;
+    }
+
+    /* scan the env chains grabbed by cont chain */
+    c = CONT;
+    while (IN_STACK_P((ScmObj*)c)) {
+        e = c->env;
+        while (IN_STACK_P((ScmObj*)e)) {
+            for (i = 0; i < visited_index; i++) {
+                if (visited[i] == e) goto next2;
+            }
+            if (visited_index < ENV_CACHE_SIZE) {
+                visited[visited_index++] = e;
+            }
+            for (i = 0; i < e->size; i++) {
+                p = &ENV_DATA(e, i);
+                SCM_FLONUM_ENSURE_MEM(*p);
+            }
+          next2:
+            e = e->up;
+        }
+        if (IN_STACK_P(c->argp) && c->size > 0) {
+            p = c->argp;
+            for (i=0; i<c->size; i++, p++) SCM_FLONUM_ENSURE_MEM(*p);
+        }
+        c = c->prev;
+    }
+
+    vm->fpsp = vm->fpstack;
+
+#ifdef COUNT_FLUSH_FPSTACK
+    flush_fpstack_count++;
+    gettimeofday(&t1, NULL);
+    flush_fpstack_time +=
+        (t1.tv_sec - t0.tv_sec)*1000000+(t1.tv_usec - t0.tv_usec);
+#endif
+}
+#undef ENV_CACHE_SIZE
+
+#endif /*GAUCHE_FFX*/
+
+
 /*==================================================================
  * Function application from C
  */
@@ -1057,7 +1188,7 @@ ScmObj Scm_VMEval(ScmObj expr, ScmObj e)
     if (restore_module) {
         /* if we swap the module, we need to make sure it is recovered
            after eval */
-        ScmObj body = Scm_MakeClosure(v, get_env(vm));
+        ScmObj body = Scm_MakeClosure(v, NULL);
         ScmObj before = Scm_MakeSubr(eval_restore_env, SCM_MODULE(e),
                                      0, 0, SCM_SYM_EVAL_BEFORE);
         ScmObj after = Scm_MakeSubr(eval_restore_env, (void*)vm->module,
@@ -1154,17 +1285,27 @@ static ScmObj user_eval_inner(ScmObj program, ScmWord *codevec)
   restart:
     vm->escapeReason = SCM_VM_ESCAPE_NONE;
     if (sigsetjmp(cstack.jbuf, FALSE) == 0) {
-        run_loop();
+        run_loop();             /* VM loop */
         VAL0 = vm->val0;
         if (vm->cont == cstack.cont) {
             POP_CONT();
             PC = prev_pc;
+        } else if (vm->cont == NULL) {
+            /* we're finished with executing partial continuation.*/
+            vm->cont = cstack.cont;
+            POP_CONT();
+            PC = prev_pc;
+        } else {
+            /* If we come here, we've been executing a ghost continuation.
+               The C world the ghost should return no longer exists, so we
+               raise an error. */
+            Scm_Error("attempt to return from a ghost continuation.");
         }
     } else {
         Scm_SetSigmask(&cstack.mask);
         /* An escape situation happened. */
         if (vm->escapeReason == SCM_VM_ESCAPE_CONT) {
-             ScmEscapePoint *ep = (ScmEscapePoint*)vm->escapeData[0];
+            ScmEscapePoint *ep = (ScmEscapePoint*)vm->escapeData[0];
             if (ep->cstack == vm->cstack) {
                 ScmObj handlers = throw_cont_calculate_handlers(ep, vm);
                 /* force popping continuation when restarted */
@@ -1439,7 +1580,7 @@ ScmObj Scm_VMDynamicWind(ScmObj before, ScmObj body, ScmObj after)
 {
     void *data[3];
 
-    /* NB: we don't check types of arguments, since we allo object-apply
+    /* NB: we don't check types of arguments, since we allow object-apply
        hooks can be used for them. */
     data[0] = (void*)before;
     data[1] = (void*)body;
@@ -1887,7 +2028,7 @@ static ScmObj throw_cont_body(ScmObj handlers,    /* after/before thunks
                                                      to be called */
                               ScmEscapePoint *ep, /* target continuation */
                               ScmObj args)        /* args to pass to the
-                                                     target continuation */ 
+                                                     target continuation */
 {
     void *data[3];
     int nargs, i;
@@ -1911,6 +2052,16 @@ static ScmObj throw_cont_body(ScmObj handlers,    /* after/before thunks
         return Scm_VMApply0(handler);
     }
 
+    /*
+     * If the target continuation is a full continuation, we can abandon
+     * the current continuation.  However, if the target continuation is
+     * partial, we must return to the current continuation after executing
+     * the partial continuation.  The returning part is handled by
+     * user_level_inner, but we have to make sure that our current continuation
+     * won't be overwritten by execution of the partial continuation.
+     */
+    if (ep->cstack == NULL) save_cont(vm);
+    
     /*
      * now, install the target continuation
      */
@@ -1948,27 +2099,29 @@ static ScmObj throw_continuation(ScmObj *argframe, int nargs, void *data)
     ScmEscapePoint *ep = (ScmEscapePoint*)data;
     ScmObj args = argframe[0];
     ScmVM *vm = theVM;
+    ScmObj handlers_to_call;
 
-    if (vm->cstack != ep->cstack) {
-        ScmCStack *cstk;
-        for (cstk = vm->cstack; cstk; cstk = cstk->prev) {
-            if (ep->cstack == cstk) break;
+    if (ep->cstack && vm->cstack != ep->cstack) {
+        ScmCStack *cs;
+        for (cs = vm->cstack; cs; cs = cs->prev) {
+            if (ep->cstack == cs) break;
         }
-        if (cstk == NULL) {
-            Scm_Error("a continuation is thrown outside of it's extent: %p",
-                      ep);
-        } else {
-            /* Rewind C stack */
+
+        /* If the continuation captured below the current C stack, we rewind
+           to the captured stack first.  If not, the continuation is 'ghost'.
+           We execute the scheme portion of the continuation on the current
+           C stack (no rewinding), but we'll catch it if it tries to return
+           to the C world.   See user_eval_inner().  */
+        if (cs != NULL) {
             vm->escapeReason = SCM_VM_ESCAPE_CONT;
             vm->escapeData[0] = ep;
             vm->escapeData[1] = args;
             siglongjmp(vm->cstack->jbuf, 1);
         }
-    } else {
-        ScmObj handlers_to_call = throw_cont_calculate_handlers(ep, vm);
-        return throw_cont_body(handlers_to_call, ep, args);
     }
-    return SCM_UNDEFINED; /*dummy*/
+
+    handlers_to_call = throw_cont_calculate_handlers(ep, vm);
+    return throw_cont_body(handlers_to_call, ep, args);
 }
 
 ScmObj Scm_VMCallCC(ScmObj proc)
@@ -1990,6 +2143,48 @@ ScmObj Scm_VMCallCC(ScmObj proc)
     return Scm_VMApply1(proc, contproc);
 }
 
+/* call with partial continuation.  this corresponds to the 'shift' operator
+   in shift/reset controls (Gasbichler&Sperber, "Final Shift for Call/cc",
+   ICFP02.)   Note that we treat the boundary frame as the bottom of
+   partial continuation. */
+ScmObj Scm_VMCallPC(ScmObj proc)
+{
+    ScmObj contproc;
+    ScmEscapePoint *ep;
+    ScmContFrame *c, *cp;
+    ScmVM *vm = theVM;
+    
+    /* save the continuation.  we only need to save the portion above the
+       latest boundary frame (+environmentns pointed from them), but for now,
+       we save everything to make things easier.  If we want to squeeze
+       performance we'll optimize it later. */
+    save_cont(vm);
+    
+    /* find the latest boundary frame */
+    for (c = vm->cont, cp = NULL;
+         c && !BOUNDARY_FRAME_P(c);
+         cp = c, c = c->prev)
+        /*empty*/;
+
+    if (cp != NULL) cp->prev = NULL; /* cut the dynamic chain */
+
+    ep = SCM_NEW(ScmEscapePoint);
+    ep->prev = NULL;
+    ep->ehandler = SCM_FALSE;
+    ep->cont = vm->cont;
+    ep->handlers = vm->handlers;
+    ep->cstack = NULL; /* so that the partial continuation can be run
+                          on any cstack state. */
+    contproc = Scm_MakeSubr(throw_continuation, ep, 0, 1,
+                            SCM_MAKE_STR("partial continuation"));
+    /* Remove the saved continuation chain.
+       NB: c can be NULL if we've been executing a partial continuation.
+       It's ok, for a continuation pointed by cstack will be restored
+       in user_eval_inner. */
+    vm->cont = c;
+    return Scm_VMApply1(proc, contproc);
+}
+
 /*==============================================================
  * Unwind protect API
  */
@@ -2163,14 +2358,35 @@ static void process_queued_requests(ScmVM *vm)
     }
     Scm_VMPushCC(process_queued_requests_cc, data, 3);
 
+    /* NB: it is safe to turn off attentionRequest here; if attentionRequest
+       is turned on again after this and before SigCheck() or FinalizerRun(),
+       the new request is processed within these procedures; we'll enter
+       process_queued_requests() again without anything to process, but
+       that's an acceptable overhead. */
+    vm->attentionRequest = FALSE;
+
     /* Process queued stuff.  Currently they call VM recursively,
        but we'd better to arrange them to be processed in the same
        VM level. */
-    if (vm->queueNotEmpty & SCM_VM_SIGQ_MASK) {
-        Scm_SigCheck(vm);
-    }
-    if (vm->queueNotEmpty & SCM_VM_FINQ_MASK) {
-        Scm_VMFinalizerRun(vm);
+    if (vm->signalPending)   Scm_SigCheck(vm);
+    if (vm->finalizerPending) Scm_VMFinalizerRun(vm);
+
+    /* VM STOP is required from other thread.
+       See Scm_ThreadStop() in ext/threads/threads.c */
+    if (vm->stopRequest) {
+        SCM_INTERNAL_MUTEX_SAFE_LOCK_BEGIN(vm->vmlock);
+        /* Double check, since stopRequest can be canceled between the above
+           two lines. */
+        if (vm->stopRequest) {
+            vm->stopRequest = FALSE;
+            vm->state = SCM_VM_STOPPED;
+            (void)SCM_INTERNAL_COND_BROADCAST(vm->cond);
+            while (vm->state == SCM_VM_STOPPED) {
+                /* Here the inspector thread examines VM state */
+                (void)SCM_INTERNAL_COND_WAIT(vm->cond, vm->vmlock);
+            }
+        }
+        SCM_INTERNAL_MUTEX_SAFE_LOCK_END();
     }
 }
 
@@ -2436,5 +2652,8 @@ void Scm__InitVM(void)
 #ifdef COUNT_INSN_FREQUENCY
     Scm_AddCleanupHandler(dump_insn_frequency, NULL);
 #endif /*COUNT_INSN_FREQUENCY*/
-}
 
+#ifdef COUNT_FLUSH_FPSTACK
+    Scm_AddCleanupHandler(print_flush_fpstack_count, NULL);
+#endif
+}
