@@ -1,7 +1,7 @@
 /*
  * write.c - writer
  *
- *   Copyright (c) 2000-2008  Shiro Kawai  <shiro@acm.org>
+ *   Copyright (c) 2000-2011  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -29,8 +29,6 @@
  *   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  $Id: write.c,v 1.71 2008-05-10 13:36:24 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -279,7 +277,7 @@ static const char *char_names[] = {
 static void write_general(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
 {
     ScmClass *c = Scm_ClassOf(obj);
-    if (c->print) c->print(obj, out, ctx); 
+    if (c->print) c->print(obj, out, ctx);
     else          write_object(obj, out, ctx);
 }
 
@@ -351,54 +349,54 @@ static ScmPort *make_walker_port(void)
 /* pass 1 */
 static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    ScmHashEntry *e;
-    ScmHashTable *ht;
+    ScmHashTable *ht = SCM_HASH_TABLE(SCM_CDR(port->data));
     ScmObj elt;
-    
-    ht = SCM_HASH_TABLE(SCM_CDR(port->data));
 
+#define REGISTER(obj)                                           \
+    do {                                                        \
+        ScmObj e = Scm_HashTableRef(ht, (obj), SCM_UNBOUND);    \
+        if (!SCM_UNBOUNDP(e)) {                                 \
+            Scm_HashTableSet(ht, (obj), SCM_TRUE, 0);           \
+            return;                                             \
+        }                                                       \
+        Scm_HashTableSet(ht, obj, SCM_FALSE, 0);                \
+    } while (0)
+    
     for (;;) {
-        if (!SCM_PTRP(obj) || SCM_SYMBOLP(obj) || SCM_KEYWORDP(obj)
-            || SCM_NUMBERP(obj)) {
+        if (!SCM_PTRP(obj) || SCM_KEYWORDP(obj) || SCM_NUMBERP(obj)
+            || (SCM_SYMBOLP(obj) && SCM_SYMBOL_INTERNED(obj))) {
             return;
         }
             
         if (SCM_PAIRP(obj)) {
-            e = Scm_HashTableGet(ht, obj);
-            if (e) { e->value = SCM_TRUE; return; }
-            Scm_HashTablePut(ht, obj, SCM_FALSE);
-
+            REGISTER(obj);
             elt = SCM_CAR(obj);
             if (SCM_PTRP(elt)) write_walk(SCM_CAR(obj), port, ctx);
             obj = SCM_CDR(obj);
             continue;
         }
         if (SCM_STRINGP(obj) && !SCM_STRING_NULL_P(obj)) {
-            e = Scm_HashTableGet(ht, obj);
-            if (e) { e->value = SCM_TRUE; return; }
-            Scm_HashTablePut(ht, obj, SCM_FALSE);
+            REGISTER(obj);
             return;
         }
         if (SCM_VECTORP(obj) && SCM_VECTOR_SIZE(obj) > 0) {
             int i, len = SCM_VECTOR_SIZE(obj);
-
-            e = Scm_HashTableGet(ht, obj);
-            if (e) { e->value = SCM_TRUE; return; }
-            Scm_HashTablePut(ht, obj, SCM_FALSE);
-
+            REGISTER(obj);
             for (i=0; i<len; i++) {
                 elt = SCM_VECTOR_ELEMENT(obj, i);
                 if (SCM_PTRP(elt)) write_walk(elt, port, ctx);
             }
             return;
         }
+        if (SCM_SYMBOLP(obj)) {
+            SCM_ASSERT(!SCM_SYMBOL_INTERNED(obj));
+            REGISTER(obj);
+            return;
+        }
         else {
             /* Now we have user-defined object.
                Call the user's print routine. */
-            e = Scm_HashTableGet(ht, obj);
-            if (e) { e->value = SCM_TRUE; return; }
-            Scm_HashTablePut(ht, obj, SCM_FALSE);
-
+            REGISTER(obj);
             write_general(obj, port, ctx);
             return;
         }
@@ -408,7 +406,7 @@ static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 /* pass 2 */
 static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    ScmHashEntry *e;
+    ScmObj e;
     char numbuf[50];  /* enough to contain long number */
     ScmHashTable *ht = NULL;
 
@@ -438,6 +436,10 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
             snprintf(buf, SPBUFSIZ, "%ld", SCM_INT_VALUE(obj));
             Scm_PutzUnsafe(buf, -1, port);
         }
+        else if (SCM_FLONUMP(obj)) {
+            write_general(obj, port, ctx);
+            return;
+        }
         else if (SCM_CHARP(obj)) {
             ScmChar ch = SCM_CHAR_VALUE(obj);
             if (SCM_WRITE_MODE(ctx) == SCM_WRITE_DISPLAY) {
@@ -466,18 +468,18 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     }
 
     if (ht) {
-        e = Scm_HashTableGet(ht, obj);
-        if (e && e->value != SCM_FALSE) {
-            if (SCM_INTP(e->value)) {
+        e = Scm_HashTableRef(ht, obj, SCM_FALSE);
+        if (!SCM_FALSEP(e)) {
+            if (SCM_INTP(e)) {
                 /* This object is already printed. */
-                snprintf(numbuf, 50, "#%ld#", SCM_INT_VALUE(e->value));
+                snprintf(numbuf, 50, "#%ld#", SCM_INT_VALUE(e));
                 Scm_PutzUnsafe(numbuf, -1, port);
                 return;
             } else {
                 /* This object will be seen again. Put a reference tag. */
                 int count = SCM_INT_VALUE(SCM_CAR(port->data));
                 snprintf(numbuf, 50, "#%d=", count);
-                e->value = SCM_MAKE_INT(count);
+                Scm_HashTableSet(ht, obj, SCM_MAKE_INT(count), 0);
                 SCM_SET_CAR(port->data, SCM_MAKE_INT(count+1));
                 Scm_PutzUnsafe(numbuf, -1, port);
             }
@@ -486,21 +488,24 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 
     /* Writes aggregates */
     if (SCM_PAIRP(obj)) {
-        /* special case for quote etc.*/
-        if (SCM_PAIRP(SCM_CDR(obj)) && SCM_NULLP(SCM_CDDR(obj))) {
-            int special = TRUE;
+        /* special case for quote etc.
+           NB: we need to check if we've seen SCM_CDR(obj), otherwise we'll
+           get infinite recursion for the case like (cdr '#1='#1#). */
+        if (SCM_PAIRP(SCM_CDR(obj)) && SCM_NULLP(SCM_CDDR(obj))
+            && (!ht
+                || SCM_FALSEP(Scm_HashTableRef(ht, SCM_CDR(obj), SCM_FALSE)))){
+            const char *prefix = NULL;
             if (SCM_CAR(obj) == SCM_SYM_QUOTE) {
-                Scm_PutcUnsafe('\'', port);
+                prefix = "'";
             } else if (SCM_CAR(obj) == SCM_SYM_QUASIQUOTE) {
-                Scm_PutcUnsafe('`', port);
+                prefix = "`";
             } else if (SCM_CAR(obj) == SCM_SYM_UNQUOTE) {
-                Scm_PutcUnsafe(',', port);
+                prefix = ",";
             } else if (SCM_CAR(obj) == SCM_SYM_UNQUOTE_SPLICING) {
-                Scm_PutzUnsafe(",@", -1, port);
-            } else {
-                special = FALSE;
+                prefix = ",@";
             }
-            if (special) {
+            if (prefix) {
+                Scm_PutzUnsafe(prefix, -1, port);
                 write_ss_rec(SCM_CADR(obj), port, ctx);
                 return;
             }
@@ -521,8 +526,8 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
                 return;
             }
             if (ht) {
-                e = Scm_HashTableGet(ht, obj); /* check for shared cdr */
-                if (e && e->value != SCM_FALSE) {
+                e = Scm_HashTableRef(ht, obj, SCM_FALSE); /* check for shared cdr */
+                if (!SCM_FALSEP(e)) {
                     Scm_PutzUnsafe(" . ", -1, port);
                     write_ss_rec(obj, port, ctx);
                     Scm_PutcUnsafe(')', port);
@@ -602,6 +607,7 @@ static void format_write(ScmObj obj, ScmPort *port, ScmWriteContext *ctx,
         return;
     }
     if (sharedp) {
+
         write_ss(obj, port, ctx);
     } else {
         write_ss_rec(obj, port, ctx);
@@ -692,7 +698,7 @@ static void format_integer(ScmPort *out, ScmObj arg,
         format_write(arg, out, &ictx, FALSE);
         return;
     }
-    if (SCM_FLONUMP(arg)) arg = Scm_InexactToExact(arg);
+    if (SCM_FLONUMP(arg)) arg = Scm_Exact(arg);
     if (nparams>0 && SCM_INTP(params[0])) mincol = SCM_INT_VALUE(params[0]);
     if (nparams>1 && SCM_CHARP(params[1])) padchar = SCM_CHAR_VALUE(params[1]);
     if (nparams>2 && SCM_CHARP(params[2])) commachar = SCM_CHAR_VALUE(params[2]);
@@ -869,7 +875,7 @@ static void format_proc(ScmPort *out, ScmString *fmt, ScmObj args, int sharedp)
                     } else {
                         backtracked = TRUE;
                     }
-                    if (argindex < 0 || argindex >= arglen) {
+                    if (argindex < 0 || argindex > arglen) {
                         Scm_Error("'~*' format directive refers outside of argument list in %S", fmt);
                     }
                     argcnt = argindex;
@@ -1162,6 +1168,15 @@ static void vprintf_proc(ScmPort *out, const char *fmt, ScmObj args,
             case '#':
                 pound_appeared++;
                 goto fallback;
+            case '*':
+                SCM_ASSERT(SCM_PAIRP(args));
+                if (dot_appeared) {
+                    prec = Scm_GetInteger(SCM_CAR(args));
+                } else {
+                    width = Scm_GetInteger(SCM_CAR(args));
+                }
+                args = SCM_CDR(args);
+                goto fallback;
             fallback:
             default:
                 SCM_DSTRING_PUTB(&argbuf, c);
@@ -1240,6 +1255,12 @@ void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
                     SCM_APPEND1(h, t, Scm_MakeInteger(c));
                     break;
                 }
+            case '*':
+                {
+                    int c = va_arg(ap, int);
+                    SCM_APPEND1(h, t, Scm_MakeInteger(c));
+                    continue;
+                }
             default:
                 continue;
             }
