@@ -11,13 +11,13 @@
 
 (sys-system "rm -rf tmp2.o")
 
-(test* "open-input-file" *test-error*
+(test* "open-input-file" (test-error <system-error>)
        (open-input-file "tmp2.o"))
 
 (test* "open-input-file :if-does-not-exist #f" #f
        (open-input-file "tmp2.o" :if-does-not-exist #f))
 
-(test* "open-output-file :if-does-not-exist :error" *test-error*
+(test* "open-output-file :if-does-not-exist :error" (test-error)
        (open-output-file "tmp2.o" :if-does-not-exist :error))
 
 (test* "open-output-file :if-does-not-exit #f" #f
@@ -36,7 +36,7 @@
          (close-input-port p)
          s))
 
-(test* "open-output-file :if-exists :error" *test-error*
+(test* "open-output-file :if-exists :error" (test-error)
        (open-output-file "tmp2.o" :if-exists :error))
 
 (test* "open-output-file :if-exists :supersede" 'cdefg
@@ -128,7 +128,7 @@
            (port-fd-dup! p1 p2)
            (list s1 (read-line p1))))
 
-  (test* "port-fd-dup!" *test-error*
+  (test* "port-fd-dup!" (test-error)
          (let* ((p1 (open-output-file "tmp1.o"))
                 (p2 (open-input-file "tmp2.o")))
            (guard (e (else
@@ -137,7 +137,7 @@
                       (raise e)))
              (port-fd-dup! p1 p2))))
 
-  (test* "port-fd-dup!" *test-error*
+  (test* "port-fd-dup!" (test-error)
          (let* ((p1 (open-input-file "tmp2.o")))
            (guard (e (else
                       (close-input-port p1)
@@ -663,11 +663,28 @@
 (test* "format ~@*" "1 2 5"
        (format #f "~a ~a ~4@*~a" 1 2 3 4 5))
 
-(test* "format incomplete tilde sequence" *test-error*
+(test* "format incomplete tilde sequence" (test-error)
        (format #f "~"))
-(test* "format incomplete tilde sequence" *test-error*
+(test* "format incomplete tilde sequence" (test-error)
        (format #f "~123"))
 
+(test* "format skip to the end" "||"
+       (format #f "|~*|" 1))
+
+;;-------------------------------------------------------------------
+(test-section "some corner cases in list reader")
+
+(define (dot-reader-tester str expect)
+  (test* (format "dot ~a" str) expect (read-from-string str)))
+
+(dot-reader-tester "(().())"  '(()))
+(dot-reader-tester "([].[])"  '(()))
+(dot-reader-tester "(x .,y)"  '(x unquote y))
+(dot-reader-tester "(x .,@y)" '(x unquote-splicing y))
+(dot-reader-tester "(().)"    (test-error <read-error>))
+(dot-reader-tester "((). .)"  (test-error <read-error>))
+
+
 ;;-------------------------------------------------------------------
 (test-section "nested multi-line comments")
 
@@ -721,6 +738,35 @@
        (begin (list #,(countup) #;#,(countup) #,(countup))
               *counter*))
 
+;;-------------------------------------------------------------------
+(test-section "port->* basic")
+
+;; testing port->string etc.
+
+(define (%test-port->* name proc data writer)
+  (test* (format "~a ~s" name data) data
+         (begin
+           (sys-unlink "tmp2.o")
+           (with-output-to-file "tmp2.o" (cut writer data))
+           (call-with-input-file "tmp2.o" proc))))
+(define-syntax test-port->*
+  (syntax-rules ()
+    [(_ proc data writer) (%test-port->* 'proc proc data writer)]))
+
+(test-port->* port->string "" display)
+(test-port->* port->string "abc" display)
+(test-port->* port->string "abc\ndef\n" display)
+(test-port->* (.$ string-complete->incomplete port->string)
+              #*"\x00\x80\xc0\xd0\xff\xfe\xef" display)
+
+(test-port->* port->string-list '("abc") (cut for-each print <>))
+(test-port->* port->string-list '("abc" "def") (cut for-each print <>))
+(test-port->* (.$ (cut map string-complete->incomplete <>) port->string-list)
+              '(#*"\x00\x80\xc0\xd0\xff\xfe\xef" #*"abc")
+              (cut for-each (^z (display z) (newline)) <>))
+
+(test-port->* port->sexp-list '(abc) (cut for-each print <>))
+
 ;;-------------------------------------------------------------------
 (test-section "coding-aware-port basic")
 
