@@ -101,22 +101,22 @@
             (mixlevel3 1 (2 3 4 5 6) (7 8 9 10)))
 
 ;; test that wrong usage of ellipsis is correctly identified
-(test "bad epplisis 1" *test-error*
+(test "bad epplisis 1" (test-error)
       (lambda () 
         (eval '(define-syntax badellipsis
                  (syntax-rules () (t) (3 ...)))
               (interaction-environment))))
-(test "bad epplisis 2" *test-error*
+(test "bad epplisis 2" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
                  (syntax-rules () (t a) (a ...)))
               (interaction-environment))))
-(test "bad epplisis 3" *test-error*
+(test "bad epplisis 3" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
                  (syntax-rules () (t a b ...) (a ...)))
               (interaction-environment))))
-(test "bad epplisis 4" *test-error*
+(test "bad epplisis 4" (test-error)
       (lambda ()
         (eval '(define-syntax badellipsis
                  (syntax-rules () (t a ...) ((a ...) ...)))
@@ -318,6 +318,18 @@
                       ((>= x 10) y)
                       (set! y (* y 2)))))
 
+;;----------------------------------------------------------------------
+;; non-syntax-rule transformers
+
+(test-section "transformers other than syntax-rules")
+
+(define-syntax xif if)
+(test "xif" 'ok (lambda () (xif #f 'ng 'ok)))
+
+(define-syntax fi (syntax-rules () [(_ a b c) (xif a c b)]))
+(define-syntax xfi fi)
+(test "xfi" 'ok (lambda () (xfi #f 'ok 'ng)))
+
 ;;----------------------------------------------------------------------
 ;; local syntactic bindings.
 
@@ -426,6 +438,15 @@
           (let-syntax((l(syntax-rules()((l((x(y ...))...)b ...)(let-syntax((x (syntax-rules()y ...))...) b ...)))))(l('(('(a b ...)(lambda a b ...)))`((`(a b c)(if a b c))(`(a)(car a))),((,(a b)(set! a b))(,(a)(cdr a))),@((,@z(call-with-current-continuation z))))'((ls)('((s)('((i) ('((d)('((j)('((c)('((p)('((l)('(()(l l))))'((k)`((pair?,(p))('((c) ,(p(append,(,(p))(d c)))(k k))(c`(p)`(,(p))c))`(p)))))(cons(d)(map d ls))))'((x y c),@'((-)(s x y null? - s)(j x y c)))))'((x y c)('((q)('((f)(cons`(q)(c((f x)x)((f y)y)c)))'((h)`((eq? q h)'((x),(x)) i)))),@'((-)(s x y'((z)(>=`(z)(sqrt(*`(x)`(y)))))- s))))))list)) '((z)z)))'((x y p k l),@'((-)`((p x)(k y)(l y x'((z)`((p z)-(- #f)))k l)))))))))
         (the-procedure '(5 1 9 3))))
 
+
+(test "let-syntax, rebinding syntax" 'ok
+      (lambda ()
+        (let-syntax ([xif if] [if when]) (xif #f 'ng 'ok))))
+
+(test "let-syntax, rebinding macro" 'ok
+      (lambda ()
+        (let-syntax ([if fi]) (if #f 'ok 'ng))))
+
 ;;----------------------------------------------------------------------
 ;; macro and internal define
 
@@ -458,12 +479,12 @@
         (gen-idef-1 4)
         (list foo bar)))
 
-(test "define foo (error)" *test-error*
+(test "define foo (error)" (test-error)
       (lambda ()
         (eval '(let ()
                  (list 3 4)
                  (gen-idef-1 5)))))
-(test "define foo (error)" *test-error*
+(test "define foo (error)" (test-error)
       (lambda ()
         (eval '(let ()
                  (gen-idef-1 5)))))
@@ -513,13 +534,13 @@
           (gen-idef-2 3 4))
         (define baz 20)
         (list foo bar baz biz)))
-(test "define foo, bar (error)" *test-error*
+(test "define foo, bar (error)" (test-error)
       (lambda ()
         (eval '(let ()
                  (list 3)
                  (gen-idef-2 -1 -2)
                  (list foo bar)))))
-(test "define foo, bar (error)" *test-error*
+(test "define foo, bar (error)" (test-error)
       (lambda ()
         (eval '(let ()
                  (gen-idef-2 -1 -2)))))
@@ -881,6 +902,13 @@
 (test "if-let1" 'bar
       (lambda () (if-let1 it (memq 'a '(b c d)) 'boo 'bar)))
 
+(test "ecase" 'b
+      (lambda () (ecase 3 ((1) 'a) ((2 3) 'b) ((4) 'c))))
+(test "ecase" (test-error)
+      (lambda () (ecase 5 ((1) 'a) ((2 3) 'b) ((4) 'c))))
+(test "ecase" 'd
+      (lambda () (ecase 5 ((1) 'a) ((2 3) 'b) ((4) 'c) (else 'd))))
+
 ;;----------------------------------------------------------------------
 ;; macro-expand
 
@@ -894,4 +922,17 @@
 (test "macroexpand-1" '(bar 1 1)
       (lambda () (macroexpand-1 '(foo 1))))
 
+;;----------------------------------------------------------------------
+;; not allowing first-class macro
+
+(test-section "failure cases")
+
+(define-macro (bad-if a b c) `(,if ,a ,b ,c))
+(test "reject first-class syntax usage" (test-error)
+      (lambda () (bad-if #t 'a 'b)))
+
+(define-macro (bad-fi a b c) `(,fi ,a ,b ,c))
+(test "reject first-class macro usage" (test-error)
+      (lambda () (bad-fi #t 'a 'b)))
+
 (test-end)
