@@ -91,8 +91,20 @@
 ;;----------------------------------------------------------------
 (test-section "application")
 
-(prim-test "apply" '(1 2 3) (lambda ()  (apply list 1 '(2 3))))
-(prim-test "apply" '(1 2 3) (lambda ()  (apply apply (list list 1 2 '(3)))))
+(prim-test "apply" '(1 2 3) (lambda ()  (apply list '(1 2 3))))
+(prim-test "apply" '(2 3 4) (lambda ()  (apply list 2 '(3 4))))
+(prim-test "apply" '(3 4 5) (lambda ()  (apply list 3 4 '(5))))
+(prim-test "apply" '(4 5 6) (lambda ()  (apply list 4 5 6 '())))
+
+(prim-test "apply^2" '() (lambda () (apply apply list '() '())))
+(prim-test "apply^2" '() (lambda () (apply apply list '(()))))
+(prim-test "apply^2" '(1 . 2) (lambda () (apply apply cons '((1 2)))))
+(prim-test "apply^2" '(3 . 4) (lambda () (apply apply cons 3 '((4)))))
+(prim-test "apply^2" '(5 . 6) (lambda () (apply apply (list cons 5 '(6)))))
+                                          
+
+(prim-test "apply" '(6 7 8) (lambda ()  (apply apply (list list 6 7 '(8)))))
+
 
 ;; This tests 'unfolding' path in ADJUST_ARGUMENT_FRAME.
 (prim-test "apply, copying args" '(1 2 3)
@@ -110,6 +122,25 @@
                  (set-car! (cdr new) '100)
                  orig))))
 
+;; This test exhibits the optimizer bug reported by Michael Campbell.
+(define bug-optimizer-local-inliner
+  (lambda (flag)
+    (define (a . args)
+      (receive x args
+        (cons x x)
+        (apply values x))
+      (apply format args))
+    (define (b bar)
+      (a "~a" bar))
+    (b 1)
+    (cond
+     (flag (b 1))
+     (else (a "~a" 1)))))
+(prim-test "apply local inliner optimizer" "1"
+           (lambda () (bug-optimizer-local-inliner #f)) equal?)
+(prim-test "apply local inliner optimizer" "1"
+           (lambda () (bug-optimizer-local-inliner #t)) equal?)
+
 (prim-test "map" '()         (lambda ()  (map car '())))
 (prim-test "map" '(1 2 3)    (lambda ()  (map car '((1) (2) (3)))))
 (prim-test "map" '(() () ()) (lambda ()  (map cdr '((1) (2) (3)))))
@@ -160,17 +191,36 @@
 (prim-test "qq"  101            (lambda ()  `,quasi1))
 (prim-test "qq," '((1 . 2))     (lambda ()  `(,(cons 1 2))))
 (prim-test "qq," '((1 . 2) 3)   (lambda ()  `(,(cons 1 2) 3)))
+(prim-test "qq," '(0 (1 . 2))   (lambda ()  `(0 ,(cons 1 2))))
+(prim-test "qq," '(0 (1 . 2) 3) (lambda ()  `(0 ,(cons 1 2) 3)))
+(prim-test "qq," '(((1 . 2)))   (lambda ()  `((,(cons 1 2)))))
+(prim-test "qq," '(((1 . 2)) 3) (lambda ()  `((,(cons 1 2)) 3)))
 (prim-test "qq," '(99 3)        (lambda ()  `(,quasi0 3)))
 (prim-test "qq," '(3 99)        (lambda ()  `(3 ,quasi0)))
+(prim-test "qq," '(3 99 3)      (lambda ()  `(3 ,quasi0 3)))
 (prim-test "qq," '(100 3)       (lambda ()  `(,(+ quasi0 1) 3)))
 (prim-test "qq," '(3 100)       (lambda ()  `(3 ,(+ quasi0 1))))
 (prim-test "qq," '(101 3)       (lambda ()  `(,quasi1 3)))
 (prim-test "qq," '(3 101)       (lambda ()  `(3 ,quasi1)))
 (prim-test "qq," '(102 3)       (lambda ()  `(,(+ quasi1 1) 3)))
 (prim-test "qq," '(3 102)       (lambda ()  `(3 ,(+ quasi1 1))))
+(prim-test "qq,(r6rs)" '(98 99 (a b) 100)
+           (lambda () `(98 (unquote quasi0 quasi2) 100)))
+(prim-test "qq,(r6rs)" '(98 99 101 100)
+           (lambda () `(98 (unquote quasi0 quasi1) 100)))
+(prim-test "qq,(r6rs)" '(98 99 (a b) 100)
+           (lambda () `(98 (unquote quasi0 quasi2) 100)))
+(prim-test "qq,(r6rs)" '(98 99 (a b) (1 2) (3 4))
+           (lambda () `(98 (unquote quasi0 quasi2) (unquote (list 1 2) (list 3 4)))))
 (prim-test "qq@" '(1 2 3 4)     (lambda ()  `(1 ,@(list 2 3) 4)))
 (prim-test "qq@" '(1 2 3 4)     (lambda ()  `(1 2 ,@(list 3 4))))
 (prim-test "qq@" '(a b c d)     (lambda ()  `(,@quasi2 ,@quasi3)))
+(prim-test "qq@(r6rs)" '(1 a b a b 2)
+           (lambda () `(1 (unquote-splicing quasi2 quasi2) 2)))
+(prim-test "qq@(r6rs)" '(1 a b c d 2)
+           (lambda () `(1 (unquote-splicing quasi2 quasi3) 2)))
+(prim-test "qq@(r6rs)" '(1 a b c d 2)
+           (lambda () `(1 (unquote-splicing (list 'a 'b) '(c d)) ,@(list 2))))
 (prim-test "qq." '(1 2 3 4)     (lambda ()  `(1 2 . ,(list 3 4))))
 (prim-test "qq." '(a b c d)     (lambda ()  `(,@quasi2 . ,quasi3)))
 (prim-test "qq#," '#((1 . 2) 3) (lambda ()  `#(,(cons 1 2) 3)))
@@ -214,6 +264,10 @@
            (lambda ()  `(1 `(1 ,,quasi0 ,,quasi1) 1)))
 (prim-test "qqq"   '(1 `(1 ,@2 ,@(1 2)))
            (lambda () `(1 `(1 ,@2 ,@,(list 1 2)))))
+(prim-test "qqq"   '(1 `(1 ,@2 (unquote 1 2)))
+           (lambda () `(1 `(1 ,@2 ,,@(list 1 2)))))
+(prim-test "qqq"   '(1 `(1 ,@2 (unquote-splicing 1 2)))
+           (lambda () `(1 `(1 ,@2 ,@,@(list 1 2)))))
 (prim-test "qqq"   '(1 `(1 ,@(a b) ,@(c d)))
            (lambda () `(1 `(1 ,@,quasi2 ,@,quasi3))))
 (prim-test "qqq"   '(1 `(1 ,(a b x) ,(y c d)))
@@ -319,6 +373,39 @@
            (lambda ()
              (let ((f (lambda (i) (set! i 0) i))) (f (f 1)))))
 
+;; this caused an internal compiler error in 0.9.1
+(define (zero) 0)
+(prim-test "pass3 inlining with pass3/$call optimization" #t
+           (lambda ()
+             (eval '((letrec ((f (lambda (a b)
+                                   (do ((x a (+ x 1)))
+                                       ((>= x b))))))
+                       f)
+                     (zero) (zero))
+                   (interaction-environment))))
+
+;; This caused internal error in 0.9.1, and infinite loop in dev version
+;; after it.
+(prim-test "pass3/$call inlining problem" #t
+           (lambda ()
+             (procedure?
+              (eval '(lambda (n p t)
+                       (define (y a r s f)
+                         (let loop ([e 0])
+                           (cond [(a n) (unwind-protect (s) (r n))]
+                                 [(< e 10) (loop (+ 1 e))]
+                                 [else (f)])))
+                       (define (l0 a r)
+                         (y a r (^() (r n)) (^() (error "oo"))))
+                       ;; Main locker
+                       (define (l1 a r)
+                         (y a r p (^() (if (and-let* ([ t ]
+                                                      [m (file-mtime n)])
+                                             (< (+ m t) 10))
+                                         (begin (l0 a r) (l1 a r))))))
+                       (error "zz"))
+                    (interaction-environment)))))
+
 ;;----------------------------------------------------------------
 (test-section "lazy, delay & force")
 
