@@ -6,6 +6,8 @@
 (use gauche.sequence)
 (use gauche.process)
 (use util.list)
+(use util.match)
+(use srfi-19)
 (test-start "rfc")
 
 ;;--------------------------------------------------------------------
@@ -121,6 +123,16 @@ Content-Length: 4349
 (test* "rfc822-parse-date (invalid)" '(#f #f #f #f #f #f #f #f)
        (receive r (rfc822-parse-date "Sun 2 Mar 2002") r))
 
+(test* "date->rfc822-date" "Sun, 29 Nov 2009 01:23:45 +0000"
+       (date->rfc822-date (make-date 0 45 23 01 29 11 2009 0)))
+(test* "date->rfc822-date" "Sun, 29 Nov 2009 01:23:45 +0900"
+       (date->rfc822-date (make-date 0 45 23 01 29 11 2009 32400)))
+(test* "date->rfc822-date" "Sun, 29 Nov 2009 01:23:45 -0830"
+       (date->rfc822-date (make-date 0 45 23 01 29 11 2009 -30600)))
+(test* "date->rfc822-date" "Sun, 29 Nov 2009 01:23:45 +0030"
+       (date->rfc822-date (make-date 0 45 23 01 29 11 2009 1800)))
+
+
 (test* "rfc822-invalid-header-field" #f
        (rfc822-invalid-header-field "abcde"))
 (test* "rfc822-invalid-header-field" 'incomplete-string
@@ -398,26 +410,31 @@ Content-Length: 4349
 
 (test* "ip-version" 4
        (ip-version '#u8(69 0 0 36 139 12 0 0 64 1 241 202
-                        127 0 0 1 127 0 0 1 0 0 205 245 50 10 0 0)))
+                        127 0 0 1 127 0 0 1 0 0 205 245 50 10 0 0)
+                   0))
 (test* "ip-version" 6
        (ip-version '#u8(#x60 0 0 0 0 0 17 0
                         0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
-                        0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1)))
+                        0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1)
+                   0))
 
 (test* "ip-protocol" 1
        (ip-protocol '#u8(69 0 0 36 139 12 0 0 64 1 241 202
-                         127 0 0 1 127 0 0 1 0 0 205 245 50 10 0 0)))
+                         127 0 0 1 127 0 0 1 0 0 205 245 50 10 0 0)
+                    0))
 (test* "ip-protocol" 17
        (ip-protocol '#u8(#x60 0 0 0 0 0 17 0
                          0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
-                         0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1)))
+                         0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1)
+                    0))
 (test* "ip-protocol" 17
        (ip-protocol '#u8(#x60 0 0 0 0 0 0 0
                          0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
                          0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1
                          60 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0
                          43 2 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
-                         17 0 0 0 0 0 0 0)))
+                         17 0 0 0 0 0 0 0)
+                    0))
 
 ;;--------------------------------------------------------------------
 ;; NB: rfc.json test is moved to under ext/peg, since it totally
@@ -485,6 +502,18 @@ Content-Length: 4349
            a=b"))
 |#
 
+(test* "mime-compose-parameters (simple)"
+       "; ab=cd; ef=gh"
+       (mime-compose-parameters '((ab . cd) (ef . gh)) #f))
+(test* "mime-compose-parameters (quote)"
+       "; ab=\"c d\"; ef=\"\\\"\\\\\""
+       (mime-compose-parameters '((ab . "c d") (ef . "\"\\")) #f))
+(test* "mime-compose-parameters (long)"
+       "; ab=cd;\r\n foo=012345678901234567890123456789012345679012345678901234567890123456789"
+       (mime-compose-parameters
+        '((ab . cd) (foo . "012345678901234567890123456789012345679012345678901234567890123456789"))
+        #f))
+
 (test* "mime-encode-text (pass-through)" "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod\r\n tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim\r\n veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea\r\n commodo consequat. Duis aute irure dolor in reprehenderit in voluptate\r\n velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat\r\n cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id\r\n est laborum."
        (mime-encode-text "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."))
 (test* "mime-encode-text (pass-through, nonbreak)" "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
@@ -597,6 +626,60 @@ Content-Length: 4349
         '(("mime-version" " 1.0")
           ("content-type" "multipart/form-data; boundary=\"---------------------------6578815652962098482130719379\"")))))
 
+(let1 b (mime-make-boundary)
+  (test* "mime-compose-message (simple)"
+         (string-append "\r\n--"b"\r\n"
+                        "Content-type: text/plain\r\n"
+                        "Content-transfer-encoding: 7bit\r\n\r\n"
+                        "This is a pen."
+                        "\r\n--"b"\r\n"
+                        "Content-type: text/html; charset=us-ascii\r\n\r\n"
+                        "<html><head></head><body></body></html>"
+                        "\r\n--"b"\r\n"
+                        "Content-type: application/octet-stream\r\n"
+                        "Content-transfer-encoding: base64\r\n\r\n"
+                        "YWJjZGVmZw=="
+                        "\r\n--"b"--\r\n")
+         (receive (s bb)
+           (mime-compose-message-string
+            '((("text" "plain")
+               (("content-transfer-encoding" "7bit"))
+               "This is a pen.")
+              (("text" "html" ("charset" . "us-ascii"))
+               ()
+               "<html><head></head><body></body></html>")
+              (("application" "octet-stream")
+               (("content-transfer-encoding" "base64"))
+               "abcdefg"))
+            :boundary b)
+           (and (equal? b bb) s))))
+
+(define (mime-roundtrip-tester num)
+  (define (gen-parts mesg)
+    (match mesg
+      [(ctype _ (? string? body))
+       `(,(mime-parse-content-type ctype) '() ,body)]
+      [("message/rfc822" _ ("text/plain" _ body))
+       `(("message" "rfc822") '()
+         ,#`"content-type: text/plain\r\n\r\n,body")]
+      [(ctype _ children ...)
+       `(,(mime-parse-content-type ctype) '()
+         (subparts ,@(map gen-parts children)))]))
+  (let1 src (call-with-input-file #`"../test/data/rfc-mime-,|num|.res.txt" read)
+    (receive (composed boundary)
+        (mime-compose-message-string (list (gen-parts src)))
+      (test* #`"mime-roundtrip (,num)"
+             `("multipart/mixed" 0 ,src)
+             (mime-message-resolver
+              (call-with-input-string composed
+                (cut mime-parse-message <>
+                     `(("mime-version" "1.0")
+                       ("content-type" ,#`"multipart/mixed; boundary=\",|boundary|\""))
+                     (cut mime-body->string <> <>)))
+              #f)))))
+                     
+(dotimes (n 8) (mime-roundtrip-tester n))
+    
 ;;--------------------------------------------------------------------
 (test-section "rfc.uri")
 (use rfc.uri)
@@ -656,6 +739,20 @@ Content-Length: 4349
             "//www.example.com:8080/?abc")
          r))
 
+(test* "uri-decompose-hierarchical" '("www.example.com:8080"
+                                      #f
+                                      #f
+                                      #f)
+       (receive r (uri-decompose-hierarchical "//www.example.com:8080") r))
+
+(test* "uri-decompose-hierarchical" '((#f #f  #f #f)
+                                      ("" #f  #f #f)
+                                      (#f "/" #f #f)
+                                      ("" "/" #f #f))
+       (map (lambda (specific)
+              (receive r (uri-decompose-hierarchical specific) r))
+            '("" "//" "/" "///")))
+
 (test* "uri-decompose-authority" '(#f "www.example.com" #f)
        (receive r (uri-decompose-authority "www.example.com") r))
 (test* "uri-decompose-authority" '(#f "www.example.com" "8080")
@@ -678,6 +775,68 @@ Content-Length: 4349
 (test* "uri-parse" '("mailto" #f #f #f "shiro@example.com" #f #f)
        (receive r (uri-parse "mailto:shiro@example.com") r))
 
+(let ([base0 "http://a/b/c/d;p?q"])
+  (define (t base rel expect) 
+    (test* (format "merging ~s onto ~s" rel base)
+           expect (uri-merge base rel)))
+  (define t0 (pa$ t base0))
+  ;; examples given in RFC3986 section 5.4
+  ;; normal path
+  (t0 "g:h" "g:h")
+  (t0 "g" "http://a/b/c/g")
+  (t0 "./g" "http://a/b/c/g")
+  (t0 "g/" "http://a/b/c/g/")
+  (t0 "/g" "http://a/g")
+  (t0 "//g" "http://g")
+  (t0 "?y" "http://a/b/c/d;p?y")
+  (t0 "g?y" "http://a/b/c/g?y")
+  (t0 "#s" "http://a/b/c/d;p?q#s")
+  (t0 "g#s" "http://a/b/c/g#s")
+  (t0 "g?y#s" "http://a/b/c/g?y#s")
+  (t0 ";x" "http://a/b/c/;x")
+  (t0 "g;x" "http://a/b/c/g;x")
+  (t0 "g;x?y#s" "http://a/b/c/g;x?y#s")
+  (t0 "" "http://a/b/c/d;p?q")
+  (t0 "." "http://a/b/c/")
+  (t0 "./" "http://a/b/c/")
+  (t0 ".." "http://a/b/")
+  (t0 "../" "http://a/b/")
+  (t0 "../g" "http://a/b/g")
+  (t0 "../.." "http://a/")
+  (t0 "../../" "http://a/")
+  (t0 "../../g" "http://a/g")
+  ;; failure path
+  (t0 "../../../g" "http://a/g")
+  (t0 "../../../../g" "http://a/g")
+  (t0 "/./g" "http://a/g")
+  (t0 "/../g" "http://a/g")
+  (t0 "g." "http://a/b/c/g.")
+  (t0 ".g" "http://a/b/c/.g")
+  (t0 "g.." "http://a/b/c/g..")
+  (t0 "..g" "http://a/b/c/..g")
+  (t0 "./../g" "http://a/b/g")
+  (t0 "./g/." "http://a/b/c/g/")
+  (t0 "g/./h" "http://a/b/c/g/h")
+  (t0 "g/../h" "http://a/b/c/h")
+  (t0 "g;x=1/./y" "http://a/b/c/g;x=1/y")
+  (t0 "g;x=1/../y" "http://a/b/c/y")
+  (t0 "g?y/./x" "http://a/b/c/g?y/./x")
+  (t0 "g?y/../x" "http://a/b/c/g?y/../x")
+  (t0 "g#s/./x" "http://a/b/c/g#s/./x")
+  (t0 "g#s/../x" "http://a/b/c/g#s/../x")
+  (t0 "http:g" "http:g") ;; for strict parser
+
+  ;; some edge cases.  the first case works since we do pre-normalization
+  ;; of the base URI (RFC3986 5.2.1), which is optional.
+  (t "http://example.com/foo/.." "./" "http://example.com/")
+  (t "http://example.com/" "./foo/bar/.." "http://example.com/foo/")
+
+  ;; empty base-path case
+  (t "http://example.com"  "foo" "http://example.com/foo")
+  (t "http://example.com"  "./foo" "http://example.com/foo")
+  (t "http://example.com"  "../foo" "http://example.com/foo")
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "rfc.http")
 (use rfc.http)
@@ -685,16 +844,21 @@ Content-Length: 4349
 
 (use gauche.parameter)
 
-(test* "http-user-agent" "gauche.http/0.1"
+(test* "http-user-agent" #`"gauche.http/,(gauche-version)"
        (and (is-a? http-user-agent <parameter>)
             (http-user-agent)))
 
+(test* "http-compose-query" "/search?q=foo%20bar&n=20"
+       (http-compose-query "/search" '((q "foo bar") (n 20))))
+
 (define *http-port* 6726)
 
 (define (alist-equal? alis1 alis2)
   (define (%sort alis)
     (sort alis (lambda (a b) (string<? (car a) (car b)))))
-  (equal? (%sort alis1) (%sort alis2)))
+  (and (list? alis1)
+       (list? alis2)
+       (equal? (%sort alis1) (%sort alis2))))
 
 (define *simple-httpd*
   '(
@@ -720,35 +884,38 @@ Content-Length: 4349
         (hash-table-put! ht "/chunked"
                          '("HTTP/1.x 200 OK\nTransfer-Encoding: chunked\n\n"
                            "2\r\nOK\n0\r\n\r\n"))
+        (hash-table-put! ht "/void"
+                         '("HTTP/1.x 404 Not found\nContent-type: text/plain\n"
+                           "Content-length: 9\n\nNot found"))
         ht))
 
     (define (http-server socket)
       (let loop ()
-        (let* ((client (socket-accept socket))
-               (in  (socket-input-port client))
-               (out (socket-output-port client))
-               (request-line (read-line in)))
+        (let* ([client (socket-accept socket)]
+               [in  (socket-input-port client)]
+               [out (socket-output-port client)]
+               [request-line (read-line in)])
           (rxmatch-if (#/^(\S+) (\S+) HTTP\/1\.1$/ request-line)
               (#f method request-uri)
-            (let* ((headers (rfc822-read-headers in))
-                   (bodylen
-                    (cond ((assoc-ref headers "content-length")
-                           => (lambda (e) (string->number (car e))))
-                          (else 0)))
-                   (body (read-block bodylen in)))
+            (let* ([headers (rfc822-read-headers in)]
+                   [bodylen
+                    (cond [(assoc-ref headers "content-length")
+                           => (lambda (e) (string->number (car e)))]
+                          [else 0])]
+                   [body (read-block bodylen in)])
               (cond
-               ((equal? request-uri "/exit")
-                (sys-exit 0))
-               ((hash-table-get %predefined-contents request-uri #f)
-                => (lambda (contents)
-                     (for-each (cut display <> out) contents)))
-               (else
+               [(equal? request-uri "/exit")
+		(socket-close client)
+                (sys-exit 0)]
+               [(hash-table-get %predefined-contents request-uri #f)
+                => (cut for-each (cut display <> out) <>)]
+               [else
                 (display "HTTP/1.x 200 OK\nContent-Type: text/plain\n\n" out)
                 (write `(("method" ,method)
                          ("request-uri" ,request-uri)
                          ("request-body" ,(string-incomplete->complete body))
                          ,@headers)
-                       out)))
+                       out)])
               (socket-close client)
               (loop))
             (error "malformed request line:" request-line)))))
@@ -767,63 +934,174 @@ Content-Length: 4349
   (read-line (process-output p)) ; handshake
   )
 
-(test* "http-get" `(("method" "GET")
-                    ("request-uri" "/get")
-                    ("host" ,#`"localhost:,*http-port*")
-                    ("my-header" "foo")
-                    ("request-body" ""))
-       (receive (code headers body)
-           (http-get #`"localhost:,*http-port*" "/get"
-             :my-header "foo")
-         (and (equal? code "200")
-              (equal? headers '(("content-type" "text/plain")))
-              (read-from-string body)))
-       alist-equal?)
+(let ([expected `(("method" "GET")
+                  ("request-uri" "/get")
+                  ("request-body" "")
+                  ("host" ,#`"localhost:,*http-port*")
+                  ("user-agent" ,#`"gauche.http/,(gauche-version)")
+                  ("my-header" "foo"))]
+      [host #`"localhost:,*http-port*"])
+  (define (req-body . args)
+    (values-ref (apply http-request args) 2))
+
+  (test* "http-get, default string receiver" expected
+         (receive (code headers body)
+             (http-request 'GET host "/get" :my-header "foo")
+           (and (equal? code "200")
+                (equal? headers '(("content-type" "text/plain")))
+                (read-from-string body)))
+         alist-equal?)
+
+  (test* "http-get, custom receiver" expected
+         (req-body 'GET host "/get"
+                   :receiver (lambda (code hdrs total retr)
+                               (let loop ((result '()))
+                                 (receive (port size) (retr)
+                                   (if (and size (= size 0))
+                                     result
+                                     (loop (append result (read port)))))))
+                   :my-header "foo")
+         alist-equal?)
+
+  (sys-unlink "test.o")
+  (test* "http-get, file receiver" expected
+         (let1 f (req-body 'GET host "/get"
+                           :receiver (http-file-receiver "test.o")
+                           :my-header :foo)
+           (and (equal? f "test.o")
+                (with-input-from-file "test.o" read)))
+         alist-equal?)
+  (sys-unlink "test.o")
+
+  (test* "http-get, file receiver (tmp)" expected
+         (let1 f (req-body 'GET host "/get"
+                           :receiver (http-file-receiver "test.o"
+                                                         :temporary #t)
+                           :my-header :foo)
+           (and (not (equal? f "test.o"))
+                (begin0 (with-input-from-file f read)
+                  (sys-unlink f))))
+         alist-equal?)
+
+  (let ()
+    (define cond-receiver
+      (http-cond-receiver
+       ["404" (lambda (code hdrs total retr)
+                (let1 sink (open-output-file
+                            (cond-expand
+                             [gauche.os.windows "NUL"]
+                             [else "/dev/null"]))
+                  (let loop ()
+                    (receive (port size) (retr)
+                      (cond
+                       [(and size (= size 0)) (close-output-port sink) 404]
+                       [else (copy-port port sink :size size) (loop)])))))]            ["200" (lambda (code hdrs total retr)
+                (let loop ((result '()))
+                  (receive (port size) (retr)
+                    (if (and size (= size 0))
+                      result
+                      (loop (append result (read port)))))))]
+       ))
+
+    (test* "http-get, cond-receiver" expected
+           (req-body 'GET host "/get"
+                     :receiver cond-receiver
+                     :my-header :foo))
+
+    (test* "http-get, cond-receiver" 404
+           (req-body 'GET host "/void"
+                     :receiver cond-receiver
+                     :my-header :foo)))
+  )
 
 (test* "http-get (redirect)" "/redirect02"
        (receive (code headers body)
-           (http-get #`"localhost:,*http-port*" "/redirect01")
+           (http-request 'GET #`"localhost:,*http-port*" "/redirect01")
          (cond ((assoc-ref (read-from-string body) "request-uri")
                 => car))))
 
 (test* "http-get (redirect)" "/redirect12"
        (receive (code headers body)
-           (http-get #`"localhost:,*http-port*" "/redirect11")
+           (http-request 'GET #`"localhost:,*http-port*" "/redirect11")
          (cond ((assoc-ref (read-from-string body) "request-uri")
                 => car))))
 
-(test* "http-get (loop)" 'ok
-       (guard (e ((<http-error> e) 'ok))
-         (http-get #`"localhost:,*http-port*" "/loop1")
-         'error))
+(test* "http-get (loop)" (test-error <http-error>)
+       (http-request 'GET #`"localhost:,*http-port*" "/loop1"))
 
 (test* "http-get (chunked body)" "OK"
        (receive (code headers body)
-           (http-get #`"localhost:,*http-port*" "/chunked")
+           (http-request 'GET #`"localhost:,*http-port*" "/chunked")
          body))
 
 (test* "http-head" #t
        (receive (code headers body)
-           (http-head #`"localhost:,*http-port*" "/")
+           (http-request 'HEAD #`"localhost:,*http-port*" "/")
          (and (equal? code "200")
               (equal? headers '(("content-type" "text/plain")))
               (not body))))
 
-(test* "http-post" `(("method" "POST")
-                     ("request-uri" "/post")
-                     ("content-length" "4")
-                     ("host" ,#`"localhost:,*http-port*")
-                     ("request-body" "data"))
-       (receive (code headers body)
-           (http-post #`"localhost:,*http-port*" "/post" "data")
-         (and (equal? code "200")
-              (equal? headers '(("content-type" "text/plain")))
-              (read-from-string body)))
-       alist-equal?)
+(let1 expected `(("method" "POST")
+                 ("request-uri" "/post")
+                 ("content-length" "4")
+                 ("host" ,#`"localhost:,*http-port*")
+                 ("user-agent" ,#`"gauche.http/,(gauche-version)")
+                 ("request-body" "data"))
+  (define (tester msg thunk)
+    (test* #`"http-post ,msg" expected
+           (receive (code headers body) (thunk)
+             (and (equal? code "200")
+                  (equal? headers '(("content-type" "text/plain")))
+                  (read-from-string body)))
+           alist-equal?))
+
+  (tester "(new API)"
+          (lambda ()
+            (http-request 'POST #`"localhost:,*http-port*" "/post"
+                          :sender (http-string-sender "data"))))
+
+  (tester "(old API)"
+          (lambda ()
+            (http-post #`"localhost:,*http-port*" "/post" "data")))
+  )
+
+(let1 expected '(("a" "b") ("c" "d"))
+  (define (tester msg thunk)
+    (test* #`"http-post (multipart/form-data) ,msg" expected
+           (receive (code headers body) (thunk)
+             
+             (and-let* ([ (equal? code "200") ]
+                        [ (equal? headers '(("content-type" "text/plain"))) ]
+                        [r (read-from-string body)]
+                        [body (assoc "request-body" r)]
+                        [part (call-with-input-string (cadr body)
+                                (cut mime-parse-message <> r mime-body->string))]
+                        [ (is-a? part <mime-part>) ]
+                        [ (list? (ref part'content)) ])
+               (map (lambda (p)
+                      (match (mime-parse-content-disposition
+                              (rfc822-header-ref (ref p'headers)
+                                                 "content-disposition"))
+                        [("form-data" ("name" . name))
+                         (list name (ref p'content))]
+                        [else
+                         (list (ref p'headers) (ref p'content))]))
+                    (ref part'content))))))
+
+  (tester "(new API)"
+          (lambda ()
+            (http-request 'POST #`"localhost:,*http-port*" "/post"
+                          :sender
+                          (http-multipart-sender '(("a" "b") ("c" "d"))))))
+  (tester "(old API)"
+          (lambda ()
+            (http-post #`"localhost:,*http-port*" "/post"
+                       '(("a" "b") ("c" "d")))))
+  )
 
 (test* "<http-error>" #t
        (guard (e (else (is-a? e <http-error>)))
-         (http-get #`"localhost:,*http-port*" "/exit")))
+         (http-request 'GET #`"localhost:,*http-port*" "/exit")))
 
 (sys-waitpid -1)
 
