@@ -1,10 +1,8 @@
-;; -*- coding: shift-jis -*-
-
-;; $Id: sjis.scm,v 1.10 2007-03-02 07:20:42 shirok Exp $
+;; -*- coding: shift_jis -*-
 
 (use gauche.test)
 
-(test-start "SJIS")
+(test-start "multibyte (sjis)")
 (use srfi-1)
 
 ;;-------------------------------------------------------------------
