@@ -2,8 +2,6 @@
 ;; Test for SRFIs
 ;;
 
-;; $Id: srfi.scm,v 1.45 2007-04-05 05:56:58 shirok Exp $
-
 (use gauche.test)
 
 (test-start "SRFIs")
@@ -134,7 +132,7 @@
                (code (and-let* ((x (circular-list? foo)))
                        (take foo 3))))
              (make-module #f)))
-(test* "program (requires, no such feature)" *test-error*
+(test* "program (requires, no such feature)" (test-error)
        (eval '(program
                (requires no-such-feature))
              (make-module #f)))
@@ -175,7 +173,7 @@
                 (else (code (define x 2))))
                (code (+ x x)))
              (make-module #f)))
-(test* "program (feature-cond w/o else)" *test-error*
+(test* "program (feature-cond w/o else)" (test-error)
        (eval '(program
                (feature-cond
                 ((not srfi-1) (code (define x 5)))))
@@ -183,37 +181,29 @@
 
 (sys-system "rm -rf test.o")
 
-;;-----------------------------------------------------------------------
-(test-section "srfi-9")
-(use srfi-9)
-(test-module 'srfi-9)
-
-(define-record-type pare
-  (kons x y)
-  pare?
-  (x kar set-kar!)
-  (y kdr))
-
-(test* "pare kons" #t (pare? (kons 1 2)))
-(test* "pare kons" #f (pare? (cons 1 2)))
-(test* "pare kar" 1 (kar (kons 1 2)))
-(test* "pare kdr" 2 (kdr (kons 1 2)))
-(test* "pare set-kar!" 3 (let ((k (kons 1 2))) (set-kar! k 3) (kar k)))
-
-(define-record-type xpare
-  (xkons y x)
-  xpare?
-  (x kar)
-  (y kdr))
-
-(test* "xpare kons" '(1 . 2)
-       (let ((k (xkons 2 1))) (cons (kar k) (kdr k))))
-
 ;;-----------------------------------------------------------------------
 (test-section "srfi-14")
 (use srfi-14)
 (test-module 'srfi-14)
 
+;; char-set writer
+(let ()
+  (define (char-set-printer-tester p)
+    (test* "char-set-printer" (car p)
+           (write-to-string (list->char-set (cdr p)))))
+  (for-each char-set-printer-tester
+            '(("#[ace]" #\a #\e #\c)
+              ("#[ab]"  #\a #\b)
+              ("#[a-c]" #\a #\b #\c)
+              ("#[a-d]" #\a #\b #\c #\d)
+              ("#[a-ce]" #\a #\b #\c #\e)
+              ("#[acd]" #\a #\c #\d)
+              ("#[ac-e]" #\a #\c #\d #\e)
+              ("#[ac-e]" #\a #\c #\d #\e)
+              ("#[\\-\\[\\]]" #\[ #\] #\-)
+              ("#[\\^a]" #\^ #\a)
+              ("#[!^]" #\^ #\!))))
+
 ;; Test samples taken from Olin Shivers' test suite,
 ;; http://srfi.schemers.org/srfi-14/srfi-14-tests.scm
 ;; TODO: This doesn't test characters beyond ASCII.  See char-set.euc.scm.
@@ -367,6 +357,56 @@
        (char-set-contains? (->char-set "xyz") #\x))
 (test* "char-set-contains?" #f
        (char-set-contains? (->char-set "xyz") #\a))
+(test* "char-set-complement (ascii, nohit)" #f
+       (char-set-contains? (char-set-complement (char-set #\a)) #\a))
+(test* "char-set-complement (ascii, hit)" #t
+       (char-set-contains? (char-set-complement (char-set #\a)) #\b))
+(test* "char-set-complement (~#x80, nohit)" #f
+       (char-set-contains? (char-set-complement (char-set (integer->char #x80)))
+                           (integer->char #x80)))
+(test* "char-set-complement (~#x80, hit)" #t
+       (char-set-contains? (char-set-complement (char-set (integer->char #x80)))
+                           (integer->char #x81)))
+(test* "char-set-complement (~~#x80, hit)" #t
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x80))))
+                           (integer->char #x80)))
+(test* "char-set-complement (~~#x80, nohit)" #f
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x80))))
+                           (integer->char #x81)))
+(test* "char-set-complement (~#x82, nohit)" #f
+       (char-set-contains? (char-set-complement (char-set (integer->char #x82)))
+                           (integer->char #x82)))
+(test* "char-set-complement (~#x82, hit-upper)" #t
+       (char-set-contains? (char-set-complement (char-set (integer->char #x82)))
+                           (integer->char #x83)))
+(test* "char-set-complement (~#x82, hit-lower)" #t
+       (char-set-contains? (char-set-complement (char-set (integer->char #x82)))
+                           (integer->char #x81)))
+(test* "char-set-complement (~~#x82, hit)" #t
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x82))))
+                           (integer->char #x82)))
+(test* "char-set-complement (~~#x82, nohit-upper)" #f
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x82))))
+                           (integer->char #x83)))
+(test* "char-set-complement (~~#x82, nohit-lower)" #f
+       (char-set-contains? (char-set-complement
+                            (char-set-complement
+                             (char-set (integer->char #x82))))
+                           (integer->char #x81)))
+(test* "char-set-complement (~empty)" #t
+       (char-set-contains? (char-set-complement (char-set))
+                           (integer->char #x80)))
+(test* "char-set-complement (~~empty)" #f
+       (char-set-contains? (char-set-complement (char-set-complement (char-set)))
+                           (integer->char #x80)))
 (test* "char-set-every" #t
        (char-set-every char-lower-case? (->char-set "abcd")))
 (test* "char-set-every" #f
@@ -464,33 +504,7 @@
                 (char-set= i (->char-set "abcdefABCDEF"))))))
 
 ;;-----------------------------------------------------------------------
-(test-section "srfi-16")
-
-(test* "case-lambda (plus)" '(0 1 3 6 10)
-       (let ()
-         (define plus
-           (case-lambda 
-            (() 0)
-            ((x) x)
-            ((x y) (+ x y))
-            ((x y z) (+ (+ x y) z))
-            (args (apply + args))))
-         (list (plus) (plus 1) (plus 1 2) (plus 1 2 3) (plus 1 2 3 4))))
-
-(test* "case-lambda (mul2)" *test-error*
-       (let ()
-         (define mul2
-           (case-lambda 
-            (() 1)
-            ((x) x)
-            ((x y) (* x y))))
-         (mul2 1 2 3)))
-
-(test* "case-lambda (matching order)" '(1 (2))
-       ((case-lambda
-         ((x . y) (list x y))
-         ((x y) (list x y)))
-        1 2))
+;; srfi-16 case-lambda : moved to procedure.scm (builtin)
 
 ;;-----------------------------------------------------------------------
 (test-section "srfi-17")
@@ -1263,4 +1277,56 @@
            (read-lines "tmp1.o")))
   )
 
+;;-----------------------------------------------------------------------
+(test-section "srfi-60")
+
+(use srfi-60)
+(test-module 'srfi-60)
+
+;; Most procedures are builtin and tested in test/numbers.scm.
+
+(test* "bitwise-if" #b01100110
+       (bitwise-if #b10101100 #b00110101 #b11001010))
+
+(for-each (^(n r) (test* (format "log2-binary-factors(~a)" n) r
+                         (log2-binary-factors n)))
+          '(0  1  2  3  4  5  6  7  8  9  10  11  12  13  14  15  16
+              -1 -2 -3 -4 -5 -6 -7 -8 -9 -10 -11 -12 -13 -14 -15 -16)
+          '(-1 0  1  0  2  0  1  0  3  0   1   0   2   0   1   0   4
+               0  1  0  2  0  1  0  3  0   1   0   2   0   1   0   4))
+
+(define (test-rotate-bit-field n c s e r)
+  (test* (format "rotate-bit-field(~s,~a,~a,~a)" (number->string n 2) c s e) r
+         (number->string (rotate-bit-field n c s e) 2)))
+(test-rotate-bit-field #b0100 3 4 0 "100") ; trivial path
+(test-rotate-bit-field #b0100 3 0 4 "10")
+(test-rotate-bit-field #b0100 -1 0 4 "10")
+(test-rotate-bit-field #b0100 10 0 4 "1")
+(test-rotate-bit-field #b110100100010000 -1 5 9 "110100010010000")
+(test-rotate-bit-field #b110100100010000 1 5 9 "110100000110000")
+
+(define (test-reverse-bit-field n s e r)
+  (test* (format "reverse-bit-field(~s,~a,~a)" (number->string n 2) s e) r
+         (number->string (reverse-bit-field n s e) 2)))
+(test-reverse-bit-field #xa7 8 0 "10100111")
+(test-reverse-bit-field #xa7 0 8 "11100101")
+(test-reverse-bit-field #xa7 1 5 "10111001")
+
+(test* "integer->list" '(#t #f #f #t) (integer->list 9))
+(test* "integer->list" '(#f #f #t #f #f #t) (integer->list 9 6))
+(test* "list->integer" 9 (list->integer '(#t #f #f #t)))
+(test* "list->integer" 9 (list->integer '(#f #f #t #f #f #t)))
+;; tests bignum path
+(test* "list->integer" (+ (expt 2 63) (expt 2 62) (expt 2 31) (expt 2 30) 1)
+       (list->integer '(#t #t #f #f #f #f #f #f
+                        #f #f #f #f #f #f #f #f
+                        #f #f #f #f #f #f #f #f
+                        #f #f #f #f #f #f #f #f
+                        #t #t #f #f #f #f #f #f
+                        #f #f #f #f #f #f #f #f
+                        #f #f #f #f #f #f #f #f
+                        #f #f #f #f #f #f #f #t)))
+
+(test* "booleans->integer" 9 (booleans->integer #f #f #t #f #f #t))
+ 
 (test-end)
