@@ -405,7 +405,8 @@
    (else (sys-sleep 1))))
 
 (cond-expand
- ((not gauche.os.windows)  ;; win32 doesn't support fork at all.
+ ((and (not gauche.os.windows)  ;; win32 doesn't support fork at all.
+       (not gauche.os.cygwin))	;; cygwin's fork is not reliable.
   (test* "fork & wait" #t
          (let ((pid (sys-fork)))
            (if (= pid 0)
@@ -493,6 +494,33 @@
                  (sys-sigmask SIG_SETMASK omask)
                  (sys-waitpid pid)
                  #t)))))
+
+  ;; Testing fork&exec and detached process
+  ;; NB: these tests assume we're running the testing gosh in the
+  ;; current directory.
+  (cmd-rmrf "test.out")
+  (with-output-to-file "test.out"
+    (lambda ()
+      (write '(begin
+                (sys-nanosleep #e2e8)
+                (write (list (sys-getpid) (sys-getppid) (sys-getpgrp)))))))
+  (receive (in out) (sys-pipe :buffering :none)
+    (define (run-and-read detached)
+      (let1 pid (sys-fork-and-exec "./gosh"
+                                   `("./gosh" "-ftest" "./test.out")
+                                   :iomap `((1 . ,out))
+                                   :detached detached)
+        (begin0 (read in) (sys-waitpid pid))))
+    (test* "fork, exec and detached process (not detached)"
+           `(parent ,(sys-getpid) pgrp ,(sys-getpgrp))
+           (let1 r (run-and-read #f)
+             `(parent ,(cadr r) pgrp ,(caddr r))))
+    (test* "fork, exec and detached process (detached)"
+           `(parent 1 pgrp #t)
+           (let1 r (run-and-read #t)
+             `(parent ,(cadr r) pgrp ,(= (car r) (caddr r)))))
+    )
+  (cmd-rmrf "test.out")
   ) ; !gauche.os.windows
  (else))
 
@@ -574,7 +602,7 @@
 (test-section "signal handling")
 
 (cond-expand
- ((not gauche.os.windows)
+ [(not gauche.os.windows)
 
   (test* "sigalrm1" SIGALRM
          (call/cc
@@ -596,7 +624,7 @@
                (sys-alarm 1)
                (sys-pause))))))
 
-  (test* "sigalrm3" *test-error*
+  (test* "sigalrm3" (test-error)
          (call/cc
           (lambda (k)
             (with-signal-handlers
@@ -667,7 +695,8 @@
 
   ;; NB: on cygwin (as of 1.5.25), sigmask doesn't work reliably so
   ;; we skip test for now.
-  (unless (string-contains (gauche-architecture) "-cygwin")
+  (cond-expand
+   [(not gauche.os.cygwin)
     (test* "sigmask" 'hup
            (let ((sig #f)
                  (chld #f)
@@ -735,12 +764,13 @@
                                  (sys-sigset) SIGHUP))
 
       (set-signal-handler! SIGINT #f)
-      )
-    ) ;; !cygwin
+      )]
+   [else]);; !cygwin
 
   ;; sys-sigwait
   (cond-expand
-   (gauche.sys.sigwait
+   [(and gauche.sys.sigwait
+         (not gauche.os.cygwin))
     (let ()
       (define z (lambda (n) (raise 'foo)))
       
@@ -770,10 +800,11 @@
       (test* "sys-sigwait / signal handler restoration" 'foo
              (guard (e (else e))
                (sys-kill (sys-getpid) SIGINT))))
-    )) ;; gauche.sys.sigwait
+    ]
+   [else]) ; (not (and gauche.sys.sigwait (not gauche.os.cygwin)))
 
-  ) ;; (not gauche.os.windows)
- (else))
+  ] 
+ [else]) ; gauche.os.windows
 
 (test-end)
 
